# The LearningOnline Network
# Cookie Based Access Handler
#
# $Id: lonacc.pm,v 1.46 2003/05/13 00:52:46 www Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
# YEAR=1999
# 5/21/99,5/22,5/29,5/31,6/15,16/11,22/11,
# YEAR=2000
# 01/06,01/13,05/31,06/01,09/06,09/25,09/28,10/30,11/6,
# 12/25,12/26,
# YEAR=2001
# 01/06/01,05/28,8/11,9/26,11/29 Gerd Kortemeyer
# YEAR=2002
# 1/4,2/25 Gerd Kortemeyer
#
###

package Apache::lonacc;

use strict;
use Apache::Constants qw(:common :http :methods);
use Apache::File;
use Apache::lonnet;
use Apache::loncommon();
use CGI::Cookie();
use Fcntl qw(:flock);

sub handler {
    my $r = shift;
    my $requrl=$r->uri;
    my %cookies=CGI::Cookie->parse($r->header_in('Cookie'));
    my $lonid=$cookies{'lonID'};
    my $cookie;
    if ($lonid) {
	my $handle=$lonid->value;
        $handle=~s/\W//g;
        my $lonidsdir=$r->dir_config('lonIDsDir');
        if ((-e "$lonidsdir/$handle.id") && ($handle ne '')) {

# ------------------------------------------------------ Initialize Environment

            &Apache::lonnet::transfer_profile_to_env($lonidsdir,$handle);

# -------------------------------------------------------------- Resource State

            if ($requrl=~/^\/res\//) {
               $ENV{'request.state'} = "published";
	    } else {
	       $ENV{'request.state'} = 'unknown';
            }
            $ENV{'request.filename'} = $r->filename;

# -------------------------------------------------------- Load POST parameters

	    &Apache::loncommon::get_posted_cgi($r);

# ---------------------------------------------------------------- Check access

            if ($requrl!~/^\/adm|public|prtspool\//) {
		my $access=&Apache::lonnet::allowed('bre',$requrl);
                if ($access eq '1') {
		   $ENV{'user.error.msg'}="$requrl:bre:0:0:Choose Course";
	           return HTTP_NOT_ACCEPTABLE; 
                }
                if (($access ne '2') && ($access ne 'F')) {
		   $ENV{'user.error.msg'}="$requrl:bre:1:1:Access Denied";
	           return HTTP_NOT_ACCEPTABLE; 
                }
            }
	    if ($requrl =~ m|^/prtspool/|) {
		my $start='/prtspool/'.$ENV{'user.name'}.'_'.
		    $ENV{'user.domain'};
		if ($requrl !~ /^\Q$start\E/) {
		    $ENV{'user.error.msg'}="$requrl:bre:1:1:Access Denied";
		    return HTTP_NOT_ACCEPTABLE;
		}
	    }
# ------------------------------------------------------------- This is allowed
          if ($ENV{'request.course.id'}) {
	    &Apache::lonnet::countacc($requrl);
            $requrl=~/\.(\w+)$/;
            if ((&Apache::loncommon::fileembstyle($1) eq 'ssi') ||
 ($requrl=~/^\/adm\/.*\/(aboutme|navmaps|smppg|bulletinboard)(\?|$)/) ||
 ($requrl=~/^\/adm\/wrapper\//) ||
 ($requrl=~/^\/public\/.*\/syllabus$/)) {
# ------------------------------------- This is serious stuff, get symb and log
		my $query=$r->args;
                my $symb;
                if ($query) {
		    &Apache::loncommon::get_unprocessed_cgi($query,['symb']);
                }
                if ($ENV{'form.symb'}) {
		    $symb=&Apache::lonnet::symbclean($ENV{'form.symb'});
                    if (&Apache::lonnet::symbverify($symb,$requrl)) {
                      my ($map,$mid,$murl)=split(/\_\_\_/,$symb);
                      &Apache::lonnet::symblist($map,$murl => $mid,
                                               'last_known' => $murl);
		    } else {
			$r->log_reason('Invalid symb for '.$requrl.': '.
                                       $symb);
		        $ENV{'user.error.msg'}=
                                "$requrl:bre:1:1:Invalid Access";
  	                return HTTP_NOT_ACCEPTABLE; 
                    }
                } else {
	            $symb=&Apache::lonnet::symbread($requrl);
                    my ($map,$mid,$murl)=split(/\_\_\_/,$symb);
		    &Apache::lonnet::symblist($map,$murl => $mid,
                                              'last_known' => $murl);
                }
                $ENV{'request.symb'}=$symb;
                &Apache::lonnet::courseacclog($symb);
            } else {
# ------------------------------------------------------- This is other content
                &Apache::lonnet::courseacclog($requrl);    
            }
	  }
            return OK; 
        } else { 
            $r->log_reason("Cookie $handle not valid", $r->filename) 
        };
    }

# -------------------------------------------- See if this is a public resource
    if ($requrl=~m|^/public/|
	|| (&Apache::lonnet::metadata($requrl,'copyright') eq 'public')) {
        &Apache::lonnet::logthis('Granting public access: '.$requrl);
	my $buffer;
	$r->read($buffer,$r->header_in('Content-length'),0);
	&Apache::loncommon::get_unprocessed_cgi($buffer);
	$ENV{'user.name'}='public';
        $ENV{'user.domain'}='public';
        $ENV{'request.state'} = "published";
        $ENV{'request.publicaccess'} = 1;
        $ENV{'request.filename'} = $r->filename;
        return OK;
    }
# -------------------------------------------------------------- Not authorized
    $requrl=~/\.(\w+)$/;
    if ((&Apache::loncommon::fileembstyle($1) eq 'ssi') ||
        ($requrl=~/^\/adm\/(roles|logout|email|menu|remote)/) ||
        ($requrl=~m|^/prtspool/|)) {
# -------------------------- Store where they wanted to go and get login screen
	$ENV{'request.querystring'}=$r->args;
	$ENV{'request.firsturl'}=$requrl;
       return FORBIDDEN;
   } else {
# --------------------------------------------------------------------- Goodbye
       return HTTP_BAD_REQUEST;
   }
}

1;
__END__

=head1 NAME

Apache::lonacc - Cookie Based Access Handler

=head1 SYNOPSIS

Invoked (for various locations) by /etc/httpd/conf/srm.conf:

 PerlAccessHandler       Apache::lonacc

=head1 INTRODUCTION

This module enables cookie based authentication and is used
to control access for many different LON-CAPA URIs.

Whenever the client sends the cookie back to the server, 
this cookie is handled by either lonacc.pm or loncacc.pm
(see srm.conf for what is invoked when).  If
the cookie is missing or invalid, the user is re-challenged
for login information.

This is part of the LearningOnline Network with CAPA project
described at http://www.lon-capa.org.

=head1 HANDLER SUBROUTINE

This routine is called by Apache and mod_perl.

=over 4

=item *

transfer profile into environment

=item *

load POST parameters

=item *

check access

=item *

if allowed, get symb, log, generate course statistics if applicable

=item *

otherwise return error

=item *

see if public resource

=item *

store attempted access

=back

=cut
