# The LearningOnline Network
# Login Screen
#
# $Id: lonlogin.pm,v 1.44 2003/08/05 12:52:23 www Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
# 5/21/99,5/22,5/25,5/26,5/31,6/2,6/10,7/12,7/14,
# 1/14/00,5/29,5/30,6/1,6/29,7/1,11/9,
# 1/17/01 Gerd Kortemeyer
#
# 2/7/02,2/8,2/12,2/14,2/15,2/19 Josh Brunskole
# 
# 7/10/02 Jeremy Bowers

package Apache::lonlogin;

use strict;
use Apache::Constants qw(:common);
use Apache::File ();
use Apache::lonnet();
use Apache::loncommon();

sub handler {
    my $r = shift;
    $r->content_type('text/html');
    &Apache::loncommon::no_cache($r);
    $r->send_http_header;
    return OK if $r->header_only;


    &Apache::loncommon::get_unprocessed_cgi
     ($ENV{'QUERY_STRING'}.'&'.$ENV{'request.querystring'},
      ['interface','username','domain','firsturl','localpath','localres']);
 
# ----------------------------------------------------------- Process Interface
    $ENV{'form.interface'}=~s/\W//g;

    my $textbrowsers=$r->dir_config('lonTextBrowsers');
    my $httpbrowser=$ENV{"HTTP_USER_AGENT"};
    
    foreach (split(/\:/,$textbrowsers)) {
	if ($httpbrowser=~/$_/i) {
	    $ENV{'form.interface'}='textual';
        }
    }

    my $fullgraph=($ENV{'form.interface'} ne 'textual');
    my $port_to_use=$r->dir_config('lonhttpdPort');
    if (!defined($port_to_use)) {
	$port_to_use='8080';
    }
    my $iconpath= 'http://'.$ENV{'HTTP_HOST'}.':'.$port_to_use.
                  $r->dir_config('lonIconsURL');
    my $domain  = $r->dir_config('lonDefDomain');
    my $role    = $r->dir_config('lonRole');
    my $loadlim = $r->dir_config('lonLoadLim');
    my $servadm = $r->dir_config('lonAdmEMail');
    my $sysadm  = $r->dir_config('lonSysEMail');
    my $lonhost = $r->dir_config('lonHostID');
    my $tabdir  = $r->dir_config('lonTabDir');
    my $include = $r->dir_config('lonIncludes');
    my $expire  = $r->dir_config('lonExpire');
    my $version = $r->dir_config('lonVersion');

# --------------------------------------------- Default values for login fields

    my $authusername=($ENV{'form.username'}?$ENV{'form.username'}:'');
    my $authdomain=($ENV{'form.domain'}?$ENV{'form.domain'}:$domain);

# ---------------------------------------------------------- Determine own load
    my $loadavg;
    {
	my $loadfile=Apache::File->new('/proc/loadavg');
	$loadavg=<$loadfile>;
    }
    $loadavg =~ s/\s.*//g;
    my $loadpercent=100*$loadavg/$loadlim;
    my $userloadpercent=&Apache::lonnet::userload();

# ------------------------------------------------------- Do the load balancing
    my $otherserver='http://'.$ENV{'SERVER_NAME'};
    my $firsturl=
    ($ENV{'request.firsturl'}?$ENV{'request.firsturl'}:$ENV{'form.firsturl'});
# ---------------------------------------- Are we access server and overloaded?
    if (($role eq 'access') &&
	(($userloadpercent>100.0)||($loadpercent>100.0))) {
        $otherserver=Apache::lonnet::spareserver($loadpercent,$userloadpercent);
    }

# -------------------------------------------------------- Set login parameters

    my @hexstr=('0','1','2','3','4','5','6','7',
                '8','9','a','b','c','d','e','f');
    my $lkey='';
    for (0..7) {
        $lkey.=$hexstr[rand(15)];
    }

    my $ukey='';
    for (0..7) {
        $ukey.=$hexstr[rand(15)];
    }

    my $lextkey=hex($lkey);
    if ($lextkey>2147483647) { $lextkey-=4294967296; }

    my $uextkey=hex($ukey);
    if ($uextkey>2147483647) { $uextkey-=4294967296; }

# -------------------------------------------------------- Store away log token
    my $logtoken=Apache::lonnet::reply(
       'tmpput:'.$ukey.$lkey.'&'.$firsturl,
       $lonhost);

# ------------------- If we cannot talk to ourselves, we are in serious trouble

    if ($logtoken eq 'con_lost') {
        my $spares='';
        foreach (keys %Apache::lonnet::hostname) {
            if ($_ ne $lonhost) {
               $spares.='<br /><a href="http://'.$Apache::lonnet::hostname{$_}.
		 '/adm/login?domain='.$authdomain.'">'.
                 $Apache::lonnet::hostname{$_}.'</a>';
               if ($Apache::lonnet::spareid{$_}) {
		   $spares.=' (preferred)';
               }
	   }
        }
	$r->print(<<ENDTROUBLE);
<html>
<head><title>The LearningOnline Network with CAPA</title></head>
<body bgcolor="#FFFFFF">
<img src="/adm/lonKaputt/lonlogo_broken.gif" align="right" />
<h3>This LON-CAPA server is temporarily not available for login</h3>
<p>Please attempt to login to one of the following servers:</p>$spares
<p>If the problem persists, please contact <tt>$servadm</tt>.</p>
</body>
</html>
ENDTROUBLE
        return OK;
    }

# ----------------------------------------------- Apparently we are in business

    my $domainlogo=&Apache::loncommon::domainlogo();
    $servadm=~s/\,/\<br \/\>/g;
    $sysadm=~s/\,/\<br \/\>/g;

# --------------------------------------------------- Print login screen header
    $r->print(<<ENDHEADER);
<html>
<head>
<meta HTTP-EQUIV="Refresh" CONTENT="$expire; url=/adm/roles" />
<title>The LearningOnline Network with CAPA Login</title>
</head>
ENDHEADER
# ---------------------------------------------------- Serve out DES JavaScript
    {
	my $jsh=Apache::File->new($include."/londes.js");
        $r->print(<$jsh>);
    }

# ----------------------------------------------------------- Front page design
    my $pgbg=
      ($fullgraph?&Apache::loncommon::designparm('login.pgbg'):'#FFFFFF');
    my $font=
      ($fullgraph?&Apache::loncommon::designparm('login.font'):'#000000');
    my $link=
      ($fullgraph?&Apache::loncommon::designparm('login.link'):'#0000FF');
    my $vlink=
      ($fullgraph?&Apache::loncommon::designparm('login.vlink'):'#0000FF');
    my $alink=&Apache::loncommon::designparm('login.alink');
    my $mainbg=
      ($fullgraph?&Apache::loncommon::designparm('login.mainbg'):'#FFFFFF');
    my $sidebg=
      ($fullgraph?&Apache::loncommon::designparm('login.sidebg'):'#FFFFFF');
    my $logo=&Apache::loncommon::designparm('login.logo');
    my $img=&Apache::loncommon::designparm('login.img');


# ---------------------------------------------------------- Serve rest of page
    $r->print(<<ENDSCRIPT);

<body bgcolor="$pgbg" text="$font" link="$link" vlink="$vlink" alink="$alink"
  topmargin=0 leftmargin=0 marginwidth=0 marginheight=0>

 <script language="JavaScript">
    function send()
    {
	this.document.server.elements.uname.value
       =this.document.client.elements.uname.value;

        this.document.server.elements.udom.value
       =this.document.client.elements.udom.value;

        this.document.server.elements.imagesuppress.value
       =this.document.client.elements.imagesuppress.checked;

        this.document.server.elements.embedsuppress.value
       =this.document.client.elements.embedsuppress.checked;

        this.document.server.elements.appletsuppress.value
       =this.document.client.elements.appletsuppress.checked;

        this.document.server.elements.fontenhance.value
       =this.document.client.elements.fontenhance.checked;

        this.document.server.elements.blackwhite.value
       =this.document.client.elements.blackwhite.checked;

        this.document.server.elements.remember.value
       =this.document.client.elements.remember.checked;

        uextkey=this.document.client.elements.uextkey.value;
        lextkey=this.document.client.elements.lextkey.value;
        initkeys();

        this.document.server.elements.upass.value
	    =crypted(this.document.client.elements.upass.value);

        this.document.server.submit();
	return false;
    }
 </script>
ENDSCRIPT

    if ($fullgraph) {
	$r->print(
		  '<table width="100%" cellpadding=0 cellspacing=0 border=0>');
    }

    $r->print(<<ENDSERVERFORM);
  <form name="server" action="$otherserver/adm/authenticate" method="post" target="_top">
   <input type="hidden" name="logtoken" value="$logtoken" />
   <input type="hidden" name="serverid" value="$lonhost" />
   <input type="hidden" name="interface" value="$ENV{'form.interface'}" />
   <input type="hidden" name="uname" value="" />
   <input type="hidden" name="upass" value="" />
   <input type="hidden" name="udom" value="" />
   <input type="hidden" name="imagesuppress"  value="" />
   <input type="hidden" name="appletsuppress"  value="" />
   <input type="hidden" name="embedsuppress"  value="" />
   <input type="hidden" name="fontenhance"  value="" />
   <input type="hidden" name="blackwhite"  value="" />
   <input type="hidden" name="remember"  value="" />
   <input type="hidden" name="localpath" value="$ENV{'form.localpath'}" />
   <input type="hidden" name="localres" value="$ENV{'form.localres'}" />
  </form>
ENDSERVERFORM
    if ($fullgraph) { $r->print(<<ENDTOP);
  <!-- The LON-CAPA Header -->
  <tr>

   <!-- Row 1 Columns 2-4 -->
   <td width="100%" height=75 colspan=4 align="left" valign="top" bgcolor="$pgbg"><img src="$img" border=0 alt="The Learning Online Network with CAPA" /></td>
  </tr>

  <!-- The gray bar that starts the two table frames -->
  <tr>

   <!-- Row 2 Column 1 -->
   <td width=182 height=27 bgcolor="$sidebg">&nbsp;</td>

   <!-- Row 2 Column 2 -->
   <td width=27 height=27 align="left" background="$iconpath/filltop.gif"><img src="$iconpath/upperleft.gif" border=0 alt="" /></td>

   <!-- Row 2 Column 3 -->
   <td height=27 background="$iconpath/filltop.gif"><img src="$iconpath/filltop.gif" alt="" /></td>

   <!-- Row 2 Column 4 -->
   <td width=27 height=27 align="right" background="$iconpath/filltop.gif"><img src="$iconpath/upperright.gif" border=0 alt="" /></td>
  </tr>
  <tr>
   
   <!-- A cell that will hold the 'access' and 'about' buttons -->
   <!-- Row 3 Column 1 -->
   <td valign="top" height=60 align="center" bgcolor="$sidebg">
    <a href="/adm/login?interface=textual"><img src="$iconpath/accessbutton.gif" border=0 alt="Accessibility Options" /></a>
    <br />
    <a href="/adm/about.html"><img src="$iconpath/aboutlon.gif" border=0 alt="About LON-CAPA" /></a>
   </td>

   <!-- The shaded space between the two main columns -->
   <!-- Row 3 Column 2 -->
   <td width=27 height=60 background="$iconpath/fillleft.gif"><img src="$iconpath/fillleft.gif" alt="" /></td>

   <!-- The right main column holding the large LON-CAPA logo-->
   <!-- Rows 3-4 Column 3 -->
   <td align="center" valign="top" width="100%" height="100%" rowspan=2 bgcolor="$mainbg">
    <center>
     <img src="$logo" alt="" />
    </center>
   </td>

   <!-- Row 3 Column 4 -->
   <td width=27 background="$iconpath/fillright.gif"><img src="$iconpath/fillright.gif" alt="" /></td>
  </tr>
  <tr>

   <!-- The entry form -->
   <!-- Row 4 Column 1 -->
   <td align="center" valign="middle" bgcolor="$sidebg">
ENDTOP
} else {
    $r->print('<h1>The Learning<i>Online</i> Network with CAPA</h1><h2>Text-based Interface Login</h2>');
}
    $r->print('<form name="client" onsubmit="return(send())">');
    unless ($fullgraph) {
        $r->print(<<ENDACCESSOPTIONS);
<h3>Select Accessibility Options</h3>
<input type="checkbox" name="imagesuppress" /> Suppress rendering of images<br />
<input type="checkbox" name="appletsuppress" /> Suppress Java applets<br />
<input type="checkbox" name="embedsuppress" /> Suppress rendering of embedded multimedia<br />
<input type="checkbox" name="fontenhance" /> Increase font size<br />
<input type="checkbox" name="blackwhite" /> Switch to black and white mode<br />
<p>If you have accessibility needs that are not addressed by this interface, 
please
contact the system administrator at <tt>$sysadm</tt>.</p><br />
<input type="checkbox" name="remember" /> Remember these settings for next login<hr />
ENDACCESSOPTIONS
} else {
    $r->print(<<ENDNOOPT);
<input type="hidden" name="imagesuppress"  value="" />
<input type="hidden" name="embedsuppress"  value="" />
<input type="hidden" name="appletsuppress"  value="" />
<input type="hidden" name="fontenhance"  value="" />
<input type="hidden" name="blackwhite"  value="" />
<input type="hidden" name="remember"  value="" />
ENDNOOPT
}
    $r->print(<<ENDLOGIN);
     <input type="hidden" name="lextkey" value="$lextkey">
     <input type="hidden" name="uextkey" value="$uextkey">

     <!-- Start the sub-table for text and input alignment -->
     <table border=0 cellspacing=0 cellpadding=0>
      <tr><td bgcolor="$sidebg" colspan=2><img src="$iconpath/userauthentication.gif" alt="User Authentication" /></td></tr>
      <tr>
       <td bgcolor="$mainbg"><br /><font size=-1><b>&nbsp;&nbsp;&nbsp;User Name:</b></font></td>
       <td bgcolor="$mainbg"><br /><input type="text" name="uname" size="10" value="$authusername" /></td>
      </tr>
      <tr>
       <td bgcolor="$mainbg"><font size=-1><b>&nbsp;&nbsp;&nbsp;Password:</b></font></td>
       <td bgcolor="$mainbg"><input type="password" name="upass" size="10" /></td>
      </tr>
      <tr>
       <td bgcolor="$mainbg"><font size=-1><b>&nbsp;&nbsp;&nbsp;Domain:</b></font></td>
       <td bgcolor="$mainbg"><input type="text" name="udom" size="10" value="$authdomain" /></td>
      </tr>
      <tr>
       <td bgcolor="$mainbg">&nbsp;&nbsp;&nbsp;<a href="/adm/loginproblems.html">Help</a></td>
       <td bgcolor="$mainbg" valign="bottom" align="center">
        <br />
        <input type="submit" value="Log In" />
       </td>
      </tr>
     </table>
     <!-- End sub-table -->
    </form>
ENDLOGIN
    if ($fullgraph) {
	$r->print(<<ENDDOCUMENT);
   </td>

   <!-- Row 4 Column 2 -->
   <td width=27 background="$iconpath/fillleft.gif"><img src="$iconpath/fillleft.gif" alt="" /></td>

   <!-- Row 4 Column 4 -->
   <td width=27 background="$iconpath/fillright.gif"><img src="$iconpath/fillright.gif" alt="" /></td>
  </tr>
  <tr>

   <!-- Row 5 Column 1 -->
   <td bgcolor="$sidebg" valign="middle" align="left">
     <br />
     <table border=0 cellspacing=0 cellpadding=0>
      <tr>
       <td bgcolor="$sidebg" align="left" valign="top">
        <small><b>&nbsp;&nbsp;&nbsp;Domain:&nbsp;</b></small>
       </td>
       <td bgcolor="$sidebg" align="left" valign="top">
        <small><tt>&nbsp;$domain</tt></small>
       </td>
      </tr>
      <tr>
       <td bgcolor="$sidebg" align="left" valign="top">
        <small><b>&nbsp;&nbsp;&nbsp;Server:&nbsp;</b></small>
       </td>
       <td bgcolor="$sidebg" align="left" valign="top">
        <small><tt>&nbsp;$lonhost ($role)</tt></small>
       </td>
      </tr>
      <tr>
       <td bgcolor="$sidebg" align="left" valign="top">
        <small><b>&nbsp;&nbsp;&nbsp;Load:&nbsp;</b></small>
       </td>
       <td bgcolor="$sidebg" align="left" valign="top">
        <small><tt>&nbsp;$loadpercent percent</tt></small>
       </td>
      </tr>
      <tr>
       <td bgcolor="$sidebg" align="left" valign="top">
        <small><b>&nbsp;&nbsp;&nbsp;User&nbsp;Load:&nbsp;</b></small>
       </td>
       <td bgcolor="$sidebg" align="left" valign="top">
        <small><tt>&nbsp;$userloadpercent percent</tt></small>
       </td>
      </tr>
     </table>
     <br />
    <small>
     <b>&nbsp;&nbsp;&nbsp;System Administration:</b><br />
     <tt>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;$sysadm</tt><br />
     <b>&nbsp;&nbsp;&nbsp;Server Administration:</b><br />
     <tt>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;$servadm</tt><br />&nbsp;<br />
     &nbsp;&nbsp;&nbsp;$version
    </small>
   </td>

   <!-- Row 5 Column 2 -->
   <td width=27 background="$iconpath/fillleft.gif"><img src="$iconpath/fillleft.gif" alt="" /></td>

   <!-- Row 5 Column 3 -->
   <td width="100%" valign="bottom" bgcolor="$mainbg">
$domainlogo
</td>

   <!-- Row 5 Column 4 -->
   <td width=27 background="$iconpath/fillright.gif"><img src="$iconpath/fillright.gif" alt="" /></td>
  </tr>
  <tr>

   <!-- Row 6 Column 1 -->
   <td bgcolor="$sidebg">&nbsp;</td>

   <!-- Row 6 Column 2 -->
   <td align="left" background="$iconpath/fillbottom.gif"><img src="$iconpath/lowerleft.gif" alt="" /></td>

   <!-- Row 6 Column 3 -->
   <td background="$iconpath/fillbottom.gif"><img src="$iconpath/fillbottom.gif" alt="" /></td>

   <!-- Row 6 Column 4 -->
   <td align="right" background="$iconpath/fillbottom.gif"><img src="$iconpath/lowerright.gif" alt="" /></td>
  </tr>
 </table>

<script>
// the if prevents the script error if the browser can't handle this
if ( document.client.uname ) { document.client.uname.focus(); }
</script>

ENDDOCUMENT
}
    $r->print('</body></html>');
    return OK;
} 

1;
__END__
