#!/usr/bin/perl
#
# The LearningOnline Network with CAPA
#
# Gets keywords from metadata database.
#
# YEAR=2001
#
# YEAR=2002
#

###############################################################################
##                                                                           ##
## ORGANIZATION OF THIS PERL CGI SCRIPT                                      ##
##                                                                           ##
## 1. Status of this code                                                    ##
## 2. Purpose and description of program                                     ##
## 3. Modules used by this script                                            ##
## 4. Print MIME Content-type and other initialization                       ##
## 5. Make sure database can be accessed and that this is a library server   ##
## 6. Loop through database records and print out keywords                   ##
##                                                                           ##
###############################################################################

# --------------------------------------------------------- Status of this code
#
# 1=horrible 2=poor 3=fair 4=good 5=excellent
# Organization 5
# Functionality 4
# Has it been tested? 4
#

# ------------------------------------------ Purpose and description of program
#
# This program outputs one line per database entry.
# The line is to be a list of keywords separated by commas.
# The file is to be output as a text file on a browser (text/plain).
# This provides initial data by which to study common and uncommon
# keywords being used.
# Note that the authoritative copy of metadata "keywords" is in the
# .meta files that are native to the library server.  We rely
# on the assumption that it is okay to use the MySQL server (which
# should reflect this information) instead.  This is a speedier approach.

# ------------------------------------------------- Modules used by this script

use lib '/home/httpd/lib/perl/';
use LONCAPA::Configuration;

use strict;
use DBI;

# ---------------------------- Print MIME Content-type and other initialization
$|=1;
print 'Content-type: text/plain'."\n\n";

# --- Make sure that database can be accessed and that this is a library server
# library server test

# By default, loncapa_apache.conf is also read by the read_conf subroutine.
my $perlvarref=LONCAPA::Configuration::read_conf('loncapa.conf');
my %perlvar=%{$perlvarref};
undef($perlvarref);

unless ($perlvar{'lonRole'} eq 'library') {
    print "This can only be run on a library server!\n";
    exit;
}
# database test
my $dbh;
{
    unless (
	    $dbh = DBI->connect("DBI:mysql:loncapa","www",
				$perlvar{'lonSqlAccess'},
				{ RaiseError =>0,PrintError=>0})
	    ) { 
	print "Cannot connect to database!\n";
	exit;
    }
}
%perlvar=(); # undefine it

# ------------------------ Loop through database records and print out keywords
my $sth=$dbh->prepare("select * from metadata");
$sth->execute();
my @row;
while (@row=$sth->fetchrow_array) {
    print $row[4]."\n";
}

# --------------------------------------------------- Close database connection
$dbh->disconnect();
