# The LearningOnline Network with CAPA
# The LON-CAPA Homework handler
#
# $Id: lonhomework.pm,v 1.138 2003/07/22 19:55:26 albertel Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
# Guy Albertelli
# 11/30 Gerd Kortemeyer
# 6/1,8/17,8/18 Gerd Kortemeyer
# 7/18 Jeremy Bowers

package Apache::lonhomework;
use strict;
use Apache::style();
use Apache::lonxml();
use Apache::lonnet();
use Apache::lonplot();
use Apache::inputtags();
use Apache::structuretags();
use Apache::randomlabel();
use Apache::response();
use Apache::hint();
use Apache::outputtags();
use Apache::caparesponse();
use Apache::radiobuttonresponse();
use Apache::optionresponse();
use Apache::imageresponse();
use Apache::essayresponse();
use Apache::externalresponse();
use Apache::rankresponse();
use Apache::matchresponse();
use Apache::chemresponse();
use Apache::Constants qw(:common);
use HTML::Entities();
use Apache::loncommon();
#use Time::HiRes qw( gettimeofday tv_interval );

BEGIN {
  &Apache::lonxml::register_insert();
}

sub get_target {
  if (($ENV{'request.state'} eq "published") ||
      ($ENV{'request.state'} eq "uploaded")) {
    if ( defined($ENV{'form.grade_target'}  ) 
	 && ($ENV{'form.grade_target'} eq 'tex')) {
      return ($ENV{'form.grade_target'});
    } elsif ( defined($ENV{'form.grade_target'}  ) 
	 && ($Apache::lonhomework::viewgrades eq 'F' )) {
      return ($ENV{'form.grade_target'});
    }
 
    if ( defined($ENV{'form.submitted'}) &&
	 ( !defined($ENV{'form.resetdata'})) &&
	 ( !defined($ENV{'form.newrandomization'}))) {
      return ('grade', 'web');
    } else {
      return ('web');
    }
  } elsif ($ENV{'request.state'} eq "construct") {
    if ( defined($ENV{'form.grade_target'}) ) {
      return ($ENV{'form.grade_target'});
    }
    if ( defined($ENV{'form.preview'})) {
      if ( defined($ENV{'form.submitted'})) {
	return ('grade', 'web');
      } else {
	return ('web');
      }
    } else {
      if ( $ENV{'form.problemmode'} eq 'View' ||
	   $ENV{'form.problemmode'} eq 'Discard Edits and View') {
	if ( defined($ENV{'form.submitted'}) &&
	     (!defined($ENV{'form.resetdata'})) &&
	     (!defined($ENV{'form.newrandomization'}))) {
	  return ('grade', 'web','answer');
	} else {
	  return ('web','answer');
	}
      } elsif ( $ENV{'form.problemmode'} eq 'Edit' ) {
	if ( $ENV{'form.submitted'} eq 'edit' ) {
	  if ( $ENV{'form.submit'} eq 'Submit Changes and View' ) {
	    return ('modified','web','answer');
	  } else {
	    return ('modified','edit');
	  }
	} else {
	  return ('edit');
	}
      } else {
	return ('web');
      }
    }
  }
  return ();
}

sub setup_vars {
  my ($target) = @_;
  return ';'
#  return ';$external::target='.$target.';';
}

sub send_header {
  my ($request)= @_;
  $request->print(&Apache::lontexconvert::header());
#  $request->print('<form name='.$ENV{'form.request.prefix'}.'lonhomework method="POST" action="'.$request->uri.'">');
}

sub createmenu {
  my ($which,$request)=@_;
  if ($which eq 'grade') {
    $request->print('<script language="JavaScript"> 
          hwkmenu=window.open("/res/adm/pages/homeworkmenu.html","homeworkremote",
                 "height=350,width=150,menubar=no");
          </script>');
  }
}

sub send_footer {
  my ($request)= @_;
#  $request->print('</form>');
  $request->print(&Apache::lontexconvert::footer());
}

$Apache::lonxml::browse='';

# JB, 9/24/2002: Any changes in this function may require a change
# in lonnavmaps::resource::getDateStatus.
sub check_access {
  my ($id) = @_;
  my $date ='';
  my $status;
  my $datemsg = '';
  my $lastdate = '';
  my $temp;
  my $type;
  my $passed;

  if ($ENV{'request.state'} eq "construct") {
    &Apache::lonxml::debug("in construction ignoring dates");
    $status='CAN_ANSWER';
    $datemsg='is in under construction';
    return ($status,$datemsg);
  }

  &Apache::lonxml::debug("checking for part :$id:");
  &Apache::lonxml::debug("time:".time);
  foreach $temp ("opendate","duedate","answerdate") {
    $lastdate = $date;
    $date = &Apache::lonnet::EXT("resource.$id.$temp");
    my $thistype = &Apache::lonnet::EXT("resource.$id.$temp.type");
    if ($thistype =~ /^(con_lost|no_such_host)/ ||
	$date     =~ /^(con_lost|no_such_host)/) {
	$status='UNAVAILABLE';
	$date="may open later.";
	return($status,$date);
    }
    if ($thistype eq 'date_interval') {
	if ($temp eq 'opendate') {
           $date=&Apache::lonnet::EXT("resource.$id.duedate")-$date;
        }
        if ($temp eq 'answerdate') {
           $date=&Apache::lonnet::EXT("resource.$id.duedate")+$date;
        }
    }
    &Apache::lonxml::debug("found :$date: for :$temp:");
    if ($date eq '') {
      $date = "an unknown date"; $passed = 0;
    } elsif ($date eq 'con_lost') {
      $date = "an indeterminate date"; $passed = 0;
    } else {
      if (time < $date) { $passed = 0; } else { $passed = 1; }
      $date = localtime $date;
    }
    if (!$passed) { $type=$temp; last; }
  }
  &Apache::lonxml::debug("have :$type:$passed:");
  if ($passed) {
    $status='SHOW_ANSWER';
    $datemsg=$date;
  } elsif ($type eq 'opendate') {
    $status='CLOSED';
    $datemsg = "will open on $date";
  } elsif ($type eq 'duedate') {
    $status='CAN_ANSWER';
    $datemsg = "is due at $date";
  } elsif ($type eq 'answerdate') {
    $status='CLOSED';
    $datemsg = "was due on $lastdate, and answers will be available on $date";
  }
  if ($status eq 'CAN_ANSWER') {
    #check #tries, and if correct.
    my $tries = $Apache::lonhomework::history{"resource.$id.tries"};
    my $maxtries = &Apache::lonnet::EXT("resource.$id.maxtries");
    if ( $tries eq '' ) { $tries = '0'; }
    if ( $maxtries eq '' ) { $maxtries = '2'; } 
    if ($tries >= $maxtries) { $status = 'CANNOT_ANSWER'; }
    # if (correct and show prob status) or excused then CANNOT_ANSWER
    if(($Apache::lonhomework::history{"resource.$id.solved"}=~/^correct/
	&&
	lc($Apache::lonhomework::problemstatus) ne 'no')
       ||
       $Apache::lonhomework::history{"resource.$id.solved"}=~/^excused/) {
	$status = 'CANNOT_ANSWER';
    }
  }

  #if (($status ne 'CLOSED') && ($Apache::lonhomework::type eq 'exam') &&
  #    (!$Apache::lonhomework::history{"resource.0.outtoken"})) {
  #    return ('UNCHECKEDOUT','needs to be checked out');
  #}


  &Apache::lonxml::debug("sending back :$status:$datemsg:");
  if (($Apache::lonhomework::browse eq 'F') && ($status eq 'CLOSED')) {
    &Apache::lonxml::debug("should be allowed to browse a resource when closed");
    $status='CAN_ANSWER';
    $datemsg='is closed but you are allowed to view it';
  }

  return ($status,$datemsg);
}

sub showhash {
  my (%hash) = @_;
  &showhashsubset(\%hash,'.');
  return '';
}

sub showarray {
    my ($array)=@_;
    my $string="(";
    foreach my $elm (@{ $array }) {
	if (ref($elm)) {
	    if ($elm =~ /ARRAY/ ) {
		$string.=&showarray($elm);
	    }
	} else {
	    $string.="$elm,"
	}
    }
    chop($string);
    $string.=")";
    return $string;
}

sub showhashsubset {
  my ($hash,$keyre) = @_;
  my $resultkey;
  foreach $resultkey (sort keys %$hash) {
    if ($resultkey =~ /$keyre/) {
      if (ref($$hash{$resultkey})) {
	if ($$hash{$resultkey} =~ /ARRAY/ ) {
	    &Apache::lonxml::debug("$resultkey ---- ".
				   &showarray($$hash{$resultkey}));
	} elsif ($$hash{$resultkey} =~ /HASH/ ) {
	    &Apache::lonxml::debug("$resultkey ---- $$hash{$resultkey}");
	    &showhashsubset($$hash{$resultkey},'.');
	} else {
	    &Apache::lonxml::debug("$resultkey ---- $$hash{$resultkey}");
	}
      } else {
	&Apache::lonxml::debug("$resultkey ---- $$hash{$resultkey}");
      }
    }
  }
  &Apache::lonxml::debug("\n<br />restored values^</br>\n");
  return '';
}

sub setuppermissions {
  $Apache::lonhomework::browse= &Apache::lonnet::allowed('bre',$ENV{'request.filename'});
  my $viewgrades = &Apache::lonnet::allowed('vgr',$ENV{'request.course.id'});
  if (! $viewgrades && 
      exists($ENV{'request.course.sec'}) && 
      $ENV{'request.course.sec'} !~ /^\s*$/) {
      $viewgrades = &Apache::lonnet::allowed('vgr',$ENV{'request.course.id'}.
                                               '/'.$ENV{'request.course.sec'});
  }
  $Apache::lonhomework::viewgrades = $viewgrades;
  return ''
}

sub setupheader {
    my $request=$_[0];
    if ($ENV{'browser.mathml'}) {
	$request->content_type('text/xml');
    } else {
	$request->content_type('text/html');
    }
    $request->content_encoding('UTF-8');
    if (!$Apache::lonxml::debug && ($ENV{'REQUEST_METHOD'} eq 'GET')) {
	&Apache::loncommon::no_cache($request);
    }
    $request->send_http_header;
    return OK if $request->header_only;
    return ''
}

sub handle_save_or_undo {
  my ($request,$problem,$result) = @_;
  my $file    = &Apache::lonnet::filelocation("",$request->uri);
  my $filebak =$file.".bak";
  my $filetmp =$file.".tmp";
  my $error=0;

  if ($ENV{'form.Undo'} eq 'undo') {
    my $error=0;
    if (!copy($file,$filetmp)) { $error=1; }
    if ((!$error) && (!copy($filebak,$file))) { $error=1; }
    if ((!$error) && (!move($filetmp,$filebak))) { $error=1; }
    if (!$error) {
      $request->print("<p><b>Undid changes, Switched $filebak and $file</b></p>");
    } else {
      $request->print("<p><font color=\"red\" size=\"+1\"><b>Unable to undo, unable to switch $filebak and $file</b></font></p>");
      $error=1;
    }
  } else {
    my $fs=Apache::File->new(">$filebak");
    if (defined($fs)) {
      print $fs $$problem;
      $request->print("<b>Making Backup to $filebak</b><br />");
    } else {
      $request->print("<font color=\"red\" size=\"+1\"><b>Unable to make backup $filebak</b></font>");
      $error=2;
    }
    my $fh=Apache::File->new(">$file");
    if (defined($fh)) {
      print $fh $$result;
      $request->print("<b>Saving Modifications to $file</b><br />");
    } else {
      $request->print("<font color=\"red\" size=\"+1\"><b>Unable to write to $file</b></font>");
      $error|=4;
    }
  }
  return $error;
}

sub analyze_header {
    my ($request) = @_;
    my $result.='<html>
            <head><title>Analyzing a problem</title></head>
            <body bgcolor="#FFFFFF">
            <form name="lonhomework" method="POST" action="'.
	      $ENV{'request.uri'}.'">
            <input type="submit" name="problemmode" value="EditXML" />
            <input type="submit" name="problemmode" value="Edit" />
            <hr />
            <input type="submit" name="submit" value="View" />
            <hr />
            List of possible answers:
            </form>';
    $request->print($result);
    $request->rflush();
}

sub analyze_footer {
    my ($request) = @_;
    my $result='</body></html>';
    $request->print($result);
    $request->rflush();
}

sub analyze {
    my ($request,$file) = @_;
    &Apache::lonxml::debug("Analyze");
    my $result;
    my %overall;
    my %allparts;
    my $rndseed=$ENV{'form.rndseed'};
    &analyze_header($request);
    my %prog_state=
	&Apache::lonhtmlcommon::Create_PrgWin($request,'Analyze Progress',
					      'Getting Problem Variants',
					      $ENV{'form.numtoanalyze'});
    for(my $i=1;$i<$ENV{'form.numtoanalyze'}+1;$i++) {
	&Apache::lonhtmlcommon::Increment_PrgWin($request,\%prog_state,
						 'last problem');
	my $subresult=&Apache::lonnet::ssi($request->uri,
					   ('grade_target' => 'analyze'),
					   ('rndseed' => $i+$rndseed));
	(my $garbage,$subresult)=split(/_HASH_REF__/,$subresult,2);
	my %analyze=&Apache::lonnet::str2hash($subresult);
	my @parts;
	if (defined(@{ $analyze{'parts'} })) {
	    @parts=@{ $analyze{'parts'} };
	}
	foreach my $part (@parts) {
	    if (!exists($allparts{$part})) {$allparts{$part}=1;};
	    if ($analyze{$part.'.type'} eq 'numericalresponse'	||
		$analyze{$part.'.type'} eq 'stringresponse'	||
		$analyze{$part.'.type'} eq 'formularesponse'   ) {
		push( @{ $overall{$part.'.answer'} },
		      [@{ $analyze{$part.'.answer'} }]);
	    }
	}
    }
    &Apache::lonhtmlcommon::Update_PrgWin($request,\%prog_state,
					  'Analyzing Results');
    foreach my $part (sort(keys(%allparts))) {
	if (defined(@{ $overall{$part.'.answer'} })) {
	    my $num_cols=scalar(@{ $overall{$part.'.answer'}->[0] });
	    $request->print('<table><tr><td colspan="'.($num_cols+1).'">Part '.$part.'</td></tr>');
	    my %frequency;
	    foreach my $answer (sort {$a->[0] <=> $b->[0]} (@{ $overall{$part.'.answer'} })) {
		$frequency{join("\0",@{ $answer })}++;
	    }
	    $request->print('<tr><td colspan="'.($num_cols).'">Answer</td><td>Frequency</td></tr>');
	    foreach my $answer (sort {(split("\0",$a))[0] <=> (split("\0",$b))[0]} (keys(%frequency))) {
		$request->print('<tr><td align="right">'.
				join('</td><td align="right">',split("\0",$answer)).
				'</td><td>('.$frequency{$answer}.
				')</td></tr>');
	    }
	    $request->print('</table>');
	} else {
	    $request->print('<p>Part '.$part.
			    ' is not analyzable at this time</p>');
	}
    }
    if (scalar(keys(%allparts)) == 0 ) {
	$request->print('<p>Found no analyzable parts in this problem,
                         currently only Numerical, Formula and String response
                         styles are supported.</p>');
    }
    &Apache::lonhtmlcommon::Close_PrgWin($request,\%prog_state);
    &analyze_footer($request);
    &Apache::lonhomework::showhash(%overall);
    return $result;
}

sub editxmlmode {
  my ($request,$file) = @_;
  my $result;
  my $problem=&Apache::lonnet::getfile($file);
  if ($problem eq -1) {
    &Apache::lonxml::error("<b> Unable to find <i>$file</i></b>");
    $problem='';
  }
  if (defined($ENV{'form.editxmltext'}) || defined($ENV{'form.Undo'})) {
    my $error=&handle_save_or_undo($request,\$problem,
				   \$ENV{'form.editxmltext'});
    if (!$error) { $problem=&Apache::lonnet::getfile($file); }
  }
  &Apache::lonhomework::showhashsubset(\%ENV,'^form');
  if ( $ENV{'form.submit'} eq 'Submit Changes and View' ) {
    &Apache::lonhomework::showhashsubset(\%ENV,'^form');
    $ENV{'form.problemmode'}='View';
    &renderpage($request,$file);
  } else {
    my ($rows,$cols) = &Apache::edit::textarea_sizes(\$problem);
    my $xml_help = Apache::loncommon::helpLatexCheatsheet("Problem_Editor_XML_Index",
							  "Problem Editing Help");
    if ($cols > 80) { $cols = 80; }
    if ($cols < 70) { $cols = 70; }
    if ($rows < 20) { $rows = 20; }
    $result.='<html><body bgcolor="#FFFFFF">
            <form name="lonhomework" method="POST" action="'.
	      $ENV{'request.uri'}.'">
            <input type="hidden" name="problemmode" value="EditXML" />
            <input type="submit" name="problemmode" value="Discard Edits and View" />
            <input type="submit" name="problemmode" value="Edit" />
            <hr />
            <input type="submit" name="submit" value="Submit Changes" />
            <input type="submit" name="submit" value="Submit Changes and View" />
            <input type="submit" name="Undo" value="undo" />
            <hr />
            ' . $xml_help . '
            <textarea rows="'.$rows.'" cols="'.$cols.'" name="editxmltext">'.
	      &HTML::Entities::encode($problem).'</textarea>
            </form></body></html>';
    $request->print($result);
  }
  return '';
}

sub renderpage {
  my ($request,$file) = @_;

  my (@targets) = &get_target();
  &Apache::lonxml::debug("Running targets ".join(':',@targets));
  foreach my $target (@targets) {
    #my $t0 = [&gettimeofday()];
    my $problem=&Apache::lonnet::getfile($file);
    if ($problem eq -1) {
      &Apache::lonxml::error("<b> Unable to find <i>$file</i></b>");
      $problem='';
    }

    my %mystyle;
    my $result = '';
    if ($target eq 'analyze') { %Apache::lonhomework::analyze=(); }
    if ($target eq 'answer') { &showhash(%Apache::lonhomework::history); }
    if ($target eq 'web') {&Apache::lonhomework::showhashsubset(\%ENV,'^form');}

    &Apache::lonxml::debug("Should be parsing now");
    $result = &Apache::lonxml::xmlparse($request, $target, $problem,
			&setup_vars($target),%mystyle);
    undef($Apache::lonhomework::parsing_a_problem);
    #$request->print("Result follows:");
    if ($target eq 'modified') {
      &handle_save_or_undo($request,\$problem,\$result);
    } else {
      if ($target eq 'analyze') {
	$result=&Apache::lonnet::hashref2str(\%Apache::lonhomework::analyze);
	undef(%Apache::lonhomework::analyze);
      }
      #my $td=&tv_interval($t0);
      #if ( $Apache::lonxml::debug) {
	#$result =~ s:</body>::;
	#$result.="<br />Spent $td seconds processing target $target\n</body>";
      #}
      $request->print($result);
      $request->rflush();
    }
    #$request->print(":Result ends");
    #my $td=&tv_interval($t0);
  }
}

# with no arg it returns a HTML <option> list of the template titles
# with one arg it returns the filename associated with the arg passed
sub get_template_list {
  my ($namewanted,$extension) = @_;
  my $result;
  my @allnames;
  &Apache::lonxml::debug("Looking for :$extension:");
  foreach my $file (</home/httpd/html/res/adm/includes/templates/*.$extension>) {
    my $name=&Apache::lonnet::metadata($file,'title');
    if ($namewanted && ($name eq $namewanted)) {
      $result=$file;
      last;
    } else {
	if ($name) { push (@allnames, $name); }
    }
  }
  if (@allnames && !$result) {
    $result="<option>Select a $extension template</option>\n<option>".
	join('</option><option>',sort(@allnames)).'</option>';
  }
  return $result;
}

sub newproblem {
    my ($request) = @_;
    my $extension=$request->uri;
    $extension=~s:^.*\.([\w]+)$:$1:;
    &Apache::lonxml::debug("Looking for :$extension:");
    my $templatelist=&get_template_list('',$extension);
    if ($ENV{'form.template'} &&
	$ENV{'form.template'} ne "Select a $extension template") {
	use File::Copy;
	my $file = &get_template_list($ENV{'form.template'},$extension);
	my $dest = &Apache::lonnet::filelocation("",$request->uri);
	copy($file,$dest);
	&renderpage($request,$dest);
    } elsif($ENV{'form.newfile'} && !$templatelist) {
	# I don't like hard-coded filenames but for now, this will work.
	use File::Copy;
	my $templatefilename =
	    $request->dir_config('lonIncludes').'/templates/blank.problem';
	&Apache::lonxml::debug("$templatefilename");
	my $dest = &Apache::lonnet::filelocation("",$request->uri);
	copy($templatefilename,$dest);
	&renderpage($request,$dest);
    } else {
	my $url=$request->uri;
	my $dest = &Apache::lonnet::filelocation("",$request->uri);
	my $errormsg;
	if ($ENV{'form.newfile'}) {
	    $errormsg='<p><font color="red">You did not select a template.</font></p>'."\n";
	}
	my $instructions;
	if ($templatelist) { $instructions=", select a template from the pull-down menu below.<br />Then";}
	$request->print(<<ENDNEWPROBLEM);
<body bgcolor="#FFFFFF">
<h1>Creating a new $extension resource</h1>
$errormsg
The requested file <tt>$url</tt> currently does not exist.
<p>
<b>To create a new $extension$instructions click on the "Create $extension" button.</b>
</p>
<p><form action="$url" method="POST">
ENDNEWPROBLEM
	if (defined($templatelist)) {
	    $request->print("<select name=\"template\">$templatelist</select>");
	}
	$request->print("<br /><input type=\"submit\" name=\"newfile\" value=\"Create $extension\" />");
	$request->print("</form></p></body>");
    }
    return '';
}

sub view_or_edit_menu {
  my ($request) = @_;
  my $url=$request->uri;
  $request->print(<<EDITMENU);
<body bgcolor="#FFFFFF">
<form action="$url" method="POST">
Would you like to <input type="submit" name="problemmode" value="View"> or
<input type="submit" name="problemmode" value="Edit"> the problem.
</form>
</body>
EDITMENU
}

sub handler {
  #my $t0 = [&gettimeofday()];
  my $request=$_[0];

  $Apache::lonxml::debug=$ENV{'user.debug'};

  if (&setupheader($request)) { return OK; }
  $ENV{'request.uri'}=$request->uri;

  #setup permissions
  $Apache::lonhomework::browse= &Apache::lonnet::allowed('bre',$ENV{'request.filename'});
  $Apache::lonhomework::viewgrades=&Apache::lonnet::allowed('vgr',$ENV{'request.course.id'});
  &Apache::lonxml::debug("Permissions:$Apache::lonhomework::browse:$Apache::lonhomework::viewgrades:");
  # some times multiple problemmodes are submitted, need to select
  # the last one
  &Apache::lonxml::debug("Problem Mode ".$ENV{'form.problemmode'});
  if ( defined($ENV{'form.problemmode'}) &&
       ref($ENV{'form.problemmode'}) ) {
    &Apache::lonxml::debug("Problem Mode ".join(",",@$ENV{'form.problemmode'}));
    my $mode=$ENV{'form.problemmode'}->[-1];
    undef $ENV{'form.problemmode'};
    $ENV{'form.problemmode'}=$mode;
  }
  &Apache::lonxml::debug("Problem Mode ".$ENV{'form.problemmode'});
  my $file=&Apache::lonnet::filelocation("",$request->uri);

  #check if we know where we are
  if ($ENV{'request.course.fn'} && !&Apache::lonnet::symbread()) { 
    # if we are browsing we might not be able to know where we are
    if ($Apache::lonhomework::browse ne 'F') {
      #should know where we are, so ask
      $request->internal_redirect('/adm/ambiguous'); return;
    }
  }

  my ($symb) = &Apache::lonxml::whichuser();
  &Apache::lonxml::debug('symb is '.$symb);
  if ($ENV{'request.state'} eq "construct" || $symb eq '') {
      if ($ENV{'form.resetdata'} eq 'Reset Submissions' ||
	  $ENV{'form.resetdata'} eq 'New Problem Variation' ||
          $ENV{'form.newrandomization'} eq 'New Randomization') {
	  my ($symb,$courseid,$domain,$name) = &Apache::lonxml::whichuser();
	  &Apache::lonnet::tmpreset($symb,'',$domain,$name);
&Apache::lonxml::debug("Attempt reset");
      }
  }
  if ($ENV{'request.state'} eq "construct") {
    if ( -e $file ) {
      &Apache::loncommon::get_unprocessed_cgi
                          ($ENV{'QUERY_STRING'},['problemmode']);
      if (!(defined $ENV{'form.problemmode'})) {
	#first visit to problem in construction space
	#&view_or_edit_menu($request);
	$ENV{'form.problemmode'}='View';
	&renderpage($request,$file);
      } elsif ($ENV{'form.problemmode'} eq 'EditXML') {
	&editxmlmode($request,$file);
      } elsif ($ENV{'form.problemmode'} eq 'Calculate answers') {
	&analyze($request,$file);
      } else {
	&renderpage($request,$file);
      }
    } else {
      # requested file doesn't exist in contruction space
      &newproblem($request);
    }
  } else {
    # just render the page normally outside of construction space
    &Apache::lonxml::debug("not construct");
    &renderpage($request,$file);
  }
  #my $td=&tv_interval($t0);
  #&Apache::lonxml::debug("Spent $td seconds processing");
  # &Apache::lonhomework::send_footer($request);
  # always turn off debug messages
  $Apache::lonxml::debug=0;
  return OK;

}

1;
__END__
