# The LearningOnline Network with CAPA
# Full matching style response
#
# $Id: matchresponse.pm,v 1.20 2003/07/01 20:13:30 sakharuk Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
# 2/21 Guy

package Apache::matchresponse;
use strict;
use HTML::Entities();
use Math::Random();
use Apache::optionresponse;

BEGIN {
    &Apache::lonxml::register('Apache::matchresponse',('matchresponse'));
}

sub start_matchresponse {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    my $result;
    #when in a matchresponse use these
    &Apache::lonxml::register('Apache::matchresponse',
			      ('foilgroup','foil','conceptgroup','item',
			       'itemgroup'));
    push (@Apache::lonxml::namespace,'matchresponse');
    my $id = &Apache::response::start_response($parstack,$safeeval);
    %Apache::hint::match=();
    if ($target eq 'meta') {
	$result=&Apache::response::meta_package_write('matchresponse');
    } elsif ($target eq 'edit' ) {
	$result.=&Apache::edit::start_table($token).
	    '<tr><td>'.&Apache::lonxml::description($token)."</td><td>Delete:".
	    &Apache::edit::deletelist($target,$token)
	    ."</td><td>&nbsp".&Apache::edit::end_row()
	    .&Apache::edit::start_spanning_row();
	
	$result.=
	    &Apache::edit::text_arg('Max Number Of Shown Foils:','max',$token,'4').
	    &Apache::edit::select_arg('Randomize Foil Order','randomize',
				      ['yes','no'],$token).
	    &Apache::edit::end_row().&Apache::edit::start_spanning_row()."\n";
    } elsif ($target eq 'modified') {
	my $constructtag=&Apache::edit::get_new_args($token,$parstack,
						     $safeeval,'max',
						     'randomize');
	if ($constructtag) { $result = &Apache::edit::rebuild_tag($token); }
    }
    return $result;
}

sub end_matchresponse {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    my $result;
    if ($target eq 'edit') { $result=&Apache::edit::end_table(); }
    &Apache::response::end_response;
    pop @Apache::lonxml::namespace;
    &Apache::lonxml::deregister('Apache::matchresponse',
				('foilgroup','foil','conceptgroup'));
    return $result;
}

%Apache::response::itemgroup=();
sub start_itemgroup {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    my $result;
    %Apache::response::itemgroup=();
    %Apache::matchresponse::itemtable=();

    if ($target eq 'edit') {
	$result=&Apache::edit::tag_start($target,$token);
	$result.=&Apache::edit::select_arg('Randomize Order:','randomize',
					   ['yes','no'],$token);
	$result.=&Apache::edit::select_arg('Items Display Location:',
					   'location',
					   ['top','bottom','left','right'],
					   $token);
	$result.=&Apache::edit::end_row().&Apache::edit::start_spanning_row();
    } elsif ($target eq 'modified') {
	my $constructtag=&Apache::edit::get_new_args($token,$parstack,
						     $safeeval,'randomize',
						     'location');
	if ($constructtag) { $result = &Apache::edit::rebuild_tag($token); }
    } elsif ($target eq 'web') {
	$Apache::matchresponse::itemtable{'location'}=
	    &Apache::lonxml::get_param('location',$parstack,$safeeval);
    }
    return $result;
}

sub end_itemgroup {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    my $result;

    if ($target eq 'edit') { $result=&Apache::edit::end_table(); }
    if (!defined(@{ $Apache::response::itemgroup{'names'} })) { return; }
    my @names=@{ $Apache::response::itemgroup{'names'} };
    my $randomize =&Apache::lonxml::get_param('randomize',$parstack,$safeeval);
    if ($randomize ne 'no' ) {
	@names=&Apache::response::whichorder($#names+1,$randomize,0,
					 \%Apache::response::itemgroup);
    }
    my %letter_name_map;
    my %name_letter_map;
    my @alphabet=('A'..'Z');
    my $i=0;
    foreach my $name (@names) {
	$letter_name_map{$alphabet[$i]}=$name;
	$name_letter_map{$name}=$alphabet[$i];
	$i++;
    }
    $Apache::response::itemgroup{'letter_name_map'}=\%letter_name_map;
    $Apache::response::itemgroup{'name_letter_map'}=\%name_letter_map;
    if ($target eq 'web') {
	my $table='<table>';
	my $i=0;
	foreach my $name (@names) {
	    $table.='<tr><td>'.$alphabet[$i].'</td><td>'.
		$Apache::response::itemgroup{$name.'.text'}.
		    '</td></tr>';
	    $i++;
	}
	$table.='</table>';
	$Apache::matchresponse::itemtable{'display'}=$table;
    } elsif ($target eq 'tex') {
	my $table=' \\\\\\\\ \begin{description} ';
	my $i=0;
	foreach my $name (@names) {
	    $Apache::response::itemgroup{$name.'.text'}=~s/\$\$/\$/g;
	    $table.='\item['.$alphabet[$i].'] '.
		$Apache::response::itemgroup{$name.'.text'}.
		    ' \\\\ ';
	    $i++;
	}
	$table.=' \end{description} \strut \\\\ ';
	$Apache::matchresponse::itemtable{'display'}=$table;
    }
    return $result;
}

sub start_item {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    my $result='';
    if ($target eq 'web' || $target eq 'tex') {
	&Apache::lonxml::startredirection;
    } elsif ($target eq 'edit') {
	my $randomize=&Apache::lonxml::get_param('randomize',$parstack,
						 $safeeval,'-2');
	$result=&Apache::edit::tag_start($target,$token,"Item");
	$result.=&Apache::edit::text_arg('Name:','name',$token);
	if ($randomize ne 'no') {
	    $result.=&Apache::edit::select_arg('Location:','location',
					       ['random','top','bottom'],
					       $token);
	}
	$result.=&Apache::edit::end_row().&Apache::edit::start_spanning_row();
    } elsif ($target eq 'modified') {
	my $constructtag=&Apache::edit::get_new_args($token,$parstack,
						     $safeeval,'name',
						     'location');
	if ($constructtag) { $result = &Apache::edit::rebuild_tag($token); }
    }
    return $result;
}

sub end_item {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    my $text ='';
    my $result = '';
    if ($target eq 'web' || $target eq 'tex') {
	$text=&Apache::lonxml::endredirection;
    }
    if ($target eq 'web' || $target eq 'grade' || $target eq 'answer' ||
	$target eq 'edit' || $target eq 'tex') {
	my $name = &Apache::lonxml::get_param('name',$parstack,$safeeval);
	my $location=&Apache::lonxml::get_param('location',$parstack,
						$safeeval);
	&Apache::lonxml::debug("Got a name of :$name:");
	if (!$name) { $name=$Apache::lonxml::curdepth; }
	&Apache::lonxml::debug("Using a name of :$name:");
	push @{ $Apache::response::itemgroup{'names'} }, $name;
	$Apache::response::itemgroup{"$name.text"} = $text;
	$Apache::response::itemgroup{"$name.location"} = $location;
    }
    if ($target eq 'edit') {
	$result.= &Apache::edit::tag_end($target,$token,'');
    }
    return $result;
}

sub insert_item {
    return '
<item name="">
<startouttext />
<endouttext />
</item>';
}

%Apache::response::foilgroup=();
sub start_foilgroup {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    my $result;
    %Apache::response::foilgroup=();
    $Apache::matchresponse::conceptgroup=0;
    &Apache::response::setrandomnumber();
    if ($target eq 'edit') {
	$result.=&Apache::edit::start_table($token)
	    ."<tr><td>Collection Of Foils</td><td>Delete:"
	    .&Apache::edit::deletelist($target,$token)
	    ."</td><td>&nbsp;".&Apache::edit::end_row()
            .&Apache::edit::start_spanning_row()."\n";
    }
    return $result;
}

sub end_foilgroup {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    my $result;
    if ($target eq 'grade' || $target eq 'web' || $target eq 'answer' || $target eq 'tex') {
	my $max = &Apache::lonxml::get_param('max',$parstack,$safeeval,'-2');
	my $randomize = &Apache::lonxml::get_param('randomize',$parstack,
						   $safeeval,'-2');
	if ($target eq 'web' || $target eq 'tex') {
	    $result=&displayfoils($target,$max,$randomize);
	} elsif ($target eq 'answer' ) {
	    $result=&displayanswers($max,$randomize);
	} elsif ( $target eq 'grade') {
	    &grade_response($max,$randomize);
	}
	&Apache::lonxml::increment_counter(&getfoilcounts($max));
    } elsif ($target eq 'edit') {
	$result=&Apache::edit::end_table();
    }
    return $result;
}

sub displayanswers {
    my ($max,$randomize,@opt)=@_;
    if (!defined(@{ $Apache::response::foilgroup{'names'} })) { return; }
    my @names = @{ $Apache::response::foilgroup{'names'} };
    my @whichfoils = &Apache::response::whichorder(&getfoilcounts($max),
						   $randomize,
					  &Apache::response::showallfoils(),
					 \%Apache::response::foilgroup);
    my $result=&Apache::response::answer_header('matchresponse');
    my %name_letter_map;
    if (defined(%{ $Apache::response::itemgroup{'name_letter_map'} })) {
	%name_letter_map=
	    %{ $Apache::response::itemgroup{'name_letter_map'} };
    }
    foreach my $name (@whichfoils) {
	my $value_name=$Apache::response::foilgroup{$name.'.value'};
	my $letter=$name_letter_map{$value_name};
	$result.=&Apache::response::answer_part('matchresponse',$letter);
    }
    $result.=&Apache::response::answer_footer('matchresponse');
    return $result;
}


sub grade_response {
    my ($max,$randomize)=@_;
    my (@whichfoils)=&Apache::response::whichorder(&getfoilcounts($max),
						   $randomize,
				            &Apache::response::showallfoils(),
				           \%Apache::response::foilgroup);
    if (!defined($ENV{'form.submitted'})) { return; }
    my %responsehash;
    my %grade;
    my ($temp,$right,$wrong,$ignored)=(0,0,0,0);
    my %letter_name_map;
    if (defined(%{ $Apache::response::itemgroup{'letter_name_map'} })) {
	%letter_name_map=
	    %{ $Apache::response::itemgroup{'letter_name_map'} };
    }
    foreach my $name (@whichfoils) {
	my $response = $ENV{'form.HWVAL_'.$Apache::inputtags::response['-1'].":$temp"};
	my $responsename = $letter_name_map{$response};
	$responsehash{$name}=$responsename;
	my $value=$Apache::response::foilgroup{$name.'.value'};
	if ( $response =~ /[^\s]/) {
	    &Apache::lonxml::debug("submitted a $response for $value<br />\n");
	    if ($value eq $responsename) {
		$grade{$name}='1'; $right++;
	    } else {
		$grade{$name}='0'; $wrong++;
	    }
	} else {
	    $ignored++;
	}
	$temp++;
    }
    my $part=$Apache::inputtags::part;
    my $id = $Apache::inputtags::response['-1'];
    my $responsestr=&Apache::lonnet::hash2str(%responsehash);
    my $gradestr   =&Apache::lonnet::hash2str(%grade);
    my %previous =&Apache::response::check_for_previous($responsestr,
							$part,$id);
    &Apache::lonxml::debug("Got $right right and $wrong wrong, and $ignored were ignored ");
    my $ad;
    if ($wrong==0 && $ignored==0) {
	$ad='EXACT_ANS';
    } elsif ($wrong==0 && $right==0) {
	#nothing submitted
    } else {
	if ($ignored==0) {
	    $ad='INCORRECT';
	} else {
	    $ad='MISSING_ANSWER';
	}
    }
    $Apache::lonhomework::results{"resource.$part.$id.submission"}=
	$responsestr;
    $Apache::lonhomework::results{"resource.$part.$id.submissiongrading"}=
	$gradestr;
    $Apache::lonhomework::results{"resource.$part.$id.awarddetail"}=$ad;
    &Apache::response::handle_previous(\%previous,$ad);
}

sub itemdisplay {
    my ($location)=@_;
    if ($location eq 'top' &&
	!defined($Apache::matchresponse::itemtable{'location'})) {
	return $Apache::matchresponse::itemtable{'display'};
    }
    if ($Apache::matchresponse::itemtable{'location'} eq $location) {
	return $Apache::matchresponse::itemtable{'display'};
    }
    return undef;
}
sub displayfoils {
    my ($target,$max,$randomize)=@_;
    my $result;
    my $question;
    my (@whichfoils)=&Apache::response::whichorder(&getfoilcounts($max),
						   $randomize,
				            &Apache::response::showallfoils(),
				           \%Apache::response::foilgroup);
    my $part=$Apache::inputtags::part;
    my $solved=$Apache::lonhomework::history{"resource.$part.solved"};
    my $status=$Apache::inputtags::status[-1];
    my %letter_name_map;
    if (defined(%{ $Apache::response::itemgroup{'letter_name_map'} })) {
	%letter_name_map=
	    %{ $Apache::response::itemgroup{'letter_name_map'} };
    }
    my %name_letter_map;
    if (defined(%{ $Apache::response::itemgroup{'name_letter_map'} })) {
	%name_letter_map=
	    %{ $Apache::response::itemgroup{'name_letter_map'} };
    }
    if (($solved =~ /^correct/) || ($status eq  'SHOW_ANSWER')) {
	foreach my $name (@whichfoils) {
	    my $text=$Apache::response::foilgroup{$name.'.text'};
	    my $value=$Apache::response::foilgroup{$name.'.value'};
	    my $letter=$name_letter_map{$value};
	    if ($target eq 'tex') {
		$question.=' \\\\ '.$letter.':'.$text;
	    } else {
		$question.='<br />'.$letter.':'.$text;
	    }
	}
    } else {
	my $i = 0;
	my $temp=0;
	my $id=$Apache::inputtags::response[-1];
	my $part=$Apache::inputtags::part;
	my $lastresponse=$Apache::lonhomework::history{"resource.$part.$id.submission"};
	my %lastresponse=&Apache::lonnet::str2hash($lastresponse);    
	my @alphabet=('A'..'Z');
	my @used_letters=sort(keys(%letter_name_map));
	my $internal_counter=$Apache::lonxml::counter;
	foreach my $name (@whichfoils) {
	    my $lastopt=$lastresponse{$name};
	    my $last_letter=$name_letter_map{$lastopt};
	    my $optionlist = '';
	    if ($target ne 'tex') {
		$optionlist="<option></option>\n";
	    } else {
		if ($Apache::lonhomework::type ne 'exam') {
		    $optionlist='\framebox[5 mm][s]{\tiny\strut}';
		}
	    }
	    my $option;
	    foreach $option (@used_letters) {
		if ($option eq $last_letter) {
		    if ($target ne 'tex') {$optionlist.="<option selected=\"on\">$option</option>\n";}
		} else {
		    if ($target ne 'tex') {$optionlist.="<option>$option</option>\n";}
		}
	    }
	    if ($target ne 'tex' && $Apache::lonhomework::type ne 'exam') {
	        $optionlist='<select name="HWVAL_'.
		    $Apache::inputtags::response[-1].':'.$temp.'">'.
		        $optionlist."</select>\n";
	    }
	    my $text=$Apache::response::foilgroup{$name.'.text'};
	    if ($target ne 'tex') {
		if ($Apache::lonhomework::type ne 'exam') {
		    $question.='<br />'.$optionlist.$text."\n";
		} else {
		    $question.='<br />'.$text."\n";
		}
		if ($Apache::lonhomework::type eq 'exam') {
		    $question.=&Apache::optionresponse::webbubbles(\@used_letters,\@used_letters,$temp);
		}
	    } else {		
		if ($Apache::lonhomework::type eq 'exam') {
		    $question.=' '.$optionlist.$text."\n";
		    my @emptyItems = ();
		    for (my $i=0;$i<=$#used_letters;$i++) {push @emptyItems, ' ';}
		    $question.='\vskip -2 mm\parbox{\textwidth}{\begin{enumerate}\item[\textbf{'.$internal_counter.'}.]\parbox{\textwidth - 5 mm}{'.&Apache::optionresponse::bubbles(\@used_letters,\@emptyItems).'}\end{enumerate}} \vskip -10 mm \strut ';
		    $internal_counter++;
	        } else {
		    $question.=' \\\\ '.$optionlist.$text."\n";
		}
            } 
	    $temp++;
	}
    }
    if ($result=&itemdisplay('top')) {
	$result.=$question;
    } elsif ($result=&itemdisplay('bottom')) {
	$result=$question.$result;
    } elsif ($result=&itemdisplay('right')) {
	$result='<table><tr><td>'.$question.'</td><td>'.$result.
	    '</td></tr></table>';
    } elsif ($result=&itemdisplay('left')) {
	$result='<table><tr><td>'.$result.'</td><td>'.$question.
	    '</td></tr></table>';
    }
    if ($target ne 'tex') {$result.="<br />";} else {$result.=' \\\\ ';}
    return $result;
}

sub getfoilcounts {
    my ($max)=@_;
    # +1 since instructors will count from 1
    my $count = $#{ $Apache::response::foilgroup{'names'} }+1;
    if (&Apache::response::showallfoils()) { $max=$count; }
    if ($count>$max) { $count=$max } 
    &Apache::lonxml::debug("Count is $count from $max");
    return $count;
}


sub start_conceptgroup {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    $Apache::matchresponse::conceptgroup=1;
    %Apache::response::conceptgroup=();
    my $result;
    if ($target eq 'edit') {
	$result.=&Apache::edit::tag_start($target,$token,
					  "Concept Grouped Foils");
	$result.=&Apache::edit::text_arg('Concept:','concept',$token,'50').
	    &Apache::edit::end_row().&Apache::edit::start_spanning_row();
    }
    if ($target eq 'modified') {
	my $constructtag=&Apache::edit::get_new_args($token,$parstack,
						     $safeeval,'concept');
	if ($constructtag) { $result = &Apache::edit::rebuild_tag($token); }
    }
    return $result;
}

sub end_conceptgroup {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    $Apache::matchresponse::conceptgroup=0;
    my $result='';
    if ($target eq 'web' || $target eq 'grade' || $target eq 'answer' ) {
	#if not there aren't any foils to display and thus no question
	if (defined(@{ $Apache::response::conceptgroup{'names'} })) {
	    my @names = @{ $Apache::response::conceptgroup{'names'} };
	    my $pick=int(&Math::Random::random_uniform() * ($#names+1));
	    my $name=$names[$pick];
	    push @{ $Apache::response::foilgroup{'names'} }, $name;
	    $Apache::response::foilgroup{"$name.value"} =
		$Apache::response::conceptgroup{"$name.value"};
	    $Apache::response::foilgroup{"$name.text"} =
		$Apache::response::conceptgroup{"$name.text"};
	    $Apache::response::foilgroup{"$name.location"} =
		$Apache::response::conceptgroup{"$name.location"};
	    my $concept = &Apache::lonxml::get_param('concept',$parstack,
						     $safeeval);
	    $Apache::response::foilgroup{"$name.concept"} = $concept;
	    &Apache::lonxml::debug("Selecting $name in $concept");
	    if ($target eq 'web') {
		my $part_id="$Apache::inputtags::part.$Apache::inputtags::response[-1]";
		push(@{ $Apache::hint::match{"$part_id.concepts"} },
		     $concept);
		$Apache::hint::match{"$part_id.concept.$concept"}=
		    $Apache::response::conceptgroup{'names'};
	    }
	}
    } elsif ($target eq 'edit') {
	$result=&Apache::edit::end_table();
    }
    return $result;
}

sub insert_conceptgroup {
    my $result="\n\t\t<conceptgroup concept=\"\">".&insert_foil()."\n\t\t</conceptgroup>\n";
    return $result;
}

sub start_foil {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    my $result='';
    if ($target eq 'web' || $target eq 'tex') {
	&Apache::lonxml::startredirection;
    } elsif ($target eq 'edit') {
	$result=&Apache::edit::tag_start($target,$token,"Foil");
	my $level='-2';
	if ($$tagstack[-2] eq 'conceptgroup') { $level = '-3'; }
	$result.=&Apache::edit::text_arg('Name:','name',$token);
	my @names;
	if (defined(@{ $Apache::response::itemgroup{'names'} })) {
	    @names=@{ $Apache::response::itemgroup{'names'} };
	}
	$result.= &Apache::edit::select_or_text_arg('Correct Option:','value',['unused',@names],$token,'15');
	my $randomize=&Apache::lonxml::get_param('randomize',$parstack,
						 $safeeval,'-3');
	if ($randomize ne 'no') {
	    $result.=&Apache::edit::select_arg('Location:','location',
					     ['random','top','bottom'],$token);
	}
	$result .=&Apache::edit::end_row().&Apache::edit::start_spanning_row();
    } elsif ($target eq 'modified') {
	my $constructtag=&Apache::edit::get_new_args($token,$parstack,
						     $safeeval,'value',
						     'name','location');
	if ($constructtag) { $result = &Apache::edit::rebuild_tag($token); }
    }
    return $result;
}

sub end_foil {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    my $text ='';
    my $result = '';
    if ($target eq 'web' || $target eq 'tex') {
	$text=&Apache::lonxml::endredirection;
    }
    if ($target eq 'web' || $target eq 'grade' || $target eq 'answer' || $target eq 'tex') {
	my $value = &Apache::lonxml::get_param('value',$parstack,$safeeval);
	if ($value ne 'unused') {
	    my $name = &Apache::lonxml::get_param('name',$parstack,$safeeval);
	    my $location =&Apache::lonxml::get_param('location',$parstack,$safeeval);
	    &Apache::lonxml::debug("Got a name of :$name:");
	    if (!$name) { $name=$Apache::lonxml::curdepth; }
	    &Apache::lonxml::debug("Using a name of :$name:");
	    if ( $Apache::matchresponse::conceptgroup
		 && !&Apache::response::showallfoils() ) {
		push @{ $Apache::response::conceptgroup{'names'} }, $name;
		$Apache::response::conceptgroup{"$name.value"} = $value;
		if ($target eq 'tex' && $Apache::lonhomework::type eq 'exam') {
		    $Apache::response::conceptgroup{"$name.text"} = ' $\triangleright$ '.$text;
		} else {
		    $Apache::response::conceptgroup{"$name.text"} = $text;
		}
		$Apache::response::conceptgroup{"$name.location"} = $location;
	    } else {
		push @{ $Apache::response::foilgroup{'names'} }, $name;
		$Apache::response::foilgroup{"$name.value"} = $value;
		if ($Apache::lonhomework::type eq 'exam') {
		    if ($target eq 'tex') {
			$Apache::response::foilgroup{"$name.text"} = '\vskip 5 mm $\triangleright$ '.$text;
		    } else {
			$Apache::response::foilgroup{"$name.text"} = $text;
		    }
		} else {
		    if ($target eq 'tex') {
			$Apache::response::foilgroup{"$name.text"} = ' $\triangleright$ '.$text;
		    } else {
			$Apache::response::foilgroup{"$name.text"} = $text;
		    }
		}
		$Apache::response::foilgroup{"$name.location"} = $location;
	    }
	}
    }
    if ($target eq 'edit') {
	$result.= &Apache::edit::tag_end($target,$token,'');
    }
    return $result;
}

sub insert_foil {
    return '
<foil name="" value="unused">
<startouttext />
<endouttext />
</foil>';
}
1;
__END__
