# LearningOnline Network with CAPA
# option list style responses
#
# $Id: optionresponse.pm,v 1.84 2003/08/01 19:04:06 sakharuk Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
# 2/21 Guy
# 01/18 Alex
package Apache::optionresponse;
use strict;
use Apache::response;

BEGIN {
  &Apache::lonxml::register('Apache::optionresponse',('optionresponse'));
}

sub start_optionresponse {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
  my $result='';
  #when in a option response use these
  &Apache::lonxml::register('Apache::optionresponse',('foilgroup','foil','conceptgroup'));
  push (@Apache::lonxml::namespace,'optionresponse');
  my $id = &Apache::response::start_response($parstack,$safeeval);
  %Apache::hint::option=();
  if ($target eq 'edit') {
    $result.=&Apache::edit::start_table($token).
	"<tr><td>Multiple Option Response Question</td><td>Delete:".
	&Apache::edit::deletelist($target,$token)
	."</td><td>&nbsp;".
        &Apache::edit::end_row().
        &Apache::edit::start_spanning_row().
        "\n";
    $result.=&Apache::edit::text_arg('Max Number Of Shown Foils:','max',
				     $token,'4').
        &Apache::edit::select_arg('Randomize Foil Order','randomize',
				  ['yes','no'],$token).
        &Apache::edit::end_row().&Apache::edit::start_spanning_row();
  } elsif ($target eq 'modified') {
    my $constructtag=&Apache::edit::get_new_args($token,$parstack,
						 $safeeval,'max','randomize');
    if ($constructtag) { $result = &Apache::edit::rebuild_tag($token); }
  } elsif ($target eq 'meta') {
    $result=&Apache::response::meta_package_write('optionresponse');
  } elsif ($target eq 'analyze') {
    my $part_id="$Apache::inputtags::part.$Apache::inputtags::response[-1]";
    push (@{ $Apache::lonhomework::analyze{"parts"} },$part_id);
  }
  return $result;
}

sub end_optionresponse {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
  &Apache::response::end_response;
  pop @Apache::lonxml::namespace;
  &Apache::lonxml::deregister('Apache::optionresponse',('foilgroup','foil','conceptgroup'));
  my $result;
  if ($target eq 'edit') { $result=&Apache::edit::end_table(); }
  return $result;
}

%Apache::response::foilgroup=();
sub start_foilgroup {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;

  my $result='';
  %Apache::response::foilgroup=();
  $Apache::optionresponse::conceptgroup=0;
  &Apache::response::setrandomnumber();
  if ($target eq 'edit') {
    my $optionlist="<option></option>\n";
    my $option;
    my @opt;
    eval '@opt ='. &Apache::lonxml::get_param('options',$parstack,$safeeval);
    my $count=1;
    foreach $option (@opt) {
      $optionlist.="<option value=\"$count\">$option</option>\n";
      $count++;
    }
    my $insertlist=&Apache::edit::insertlist($target,$token);
    $result.=&Apache::edit::start_table($token);
    $result.= (<<ENDTABLE);
      <tr><td>Select Options</td>
        <td>
	  Add new Option: <input type="text" name="$Apache::lonxml::curdepth.options" />
        </td>
        <td>Delete an Option:
	  <select name="$Apache::lonxml::curdepth.deleteopt">$optionlist</select>
ENDTABLE
    $result.= &Apache::edit::end_row();
    $result.= &Apache::edit::start_spanning_row();
    $result.= $insertlist.'<br />';
  }
  if ($target eq 'modified') {
    my @options;
    my $optchanged=0;
    eval '@options ='.&Apache::lonxml::get_param('options',$parstack,$safeeval);
    if ($ENV{"form.$Apache::lonxml::curdepth.deleteopt"}) {
      my $delopt=$ENV{"form.$Apache::lonxml::curdepth.deleteopt"};
      &Apache::lonxml::debug("Deleting :$delopt:");
      splice(@options,$delopt-1,1);
      $optchanged=1;
    }
    if ($ENV{"form.$Apache::lonxml::curdepth.options"}) {
      my $newopt = $ENV{"form.$Apache::lonxml::curdepth.options"};
      if ($options[0]) {
	push(@options,$newopt);
      } else {
	$options[0]=$newopt;
      }
      $optchanged=1;
    }
    if ($optchanged) {
      $result = "<foilgroup options=\"(";
      foreach my $option (@options) {
	$option=~s/\'/\\\'/g;
	&Apache::lonxml::debug("adding option :$option:");
	$result .="'".$option."',";
      }
      chop $result;
      $result.=')">';
    } # else nothing changed so just use the default mechanism
  }
  if ($target eq 'tex' and $Apache::lonhomework::type ne 'exam') {
      $result .= ' \begin{enumerate} ';
  }
  return $result;
}

sub end_foilgroup {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
  
  my $result;
  if ($target eq 'grade' || $target eq 'web' || $target eq 'answer' || $target eq 'tex' || $target eq 'analyze') {
    my $tex_option_switch=&Apache::lonxml::get_param('texoptions',$parstack,$safeeval);
    my $name;
    my @opt;
    eval '@opt ='.&Apache::lonxml::get_param('options',$parstack,$safeeval);
    if ($target eq 'tex' && $tex_option_switch eq 'nochoice') {@opt=();}
    &Apache::lonxml::debug("Options are $#opt");
    my $max = &Apache::lonxml::get_param('max',$parstack,$safeeval,'-2');
    my $randomize = &Apache::lonxml::get_param('randomize',$parstack,
					       $safeeval,'-2');
    if ($target eq 'web' || $target eq 'tex') {
      $result.=&displayfoils($target,$max,$randomize,@opt);
    } elsif ( $target eq 'answer') {
      $result.=&displayanswers($max,$randomize,@opt);
    } elsif ( $target eq 'analyze') {
      my $part_id="$Apache::inputtags::part.$Apache::inputtags::response[-1]";
      foreach my $name (@{ $Apache::response::foilgroup{'names'} }) {
	if (defined($Apache::lonhomework::analyze{"$part_id.foil.value.$name"})) { next; }
	push (@{ $Apache::lonhomework::analyze{"$part_id.foils"} },$name);
	$Apache::lonhomework::analyze{"$part_id.foil.value.$name"} =
	  $Apache::response::foilgroup{"$name.value"};
	$Apache::lonhomework::analyze{"$part_id.foil.text.$name"} =
	  $Apache::response::foilgroup{"$name.text"};
      }
      push (@{ $Apache::lonhomework::analyze{"$part_id.options"} },@opt);
      push (@{ $Apache::lonhomework::analyze{"$part_id.shown"} },&whichfoils($max,$randomize));
    } elsif ( $target eq 'grade') {
      if ( defined $ENV{'form.submitted'}) {
	my @whichopt = &whichfoils($max,$randomize);
	my $temp=1;my $name;
	my %responsehash;
	my %grade;
	my $right=0;
	my $wrong=0;
	my $ignored=0;
	foreach $name (@whichopt) {
	  my $response=&Apache::response::getresponse($temp);
	  if ($ENV{'form.submitted'} eq 'scantron') {
	      $response = $opt[$response];
	  }
	  $responsehash{$name}=$response;
	  if ( $response =~ /[^\s]/) {
	    my $value=$Apache::response::foilgroup{$name.'.value'};
	    &Apache::lonxml::debug("submitted a $response for $value<br />\n");
	    if ($value eq $response) {
		$grade{$name}='1'; $right++;
	    } else {
		$grade{$name}='0'; $wrong++;
	    }
	  } else {
	    $ignored++;
	  }
	  $temp++;
	}
	my $part=$Apache::inputtags::part;
	my $id = $Apache::inputtags::response['-1'];
	my $responsestr=&Apache::lonnet::hash2str(%responsehash);
	my $gradestr   =&Apache::lonnet::hash2str(%grade);
	my %previous=&Apache::response::check_for_previous($responsestr,
							   $part,$id);
	&Apache::lonxml::debug("Got $right right and $wrong wrong, and $ignored were ignored");
	$Apache::lonhomework::results{"resource.$part.$id.submission"}=
	    $responsestr;
	$Apache::lonhomework::results{"resource.$part.$id.submissiongrading"}=$gradestr;
	
	if (!$Apache::lonhomework::scantronmode) {
	    my $ad;
	    if ($wrong==0 && $ignored==0) {
		$ad='EXACT_ANS';
	    } elsif ($wrong==0 && $right==0) {
		#nothing submitted
	    } else {
		if ($ignored==0) {
		    $ad='INCORRECT';
		} else {
		    $ad='MISSING_ANSWER';
		}
	    }
	    $Apache::lonhomework::results{"resource.$part.$id.awarddetail"}=$ad;
	    &Apache::response::handle_previous(\%previous,$ad);
	} else {
	    my $ad;
	    if ($wrong==0 && $right==0) {
		#nothing submitted
	    } else {
		$ad='ASSIGNED_SCORE';
	    }
	    $Apache::lonhomework::results{"resource.$part.$id.awarddetail"}=$ad;
	    $Apache::lonhomework::results{"resource.$part.$id.awarded"}=
		$right/(scalar(@whichopt));
	    $Apache::lonhomework::results{"resource.$part.$id.numfoils"}=
		scalar(@whichopt);
	}
      }
    }
    &Apache::lonxml::increment_counter(&getfoilcounts($max));
  } elsif ($target eq 'edit') {
    $result.=&Apache::edit::end_table();
  }
  if ($target eq 'tex' and $Apache::lonhomework::type ne 'exam') {
      $result .= '\end{enumerate}';
  }
  return $result;
}

sub getfoilcounts {
  my ($max)=@_;
  # +1 since instructors will count from 1
  my $count = $#{ $Apache::response::foilgroup{'names'} }+1;
  if (&Apache::response::showallfoils()) { $max=$count; }
  if ($count>$max) { $count=$max } 
  &Apache::lonxml::debug("Count is $count from $max");
  return $count;
}

sub whichfoils {
    my ($max,$randomize)=@_;
    return &Apache::response::whichorder($max,$randomize,
					 &Apache::response::showallfoils(),
					 \%Apache::response::foilgroup);
}

sub displayanswers {
  my ($max,$randomize,@opt)=@_;
  if (!defined(@{ $Apache::response::foilgroup{'names'} })) {return;}
  my @names = @{ $Apache::response::foilgroup{'names'} };
  my @whichopt = &whichfoils($max,$randomize);
  my $result=&Apache::response::answer_header('optionresponse');
  foreach my $name (@whichopt) {
    $result.=&Apache::response::answer_part('optionresponse',
		     $Apache::response::foilgroup{$name.'.value'})
  }
  $result.=&Apache::response::answer_footer('optionresponse');
  return $result;
}

sub displayfoils {
  my ($target,$max,$randomize,@opt)=@_;
  if (!defined(@{ $Apache::response::foilgroup{'names'} })) {return;}
  my @names = @{ $Apache::response::foilgroup{'names'} };
  my @truelist;
  my @falselist;
  my $result;  
  my $name;
  my $displayoptionintex=0;
  my @alphabet = ('A'..'Z');
  my @whichopt = &whichfoils($max,$randomize);
  my $part=$Apache::inputtags::part;
  my $id=$Apache::inputtags::response[-1];
  my $break;
  my $solved=$Apache::lonhomework::history{"resource.$part.solved"};
  my $status=$Apache::inputtags::status[-1];
  if (
      ($target ne 'tex') &&
      (($solved =~ /^correct/) || ($status eq 'SHOW_ANSWER')) ) {
    foreach $name (@whichopt) {
	my $text=$Apache::response::foilgroup{$name.'.text'};
	if ($text!~/^\s*$/) {
	    if ($target eq 'tex') {
		$break='\vskip 0 mm ';
	    } elsif ($target eq 'web') {
		$break='<br />';
	    }
	}
	$result.=$break;
	if ($target eq 'web') {
	    my $value=$Apache::response::foilgroup{$name.'.value'};
	    if (!($text=~s|<drawoptionlist\s*/>|$value|)) {
		if ($text=~/^\s*$/) {
		    $text=$value.$text;
		} else {
		    $text=$value.': '.$text;
		}
	    }
	    $result.=$text."\n";
      } elsif ($target eq 'tex') {
	  $Apache::response::foilgroup{$name.'.text'}=~s/\\item//;
	  if ($max>1) {$result .='\item ';}
 	  $result .=' \textit{'.$Apache::response::foilgroup{$name.'.value'}.'}'.
	      ":".$Apache::response::foilgroup{$name.'.text'}."\n";
      }
      if ($Apache::lonhomework::type eq 'exam') {
	if ($target ne 'tex') {
	  $result.=&webbubbles(\@opt,\@alphabet);
        } else {
	  $result.=&bubbles(\@alphabet,\@opt);
        }
      }
    }
  } else {
    my $temp=1;
    my %lastresponse=&Apache::lonnet::str2hash($Apache::lonhomework::history{"resource.$part.$id.submission"});
    my $internal_counter=$Apache::lonxml::counter;
    foreach $name (@whichopt) {
      my $text=$Apache::response::foilgroup{$name.'.text'};
      if ($text!~/^\s*$/) {
	  if ($target eq 'tex') {
	      $break='\vskip 0 mm ';
	  } elsif ($target eq 'web') {
	      $break='<br />';
	  }
      }
      my $lastopt=$lastresponse{$name};
      my $optionlist="<option></option>\n";
      my $option;
      foreach $option (@opt) {
	if ($option eq $lastopt) {
	  $optionlist.="<option selected=\"on\">$option</option>\n";
	} else {
	  $optionlist.="<option>$option</option>\n";
	}
      }
      if ($target ne 'tex') {
	  if ($Apache::lonhomework::type ne 'exam') {
	    $optionlist='<select name="HWVAL_'.
	      $Apache::inputtags::response['-1'].':'.$temp.'">'.
		  $optionlist."</select>\n";
	  }
	  my $text=$Apache::response::foilgroup{$name.'.text'};
	  if (!($text=~s|<drawoptionlist\s*/>|$optionlist|) && $Apache::lonhomework::type ne 'exam') {
	      $text=$optionlist.$text;
	  }
	  $result.=$break.$text."\n";
	  if ($Apache::lonhomework::type eq 'exam') {
	    $result.=&webbubbles(\@opt,\@alphabet,$temp);
	  }
	  $temp++;
      } else {
          my $texoptionlist = &optionlist_correction($optionlist);
	  if ($displayoptionintex == 0) {
	      if ($Apache::lonhomework::type eq 'exam') {$texoptionlist='';} #if exam we do not need to show optionlist
	      if ($Apache::response::foilgroup{$name.'.text'}=~m/\\item /) {
		if ($Apache::lonhomework::type eq 'exam') {
	          $Apache::response::foilgroup{$name.'.text'}=~s/\\item/\\vskip 2 mm/;
	        }
	        if ($Apache::response::foilgroup{$name.'.text'}=~/<drawoptionlist\s*\/>/) {
	          $Apache::response::foilgroup{$name.'.text'}=~s|<drawoptionlist\s*/>| \\makebox\[0\.3in\]\[b\]\{\\hrulefill\} |;
	        }
		$result.= $texoptionlist.$Apache::response::foilgroup{$name.'.text'};
	      } else {
		  if ($Apache::lonhomework::type eq 'exam') {
		      $result.= $texoptionlist.'  '.$Apache::response::foilgroup{$name.'.text'};
		  } else {
		      $result.= $texoptionlist.'\vspace*{-2 mm}\item '.$Apache::response::foilgroup{$name.'.text'};
		  }
	      }
	      if ($Apache::lonhomework::type eq 'exam') {
		  $result.='\vskip -2 mm\parbox{\textwidth}{\begin{enumerate}\item[\textbf{'.$internal_counter.'}.]\parbox{\textwidth - 5 mm}{'.&bubbles(\@alphabet,\@opt).'}\end{enumerate}} \vskip -9 mm \strut ';
		  $internal_counter++;
	      }
	      $displayoptionintex=1;
	  } else {
	      if ($Apache::response::foilgroup{$name.'.text'}=~m/\\item /) {
		  if ($Apache::lonhomework::type eq 'exam') {
		      $Apache::response::foilgroup{$name.'.text'}=~s/\\item//;
		  }
	        if ($Apache::response::foilgroup{$name.'.text'}=~/<drawoptionlist\s*\/>/) {
	          $Apache::response::foilgroup{$name.'.text'}=~s|<drawoptionlist\s*/>| \\makebox\[0\.3in\]\[b\]\{\\hrulefill\} |;
	        }
		$result.= $Apache::response::foilgroup{$name.'.text'};
	      } else {
		  if ($Apache::lonhomework::type eq 'exam') {
		      $result.= '  '.$Apache::response::foilgroup{$name.'.text'};
		  } else {
		      $result.= '\item '.$Apache::response::foilgroup{$name.'.text'};
		  }
	      }
	      if ($Apache::lonhomework::type eq 'exam') {
		  $result.='\vskip -2 mm \parbox{\textwidth}{\begin{enumerate}\item[\textbf{'.$internal_counter.'}.]\parbox{\textwidth - 5 mm}{'.&bubbles(\@alphabet,\@opt).'}\end{enumerate}} \vskip -9 mm \strut ';
		  $internal_counter++;
	      }
	  }
      }
    }
  }
  if ($target ne 'tex') {
      return $result.$break;
  } else {
      return $result;
  }
}


sub optionlist_correction {

    my $texoptionlist = shift;
    if ($texoptionlist=~/<option selected/ or $texoptionlist=~/<option>[^<]+<\/option>/) {
	$texoptionlist =~ s/<option><\/option>/\\item \[\] Choose from: /;
	$texoptionlist =~ s/<option>/\{\\bf /g;
	$texoptionlist =~ s/<option selected="on">/\{\\bf /g;
	$texoptionlist =~ s/<\/option>/\},/g;
	$texoptionlist =~ s/,$/\./g;
	$texoptionlist =~ s/>/\$>\$/g;
	$texoptionlist =~ s/</\$<\$/g;
	$texoptionlist =~ s/=/\$=\$/g;
	$texoptionlist =~ s/\^(\d+)/<m>\$$1\$<\/m>/g;
    } else {
	$texoptionlist =~ s/<option><\/option>/\\item \[\] \\vskip -5 mm/;
    }
    return $texoptionlist;
}


sub webbubbles {

    my ($ropt,$ralphabet,$temp)=@_;
    my @opt=@$ropt; 
    my @alphabet=@$ralphabet;
    my $result='';
	my $number_of_bubbles = $#opt + 1;
	$result.= '<table border="1"><tr>';
	for (my $ind=0;$ind<$number_of_bubbles;$ind++) {
	   $result.='<td><input type="radio" name="HWVAL_'.$Apache::inputtags::response['-1'].':'.$temp.
                    '" value='.$opt[$ind].'>'.$alphabet[$ind].': '.$opt[$ind].'</td>';
	}
	$result.='</tr></table>';
	return $result;
}


sub bubbles {

    my ($ralphabit,$ropt) = @_;
    my @alphabet = @$ralphabit;
    my @opt = @$ropt;
    my ($result,$head,$line) =('','','');
    my $number_of_bubbles = $#opt + 1;
    my $current_length = 0;
    for (my $ind=0;$ind<=$number_of_bubbles;$ind++) {
	$current_length += (length($opt[$ind])+length($alphabet[$ind])+2)*2;
	if ($current_length<0.9*$ENV{'form.textwidth'} and $ind!=$number_of_bubbles) {
	    $line.='\hskip -1 mm {\small \textbf{'.$alphabet[$ind].'}}$\bigcirc$\hskip -1 mm & \hskip -3 mm {\small '.$opt[$ind].'} & ';
	    $head.='lr';
	} else {
	    $line=~s/\&\s*$//;
	    $result.='\vskip -5 mm\begin{tabular}{'.$head.'}\\\\'.$line.'\\\\\end{tabular}';
	    $line = '\hskip -1 mm {\small \textbf{'.$alphabet[$ind].'}}$\bigcirc$\hskip -1 mm & \hskip -3 mm {\small '.$opt[$ind].'} & ';;
	    $head ='lr';
	    $current_length = (length($opt[$ind])+length($alphabet[$ind]))*2;
	}

    }
    return $result;
}


sub start_conceptgroup {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
  $Apache::optionresponse::conceptgroup=1;
  %Apache::response::conceptgroup=();
  my $result;
  if ($target eq 'edit') {
    $result.=&Apache::edit::tag_start($target,$token,"Concept Grouped Foils");
    $result.=&Apache::edit::text_arg('Concept:','concept',$token,'50').
        &Apache::edit::end_row().&Apache::edit::start_spanning_row();
  }
  if ($target eq 'modified') {
    my $constructtag=&Apache::edit::get_new_args($token,$parstack,$safeeval,'concept');
    if ($constructtag) { $result = &Apache::edit::rebuild_tag($token); }
  }
  return $result;
}

sub end_conceptgroup {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
  $Apache::optionresponse::conceptgroup=0;
  my $result='';
  if ($target eq 'web' || $target eq 'grade' || $target eq 'answer'
      || $target eq 'tex' || $target eq 'analyze') {
    #if not there aren't any foils to display and thus no question
    if (defined(@{ $Apache::response::conceptgroup{'names'} })) {
      my @names = @{ $Apache::response::conceptgroup{'names'} };
      my $pick=int(&Math::Random::random_uniform() * ($#names+1));
      my $name=$names[$pick];
      push @{ $Apache::response::foilgroup{'names'} }, $name;
      $Apache::response::foilgroup{"$name.value"} =
	$Apache::response::conceptgroup{"$name.value"};
      $Apache::response::foilgroup{"$name.text"} =
	$Apache::response::conceptgroup{"$name.text"};
      $Apache::response::foilgroup{"$name.location"} =
	$Apache::response::conceptgroup{"$name.location"};
      my $concept = &Apache::lonxml::get_param('concept',$parstack,$safeeval);
      $Apache::response::foilgroup{"$name.concept"} = $concept;
      &Apache::lonxml::debug("Selecting $name in $concept");
      if ($target eq 'analyze') {
	my $part_id="$Apache::inputtags::part.$Apache::inputtags::response[-1]";
        push (@{ $Apache::lonhomework::analyze{"$part_id.concepts"} },
	      $concept);
	$Apache::lonhomework::analyze{"$part_id.concept.$concept"}=
	  $Apache::response::conceptgroup{'names'};
	foreach my $name (@{ $Apache::response::conceptgroup{'names'} }) {
	  push (@{ $Apache::lonhomework::analyze{"$part_id.foils"} },
		$name);
	  $Apache::lonhomework::analyze{"$part_id.foil.value.$name"} =
	    $Apache::response::conceptgroup{"$name.value"};
	  $Apache::lonhomework::analyze{"$part_id.foil.text.$name"} =
	    $Apache::response::conceptgroup{"$name.text"};
	  $Apache::lonhomework::analyze{"$part_id.foil.location.$name"} =
	    $Apache::response::conceptgroup{"$name.location"};
	}
      } elsif ($target eq 'web') {
	  my $part_id="$Apache::inputtags::part.$Apache::inputtags::response[-1]";
	  push(@{ $Apache::hint::option{"$part_id.concepts"} },
	       $concept);
	  $Apache::hint::option{"$part_id.concept.$concept"}=
	      $Apache::response::conceptgroup{'names'};
      }
    }
  } elsif ($target eq 'edit') {
    $result=&Apache::edit::end_table();
  }
  return $result;
}

sub insert_conceptgroup {
  my $result="\n\t\t<conceptgroup concept=\"\">".&insert_foil()."\n\t\t</conceptgroup>\n";
  return $result;
}

sub start_foil {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
  my $result='';
  if ($target eq 'web' || $target eq 'tex' || $target eq 'analyze' ) {
    &Apache::lonxml::startredirection;
  } elsif ($target eq 'edit') {
    $result=&Apache::edit::tag_start($target,$token,"Foil");
    my $level='-2';
    if ($$tagstack['-2'] eq 'conceptgroup') { $level = '-3'; }
    my @opt;
    eval '@opt ='.&Apache::lonxml::get_param('options',$parstack,$safeeval,$level);
    $result.=&Apache::edit::text_arg('Name:','name',$token);
    $result.= &Apache::edit::select_or_text_arg('Correct Option:','value',
					       ['unused',(@opt)],$token,'15');
    my $randomize=&Apache::lonxml::get_param('randomize',$parstack,
					     $safeeval,'-3');
    if ($randomize ne 'no') {
	$result.=&Apache::edit::select_arg('Location:','location',
					   ['random','top','bottom'],$token);
    }
    $result .=&Apache::edit::end_row().&Apache::edit::start_spanning_row();
  } elsif ($target eq 'modified') {
    my $constructtag=&Apache::edit::get_new_args($token,$parstack,$safeeval,
						 'value','name','location');
    if ($constructtag) { $result = &Apache::edit::rebuild_tag($token); }
  } 
  return $result;
}

sub end_foil {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
  my $text ='';
  my $result = '';
  if ($target eq 'web' || $target eq 'tex') { 
      $text=&Apache::lonxml::endredirection;
      if ($target eq 'tex' and $Apache::lonhomework::type ne 'exam') { $text = '\vspace*{-2 mm}\item '.$text; }
  }
  if ($target eq 'web' || $target eq 'grade' || $target eq 'answer' 
      || $target eq 'tex' || $target eq 'analyze') {
    my $value = &Apache::lonxml::get_param('value',$parstack,$safeeval);
    if ($value ne 'unused') {
      my $name = &Apache::lonxml::get_param('name',$parstack,$safeeval);
      my $location =&Apache::lonxml::get_param('location',$parstack,$safeeval);
      &Apache::lonxml::debug("Got a name of :$name:");
      if (!$name) { $name=$Apache::lonxml::curdepth; }
      &Apache::lonxml::debug("Using a name of :$name:");
      if ( $Apache::optionresponse::conceptgroup
	   && !&Apache::response::showallfoils() ) {
	push @{ $Apache::response::conceptgroup{'names'} }, $name;
	$Apache::response::conceptgroup{"$name.value"} = $value;
	if ($target eq 'tex' and $Apache::lonhomework::type eq 'exam') {
	    $Apache::response::conceptgroup{"$name.text"} = '\vskip 4 mm $\triangleright$ '.$text;
	} else {
	    if ($target eq 'tex' and $Apache::lonhomework::type eq 'exam') {
		$Apache::response::conceptgroup{"$name.text"} = ' $\triangleright$ '.$text;
	    } else {
		$Apache::response::conceptgroup{"$name.text"} = $text;
	    }
	}
	$Apache::response::conceptgroup{"$name.location"} = $location;
      } else {
	push @{ $Apache::response::foilgroup{'names'} }, $name;
	$Apache::response::foilgroup{"$name.value"} = $value;
	if ($target eq 'tex' and $Apache::lonhomework::type eq 'exam') {
	    $Apache::response::foilgroup{"$name.text"} = '\vskip 5 mm $\triangleright$ '.$text;
	} else {
	    if ($target eq 'tex' and $Apache::lonhomework::type eq 'exam') {
		$Apache::response::foilgroup{"$name.text"} = ' $\triangleright$ '.$text;
	    } else {
		$Apache::response::foilgroup{"$name.text"} = $text;
	    }
	}
	$Apache::response::foilgroup{"$name.location"} = $location;
      }
    }
  }
  if ($target eq 'edit') {
    $result.= &Apache::edit::tag_end($target,$token,'');
  }
  return $result;
}

sub insert_foil {
  return '
<foil name="" value="unused">
<startouttext />
<endouttext />
</foil>';
}
1;
__END__
 
