# The LearningOnline Network with CAPA 
# tags that create controlled output
#
# $Id: outputtags.pm,v 1.30 2003/07/01 21:10:43 albertel Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#

package Apache::outputtags; 

use strict;
use Apache::lonnet;
use POSIX qw(strftime);

BEGIN {
    &Apache::lonxml::register('Apache::outputtags',('displayduedate','displaytitle','displayweight'));
}

sub initialize_outputtags {
    %Apache::outputtags::showonce=();
}

sub start_displayduedate {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
    my $result;
    if (exists($Apache::outputtags::showonce{'displayduedate'})) {
	return '';
    } else {
	$Apache::outputtags::showonce{'displayduedate'}=1;
    }
    my $status=$Apache::inputtags::status['-1'];
    &Apache::lonxml::debug("got a $status in duedatebox");
    if (($status =~ /CAN.*_ANSWER/) && ($target eq 'web')) {
#	my $format = &Apache::lonxml::get_param('format',$parstack,$safeeval);
#	if ($format eq '') {
#	    &Apache::lonxml::debug("using default format");
#	    $format="%c";
#	}
	my $id = $Apache::inputtags::part;
	my $date = &Apache::lonnet::EXT("resource.$id.duedate");
	&Apache::lonxml::debug("duedatebox found $date for $id");
	$result ='<table border="on"><tr><td>Due '.
	    &Apache::lonnavmaps::timeToHumanString($date).
		'</td></tr></table>';
#        } else {
#	    $result ='<table border="on"><tr><td>No due date set.</td></tr></table>';
#	}
    } elsif ( $target eq 'edit' ) {
	$result=&Apache::edit::tag_start($target,$token);
	$result.='</td></tr>';
	$result.=&Apache::edit::end_table();
    }
    return $result;
}

sub end_displayduedate {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
    my @result;
    if ($target eq 'edit') { $result[1]='no'; }
    return @result;
}

sub start_displaytitle {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
    my $result='';
    if (exists($Apache::outputtags::showonce{'displayduetitle'})) {
	return '';
    } else {
	$Apache::outputtags::showonce{'displayduetitle'}=1;
    }
    if ($target eq 'web') {
	$result="<h1>$Apache::lonhomework::name</h1>";
    } elsif ($target eq 'edit') {
	$result=&Apache::edit::tag_start($target,$token);
	$result.='</td></tr>';
	$result.=&Apache::edit::end_table();
    } elsif ($target eq 'tex' and $Apache::lonhomework::type ne 'exam') {
	$Apache::lonhomework::name=~s/%/\%/g;
	$Apache::lonhomework::name=~s/&/\&/g;
	$result='\vskip 0 mm\noindent\textbf{'.$Apache::lonhomework::name.'}\vskip 0 mm';
    }
    return $result;
}

sub end_displaytitle {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
    my @result;
    if ($target eq 'edit') { $result[1]='no'; }
    return @result;
}

sub multipart {
    my ($uri)=@_;
    if (!defined($uri)) { $uri=$ENV{'request.uri'}; }
    my @parts;
    my $metadata = &Apache::lonnet::metadata($uri,'packages');
    foreach (split(/\,/,$metadata)) {
	if ($_ =~ /^part_(.*)$/) {
	    my $part = $1;
	    if ($part ne '0') { push(@parts,$part); }
	}
    }
    return @parts;
}

sub start_displayweight {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
    my $result;
    if (exists($Apache::outputtags::showonce{'displayweight'})) {
	if(grep(/^\Q$Apache::inputtags::part\E$/,
		@{$Apache::outputtags::showonce{'displayweight'}})) {
	    return '';
	}
    }
    push(@{$Apache::outputtags::showonce{'displayweight'}},
	 $Apache::inputtags::part);
    my $status=$Apache::inputtags::status['-1'];
    if ($target eq 'web' || $target eq 'tex') {
	my $id = $Apache::inputtags::part;
	if ($id ne '0') {
	    my $weight = &Apache::lonnet::EXT("resource.$id.weight");
	    if (!defined($weight) || ($weight eq '')) { $weight=1; }
	    $result.=$weight;
	} else {
	    my @parts=&multipart($ENV{'request.uri'});
	    my $weight;
	    if (@parts) {
	        foreach my $part (@parts) {
		    my $pweight=&Apache::lonnet::EXT("resource.$part.weight");
		    if (!defined($pweight) || ($pweight eq '')) { $pweight=1; }
		    $weight+=$pweight;
	        }
	    } else {
		$weight = &Apache::lonnet::EXT("resource.$id.weight");
                if (!defined($weight) || ($weight eq '')) { $weight=1; }
	    }
	    $result=$weight;
	}
    } elsif ( $target eq 'edit' ) {
	$result=&Apache::edit::tag_start($target,$token);
	$result.='</td></tr>';
	$result.=&Apache::edit::end_table();
    }
    return $result;
}

sub end_displayweight {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
    my @result;
    if ($target eq 'edit') { $result[1]='no'; }
    return @result;
}

1;
__END__
