# The LearningOnline Network
# "About Me" Personal Information
#
# $Id: lonaboutme.pm,v 1.16 2003/04/12 15:57:30 www Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#

package Apache::lonaboutme;

use strict;
use Apache::Constants qw(:common);
use Apache::loncommon;
use Apache::lonnet;
use Apache::lontexconvert;
use Apache::lonfeedback;

sub handler {
    my $r = shift;
    $r->content_type('text/html');
    $r->send_http_header;
    return OK if $r->header_only;

# ------------------------------------------------------------ Print the screen
    $r->print(<<ENDDOCUMENT);
<html>
<head>
<title>The LearningOnline Network with CAPA</title>
ENDDOCUMENT
    my (undef,undef,$cdom,$cnum)=split(/\//,$r->uri);
# Is this even a user?
    if (&Apache::lonnet::homeserver($cnum,$cdom) eq 'no_host') {
	$r->print('</head><body>No user information available</body></html>');
        return OK;
    }
# --------------------------------------------------------- The syllabus fields
    my %syllabusfields=(
       'aaa_contactinfo'   => 'Contact Information',
       'bbb_aboutme'       => 'About Me',
       'ccc_webreferences' => 'Web References');

# ------------------------------------------------------------ Get Query String
    &Apache::loncommon::get_unprocessed_cgi
                            ($ENV{'QUERY_STRING'},['forcestudent','register']);
# ----------------------------------------------------- Force menu registration
    my $addentries='';
    if ($ENV{'form.register'}) {
       $addentries=' onLoad="'.&Apache::lonmenu::loadevents().
	   '" onUnload="'.&Apache::lonmenu::unloadevents().'"';
       $r->print(&Apache::lonmenu::registerurl(1));
    }
# --------------------------------------------------------------- Force Student
    my $forcestudent='';
    if ($ENV{'form.forcestudent'}) { $forcestudent='student'; };
       
# --------------------------------------- There is such a user, get environment
    my %courseenv=&Apache::lonnet::dump('environment',$cdom,$cnum);
    $r->print('</head>'.&Apache::loncommon::bodytag
                  ("Personal Information",$forcestudent,$addentries,'',$cdom,
                   $ENV{'form.register'}));
    $r->print('<h1>'.&Apache::loncommon::plainname($cnum,$cdom).'</h1>');
    if ($courseenv{'nickname'}) {
       $r->print(
         '<h2>&quot;'.$courseenv{'nickname'}.
         '&quot;</h2>');
    }
    $r->print('<h3>'.$Apache::lonnet::domaindescription{$cdom}.'</h3>'.
    '<p>'.&Apache::loncommon::messagewrapper('Send me a message',$cnum,$cdom).
    '</p>');
    my %syllabus=&Apache::lonnet::dump('aboutme',$cdom,$cnum);
    my $allowed=0;

# does this user have privileges to post, etc?

       $allowed=(($ENV{'user.name'} eq $cnum) && 
		 ($ENV{'user.domain'} eq $cdom));
       if ($forcestudent) { $allowed=0; }
 
       if ($allowed) {
          $r->print('<p><b>Privacy Note:</b> The information you submit can be viewed by anybody who is logged into LON-CAPA. Do not provide information that you are not ready to share publicly.</p>'.
&Apache::loncommon::help_open_topic('Uploaded_Templates_TextBoxes','Help with filling in text boxes').'</p><p><a href="'.$r->uri.'?forcestudent=1">Show Public View</a>'.
 &Apache::loncommon::help_open_topic('Uploaded_Templates_PublicView').'</p>');
      }
      if (($ENV{'form.uploaddoc.filename'}) &&
          ($ENV{'form.storeupl'}) && ($allowed)) {
 	  if ($ENV{'form.uploaddoc.filename'}=~/\.(gif|jpg|png|jpeg)$/i) {
             $syllabus{'uploaded.photourl'}=
                 &Apache::lonnet::userfileupload('uploaddoc');
 	  }
          $syllabus{'uploaded.lastmodified'}=time;
          &Apache::lonnet::put('aboutme',\%syllabus,$cdom,$cnum);
       }
       if (($allowed) && ($ENV{'form.storesyl'})) {
	   foreach (keys %syllabusfields) {
               my $field=$ENV{'form.'.$_};
               $field=~s/\s+$//s;
               $field=&Apache::lonfeedback::clear_out_html($field,
                                                           $ENV{'user.adv'});
	       $syllabus{$_}=$field;
           }
           $syllabus{'uploaded.lastmodified'}=time;
           &Apache::lonnet::put('aboutme',\%syllabus,$cdom,$cnum);
       }

# ---------------------------------------------------------------- Get syllabus
    if (($syllabus{'uploaded.lastmodified'}) || ($allowed)) {
       my $lastmod=$syllabus{'uploaded.lastmodified'};
       $lastmod=($lastmod?localtime($lastmod):'never');
       $r->print('Last updated: '.$lastmod);
       if ($syllabus{'uploaded.photourl'}) {
	   $r->print('<img src="'.
             &Apache::lonnet::tokenwrapper($syllabus{'uploaded.photourl'}).
             '" align="right" />');
       }
       if ($allowed) {
           $r->print(
	 '<form method="post" enctype="multipart/form-data">'.
         '<h3>Upload a Photo</h3>'.
         '<input type="file" name="uploaddoc" size="50">'.
         '<input type="submit" name="storeupl" value="Upload">'.
	 '</form><form method="post">');
       }
       foreach (sort keys %syllabusfields) {
          if (($syllabus{$_}) || ($allowed)) {
              my $message=$syllabus{$_};
              $message=~s/\n/\<br \/\>/g;
              $message
             =~s/(http\:\/\/[^\s]+)/\<a href=\"$1\"\>\<tt\>$1\<\/tt\>\<\/a\>/g;
	      $message=&Apache::lontexconvert::msgtexconverted($message);
              $r->print('<h3>'.$syllabusfields{$_}.'</h3><blockquote>'.
                        $message.'</blockquote>');
              if ($allowed) {
                 $r->print('<br /><textarea cols="80" rows="6" name="'.$_.'">'.
			   $syllabus{$_}.
           '</textarea><input type="submit" name="storesyl" value="Store" />');
	      }
	  }
       }
       if ($allowed) {
	   $r->print('</form>');
       }
       $r->print('</p>');
    } else {
       $r->print('<p>No personal information provided.</p>');
    }
    if ($ENV{'request.course.id'}) {
	if (&Apache::lonnet::allowed('srm',$ENV{'request.course.id'})) {
            $r->print('<hr /><h3>User Notes, Records of Face-To-Face Discussions, and Critical Messages in Course</h3>Shared by course faculty and staff<br />');
	    &Apache::lonmsg::disfacetoface($r,$cnum,$cdom);
            $r->print(&Apache::loncommon::noteswrapper('Add Records',$cnum,$cdom));
        }
    }
    $r->print('</body></html>');
    return OK;
} 

1;
__END__
