# The LearningOnline Network with CAPA
# a pile of common html routines
#
# $Id: lonhtmlcommon.pm,v 1.28 2003/07/25 19:56:37 matthew Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
######################################################################
######################################################################

=pod

=head1 NAME

Apache::lonhtmlcommon - routines to do common html things

=head1 SYNOPSIS

Referenced by other mod_perl Apache modules.

=head1 INTRODUCTION

lonhtmlcommon is a collection of subroutines used to present information
in a consistent html format, or provide other functionality related to
html.

=head2 General Subroutines

=over 4

=cut 

######################################################################
######################################################################

package Apache::lonhtmlcommon;

use Time::Local;
use strict;


##############################################
##############################################

=pod

=item textbox

=cut

##############################################
##############################################
sub textbox {
    my ($name,$value,$size,$special) = @_;
    $size = 40 if (! defined($size));
    my $Str = '<input type="text" name="'.$name.'" size="'.$size.'" '.
        'value="'.$value.'" '.$special.' />';
    return $Str;
}

##############################################
##############################################

=pod

=item checkbox

=cut

##############################################
##############################################
sub checkbox {
    my ($name) = @_;
    my $Str = '<input type="checkbox" name="'.$name.'" />';
    return $Str;
}



##############################################
##############################################

=pod

=item &date_setter

&date_setter returns html and javascript for a compact date-setting form.
To retrieve values from it, use &get_date_from_form().

Inputs

=over 4

=item $dname 

The name to prepend to the form elements.  
The form elements defined will be dname_year, dname_month, dname_day,
dname_hour, dname_min, and dname_sec.

=item $currentvalue

The current setting for this time parameter.  A unix format time
(time in seconds since the beginning of Jan 1st, 1970, GMT.  
An undefined value is taken to indicate the value is the current time.
Also, to be explicit, a value of 'now' also indicates the current time.

=item $special

Additional html/javascript to be associated with each element in
the date_setter.  See lonparmset for example usage.

=back

Bugs

The method used to restrict user input will fail in the year 2400.

=cut

##############################################
##############################################
sub date_setter {
    my ($formname,$dname,$currentvalue,$special) = @_;
    if (! defined($currentvalue) || $currentvalue eq 'now') {
        $currentvalue = time;
    }
    # other potentially useful values:     wkday,yrday,is_daylight_savings
    my ($sec,$min,$hour,$mday,$month,$year,undef,undef,undef) = 
        localtime($currentvalue);
    $year += 1900;
    my $result = "\n<!-- $dname date setting form -->\n";
    $result .= <<ENDJS;
<script language="Javascript">
    function $dname\_checkday() {
        var day   = document.$formname.$dname\_day.value;
        var month = document.$formname.$dname\_month.value;
        var year  = document.$formname.$dname\_year.value;
        var valid = true;
        if (day < 1) {
            document.$formname.$dname\_day.value = 1;
        } 
        if (day > 31) {
            document.$formname.$dname\_day.value = 31;
        }
        if ((month == 1)  || (month == 3)  || (month == 5)  ||
            (month == 7)  || (month == 8)  || (month == 10) ||
            (month == 12)) {
            if (day > 31) {
                document.$formname.$dname\_day.value = 31;
                day = 31;
            }
        } else if (month == 2 ) {
            if ((year % 4 == 0) && (year % 100 != 0)) {
                if (day > 29) {
                    document.$formname.$dname\_day.value = 29;
                }
            } else if (day > 29) {
                document.$formname.$dname\_day.value = 28;
            }
        } else if (day > 30) {
            document.$formname.$dname\_day.value = 30;
        }
    }
</script>
ENDJS
    $result .= "  <nobr><select name=\"$dname\_month\" ".$special.' '.
        "onChange=\"javascript:$dname\_checkday()\" >\n";
    my @Months = qw/January February  March     April   May      June 
                    July    August    September October November December/;
    # Pad @Months with a bogus value to make indexing easier
    unshift(@Months,'If you can read this an error occurred');
    for(my $m = 1;$m <=$#Months;$m++) {
        $result .= "      <option value=\"$m\" ";
        $result .= "selected " if ($m-1 == $month);
        $result .= "> $Months[$m] </option>\n";
    }
    $result .= "  </select>\n";
    $result .= "  <input type=\"text\" name=\"$dname\_day\" ".
            "value=\"$mday\" size=\"3\" ".$special.' '.
            "onChange=\"javascript:$dname\_checkday()\" />\n";
    $result .= "  <input type=\"year\" name=\"$dname\_year\" ".
            "value=\"$year\" size=\"5\" ".$special.' '.
            "onChange=\"javascript:$dname\_checkday()\" />\n";
    $result .= "&nbsp;&nbsp;";
    $result .= "  <select name=\"$dname\_hour\" ".$special." >\n";
    for (my $h = 0;$h<24;$h++) {
        $result .= "      <option value=\"$h\" ";
        $result .= "selected " if ($hour == $h);
        $result .= "> ";
        if ($h == 0) {
            $result .= "12 am";
        } elsif($h == 12) {
            $result .= "12 noon";
        } elsif($h < 12) {
            $result .= "$h am";
        } else {
            $result .= $h-12 ." pm";
        }
        $result .= " </option>\n";
    } 
    $result .= "  </select>\n";
    $result .= "  <input type=\"text\" name=\"$dname\_minute\" ".$special.' '.
        "value=\"$min\" size=\"3\" /> m\n";
    $result .= "  <input type=\"text\" name=\"$dname\_second\" ".$special.' '.
        "value=\"$sec\" size=\"3\" /> s\n";
    $result .= "</nobr>\n<!-- end $dname date setting form -->\n";
    return $result;
}

##############################################
##############################################

=pod

=item &get_date_from_form

get_date_from_form retrieves the date specified in an &date_setter form.

Inputs:

=over 4

=item $dname

The name passed to &datesetter, which prefixes the form elements.

=item $defaulttime

The unix time to use as the default in case of poor inputs.

=back

Returns: Unix time represented in the form.

=cut

##############################################
##############################################
sub get_date_from_form {
    my ($dname) = @_;
    my ($sec,$min,$hour,$day,$month,$year);
    #
    if (defined($ENV{'form.'.$dname.'_second'})) {
        my $tmpsec = $ENV{'form.'.$dname.'_second'};
        if (($tmpsec =~ /^\d+$/) && ($tmpsec >= 0) && ($tmpsec < 60)) {
            $sec = $tmpsec;
        }
    }
    if (defined($ENV{'form.'.$dname.'_minute'})) {
        my $tmpmin = $ENV{'form.'.$dname.'_minute'};
        if (($tmpmin =~ /^\d+$/) && ($tmpmin >= 0) && ($tmpmin < 60)) {
            $min = $tmpmin;
        }
    }
    if (defined($ENV{'form.'.$dname.'_hour'})) {
        my $tmphour = $ENV{'form.'.$dname.'_hour'};
        if (($tmphour =~ /^\d+$/) && ($tmphour > 0) && ($tmphour < 32)) {
            $hour = $tmphour;
        }
    }
    if (defined($ENV{'form.'.$dname.'_day'})) {
        my $tmpday = $ENV{'form.'.$dname.'_day'};
        if (($tmpday =~ /^\d+$/) && ($tmpday > 0) && ($tmpday < 32)) {
            $day = $tmpday;
        }
    }
    if (defined($ENV{'form.'.$dname.'_month'})) {
        my $tmpmonth = $ENV{'form.'.$dname.'_month'};
        if (($tmpmonth =~ /^\d+$/) && ($tmpmonth > 0) && ($tmpmonth < 13)) {
            $month = $tmpmonth - 1;
        }
    }
    if (defined($ENV{'form.'.$dname.'_year'})) {
        my $tmpyear = $ENV{'form.'.$dname.'_year'};
        if (($tmpyear =~ /^\d+$/) && ($tmpyear > 1900)) {
            $year = $tmpyear - 1900;
        }
    }
    if (($year<70) || ($year>137)) { return undef; }
    if (eval(&timelocal($sec,$min,$hour,$day,$month,$year))) {
        return &timelocal($sec,$min,$hour,$day,$month,$year);
    } else {
        return undef;
    }
}

##############################################
##############################################

=pod

=item &pjump_javascript_definition()

Returns javascript defining the 'pjump' function, which opens up a
parameter setting wizard.

=cut

##############################################
##############################################
sub pjump_javascript_definition {
    my $Str = <<END;
    function pjump(type,dis,value,marker,ret,call) {
        parmwin=window.open("/adm/rat/parameter.html?type="+escape(type)
                 +"&value="+escape(value)+"&marker="+escape(marker)
                 +"&return="+escape(ret)
                 +"&call="+escape(call)+"&name="+escape(dis),"LONCAPAparms",
                 "height=350,width=350,scrollbars=no,menubar=no");
    }
END
    return $Str;
}

##############################################
##############################################

=pod

=item &javascript_nothing()

Return an appropriate null for the users browser.  This is used
as the first arguement for window.open calls when you want a blank
window that you can then write to.

=cut

##############################################
##############################################
sub javascript_nothing {
    # mozilla and other browsers work with "''", but IE on mac does not.
    my $nothing = "''";
    my $user_browser;
    my $user_os;
    $user_browser = $ENV{'browser.type'} if (exists($ENV{'browser.type'}));
    $user_os      = $ENV{'browser.os'}   if (exists($ENV{'browser.os'}));
    if (! defined($user_browser) || ! defined($user_os)) {
        (undef,$user_browser,undef,undef,undef,$user_os) = 
                           &Apache::loncommon::decode_user_agent();
    }
    if ($user_browser eq 'explorer' && $user_os =~ 'mac') {
        $nothing = "'javascript:void(0);'";
    }
    return $nothing;
}


##############################################
##############################################

=pod

=item &StatusOptions()

Returns html for a selection box which allows the user to choose the
enrollment status of students.  The selection box name is 'Status'.

Inputs:

$status: the currently selected status.  If undefined the value of
$ENV{'form.Status'} is taken.  If that is undefined, a value of 'Active'
is used.

$formname: The name of the form.  If defined the onchange attribute of
the selection box is set to document.$formname.submit().

$size: the size (number of lines) of the selection box.

$onchange: javascript to use when the value is changed.  Enclosed in 
double quotes, ""s, not single quotes.

Returns: a perl string as described.

=cut

##############################################
##############################################
sub StatusOptions {
    my ($status, $formName,$size,$onchange)=@_;
    $size = 1 if (!defined($size));
    if (! defined($status)) {
        $status = 'Active';
        $status = $ENV{'form.Status'} if (exists($ENV{'form.Status'}));
    }

    my $OpSel1 = '';
    my $OpSel2 = '';
    my $OpSel3 = '';

    if($status eq 'Any')         { $OpSel3 = ' selected'; }
    elsif($status eq 'Expired' ) { $OpSel2 = ' selected'; }
    else                         { $OpSel1 = ' selected'; }

    my $Str = '';
    $Str .= '<select name="Status"';
    if(defined($formName) && $formName ne '' && ! defined($onchange)) {
        $Str .= ' onchange="document.'.$formName.'.submit()"';
    }
    if (defined($onchange)) {
        $Str .= ' onchange="'.$onchange.'"';
    }
    $Str .= ' size="'.$size.'" ';
    $Str .= '>'."\n";
    $Str .= '<option value="Active" '.$OpSel1.'>'.
        'Currently Enrolled</option>'."\n";
    $Str .= '<option value="Expired" '.$OpSel2.'>'.
        'Previously Enrolled</option>'."\n";
    $Str .= '<option value="Any" '.$OpSel3.'>'.
        'Any Enrollment Status</option>'."\n";
    $Str .= '</select>'."\n";
}


########################################################
########################################################

=pod

=item &MultipleSectionSelect()

Inputs: 

=over 4

=item $sections A references to an array containing the names of all the
sections used in a class.

=item $selectedSections A reference to an array containing the names of the
currently selected sections.

=back 

Returns: a string containing HTML for a multiple select box for
selecting sections of a course.  

The form element name is 'Section'.  @$sections is sorted prior to output.

=cut

########################################################
########################################################
sub MultipleSectionSelect {
    my ($sections,$selectedSections)=@_;

    my $Str = '';
    $Str .= '<select name="Section" multiple="true" size="4">'."\n";

    foreach (sort @$sections) {
        $Str .= '<option';
        foreach my $selected (@$selectedSections) {
            if($_ eq $selected) {
                $Str .= ' selected=""';
            }
        }
        $Str .= '>'.$_.'</option>'."\n";
    }
    $Str .= '</select>'."\n";
    
    return $Str;
}

########################################################
########################################################

=pod

=item &Title()

Inputs: $pageName a string containing the name of the page to be sent
to &Apache::loncommon::bodytag.

Returns: string containing being <html> and complete <head> and <title>
as well as a <script> to focus the current window and change its width
and height to 500.  Why?  I do not know.  If you find out, please update
this documentation.

=cut

########################################################
########################################################
sub Title {
    my ($pageName)=@_;

    my $Str = '';

    $Str .= '<html><head><title>'.$pageName.'</title></head>'."\n";
    $Str .= &Apache::loncommon::bodytag($pageName)."\n";
    $Str .= '<script>window.focus(); window.width=500;window.height=500;';
    $Str .= '</script>'."\n";

    return $Str;
}

########################################################
########################################################

=pod

=item &CreateHeadings()

This function generates the column headings for the chart.

=over 4

Inputs: $CacheData, $keyID, $headings, $spacePadding

$CacheData: pointer to a hash tied to the cached data database

$keyID: a pointer to an array containing the names of the data 
held in a column and is used as part of a key into $CacheData

$headings: The names of the headings for the student information

$spacePadding: The spaces to go between columns

Output: $Str

$Str: A formatted string of the table column headings.

=back

=cut

########################################################
########################################################
sub CreateHeadings {
    my ($data,$keyID,$headings,$displayString,$format)=@_;
    my $Str='';
    my $formatting = '';

    for(my $index=0; $index<(scalar @$headings); $index++) {
 	my $currentHeading=$headings->[$index];
        if($format eq 'preformatted') {
            my @dataLength=split(//,$currentHeading);
            my $length=scalar @dataLength;
            $formatting = (' 'x
                      ($data->{$keyID->[$index].':columnWidth'}-$length));
        }
        my $linkdata=$keyID->[$index];

        my $tempString = $displayString;
        $tempString =~ s/LINKDATA/$linkdata/;
        $tempString =~ s/DISPLAYDATA/$currentHeading/;
        $tempString =~ s/FORMATTING/$formatting/;

        $Str .= $tempString;
    }

    return $Str;
}

########################################################
########################################################

=pod

=item &FormatStudentInformation()

This function produces a formatted string of the student\'s information:
username, domain, section, full name, and PID.

=over 4

Input: $cache, $name, $keyID, $spacePadding

$cache: This is a pointer to a hash that is tied to the cached data

$name:  The name and domain of the current student in name:domain format

$keyID: A pointer to an array holding the names used to

remove data from the hash.  They represent the name of the data to be removed.

$spacePadding: Extra spaces that represent the space between columns

Output: $Str

$Str: Formatted string.

=back

=cut

########################################################
########################################################
sub FormatStudentInformation {
    my ($data,$name,$keyID,$displayString,$format)=@_;
    my $Str='';
    my $currentColumn;

    for(my $index=0; $index<(scalar @$keyID); $index++) {
        $currentColumn=$data->{$name.':'.$keyID->[$index]};

        if($format eq 'preformatted') {
            my @dataLength=split(//,$currentColumn);
            my $length=scalar @dataLength;
            $currentColumn.= (' 'x
                     ($data->{$keyID->[$index].':columnWidth'}-$length));
        }

        my $tempString = $displayString;
        $tempString =~ s/DISPLAYDATA/$currentColumn/;

        $Str .= $tempString;
    }

    return $Str;
}

########################################################
########################################################

=pod

=item Progess Window Handling Routines

These routines handle the creation, update, increment, and closure of 
progress windows.  The progress window reports to the user the number
of items completed and an estimate of the time required to complete the rest.

=over 4


=item &Create_PrgWin

Writes javascript to the client to open a progress window and returns a
data structure used for bookkeeping.

Inputs

=over 4

=item $r Apache request

=item $title The title of the progress window

=item $heading A description (usually 1 line) of the process being initiated.

=item $number_to_do The total number of items being processed.

=back

Returns a hash containing the progress state data structure.


=item &Update_PrgWin

Updates the text in the progress indicator.  Does not increment the count.
See &Increment_PrgWin.

Inputs:

=over 4

=item $r Apache request

=item $prog_state Pointer to the data structure returned by &Create_PrgWin

=item $displaystring The string to write to the status indicator

=back

Returns: none


=item Increment_PrgWin

Increment the count of items completed for the progress window by 1.  

Inputs:

=over 4

=item $r Apache request

=item $prog_state Pointer to the data structure returned by Create_PrgWin

=item $extraInfo A description of the items being iterated over.  Typically
'student'.

=back

Returns: none


=item Close_PrgWin

Closes the progress window.

Inputs:

=over 4 

=item $r Apache request

=item $prog_state Pointer to the data structure returned by Create_PrgWin

=back

Returns: none

=back

=cut

########################################################
########################################################

# Create progress
sub Create_PrgWin {
    my ($r, $title, $heading, $number_to_do)=@_;
    $r->print('<script>'.
    "popwin=open(\'\',\'popwin\',\'width=400,height=100\');".
    "popwin.document.writeln(\'<html><head><title>$title</title></head>".
	      "<body bgcolor=\"#88DDFF\">".
              "<h4>$heading</h4>".
              "<form name=popremain>".
              '<input type="text" size="55" name="remaining" value="Starting"></form>'.
              "</body></html>\');".
    "popwin.document.close();".
    "</script>");

    my %prog_state;
    $prog_state{'done'}=0;
    $prog_state{'firststart'}=&Time::HiRes::time();
    $prog_state{'laststart'}=&Time::HiRes::time();
    $prog_state{'max'}=$number_to_do;

    $r->rflush();
    return %prog_state;
}

# update progress
sub Update_PrgWin {
    my ($r,$prog_state,$displayString)=@_;
    $r->print('<script>popwin.document.popremain.remaining.value="'.
              $displayString.'";</script>');
    $$prog_state{'laststart'}=&Time::HiRes::time();
    $r->rflush();
}

# increment progress state
sub Increment_PrgWin {
    my ($r,$prog_state,$extraInfo)=@_;
    $$prog_state{'done'}++;
    my $time_est= (&Time::HiRes::time() - $$prog_state{'firststart'})/
        $$prog_state{'done'} *
	($$prog_state{'max'}-$$prog_state{'done'});
    $time_est = int($time_est);
    if (int ($time_est/60) > 0) {
	my $min = int($time_est/60);
	my $sec = $time_est % 60;
	$time_est = $min.' minutes';
        if ($min < 10)  {
            if ($sec > 1) {
                $time_est.= ', '.$sec.' seconds';
            } elsif ($sec > 0) {
                $time_est.= ', '.$sec.' second';
            }
        }
    } else {
	$time_est .= ' seconds';
    }
    my $lasttime = &Time::HiRes::time()-$$prog_state{'laststart'};
    if ($lasttime > 9) {
        $lasttime = int($lasttime);
    } elsif ($lasttime < 0.01) {
        $lasttime = 0;
    } else {
        $lasttime = sprintf("%3.2f",$lasttime);
    }
    if ($lasttime == 1) {
        $lasttime = '('.$lasttime.' second for '.$extraInfo.')';
    } else {
        $lasttime = '('.$lasttime.' seconds for '.$extraInfo.')';
    }
    #
    my $user_browser = $ENV{'browser.type'} if (exists($ENV{'browser.type'}));
    my $user_os      = $ENV{'browser.os'}   if (exists($ENV{'browser.os'}));
    if (! defined($user_browser) || ! defined($user_os)) {
        (undef,$user_browser,undef,undef,undef,$user_os) = 
                           &Apache::loncommon::decode_user_agent();
    }
    if ($user_browser eq 'explorer' && $user_os =~ 'mac') {
        $lasttime = '';
    }
    $r->print('<script>popwin.document.popremain.remaining.value="'.
	      $$prog_state{'done'}.'/'.$$prog_state{'max'}.
	      ': '.$time_est.' remaining '.$lasttime.'";'.'</script>');
    $$prog_state{'laststart'}=&Time::HiRes::time();
    $r->rflush();
}

# close Progress Line
sub Close_PrgWin {
    my ($r,$prog_state)=@_;
    $r->print('<script>popwin.close()</script>'."\n");
    undef(%$prog_state);
    $r->rflush(); 
}

1;

__END__
