# The LearningOnline Network with CAPA
# Handler to manage course access keys 
#
# $Id: lonmanagekeys.pm,v 1.11 2003/05/02 19:00:29 www Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
###############################################################
###############################################################

package Apache::lonmanagekeys;

use strict;
use Apache::lonnet();
use Apache::loncommon();
use Apache::lonhtmlcommon();
use Apache::Constants qw(:common :http REDIRECT);
use Spreadsheet::WriteExcel;

###############################################################
###############################################################
sub header {
    my $bodytag=&Apache::loncommon::bodytag('Access Key Management');
    return(<<ENDHEAD);
<html>
<head>
<title>LON-CAPA Access Key Management</title>
</head>
$bodytag
<form method="post" enctype="multipart/form-data"  
      action="/adm/managekeys" name="keyform">
ENDHEAD
}

# =================================================== Show student list to drop
sub show_key_list {
    my ($r,$csvlist,$comment,$newonly,$checkonly,%cenv)=@_;
    $comment=~s/\W/\./g;
    my %accesskeys=&Apache::lonnet::dump
	('accesskeys',$cenv{'domain'},$cenv{'num'});
    unless ($csvlist) {
	$r->print(<<ENDTABLEHEADER);
<script>
    function copyallcom(tf) {
	for (i=0; i<tf.elements.length; i++) {
            if  (tf.elements[i].name.indexOf('com_')==0) {
	      tf.elements[i].value+=tf.copyall.value;
            }
        }

    }
</script>
<h3>List of Keys/Enter New Comments</h3>
<table border="2"><tr><th>Key</th><th>Checked Out</th>
<th>Comments/Remarks/Notes</th>
<th>Enter Additional Comments/Remarks/Notes<br />
<input type="text" size="40" name="copyall" />
<input type="button" value="Copy to All" onClick="copyallcom(this.form);" />
</th></tr>
ENDTABLEHEADER
    }
    foreach (keys %accesskeys) {
        if ($_=~/^error\:/) {
	    $r->print('<tr><td>No keys have been generated yet.</td></tr>');
        } elsif ($accesskeys{$_}=~/$comment/) {
	    my ($checkout,$com)=split(/\s*\#\s*/,$accesskeys{$_});
            unless ($checkout) {
		if ($checkonly) { next; }
            } else {
		if ($newonly) { next; }
            }
            unless ($csvlist) {
		$r->print("\n<tr><td><tt>".$_.'</tt></td><td>'.($checkout?
                     $checkout:'-').'</td><td>'.
                     join('<br />',split(/\s*\;\s*/,$com)).
		     '</td><td><input type="text" size="40" name="com_'.$_.
		     '" value="" /></td></tr>');
	    } else {
		my @line = ();
		push @line,&Apache::loncommon::csv_translate($_);
		push @line,&Apache::loncommon::csv_translate($checkout);
		foreach (split(/\s*\;\s*/,$com)) {
 		   push @line,&Apache::loncommon::csv_translate($_);
		}
		my $tmp = $";
		$" = '","';
		$r->print("\"@line\"\n");
		$" = $tmp;
	    }
       }
    }
    unless ($csvlist) {
	$r->print('</table>');
	$r->print('<input type="submit" name="addcom" value="Add Above Comments to Keys" /><hr />');
    }
    return '';
}


# ----------------------------------------------------------- Toggle Key Access

sub togglekeyaccess {
    my %cenv=@_;
    unless ($cenv{'domain'}) { return; }
    if ($cenv{'keyaccess'} eq 'yes') {
       return 'Removing key access: '.
       &Apache::lonnet::del('environment',['keyaccess'],
			    $cenv{'domain'},$cenv{'num'});
   } else {
      return 'Establishing key access: '.
       &Apache::lonnet::put('environment',{'keyaccess' => 'yes'},
			    $cenv{'domain'},$cenv{'num'});
    }
}

# --------------------------------------------------------------- Generate Keys

sub genkeys {
    my ($num,$comments,%cenv)=@_;
    unless ($comments) { $comments=''; }
    $comments=~s/\#/ /g;
    $comments=~s/\;/ /g;
    unless ($num) { return 'No number of keys given.'; }
    unless (($num=~/^\d+$/) && ($num>0)) { 
	return 'Invalid number of keys given.'; 
    }
    my $batchnumber='BATCH_'.time().'_'.$$;
    return 'Generated '.&Apache::lonnet::generate_access_keys
    ($num,$cenv{'domain'},$cenv{'num'},$batchnumber.'; '.$comments).' of '.
    $num.' access keys (Batch Number: '.$batchnumber.')',$batchnumber;
}

# ---------------------------------------------------------------- Add comments

sub addcom {
    my %cenv=@_;
    my %newcomment=();
    undef %newcomment;
    foreach (keys %ENV) {
	if ($_=~/^form\.com\_(.+)$/) {
            my $key=$1;
	    my $comment=$ENV{$_};
            $comment=~s/^\s+//gs;
            if ($comment) {
               &Apache::lonnet::comment_access_key
		   ($key,$cenv{'domain'},$cenv{'num'},$comment); 
	   }
	}
    }
    return '';
}
###################################################################
###################################################################
sub handler {
    my $r=shift;
    if ($r->header_only) {
        $r->content_type('text/html');
        $r->send_http_header;
        return OK;
    }
    &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},
					    ['state','cid']);
    if (($ENV{'form.domain'}) && ($ENV{'form.course'})) {
	$ENV{'form.cid'}=$ENV{'form.domain'}.'_'.$ENV{'form.course'};
    }

    unless (&Apache::lonnet::allowed('mky',$ENV{'request.role.domain'})) {
        $ENV{'user.error.msg'}=
            "/adm/managekeys:mky:0:0:Cannot manage access keys";
        return HTTP_NOT_ACCEPTABLE; 
    }
    if ($ENV{'form.cid'}) {
        my %cenv=&Apache::lonnet::coursedescription($ENV{'form.cid'});
	if ($ENV{'form.listkeyscsv'}) {
#
# CSV Output
#
	    $r->content_type('text/csv');
            $r->send_http_header;
#
# Do CSV
#
	    &show_key_list($r,1,$ENV{'form.listcom'},
                          $ENV{'form.newonly'},$ENV{'form.checkonly'},%cenv);

	} else {
#
# Normal web stuff
#
	    $r->content_type('text/html');
	    $r->send_http_header;
	    $r->print(&header());
	
	    $r->print(
	    '<input type="hidden" name="cid" value="'.$ENV{'form.cid'}.'" />');
# --- Actions
	    if ($ENV{'form.toggle'}) {
		$r->print(&togglekeyaccess(%cenv).'<br />');
		%cenv=&Apache::lonnet::coursedescription($ENV{'form.cid'});
	    }
            my $batchnumber='';
	    if ($ENV{'form.genkeys'}) {
		(my $msg,$batchnumber)=
		    &genkeys($ENV{'form.num'},$ENV{'form.comments'},%cenv);
                $r->print($msg.'<br />');
	    }
            if ($ENV{'form.listkeys'}) {
		&show_key_list($r,0,$ENV{'form.listcom'},
                          $ENV{'form.newonly'},$ENV{'form.checkonly'},%cenv);
            }
            if ($ENV{'form.addcom'}) {
		&addcom(%cenv);
            }
# --- Menu
	    $r->print('<h3>Key Access</h3>');
	    if ($cenv{'keyaccess'} eq 'yes') {
		$r->print('Access to this course is key controlled.<br /><input type="submit" name="toggle" value="Open Access" />')
		} else {
		    $r->print('Access to this course is open, no access keys.<br /><input type="submit" name="toggle" value="Control Access" />');
	    }
	    $r->print(<<ENDKEYMENU);
<hr /><h3>Generate New Keys</h3>
Number of keys to be generated: <input type="text" name="num" size="6" /><br />
Comments/Remarks/Notes: <input type="text" name="comments" size="30" /><br />
<input type="submit" name="genkeys" value="Generate Keys" />
<hr /><h3>List Keys</h3>
Comments/Remarks/Notes/User/Batch Number Filter:
<input type="text" name="listcom" size="30" value="$batchnumber" /><br />
<input type="checkbox" name="newonly" /> Unused keys only<br />
<input type="checkbox" name="checkonly" /> Used keys only<br />
<input type="submit" name="listkeys" value="List Keys/Add Comments" />
<input type="submit" name="listkeyscsv" value="CSV List of Keys" />
ENDKEYMENU
	    $r->print('</form></body></html>');
	}
    } else {
	# Start page no course id
	$r->content_type('text/html');
	$r->send_http_header;
	$r->print(&header().&Apache::loncommon::coursebrowser_javascript());
        $r->print(
   'Course ID: <input input type="text" size="25" name="course" value="" />');
        $r->print('Domain: '.&Apache::loncommon::select_dom_form(
               $ENV{'request.role.domain'},'domain'));
        $r->print(&Apache::loncommon::selectcourse_link(
					        'keyform','course','domain'));
        $r->print('<br /><input type="submit" value="Manage Access Keys" />');
	$r->print('</form></body></html>');
    }
    return OK;
}

###################################################################
###################################################################

1;
__END__


