# The LearningOnline Network with CAPA
# Routines to control the menu
#
# $Id: lonmenu.pm,v 1.85 2003/08/18 18:16:24 www Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
#
# There are two parameters controlling the action of this module:
#
# browser.interface - if this is 'textual', it overrides the second parameter
# and goes to screen reader PDA mode
#
# environment.remote - if this is 'on', the routines controll the remote
# control, otherwise they render the main window controls; ignored it
# browser.interface is 'textual'
#

package Apache::lonmenu;

use strict;
use Apache::lonnet;
use Apache::Constants qw(:common);
use Apache::lonhtmlcommon();
use Apache::loncommon;
use Apache::File;
use vars qw(@desklines $readdesk);
my @inlineremote;
my $font;
my $tabbg;
my $pgbg;

# ============================= This gets called at the top of the body section

sub menubuttons {
    my $forcereg=shift;
    my $target  =shift;
    my $registration=shift;
    my $navmaps='';
    my $reloadlink='';
    my $escurl=&Apache::lonnet::escape($ENV{'REQUEST_URI'});
    my $escsymb=&Apache::lonnet::escape($ENV{'request.symb'});
    if ($ENV{'browser.interface'} eq 'textual') {
# Textual display only
        if ($ENV{'request.course.id'}) {
	    $navmaps=(<<ENDNAV);
<a href="/adm/navmaps?postdata=$escurl&postsymb=$escsymb" target="_top">Navigate Contents</a>
ENDNAV
            if (($ENV{'REQUEST_URI'}=~/^\/adm\//) &&
         ($ENV{'REQUEST_URI'}!~/^\/adm\/wrapper\//) &&
         ($ENV{'REQUEST_URI'}!~/^\/adm\/.*\/(smppg|bulletinboard|aboutme)(\?|$)/)) {
                my $escreload=&Apache::lonnet::escape('return:');
                $reloadlink=(<<ENDRELOAD);
<a href="/adm/flip?postdata=$escreload" target="_top"><font color="$font">Return to Last Location</font></a>
ENDRELOAD
            }
        }
	my $output=(<<ENDMAINMENU);
<script>
// BEGIN LON-CAPA Internal
</script>
<a href="/adm/menu" target="_top">Main Menu</a>
$reloadlink $navmaps<br />
<script>
// END LON-CAPA Internal
</script>
ENDMAINMENU
        if ($registration) { $output.=&innerregister($forcereg,$target); }
	return $output."<hr />";
    } elsif ($ENV{'environment.remote'} eq 'off') {
# Remote Control is switched off
# figure out colors
	my $function='student';
        if ($ENV{'request.role'}=~/^(cc|in|ta|ep)/) {
	    $function='coordinator';
        }
	if ($ENV{'request.role'}=~/^(su|dc|ad|li)/) {
            $function='admin';
        }
        if (($ENV{'request.role'}=~/^(au|ca)/) ||
            ($ENV{'REQUEST_URI'}=~/^(\/priv|\~)/)) {
            $function='author';
        }
        my $domain=&Apache::loncommon::determinedomain();
        $pgbg=&Apache::loncommon::designparm($function.'.pgbg',$domain);
        $tabbg=&Apache::loncommon::designparm($function.'.tabbg',$domain);
        $font=&Apache::loncommon::designparm($function.'.font',$domain);
        my $link=&Apache::loncommon::designparm($function.'.link',$domain);
        my $alink=&Apache::loncommon::designparm($function.'.alink',$domain);
        my $vlink=&Apache::loncommon::designparm($function.'.vlink',$domain);
        my $sidebg=&Apache::loncommon::designparm($function.'.sidebg',$domain);
# Do we have a NAV link?
        if ($ENV{'request.course.id'}) {
	    $navmaps=(<<ENDNAVREM);
<td bgcolor="$tabbg">
<a href="/adm/navmaps?postdata=$escurl&postsymb=$escsymb" target="_top"><font color="$font">Navigate Contents</font></a></td>
ENDNAVREM
            if (($ENV{'REQUEST_URI'}=~/^\/adm\//) &&
                ($ENV{'REQUEST_URI'}!~/^\/adm\/wrapper\//) &&
         ($ENV{'REQUEST_URI'}!~/^\/adm\/.*\/(smppg|bulletinboard|aboutme)(\?|$)/)) {
                my $escreload=&Apache::lonnet::escape('return:');
                $reloadlink=(<<ENDRELOAD);
<td bgcolor="$tabbg">
<a href="/adm/flip?postdata=$escreload" target="_top"><font color="$font">Return to Last Location</font></a></td>
ENDRELOAD
            }
        }
        my $reg='';
        if ($registration) {
           $reg=&innerregister($forcereg,$target);
        }
	return (<<ENDINLINEMENU);
<script>
// BEGIN LON-CAPA Internal
</script>
<table bgcolor="$pgbg" width="100%" border="0" cellpadding="3" cellspacing="3">
<tr>
<td bgcolor="$tabbg">
<a href="/adm/menu" target="_top"><font color="$font">Main Menu</font></a>
</td>
$reloadlink
$navmaps
<td bgcolor="$tabbg">
<a href="/adm/remote?action=launch&url=$escurl" target="_top">
<font color="$font">Launch Remote Control</font></a></td>
<td bgcolor="$tabbg">
<img align="right" src="/adm/lonIcons/minilogo.gif" />
<b>LON-CAPA</b></td>
</tr>
</table>
<script>
// END LON-CAPA Internal
</script>
$reg
ENDINLINEMENU
    } else {
	return '';
    }
}

# ====================================== This gets called in the header section

sub registerurl {
    my $forcereg=shift;
    my $target = shift;
    my $result = '';
    if ($ENV{'REQUEST_URI'} eq '/res/adm/pages/menu.html') { return ''; }
    my $force_title='';
    if ($ENV{'request.state'} eq 'construct') {
	$force_title=&Apache::lonxml::display_title();
    }
    if ($target eq 'edit') {
        $result .="<script type=\"text/javascript\">\n".
            "if (typeof swmenu != 'undefined') {swmenu.currentURL=null;}\n".
            &Apache::loncommon::browser_and_searcher_javascript().
                "\n</script>\n";
    }
    if (($ENV{'browser.interface'} eq 'textual') ||
        ($ENV{'environment.remote'} eq 'off') ||
        ((($ENV{'request.publicaccess'}) || 
         (!&Apache::lonnet::is_on_map(
	   &Apache::lonnet::unescape($ENV{'REQUEST_URI'})))) &&
        (!$forcereg))) {
 	return $result.
          '<script type="text/javascript">function LONCAPAreg(){;} function LONCAPAstale(){}</script>'.$force_title;
    }
# Graphical display after login only
    if ($Apache::lonxml::registered && !$forcereg) { return ''; }
    $result.=&innerregister($forcereg,$target);
    return $result.$force_title;
}

# =========== This gets called in order to register a URL, both with the Remote
# =========== and in the body of the document

sub innerregister {
    my $forcereg=shift;
    my $target = shift;
    my $result = '';
    if ($ENV{'REQUEST_URI'} eq '/res/adm/pages/menu.html') { return ''; }

    $Apache::lonxml::registered=1;

    my $textinter=($ENV{'browser.interface'} eq 'textual');
    my $noremote=($ENV{'environment.remote'} eq 'off');
    
    my $textual=($textinter || $noremote);

    @inlineremote=();
    undef @inlineremote;

    my $reopen=&Apache::lonmenu::reopenmenu();

    my $newmail='';
    if ($noremote) {
	$newmail='<table bgcolor="'.$pgbg.'" border="0" cellspacing="3" cellpadding="3" width="100%"><tr><td bgcolor="'.$tabbg.'">';
    }
    if (($textual) && ($ENV{'request.symb'}) && ($ENV{'request.course.id'})) {
	my ($mapurl,$rid,$resurl)=split(/\_\_\_/,$ENV{'request.symb'});
        $newmail.=$ENV{'course.'.$ENV{'request.course.id'}.'.description'};
        my $maptitle=&Apache::lonnet::gettitle($mapurl);
	my $restitle=&Apache::lonnet::gettitle($resurl);
        if ($maptitle) {
	    $newmail.=', '.$maptitle;
        }
        if ($restitle) {
	    $newmail.=': '.$restitle;
        }
        $newmail.='&nbsp;&nbsp;&nbsp;';
    }
    if (&Apache::lonmsg::newmail()) { 
       $newmail=($textual?
 '<b><a href="/adm/communicate" target="_top">You have new messages</a></b><br />':
                          'swmenu.setstatus("you have","messages");');
    }
    if ($noremote) {
	$newmail.='</td></tr></table>';
    }
    my $timesync=($textual?'':'swmenu.syncclock(1000*'.time.');');
    my $tablestart=($noremote?'<table bgcolor="'.$pgbg.'" border="0" cellspacing="3" cellpadding="3" width="100%">':'');
    my $tableend=($noremote?'</table>':'');
# =============================================================================
# ============================ This is for URLs that actually can be registered
    if (($ENV{'REQUEST_URI'}!~/^\/(res\/)*adm\//) || ($forcereg)) {
# -- This applies to homework problems for users with grading privileges
        my $hwkadd='';
        if 
      ($ENV{'request.filename'}=~/\.(problem|exam|quiz|assess|survey|form)$/) {
	    my $crs='/'.$ENV{'request.course.id'};
	    if ($ENV{'request.course.sec'}) {
	        $crs.='_'.$ENV{'request.course.sec'};
	    }
	    $crs=~s/\_/\//g;

	    if (&Apache::lonnet::allowed('vgr',$crs)) {
		$hwkadd.=&switch('','',7,1,'subm.gif','view sub-','missions',
                       "gocmd('/adm/grades','submission')",
		       'View user submissions for this assessment resource');
            }
	    if (&Apache::lonnet::allowed('mgr',$crs)) {
		$hwkadd.=&switch('','',7,2,'pgrd.gif','problem','grades',
                       "gocmd('/adm/grades','gradingmenu')",
                       'Modify user grades for this assessment resource');
            }
	    if (&Apache::lonnet::allowed('opa',$crs)) {
		$hwkadd.=&switch('','',7,3,'pparm.gif','problem','parms',
                       "gocmd('/adm/parmset','set')",
                       'Modify deadlines, etc, for this assessment resource');
            }
	}
# -- End Homework
        ###
        ### Determine whether or not to display the 'cstr' button for this
        ### resource
        ###
        my $editbutton = '';
        if ($ENV{'user.author'}) {
            if ($ENV{'request.role'}=~/^(ca|au)/) {
                # Set defaults for authors
                my ($top,$bottom) = ('con-','struct');
                my $action = "go('/priv/".$ENV{'user.name'}."');";
                my $cadom  = $ENV{'request.role.domain'};
                my $caname = $ENV{'user.name'};
                my $desc = "Enter my resource construction space";
                # Set defaults for co-authors
                if ($ENV{'request.role'} =~ /^ca/) { 
                    ($cadom,$caname)=($ENV{'request.role'}=~/(\w+)\/(\w+)$/);
                    ($top,$bottom) = ('co con-','struct');
                    $action = "go('/priv/".$caname."');";
                    $desc = "Enter construction space as co-author";
                }
                # Check that we are on the correct machine
                my $home = &Apache::lonnet::homeserver($caname,$cadom);
                if ($home eq $Apache::lonnet::perlvar{'lonHostID'}) {
                    $editbutton=&switch
                        ('','',6,1,$top,,$bottom,$action,$desc);
                }
            }
            ##
            ## Determine if user can edit url.
            ##
            my $cfile='';
            my $cfuname='';
            my $cfudom='';
            if ($ENV{'request.filename'}) {
                my $file=&Apache::lonnet::declutter($ENV{'request.filename'});
                $file=~s/^(\w+)\/(\w+)/\/priv\/$2/;
                # Chech that the user has permission to edit this resource
                ($cfuname,$cfudom)=&Apache::loncacc::constructaccess($file,$1);
                if (defined($cfudom)) {
                    if (&Apache::lonnet::homeserver($cfuname,$cfudom) 
                        eq $Apache::lonnet::perlvar{'lonHostID'}) {
                        $cfile=$file;
                    }
                }
            }        
            # Finally, turn the button on or off
            if ($cfile) {
                $editbutton=&switch
                    ('','',6,1,'cstr.gif','edit','resource',
                     "go('".$cfile."');","Edit this resource");
            } elsif ($editbutton eq '') {
                $editbutton=&clear(6,1);
            }
        }
        ###
        ###
# Prepare the rest of the buttons
	my $menuitems=(<<ENDMENUITEMS);
c&3&1
s&2&1&back.gif&backward&&gopost('/adm/flip','back:'+currentURL)&Go to the previous resource in the course sequence&1
s&2&3&forw.gif&forward&&gopost('/adm/flip','forward:'+currentURL)&Go to the next resource in the course sequence&3
c&6&3
c&8&1
c&8&2
s&8&3&prt.gif&prepare&printout&gopost('/adm/printout',currentURL)&Prepare a printable document
s&9&1&sbkm.gif&set&bookmark&set_bookmark()&Set a bookmark for this resource&2
s&9&3&anot.gif&anno-&tations&annotate()&Make notes and annotations about this resource&2
ENDMENUITEMS
        unless ($ENV{'REQUEST_URI'}=~/\/(bulletinboard|smppg|navmaps|syllabus|aboutme)(\?|$)/) {
	    $menuitems.=(<<ENDREALRES);
s&6&3&catalog.gif&catalog&info&catalog_info()&Show catalog information
s&8&1&eval.gif&evaluate&this&gopost('/adm/evaluate',currentURL,1)&Provide my evaluation of this resource
s&8&2&fdbk.gif&feedback&discuss&gopost('/adm/feedback',currentURL,1)&Provide feedback messages or contribute to the course discussion about this resource
ENDREALRES
	}
        my $buttons='';
        foreach (split(/\n/,$menuitems)) {
	    my ($command,@rest)=split(/\&/,$_);
            if ($command eq 's') {
		$buttons.=&switch('','',@rest);
            } else {
                $buttons.=&clear(@rest);
            }
        }
        if ($textual) {
# Registered, textual output
            my $utility=&utilityfunctions();
            my $form=&serverform();
            my $inlinebuttons=
                        join('',map { (defined($_)?$_:'') } @inlineremote);
	    $result =(<<ENDREGTEXT);
<script>
// BEGIN LON-CAPA Internal
$utility
</script>
$timesync
$newmail
$tablestart
$inlinebuttons
$tableend
$form
<script>
// END LON-CAPA Internal
</script>

ENDREGTEXT
# Registered, graphical output
        } else {
	    my $requri=&Apache::lonnet::unescape($ENV{'REQUEST_URI'});
	    $result = (<<ENDREGTHIS);
     
<script language="JavaScript">
// BEGIN LON-CAPA Internal
var swmenu=null;

    function LONCAPAreg() {
	  swmenu=$reopen;
          swmenu.clearTimeout(swmenu.menucltim);
          $timesync
          $newmail
          $buttons
	  swmenu.currentURL="$requri";
          swmenu.reloadURL=swmenu.currentURL+window.location.search;
          swmenu.currentSymb="$ENV{'request.symb'}";
          swmenu.reloadSymb="$ENV{'request.symb'}";
          swmenu.currentStale=0;
          $hwkadd
          $editbutton
    }

    function LONCAPAstale() {
	  swmenu=$reopen
          swmenu.currentStale=1;
          if (swmenu.reloadURL!='' && swmenu.reloadURL!= null) { 
             swmenu.switchbutton
             (3,1,'reload.gif','return','location','go(reloadURL)','Return to the last known location in the course sequence');
	  }
          swmenu.clearbut(7,1);
          swmenu.clearbut(7,2);
          swmenu.clearbut(7,3);
          swmenu.menucltim=swmenu.setTimeout(
 'clearbut(2,1);clearbut(2,3);clearbut(8,1);clearbut(8,2);clearbut(8,3);'+
 'clearbut(9,1);clearbut(9,2);clearbut(9,3);clearbut(6,3);clearbut(6,1)',
			  2000);
      }

// END LON-CAPA Internal
</script>
ENDREGTHIS
        }
# =============================================================================
    } else {
# ========================================== This can or will not be registered
        if ($textual) {
# Not registered, textual
	    $result= (<<ENDDONOTREGTEXT);
ENDDONOTREGTEXT
        } else {
# Not registered, graphical
           $result = (<<ENDDONOTREGTHIS);

<script language="JavaScript">
// BEGIN LON-CAPA Internal
var swmenu=null;

    function LONCAPAreg() {
	  swmenu=$reopen
          $timesync
          swmenu.currentStale=1;
          swmenu.clearbut(2,1);
          swmenu.clearbut(2,3);
          swmenu.clearbut(8,1);
          swmenu.clearbut(8,2);
          swmenu.clearbut(8,3);
          if (swmenu.currentURL) {
             swmenu.switchbutton
              (3,1,'reload.gif','return','location','go(currentURL)');
 	  } else {
	      swmenu.clearbut(3,1);
          }
    }

    function LONCAPAstale() {
    }

// END LON-CAPA Internal
</script>
ENDDONOTREGTHIS
       }
# =============================================================================
    }
    return $result;
}

sub loadevents() {
    if ($ENV{'REQUEST_URI'} eq '/res/adm/pages/menu.html') { return ''; }
    return 'LONCAPAreg();';
}

sub unloadevents() {
    if ($ENV{'REQUEST_URI'} eq '/res/adm/pages/menu.html') { return ''; }
    return 'LONCAPAstale();';
}

# ============================================================= Start up remote

sub startupremote {
    my ($lowerurl)=@_;
    if (($ENV{'browser.interface'} eq 'textual') ||
        ($ENV{'environment.remote'} eq 'off')) {
     return ('<meta HTTP-EQUIV="Refresh" CONTENT="0.5; url='.$lowerurl.'" />');
    }
#
# The Remote actually gets launched!
#
    my $configmenu=&rawconfig();
    my $esclowerurl=&Apache::lonnet::escape($lowerurl);

    return(<<ENDREMOTESTARTUP);
<script>

function wheelswitch() {
   if (window.status=='|') { 
      window.status='/'; 
   } else {
      if (window.status=='/') {
         window.status='-';
      } else {
         if (window.status=='-') { 
            window.status='\\\\'; 
         } else {
            if (window.status=='\\\\') { window.status='|'; }
         }
      }
   } 
}

// ---------------------------------------------------------- The wait function
var canceltim;
function wait() {
   if ((menuloaded==1) || (tim==1)) {
      window.status='Done.';
      if (tim==0) {
         clearTimeout(canceltim);
         $configmenu
         window.location='$lowerurl';  
      } else {
	  window.location='/adm/remote?action=collapse&url=$esclowerurl';
      }
   } else {
      wheelswitch();
      setTimeout('wait();',200);
   }
}

function main() {
   canceltim=setTimeout('tim=1;',30000);
   window.status='-';
   wait();
}

</script>
ENDREMOTESTARTUP
}

sub setflags() {
    return(<<ENDSETFLAGS);
<script>
    menuloaded=0;
    tim=0;
</script>
ENDSETFLAGS
}

sub maincall() {
    if (($ENV{'browser.interface'} eq 'textual') ||
        ($ENV{'environment.remote'} eq 'off')) { return ''; }
    return(<<ENDMAINCALL);
<script>
    main();
</script>
ENDMAINCALL
}
# ================================================================= Reopen menu

sub reopenmenu {
   if (($ENV{'browser.interface'} eq 'textual') ||
       ($ENV{'environment.remote'} eq 'off')) { return ''; }
   my $menuname='LCmenu'.$Apache::lonnet::perlvar{'lonHostID'};
   my $nothing = &Apache::lonhtmlcommon::javascript_nothing();
   return('window.open('.$nothing.',"'.$menuname.'","",false);');
} 

# =============================================================== Open the menu

sub open {
    my $returnval='';
    if (($ENV{'browser.interface'} eq 'textual') ||
        ($ENV{'environment.remote'} eq 'off')) { return ''; }
    my $menuname='LCmenu'.$Apache::lonnet::perlvar{'lonHostID'};
    unless (shift eq 'unix') {
# resizing does not work on linux because of virtual desktop sizes
       $returnval.=(<<ENDRESIZE);
if (window.screen) {
    self.resizeTo(screen.availWidth-215,screen.availHeight-55);
    self.moveTo(190,15);
}
ENDRESIZE
    }
    $returnval.=(<<ENDOPEN);
window.status='Opening LON-CAPA Remote Control';
var menu=window.open("/res/adm/pages/menu.html","$menuname",
"height=350,width=150,scrollbars=no,menubar=no,top=5,left=5,screenX=5,screenY=5");
self.name='loncapaclient';
ENDOPEN
    return '<script>'.$returnval.'</script>';
}


# ================================================================== Raw Config

sub clear {
    my ($row,$col)=@_;
    unless (($ENV{'browser.interface'} eq 'textual') ||
            ($ENV{'environment.remote'} eq 'off')) {
       return "\n".qq(window.status+='.';swmenu.clearbut($row,$col););
   } else { 
       $inlineremote[10*$row+$col]='';
       return ''; 
   }
}

# ============================================ Switch a button or create a link
# Switch acts on the javascript that is executed when a button is clicked.  
# The javascript is usually similar to "go('/adm/roles')" or "cstrgo(..)".

sub switch {
    my ($uname,$udom,$row,$col,$img,$top,$bot,$act,$desc,$nobreak)=@_;
    $act=~s/\$uname/$uname/g;
    $act=~s/\$udom/$udom/g;
    unless (($ENV{'browser.interface'} eq 'textual')  ||
            ($ENV{'environment.remote'} eq 'off')) {
# Remote
       return "\n".
 qq(window.status+='.';swmenu.switchbutton($row,$col,"$img","$top","$bot","$act","$desc"););
   } elsif ($ENV{'browser.interface'} eq 'textual') {
# Accessibility
       if ($nobreak==2) { return ''; }
       my $text=$top.' '.$bot;
       $text=~s/\s*\-\s*//gs;
       $inlineremote[10*$row+$col]="\n".($nobreak?' ':'<br />').
        '<a href="javascript:'.$act.';">'.$text.'</a> '.
        ($nobreak?'':$desc);
   } else {
# Inline Remote
       if ($nobreak==2) { return ''; }
       my $text=$top.' '.$bot;
       $text=~s/\s*\-\s*//gs;
       $inlineremote[10*$row+$col]="\n".
         ($nobreak==3?'<td width="50%" colspan="2" align="right"':'<tr><td').
         ' bgcolor="'.$tabbg.'"'.($nobreak==1?' width="50%" colspan="2"':'').
     '"><a href="javascript:'.$act.';"><font color="'.$font.'"'.
          ($nobreak?' size="+1"':'').
     '>'.$text.'</font></a></td>'.
     ($nobreak?'':'<td colspan="3" width="80%"><font color="'.$font.'" size="-1">'.$desc.'</font>').($nobreak!=1?'</tr>':'');
   }
    return '';
}

sub secondlevel {
    my $output='';
    my 
    ($uname,$udom,$rol,$crs,$pub,$con,$row,$col,$prt,$img,$top,$bot,$act,$desc)=@_;
    if ($prt eq 'any') {
	   $output.=switch($uname,$udom,$row,$col,$img,$top,$bot,$act,$desc);
    } elsif ($prt=~/^r(\w+)/) {
        if ($rol eq $1) {
           $output.=switch($uname,$udom,$row,$col,$img,$top,$bot,$act,$desc);
        }
    }
    return $output;
}

sub openmenu {
    my $menuname='LCmenu'.$Apache::lonnet::perlvar{'lonHostID'};
    if (($ENV{'browser.interface'} eq 'textual') ||
        ($ENV{'environment.remote'} eq 'off')) { return ''; }
    my $nothing = &Apache::lonhtmlcommon::javascript_nothing();
    return "window.open(".$nothing.",'".$menuname."');";
}

sub inlinemenu {
    @inlineremote=();
    undef @inlineremote;
    &rawconfig(1);
    return join('',map { (defined($_)?$_:'') } @inlineremote);
}

sub rawconfig {
    my $textualoverride=shift;
    my $output='';
    unless (($ENV{'browser.interface'} eq 'textual') ||
            ($ENV{'environment.remote'} eq 'off')) {
       $output.=
 "window.status='Opening Remote Control';var swmenu=".&openmenu().
"\nwindow.status='Configuring Remote Control ';";
    } else {
       unless ($textualoverride) { return ''; }
    }
    my $uname=$ENV{'user.name'};
    my $udom=$ENV{'user.domain'};
    my $adv=$ENV{'user.adv'};
    my $author=$ENV{'user.author'};
    my $crs='';
    if ($ENV{'request.course.id'}) {
       $crs='/'.$ENV{'request.course.id'};
       if ($ENV{'request.course.sec'}) {
	   $crs.='_'.$ENV{'request.course.sec'};
       }
       $crs=~s/\_/\//g;
    }
    my $pub=($ENV{'request.state'} eq 'published');
    my $con=($ENV{'request.state'} eq 'construct');
    my $rol=$ENV{'request.role'};
    my $requested_domain = $ENV{'request.role.domain'};
    foreach (@desklines) {
        my ($row,$col,$pro,$prt,$img,$top,$bot,$act,$desc)=split(/\:/,$_);
        $prt=~s/\$uname/$uname/g;
        $prt=~s/\$udom/$udom/g;
        $prt=~s/\$crs/$crs/g; 
        $prt=~s/\$requested_domain/$requested_domain/g;
        if ($pro eq 'clear') {
	    $output.=&clear($row,$col);
        } elsif ($pro eq 'any') {
               $output.=&secondlevel(
	  $uname,$udom,$rol,$crs,$pub,$con,$row,$col,$prt,$img,$top,$bot,$act,$desc);
	} elsif ($pro eq 'smp') {
            unless ($adv) {
               $output.=&secondlevel(
          $uname,$udom,$rol,$crs,$pub,$con,$row,$col,$prt,$img,$top,$bot,$act,$desc);
            }
        } elsif ($pro eq 'adv') {
            if ($adv) {
               $output.=&secondlevel(
	  $uname,$udom,$rol,$crs,$pub,$con,$row,$col,$prt,$img,$top,$bot,$act,$desc);
            }
        } elsif (($pro=~/^p(\w+)/) && ($prt)) {
	    if (&Apache::lonnet::allowed($1,$prt)) {
               $output.=switch($uname,$udom,$row,$col,$img,$top,$bot,$act,$desc);
            }
        } elsif ($pro eq 'course') {
            if ($ENV{'request.course.fn'}) {
               $output.=switch($uname,$udom,$row,$col,$img,$top,$bot,$act,$desc);
	    }
        } elsif ($pro =~ /^course_(.*)$/) {
            # Check for permissions inside of a course
            if (($ENV{'request.course.id'}) &&
                (&Apache::lonnet::allowed($1,$ENV{'request.course.id'}.
            ($ENV{'request.course.sec'}?'/'.$ENV{'request.course.sec'}:''))
                 )) {
                $output.=switch($uname,$udom,$row,$col,$img,$top,$bot,$act,$desc);
	    }
        } elsif ($pro eq 'author') {
            if ($author) {
                if ((($prt eq 'rca') && ($ENV{'request.role'}=~/^ca/)) ||
                    (($prt eq 'rau') && ($ENV{'request.role'}=~/^au/))) {
                    # Check that we are on the correct machine
                    my $cadom=$requested_domain;
                    my $caname=$ENV{'user.name'};
                    if ($prt eq 'rca') {
		       ($cadom,$caname)=
                               ($ENV{'request.role'}=~/(\w+)\/(\w+)$/);
                    }                       
                    $act =~ s/\$caname/$caname/g;
                    my $home = &Apache::lonnet::homeserver($caname,$cadom);
                    if ($home eq $Apache::lonnet::perlvar{'lonHostID'}) {
                        $output.=switch($caname,$cadom,
                                        $row,$col,$img,$top,$bot,$act,$desc);
                    }
                }
            }
        }
    }
    unless (($ENV{'browser.interface'} eq 'textual') ||
            ($ENV{'environment.remote'} eq 'off')) {
       $output.="\nwindow.status='Synchronizing Time';swmenu.syncclock(1000*".time.");\nwindow.status='Remote Control Configured.';";
    }
    return $output;
}

# ======================================================================= Close

sub close {
    if (($ENV{'browser.interface'} eq 'textual') ||
        ($ENV{'environment.remote'} eq 'off')) { return ''; }
    my $menuname='LCmenu'.$Apache::lonnet::perlvar{'lonHostID'};
    return(<<ENDCLOSE);
<script>
window.status='Accessing Remote Control';
menu=window.open("/adm/rat/empty.html","$menuname",
                 "height=350,width=150,scrollbars=no,menubar=no");
window.status='Disabling Remote Control';
menu.active=0;
menu.autologout=0;
window.status='Closing Remote Control';
menu.close();
window.status='Done.';
</script>
ENDCLOSE
}

# ====================================================================== Footer

sub footer {

}

sub utilityfunctions {
    unless (($ENV{'browser.interface'} eq 'textual')  ||
        ($ENV{'environment.remote'} eq 'off')) { return ''; }
    my $currenturl=$ENV{'REQUEST_URI'};
    my $currentsymb=$ENV{'request.symb'};
return (<<ENDUTILITY)

    var currentURL="$currenturl";
    var reloadURL="$currenturl";
    var currentSymb="$currentsymb";

function go(url) {
   if (url!='' && url!= null) {
       currentURL = null;
       currentSymb= null;
       window.location.href=url;
   }
}

function gopost(url,postdata) {
   if (url!='') {
      this.document.server.action=url;
      this.document.server.postdata.value=postdata;
      this.document.server.command.value='';
      this.document.server.url.value='';
      this.document.server.symb.value='';
      this.document.server.submit();
   }
}

function gocmd(url,cmd) {
   if (url!='') {
      this.document.server.action=url;
      this.document.server.postdata.value='';
      this.document.server.command.value=cmd;
      this.document.server.url.value=currentURL;
      this.document.server.symb.value=currentSymb;
      this.document.server.submit();
   }
}

function catalog_info() {
   loncatinfo=window.open(window.location.pathname+'.meta',"LONcatInfo",'height=320,width=280,resizeable=yes,scrollbars=yes,location=no,menubar=no,toolbar=no');
}

function chat_win() {
   lonchat=window.open('/res/adm/pages/chatroom.html',"LONchat",'height=320,width=280,resizeable=yes,location=no,menubar=no,toolbar=no');
}
ENDUTILITY
}

sub serverform {
    return(<<ENDSERVERFORM);

<form name="server" action="/adm/logout" method="post" target="_top">
<input type="hidden" name="postdata" value="none" />
<input type="hidden" name="command" value="none" />
<input type="hidden" name="url" value="none" />
<input type="hidden" name="symb" value="none" />
</form>
ENDSERVERFORM
}
# ================================================ Handler when called directly


sub handler {
    my $r = shift;
    $r->content_type('text/html');
    $r->send_http_header;
    return OK if $r->header_only;

    my $form=&serverform();
    my $bodytag=&Apache::loncommon::bodytag('Main Menu');
    my $function='student';
    if ($ENV{'request.role'}=~/^(cc|in|ta|ep)/) {
	$function='coordinator';
    }
    if ($ENV{'request.role'}=~/^(su|dc|ad|li)/) {
	$function='admin';
    }
    if (($ENV{'request.role'}=~/^(au|ca)/) ||
	($ENV{'REQUEST_URI'}=~/^(\/priv|\~)/)) {
	$function='author';
    }
    my $domain=&Apache::loncommon::determinedomain();
    $pgbg=&Apache::loncommon::designparm($function.'.pgbg',$domain);
    $tabbg=&Apache::loncommon::designparm($function.'.tabbg',$domain);
    $font=&Apache::loncommon::designparm($function.'.font',$domain);
# ---- Print the screen, pretent to be in text mode to generate text-based menu
    unless ($ENV{'brower.interface'} eq 'textual') {
	$ENV{'environment.remote'}='off';
    }
    my $utility=&utilityfunctions();
    $r->print(<<ENDHEADER);
<html><head>
<title>LON-CAPA Main Menu</title>
<script>
$utility
</script>
</head>
$bodytag
ENDHEADER
    $r->print('<table>'.&inlinemenu().'</table>'.$form);
    $r->print('</body></html>');
    return OK;
}

# ================================================================ Main Program

BEGIN {
  if (! defined($readdesk)) {
   {
    my $config=Apache::File->new($Apache::lonnet::perlvar{'lonTabDir'}.
				  '/mydesk.tab');
    while (my $configline=<$config>) {
       $configline=(split(/\#/,$configline))[0];
       $configline=~s/^\s+//;
       chomp($configline);
       if ($configline) {
          $desklines[$#desklines+1]=$configline;
       }
    }
   }
   $readdesk='done';
  }
}

1;
__END__







