# The LearningOnline Network with CAPA
# Routines for messaging
#
# $Id: lonmsg.pm,v 1.57 2003/07/05 10:07:11 www Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
#
# (Routines to control the menu
#
# (TeX Conversion Module
#
# 05/29/00,05/30 Gerd Kortemeyer)
#
# 10/05 Gerd Kortemeyer)
#
# 10/19,10/20,10/30,
# 02/06/01 Gerd Kortemeyer
# 07/27 Guy Albertelli
# 07/27,07/28,07/30,08/03,08/06,08/08,08/09,08/10,8/13,8/15,
# 10/1,11/5 Gerd Kortemeyer
# YEAR=2002
# 1/1,3/18 Gerd Kortemeyer
#
package Apache::lonmsg;

use strict;
use Apache::lonnet();
use vars qw($msgcount);
use HTML::TokeParser();
use Apache::Constants qw(:common);
use Apache::loncommon();
use Apache::lontexconvert();
use HTML::Entities();
use Mail::Send;

# ===================================================================== Package

sub packagemsg {
    my ($subject,$message,$citation,$baseurl,$attachmenturl)=@_;
    $message =&HTML::Entities::encode($message);
    $citation=&HTML::Entities::encode($citation);
    $subject =&HTML::Entities::encode($subject);
    #remove machine specification
    $baseurl =~ s|^http://[^/]+/|/|;
    $baseurl =&HTML::Entities::encode($baseurl);
    #remove machine specification
    $attachmenturl =~ s|^http://[^/]+/|/|;
    $attachmenturl =&HTML::Entities::encode($attachmenturl);

    my $now=time;
    $msgcount++;
    my $partsubj=$subject;
    $partsubj=&Apache::lonnet::escape($partsubj);
    my $msgid=&Apache::lonnet::escape(
           $now.':'.$partsubj.':'.$ENV{'user.name'}.':'.
           $ENV{'user.domain'}.':'.$msgcount.':'.$$);
    my $result='<sendername>'.$ENV{'user.name'}.'</sendername>'.
           '<senderdomain>'.$ENV{'user.domain'}.'</senderdomain>'.
           '<subject>'.$subject.'</subject>'.
	   '<time>'.localtime($now).'</time>'.
	   '<servername>'.$ENV{'SERVER_NAME'}.'</servername>'.
           '<host>'.$ENV{'HTTP_HOST'}.'</host>'.
	   '<client>'.$ENV{'REMOTE_ADDR'}.'</client>'.
	   '<browsertype>'.$ENV{'browser.type'}.'</browsertype>'.
	   '<browseros>'.$ENV{'browser.os'}.'</browseros>'.
	   '<browserversion>'.$ENV{'browser.version'}.'</browserversion>'.
           '<browsermathml>'.$ENV{'browser.mathml'}.'</browsermathml>'.
	   '<browserraw>'.$ENV{'HTTP_USER_AGENT'}.'</browserraw>'.
	   '<courseid>'.$ENV{'request.course.id'}.'</courseid>'.
	   '<role>'.$ENV{'request.role'}.'</role>'.
	   '<resource>'.$ENV{'request.filename'}.'</resource>'.
           '<msgid>'.$msgid.'</msgid>'.
	   '<message>'.$message.'</message>';
    if (defined($citation)) {
	$result.='<citation>'.$citation.'</citation>';
    }
    if (defined($baseurl)) {
	$result.= '<baseurl>'.$baseurl.'</baseurl>';
    }
    if (defined($attachmenturl)) {
	$result.= '<attachmenturl>'.$attachmenturl.'</attachmenturl>';
    }
    return $msgid,$result;
}

# ================================================== Unpack message into a hash

sub unpackagemsg {
    my ($message,$notoken)=@_;
    my %content=();
    my $parser=HTML::TokeParser->new(\$message);
    my $token;
    while ($token=$parser->get_token) {
       if ($token->[0] eq 'S') {
	   my $entry=$token->[1];
           my $value=$parser->get_text('/'.$entry);
           $content{$entry}=$value;
       }
    }
    if ($content{'attachmenturl'}) {
       my ($fname,$ft)=($content{'attachmenturl'}=~/\/(\w+)\.(\w+)$/);
       if ($notoken) {
	   $content{'message'}.='<p>Attachment: <tt>'.$fname.'.'.$ft.'</tt>';
       } else {
	   $content{'message'}.='<p>Attachment: <a href="'.
	       &Apache::lonnet::tokenwrapper($content{'attachmenturl'}).
	       '"><tt>'.$fname.'.'.$ft.'</tt></a>';
       }
    }
    return %content;
}

# ======================================================= Get info out of msgid

sub unpackmsgid {
    my $msgid=&Apache::lonnet::unescape(shift);
    my ($sendtime,$shortsubj,$fromname,$fromdomain)=split(/\:/,
                          &Apache::lonnet::unescape($msgid));
    my %status=&Apache::lonnet::get('email_status',[$msgid]);
    if ($status{$msgid}=~/^error\:/) { $status{$msgid}=''; }
    unless ($status{$msgid}) { $status{$msgid}='new'; }
    return ($sendtime,$shortsubj,$fromname,$fromdomain,$status{$msgid});
} 


sub sendemail {
    my ($to,$subject,$body)=@_;
    $body=
    "*** This is an automatic message generated by the LON-CAPA system.\n".
    "*** Please do not reply to this address.\n\n".$body;
    my $msg = new Mail::Send;
    $msg->to($to);
    $msg->subject('[LON-CAPA] '.$subject);
    my $fh = $msg->open('smtp',Server => 'localhost');
    print $fh $body;
    $fh->close;
}

# ==================================================== Send notification emails

sub sendnotification {
    my ($to,$touname,$toudom,$subj,$crit)=@_;
    my $sender=$ENV{'environment.firstname'}.' '.$ENV{'environment.lastname'};
    my $critical=($crit?' critical':'');
    my $url='http://'.
      $Apache::lonnet::hostname{&Apache::lonnet::homeserver($touname,$toudom)}.
      '/adm/email?username='.$touname.'&domain='.$toudom;
    my $body=(<<ENDMSG);
You received a$critical message from $sender in LON-CAPA. The subject is

 $subj

Use

 $url

to access this message.
ENDMSG
    &sendemail($to,'New'.$critical.' message from '.$sender,$body);
}
# ============================================================= Check for email

sub newmail {
    if ((time-$ENV{'user.mailcheck.time'})>300) {
        my %what=&Apache::lonnet::get('email_status',['recnewemail']);
        &Apache::lonnet::appenv('user.mailcheck.time'=>time);
        if ($what{'recnewemail'}>0) { return 1; }
    }
    return 0;
}

# =============================== Automated message to the author of a resource

sub author_res_msg {
    my ($filename,$message)=@_;
    unless ($message) { return 'empty'; }
    $filename=&Apache::lonnet::declutter($filename);
    my ($domain,$author,@dummy)=split(/\//,$filename);
    my $homeserver=&Apache::lonnet::homeserver($author,$domain);
    if ($homeserver ne 'no_host') {
       my $id=unpack("%32C*",$message);
       my $msgid;
       ($msgid,$message)=&packagemsg($filename,$message);
       return &Apache::lonnet::reply('put:'.$domain.':'.$author.
         ':nohist_res_msgs:'.
          &Apache::lonnet::escape($filename.'_'.$id).'='.
          &Apache::lonnet::escape($message),$homeserver);
    }
    return 'no_host';
}

# ================================================== Critical message to a user

sub user_crit_msg_raw {
    my ($user,$domain,$subject,$message,$sendback)=@_;
# Check if allowed missing
    my $status='';
    my $msgid='undefined';
    unless (($message)&&($user)&&($domain)) { $status='empty'; };
    my $homeserver=&Apache::lonnet::homeserver($user,$domain);
    if ($homeserver ne 'no_host') {
       ($msgid,$message)=&packagemsg($subject,$message);
       if ($sendback) { $message.='<sendback>true</sendback>'; }
       $status=&Apache::lonnet::critical(
           'put:'.$domain.':'.$user.':critical:'.
           &Apache::lonnet::escape($msgid).'='.
           &Apache::lonnet::escape($message),$homeserver);
       if ($ENV{'request.course.id'}) {
          &user_normal_msg_raw(
            $ENV{'course.'.$ENV{'request.course.id'}.'.num'},
            $ENV{'course.'.$ENV{'request.course.id'}.'.domain'},
            'Critical ['.$user.':'.$domain.']',
	    $message);
       }
    } else {
       $status='no_host';
    }
# Notifications
    my %userenv = &Apache::lonnet::get('environment',['critnotification'],
                                       $domain,$user);
    if ($userenv{'critnotification'}) {
      &sendnotification($userenv{'critnotification'},$user,$domain,$subject,1);
    }
# Log this
    &Apache::lonnet::logthis(
      'Sending critical email '.$msgid.
      ', log status: '.
      &Apache::lonnet::log($ENV{'user.domain'},$ENV{'user.name'},
                         $ENV{'user.home'},
      'Sending critical '.$msgid.' to '.$user.' at '.$domain.' with status: '
      .$status));
    return $status;
}

# New routine that respects "forward" and calls old routine

sub user_crit_msg {
    my ($user,$domain,$subject,$message,$sendback)=@_;
    my $status='';
    my %userenv = &Apache::lonnet::get('environment',['msgforward'],
                                       $domain,$user);
    my $msgforward=$userenv{'msgforward'};
    if ($msgforward) {
       foreach (split(/\,/,$msgforward)) {
	 my ($forwuser,$forwdomain)=split(/\:/,$_);
         $status.=
	   &user_crit_msg_raw($forwuser,$forwdomain,$subject,$message,
                $sendback).' ';
       }
    } else { 
	$status=&user_crit_msg_raw($user,$domain,$subject,$message,$sendback);
    }
    return $status;
}

# =================================================== Critical message received

sub user_crit_received {
    my $msgid=shift;
    my %message=&Apache::lonnet::get('critical',[$msgid]);
    my %contents=&unpackagemsg($message{$msgid},1);
    my $status='rec: '.($contents{'sendback'}?
     &user_normal_msg($contents{'sendername'},$contents{'senderdomain'},
                     'Receipt: '.$ENV{'user.name'}.' at '.$ENV{'user.domain'},
                     'User '.$ENV{'user.name'}.' at '.$ENV{'user.domain'}.
                     ' acknowledged receipt of message'."\n".'   "'.
                     $contents{'subject'}.'"'."\n".'dated '.
                     $contents{'time'}.".\n"
                     ):'no msg req');
    $status.=' trans: '.
     &Apache::lonnet::put(
     'nohist_email',{$contents{'msgid'} => $message{$msgid}});
    $status.=' del: '.
     &Apache::lonnet::del('critical',[$contents{'msgid'}]);
    &Apache::lonnet::log($ENV{'user.domain'},$ENV{'user.name'},
                         $ENV{'user.home'},'Received critical message '.
                         $contents{'msgid'}.
                         ', '.$status);
    return $status;
}

# ======================================================== Normal communication

sub user_normal_msg_raw {
    my ($user,$domain,$subject,$message,$citation,$baseurl,$attachmenturl)=@_;
# Check if allowed missing
    my $status='';
    my $msgid='undefined';
    unless (($message)&&($user)&&($domain)) { $status='empty'; };
    my $homeserver=&Apache::lonnet::homeserver($user,$domain);
    if ($homeserver ne 'no_host') {
       ($msgid,$message)=&packagemsg($subject,$message,$citation,$baseurl,
                                     $attachmenturl);
       $status=&Apache::lonnet::critical(
           'put:'.$domain.':'.$user.':nohist_email:'.
           &Apache::lonnet::escape($msgid).'='.
           &Apache::lonnet::escape($message),$homeserver);
       &Apache::lonnet::put
                         ('email_status',{'recnewemail'=>time},$domain,$user);
    } else {
       $status='no_host';
    }
# Notifications
    my %userenv = &Apache::lonnet::get('environment',['notification'],
                                       $domain,$user);
    if ($userenv{'notification'}) {
	&sendnotification($userenv{'notification'},$user,$domain,$subject,0);
    }
    &Apache::lonnet::log($ENV{'user.domain'},$ENV{'user.name'},
                         $ENV{'user.home'},
      'Sending '.$msgid.' to '.$user.' at '.$domain.' with status: '.$status);
    return $status;
}

# New routine that respects "forward" and calls old routine

sub user_normal_msg {
    my ($user,$domain,$subject,$message,$citation,$baseurl,$attachmenturl)=@_;
    my $status='';
    my %userenv = &Apache::lonnet::get('environment',['msgforward'],
                                       $domain,$user);
    my $msgforward=$userenv{'msgforward'};
    if ($msgforward) {
       foreach (split(/\,/,$msgforward)) {
	 my ($forwuser,$forwdomain)=split(/\:/,$_);
         $status.=
	  &user_normal_msg_raw($forwuser,$forwdomain,$subject,$message,
			       $citation,$baseurl,$attachmenturl).' ';
       }
    } else { 
	$status=&user_normal_msg_raw($user,$domain,$subject,$message,
				     $citation,$baseurl,$attachmenturl);
    }
    return $status;
}


# =============================================================== Status Change

sub statuschange {
    my ($msgid,$newstatus)=@_;
    my %status=&Apache::lonnet::get('email_status',[$msgid]);
    if ($status{$msgid}=~/^error\:/) { $status{$msgid}=''; }
    unless ($status{$msgid}) { $status{$msgid}='new'; }
    unless (($status{$msgid} eq 'replied') || 
            ($status{$msgid} eq 'forwarded')) {
	&Apache::lonnet::put('email_status',{$msgid => $newstatus});
    }
    if (($newstatus eq 'deleted') || ($newstatus eq 'new')) {
	&Apache::lonnet::put('email_status',{$msgid => $newstatus});
    }
}

# ======================================================= Display a course list

sub discourse {
    my $r=shift;
    my %courselist=&Apache::lonnet::dump(
                   'classlist',
		   $ENV{'course.'.$ENV{'request.course.id'}.'.domain'},
		   $ENV{'course.'.$ENV{'request.course.id'}.'.num'});
    my $now=time;
    $r->print(<<ENDDISHEADER);
<input type=hidden name=sendmode value=group>
<script>
    function checkall() {
	for (i=0; i<document.forms.compemail.elements.length; i++) {
            if 
          (document.forms.compemail.elements[i].name.indexOf('send_to_')==0) {
	      document.forms.compemail.elements[i].checked=true;
            }
        }
    }

    function checksec() {
	for (i=0; i<document.forms.compemail.elements.length; i++) {
            if 
          (document.forms.compemail.elements[i].name.indexOf
           ('send_to_&&&'+document.forms.compemail.chksec.value)==0) {
	      document.forms.compemail.elements[i].checked=true;
            }
        }
    }

    function uncheckall() {
	for (i=0; i<document.forms.compemail.elements.length; i++) {
            if 
          (document.forms.compemail.elements[i].name.indexOf('send_to_')==0) {
	      document.forms.compemail.elements[i].checked=false;
            }
        }
    }
</script>
<input type=button onClick="checkall()" value="Check for All">&nbsp;
<input type=button onClick="checksec()" value="Check for Section/Group">
<input type=text size=5 name=chksec>&nbsp;
<input type=button onClick="uncheckall()" value="Check for None">
<p>
ENDDISHEADER
    foreach (sort keys %courselist) {
        my ($end,$start)=split(/\:/,$courselist{$_});
        my $active=1;
        if (($end) && ($now>$end)) { $active=0; }
        if ($active) {
           my ($sname,$sdom)=split(/\:/,$_);
           my %reply=&Apache::lonnet::get('environment',
              ['firstname','middlename','lastname','generation'],
              $sdom,$sname);
           my $section=&Apache::lonnet::usection
	       ($sdom,$sname,$ENV{'request.course.id'});
           $r->print(
        '<br><input type=checkbox name="send_to_&&&'.$section.'&&&_'.$_.'"> '.
		      $reply{'firstname'}.' '. 
                      $reply{'middlename'}.' '.
                      $reply{'lastname'}.' '.
                      $reply{'generation'}.
                      ' ('.$_.') '.$section);
        } 
    }
}

# ==================================================== Display Critical Message

sub discrit {
    my $r=shift;
    my $header = '<h1><font color=red>Critical Messages</font></h1>'.
        '<form action=/adm/email method=post>'.
        '<input type=hidden name=confirm value=true>';
    my %what=&Apache::lonnet::dump('critical');
    my $result = '';
    foreach (sort keys %what) {
        my %content=&unpackagemsg($what{$_});
        next if ($content{'senderdomain'} eq '');
        $content{'message'}=~s/\n/\<br\>/g;
        $result.='<hr>From: <b>'.
&Apache::loncommon::aboutmewrapper(
 &Apache::loncommon::plainname($content{'sendername'},$content{'senderdomain'}),$content{'sendername'},$content{'senderdomain'}).'</b> ('.
$content{'sendername'}.'@'.
            $content{'senderdomain'}.') '.$content{'time'}.
            '<br>Subject: '.$content{'subject'}.
            '<br><blockquote>'.
              &Apache::lontexconvert::msgtexconverted($content{'message'}).
            '</blockquote>'.
            '<input type=submit name="rec_'.$_.'" value="Confirm Receipt">'.
            '<input type=submit name="reprec_'.$_.'" '.
                  'value="Confirm Receipt and Reply">';
    }
    # Check to see if there were any messages.
    if ($result eq '') {
        $result = "<h2>You have no critical messages.</h2>";
    } else {
        $r->print($header);
    }
    $r->print($result);
    $r->print('<input type=hidden name="displayedcrit" value="true"></form>');
}

# =============================================================== Compose reply

sub comprep {
    my ($r,$msgid)=@_;
      my %message=&Apache::lonnet::get('nohist_email',[$msgid]);
      my %content=&unpackagemsg($message{$msgid},1);
      my $quotemsg='> '.$content{'message'};
      $quotemsg=~s/\r/\n/g;
      $quotemsg=~s/\f/\n/g;
      $quotemsg=~s/\n+/\n\> /g;
      my $torepl=&Apache::loncommon::aboutmewrapper(
 &Apache::loncommon::plainname($content{'sendername'},$content{'senderdomain'}),$content{'sendername'},$content{'senderdomain'}).' ('.
$content{'sendername'}.'@'.
            $content{'senderdomain'}.')';
      my $subject='Re: '.$content{'subject'};
      my $dispcrit='';
      if (&Apache::lonnet::allowed('srm',$ENV{'request.course.id'})) {
	 my $crithelp = Apache::loncommon::help_open_topic("Course_Critical_Message");
         $dispcrit=
 '<input type=checkbox name=critmsg> Send as critical message ' . $crithelp . 
 '<br>'.
 '<input type=checkbox name=sendbck> Send as critical message ' .
 ' and return receipt' . $crithelp . '<p>';
      }
      $r->print(<<"ENDREPLY");
<form action="/adm/email" method=post>
<input type=hidden name=sendreply value="$msgid">
To: $torepl<br />
Subject: <input type=text size=50 name=subject value="$subject"><p>
<textarea name=message cols=84 rows=10 wrap=hard>
$quotemsg
</textarea><p>
$dispcrit
<input type=submit value="Send Reply">
</form>
ENDREPLY
}

# ======================================================== Display all messages

sub disall {
    my $r=shift;
     $r->print(<<ENDDISHEADER);
<script>
    function checkall() {
	for (i=0; i<document.forms.disall.elements.length; i++) {
            if 
          (document.forms.disall.elements[i].name.indexOf('delmark_')==0) {
	      document.forms.disall.elements[i].checked=true;
            }
        }
    }

    function uncheckall() {
	for (i=0; i<document.forms.disall.elements.length; i++) {
            if 
          (document.forms.disall.elements[i].name.indexOf('delmark_')==0) {
	      document.forms.disall.elements[i].checked=false;
            }
        }
    }
</script>
ENDDISHEADER
   $r->print(
 '<h1>Display All Messages</h1><form method=post name=disall '.
 'action="/adm/email">'.
     '<table border=2><tr><th colspan=2>&nbsp</th><th>Date</th>'.
     '<th>Username</th><th>Domain</th><th>Subject</th><th>Status</th></tr>');
    foreach (sort split(/\&/,&Apache::lonnet::reply('keys:'.
					$ENV{'user.domain'}.':'.
                                        $ENV{'user.name'}.':nohist_email',
                                        $ENV{'user.home'}))) {
        my ($sendtime,$shortsubj,$fromname,$fromdomain,$status)=
	    &Apache::lonmsg::unpackmsgid($_);
	if (($status ne 'deleted') && defined($sendtime) && $sendtime!~/error/) {
	    if ($status eq 'new') {
		$r->print('<tr bgcolor="#FFBB77">');
	    } elsif ($status eq 'read') {
		$r->print('<tr bgcolor="#BBBB77">');
	    } elsif ($status eq 'replied') {
		$r->print('<tr bgcolor="#AAAA88">');
	    } else {
		$r->print('<tr bgcolor="#99BBBB">');
	    }
	    $r->print('<td><a href="/adm/email?display='.$_.
		      '">Open</a></td><td><a href="/adm/email?markdel='.$_.
		      '">Delete</a><input type=checkbox name="delmark_'.$_.'"></td>'.
		      '<td>'.localtime($sendtime).'</td><td>'.
		      $fromname.'</td><td>'.$fromdomain.'</td><td>'.
		      &Apache::lonnet::unescape($shortsubj).'</td><td>'.
                      $status.'</td></tr>');
	}
    }
    $r->print('</table><p>'.
              '<a href="javascript:checkall()">Check All</a>&nbsp;'.
              '<a href="javascript:uncheckall()">Uncheck All</a><p>'.
              '<input type=submit name="markeddel" value="Delete Checked">'.
              '</form></body></html>');
}

# ============================================================== Compose output

sub compout {
    my ($r,$forwarding,$broadcast)=@_;
      my $dispcrit='';
    my $dissub='';
    my $dismsg='';
    my $func='Send New';
      if (&Apache::lonnet::allowed('srm',$ENV{'request.course.id'})) {
	 my $crithelp = Apache::loncommon::help_open_topic("Course_Critical_Message");
         $dispcrit=
 '<input type=checkbox name=critmsg> Send as critical message ' . $crithelp . 
 '<br>'.
 '<input type=checkbox name=sendbck> Send as critical message ' .
 ' and return receipt' . $crithelp . '<p>';
      }
    if ($forwarding) {
       $dispcrit.='<input type=hidden name=forwid value="'.
	   $forwarding.'">';
       $func='Forward';
      my %message=&Apache::lonnet::get('nohist_email',[$forwarding]);
      my %content=&unpackagemsg($message{$forwarding});

       $dissub='Forwarding: '.$content{'subject'};
       $dismsg='Forwarded message from '.
	   $content{'sendername'}.' at '.$content{'senderdomain'};
    }
    my $defdom=$ENV{'user.domain'};
    if ($ENV{'form.recdom'}) { $defdom=$ENV{'form.recdom'}; }
      $r->print(
                '<form action="/adm/email"  name="compemail" method="post"'.
                ' enctype="multipart/form-data">'."\n".
                '<input type="hidden" name="sendmail" value="on">'."\n".
                '<table>');
    unless (($broadcast eq 'group') || ($broadcast eq 'upload')) {
        my $domform = &Apache::loncommon::select_dom_form($defdom,'recdomain');
        my $selectlink=&Apache::loncommon::selectstudent_link
	    ('compemail','recuname','recdomain');
       $r->print(<<"ENDREC");
<table>
<tr><td>Username:</td><td><input type=text size=12 name=recuname value="$ENV{'form.recname'}"></td><td rowspan="2">$selectlink</td></tr>
<tr><td>Domain:</td>
<td>$domform</td></tr>
ENDREC
    }
    my $latexHelp = Apache::loncommon::helpLatexCheatsheet();
    if ($broadcast ne 'upload') {
       $r->print(<<"ENDCOMP");
<tr><td>Additional Recipients<br><tt>username\@domain,username\@domain, ...
</tt></td><td>
<input type=text size=50 name=additionalrec></td></tr>
<tr><td>Subject:</td><td><input type=text size=50 name=subject value="$dissub">
</td></tr></table>
$latexHelp
<textarea name=message cols=80 rows=10 wrap=hard>$dismsg
</textarea><p>
$dispcrit
<input type=submit value="$func Mail">
ENDCOMP
    } else { # $broadcast is 'upload'
	$r->print(<<ENDUPLOAD);
<input type=hidden name=sendmode value=upload>
<h3>Generate messages from a file</h3>
<p>
Subject: <input type=text size=50 name=subject>
</p>
<p>General message text<br />
<textarea name=message cols=60 rows=10 wrap=hard>$dismsg
</textarea></p>
<p>
The file format for the uploaded portion of the message is:
<pre>
username1\@domain1: text
username2\@domain2: text
username3\@domain1: text
</pre>
</p>
<p>
The messages will be assembled from all lines with the respective 
<tt>username\@domain</tt>, and appended to the general message text.</p>
<p>
<input type=file name=upfile size=20><p>
$dispcrit
<input type=submit value="Upload and send">
ENDUPLOAD
    }
    if ($broadcast eq 'group') {
       &discourse;
    }
    $r->print('</form>');
}

# ---------------------------------------------------- Display all face to face

sub disfacetoface {
    my ($r,$user,$domain)=@_;
    unless ($ENV{'request.course.id'}) { return; }
    unless (&Apache::lonnet::allowed('srm',$ENV{'request.course.id'})) {
	return;
    }
    my %records=&Apache::lonnet::dump('nohist_email',
			 $ENV{'course.'.$ENV{'request.course.id'}.'.domain'},
			 $ENV{'course.'.$ENV{'request.course.id'}.'.num'},
                         '%255b'.$user.'%253a'.$domain.'%255d');
    my $result='';
    foreach (sort keys %records) {
        my %content=&unpackagemsg($records{$_});
        next if ($content{'senderdomain'} eq '');
        $content{'message'}=~s/\n/\<br\>/g;
        if ($content{'subject'}=~/^Record/) {
	    $result.='<h3>Record</h3>';
        } else {
            $result.='<h3>Sent Message</h3>';
            %content=&unpackagemsg($content{'message'});
            $content{'message'}=
                '<b>Subject: '.$content{'subject'}.'</b><br />'.
		$content{'message'};
        }
        $result.='By: <b>'.
&Apache::loncommon::aboutmewrapper(
 &Apache::loncommon::plainname($content{'sendername'},$content{'senderdomain'}),$content{'sendername'},$content{'senderdomain'}).'</b> ('.
$content{'sendername'}.'@'.
            $content{'senderdomain'}.') '.$content{'time'}.
            '<br><blockquote>'.
              &Apache::lontexconvert::msgtexconverted($content{'message'}).
	      '</blockquote>';
     }
    # Check to see if there were any messages.
    if ($result eq '') {
        $r->print("<p><b>No notes, face-to-face discussion records, or critical messages in this course.</b></p>");
    } else {
       $r->print($result);
    }
}

# ---------------------------------------------------------------- Face to face

sub facetoface {
    my ($r,$stage)=@_;
    unless (&Apache::lonnet::allowed('srm',$ENV{'request.course.id'})) {
	return;
    }
# from query string
    if ($ENV{'form.recname'}) { $ENV{'form.recuname'}=$ENV{'form.recname'}; }
    if ($ENV{'form.recdom'}) { $ENV{'form.recdomain'}=$ENV{'form.recdom'}; }

    my $defdom=$ENV{'user.domain'};
# already filled in
    if ($ENV{'form.recdomain'}) { $defdom=$ENV{'form.recdomain'}; }
# generate output
    my $domform = &Apache::loncommon::select_dom_form($defdom,'recdomain');
    my $stdbrws = &Apache::loncommon::selectstudent_link
	('stdselect','recuname','recdomain');
    $r->print(<<"ENDTREC");
<h3>User Notes, Records of Face-To-Face Discussions, and Critical Messages in Course</h3>
<form method="post" action="/adm/email" name="stdselect">
<input type="hidden" name="recordftf" value="retrieve" />
<table>
<tr><td>Username:</td><td><input type=text size=12 name=recuname value="$ENV{'form.recuname'}"></td>
<td rowspan="2">
$stdbrws
<input type="submit" value="Retrieve discussion and message records"></td>
</tr>
<tr><td>Domain:</td>
<td>$domform</td></tr>
</table>
</form>
ENDTREC
    if (($stage ne 'query') &&
        ($ENV{'form.recdomain'}) && ($ENV{'form.recuname'})) {
        chomp($ENV{'form.newrecord'});
        if ($ENV{'form.newrecord'}) {
           &user_normal_msg_raw(
            $ENV{'course.'.$ENV{'request.course.id'}.'.num'},
            $ENV{'course.'.$ENV{'request.course.id'}.'.domain'},
            'Record ['.$ENV{'form.recuname'}.':'.$ENV{'form.recdomain'}.']',
	    $ENV{'form.newrecord'});
        }
        $r->print('<h3>'.&Apache::loncommon::plainname($ENV{'form.recuname'},
				     $ENV{'form.recdomain'}).'</h3>');
        &disfacetoface($r,$ENV{'form.recuname'},$ENV{'form.recdomain'});
	$r->print(<<ENDRHEAD);
<form method="post" action="/adm/email">
<input name="recdomain" value="$ENV{'form.recdomain'}" type="hidden" />
<input name="recuname" value="$ENV{'form.recuname'}" type="hidden" />
ENDRHEAD
        $r->print(<<ENDBFORM);
<hr />New Record (record is visible to course faculty and staff)<br />
<textarea name="newrecord" cols="80" rows="10" wrap="hard"></textarea>
<br />
<input type="hidden" name="recordftf" value="post" />
<input type="submit" value="Post this record" />
</form>
ENDBFORM
    }
}

# ===================================================================== Handler

sub handler {
    my $r=shift;

# ----------------------------------------------------------- Set document type

  $r->content_type('text/html');
  $r->send_http_header;

  return OK if $r->header_only;

# --------------------------- Get query string for limited number of parameters
    &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},
        ['display','replyto','forward','markread','markdel','markunread',
         'sendreply','compose','sendmail','critical','recname','recdom',
         'recordftf']);

# ------------------------------------------------------ They checked for email
  &Apache::lonnet::put('email_status',{'recnewemail'=>0});
# --------------------------------------------------------------- Render Output
  if (!$ENV{'form.display'}) {
      $r->print('<html><head><title>EMail and Messaging</title>'.
		&Apache::loncommon::studentbrowser_javascript().'</head>'.
		&Apache::loncommon::bodytag('EMail and Messages'));
  }
  if ($ENV{'form.display'}) {
      my $msgid=$ENV{'form.display'};
      &statuschange($msgid,'read');
      my %message=&Apache::lonnet::get('nohist_email',[$msgid]);
      my %content=&unpackagemsg($message{$msgid});
      $r->print('<html><head><title>EMail and Messaging</title>');
      if (defined($content{'baseurl'})) {
	  $r->print("<base href=\"http://$ENV{'SERVER_NAME'}/$content{'baseurl'}\" />");
      }
      $r->print(&Apache::loncommon::studentbrowser_javascript().
		'</head>'.
		&Apache::loncommon::bodytag('EMail and Messages'));
      $r->print('<b>Subject:</b> '.$content{'subject'}.
             '<br><b>From:</b> '.
&Apache::loncommon::aboutmewrapper(
&Apache::loncommon::plainname($content{'sendername'},$content{'senderdomain'}),
$content{'sendername'},$content{'senderdomain'}).' ('.
                                 $content{'sendername'}.' at '.
                                 $content{'senderdomain'}.') '.
             '<br><b>Time:</b> '.$content{'time'}.'<p>'.
             '<table border=2><tr bgcolor="#FFFFAA"><td>Functions:</td>'.
           '<td><a href="/adm/email?replyto='.&Apache::lonnet::escape($msgid).
             '"><b>Reply</b></a></td>'.
           '<td><a href="/adm/email?forward='.&Apache::lonnet::escape($msgid).
             '"><b>Forward</b></a></td>'.
        '<td><a href="/adm/email?markunread='.&Apache::lonnet::escape($msgid).
             '"><b>Mark Unread</b></a></td>'.
        '<td><a href="/adm/email?markdel='.&Apache::lonnet::escape($msgid).
             '"><b>Delete</b></a></td>'.
        '<td><a href="/adm/email"><b>Display all Messages</b></a></td>'.
             '</tr></table><p><pre>'.
             &Apache::lontexconvert::msgtexconverted($content{'message'}).
             '</pre><hr>'.$content{'citation'});
  } elsif ($ENV{'form.replyto'}) {
      &comprep($r,$ENV{'form.replyto'});
  } elsif ($ENV{'form.sendreply'}) {
      my $msgid=$ENV{'form.sendreply'};
      my %message=&Apache::lonnet::get('nohist_email',[$msgid]);
      my %content=&unpackagemsg($message{$msgid},1);
      &statuschange($msgid,'replied');
      if ((($ENV{'form.critmsg'}) || ($ENV{'form.sendbck'})) && 
          (&Apache::lonnet::allowed('srm',$ENV{'request.course.id'}))) {
         $r->print('Sending critical: '.
                &user_crit_msg($content{'sendername'},
                                 $content{'senderdomain'},
                                 &Apache::lonfeedback::clear_out_html($ENV{'form.subject'}),
                                 &Apache::lonfeedback::clear_out_html($ENV{'form.message'}),
                                 $ENV{'form.sendbck'}));
      } else {
         $r->print('Sending: '.&user_normal_msg($content{'sendername'},
                                 $content{'senderdomain'},
                                 &Apache::lonfeedback::clear_out_html($ENV{'form.subject'}),
                                 &Apache::lonfeedback::clear_out_html($ENV{'form.message'})));
      }
      if ($ENV{'form.displayedcrit'}) {
          &discrit($r);
      } else {
	  &disall($r);
      }
  } elsif ($ENV{'form.confirm'}) {
      foreach (keys %ENV) {
          if ($_=~/^form\.rec\_(.*)$/) {
	      $r->print('<b>Confirming Receipt:</b> '.
                        &user_crit_received($1).'<br>');
          }
          if ($_=~/^form\.reprec\_(.*)$/) {
              my $msgid=$1;
	      $r->print('<b>Confirming Receipt:</b> '.
                        &user_crit_received($msgid).'<br>');
              &comprep($r,$msgid);
          }
      }
      &discrit($r);
  } elsif ($ENV{'form.critical'}) {
      &discrit($r);
  } elsif ($ENV{'form.forward'}) {
      &compout($r,$ENV{'form.forward'});
  } elsif ($ENV{'form.markread'}) {
  } elsif ($ENV{'form.markdel'}) {
      &statuschange($ENV{'form.markdel'},'deleted');
      &disall($r);
  } elsif ($ENV{'form.markeddel'}) {
      my $total=0;
      foreach (keys %ENV) {
          if ($_=~/^form\.delmark_(.*)$/) {
	      &statuschange(&Apache::lonnet::unescape($1),'deleted');
              $total++;
          }
      }
      $r->print('Deleted '.$total.' message(s)<p>');
      &disall($r);
  } elsif ($ENV{'form.markunread'}) {
      &statuschange($ENV{'form.markunread'},'new');
      &disall($r);
  } elsif ($ENV{'form.compose'}) {
      &compout($r,'',$ENV{'form.compose'});
  } elsif ($ENV{'form.recordftf'}) {
      &facetoface($r,$ENV{'form.recordftf'});
  } elsif ($ENV{'form.sendmail'}) {
      my %content=();
      undef %content;
      if ($ENV{'form.forwid'}) {
        my $msgid=$ENV{'form.forwid'};
        my %message=&Apache::lonnet::get('nohist_email',[$msgid]);
        %content=&unpackagemsg($message{$msgid},1);
        &statuschange($msgid,'forwarded');
        $ENV{'form.message'}.="\n\n-- Forwarded message --\n\n".
	                       $content{'message'};
      }
      my %toaddr=();
      undef %toaddr;
      if ($ENV{'form.sendmode'} eq 'group') {
          foreach (keys %ENV) {
	      if ($_=~/^form\.send\_to\_\&\&\&[^\&]*\&\&\&\_(.+)$/) {
		  $toaddr{$1}='';
              }
          }
      } elsif ($ENV{'form.sendmode'} eq 'upload') {
          foreach (split(/[\n\r\f]+/,$ENV{'form.upfile'})) {
              my ($rec,$txt)=split(/\s*\:\s*/,$_);
              if ($txt) {
		  $rec=~s/\@/\:/;
                  $toaddr{$rec}.=$txt."\n";
              }
          }
      } else {
	  $toaddr{$ENV{'form.recuname'}.':'.$ENV{'form.recdomain'}}='';
      }
      if ($ENV{'form.additionalrec'}) {
	  foreach (split(/\,/,$ENV{'form.additionalrec'})) {
              my ($auname,$audom)=split(/\@/,$_);
              $toaddr{$auname.':'.$audom}='';
          }
      }
    foreach (keys %toaddr) {
      my ($recuname,$recdomain)=split(/\:/,$_);
      my $msgtxt=&Apache::lonfeedback::clear_out_html($ENV{'form.message'});
      if ($toaddr{$_}) { $msgtxt.='<hr>'.$toaddr{$_}; }    
      if ((($ENV{'form.critmsg'}) || ($ENV{'form.sendbck'})) && 
          (&Apache::lonnet::allowed('srm',$ENV{'request.course.id'}))) {
         $r->print('Sending critical: '.
                &user_crit_msg($recuname,$recdomain,
               &Apache::lonfeedback::clear_out_html($ENV{'form.subject'}),
                                 $msgtxt,
                                 $ENV{'form.sendbck'}));
      } else {
         $r->print('Sending: '.&user_normal_msg($recuname,$recdomain,
              &Apache::lonfeedback::clear_out_html($ENV{'form.subject'}),
                                 $msgtxt,
                                 $content{'citation'}));
      }
      $r->print('<br>');
    }
      if ($ENV{'form.displayedcrit'}) {
          &discrit($r);
      } else {
	  &disall($r);
      }
  } else {
      &disall($r);
  }
  $r->print('</body></html>');
  return OK;

}
# ================================================= Main program, reset counter

BEGIN {
    $msgcount=0;
}

1;
__END__







