# The LearningOnline Network with CAPA
# Search Catalog
#
# $Id: lonsearchcat.pm,v 1.185 2003/07/14 19:26:24 matthew Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
###############################################################################
###############################################################################

=pod 

=head1 NAME

lonsearchcat - LONCAPA Search Interface

=head1 SYNOPSIS

Search interface to LON-CAPAs digital library

=head1 DESCRIPTION

This module enables searching for a distributed browseable catalog.

This is part of the LearningOnline Network with CAPA project
described at http://www.lon-capa.org.

lonsearchcat presents the user with an interface to search the LON-CAPA
digital library.  lonsearchcat also initiates the execution of a search
by sending the search parameters to LON-CAPA servers.  The progress of 
search (on a server basis) is displayed to the user in a seperate window.

=head1 Internals

=over 4

=cut

###############################################################################
###############################################################################

###############################################################################
##                                                                           ##
## ORGANIZATION OF THIS PERL MODULE                                          ##
##                                                                           ##
## 1. Modules used by this module                                            ##
## 2. Variables used throughout the module                                   ##
## 3. handler subroutine called via Apache and mod_perl                      ##
## 4. Other subroutines                                                      ##
##                                                                           ##
###############################################################################

package Apache::lonsearchcat;

# ------------------------------------------------- modules used by this module
use strict;
use Apache::Constants qw(:common :http);
use Apache::lonnet();
use Apache::File();
use CGI qw(:standard);
use Text::Query;
use GDBM_File;
use Apache::loncommon();
use Apache::lonmysql();

# ---------------------------------------- variables used throughout the module

######################################################################
######################################################################

=pod 

=item Global variables

=over 4

=item $importbutton

button to take the select results and go to group sorting

=item %groupsearch_db   

Database hash used to save values for the groupsearch RAT interface.

=item $diropendb 

The full path to the (temporary) search database file.  This is set and
used in &handler() and is also used in &output_results().

=item %Views

Hash which associates an output view description with the function
that produces it.  Adding a new view type should be as easy as
adding a line to the definition of this hash and making sure the function
takes the proper parameters.

=item $bodytag

LON-CAPA standard body tag, gotten from &Apache::lonnet::bodytag.
No title, no table, just a <body> tag.

=back 

=cut

######################################################################
######################################################################

# -- dynamically rendered interface components
my $importbutton; # button to take the selected results and go to group sorting

# -- miscellaneous variables
my %groupsearch_db;     # database hash
my $diropendb = "";    # db file
#             View Description           Function Pointer
my %Views = ("Detailed Citation View" => \&detailed_citation_view,
             "Summary View"           => \&summary_view,
             "Fielded Format"         => \&fielded_format_view,
             "XML/SGML"               => \&xml_sgml_view,
             "Compact View"           => \&compact_view);
my %persistent_db;
my $hidden_fields;
my $bodytag;

#
# For course search
#
my %alreadyseen;
my $hashtied;
my %hash;
my $totalfound;
 
######################################################################
######################################################################

=pod 

=item &handler() - main handler invoked by httpd child

=item Variables

=over 4

=item $hidden

holds 'hidden' html forms

=item $scrout

string that holds portions of the screen output

=back 

=cut

######################################################################
######################################################################
sub handler {
    my $r = shift;
    #

    my $loaderror=&Apache::lonnet::overloaderror($r);
    if ($loaderror) { return $loaderror; }

    my $closebutton;  # button that closes the search window 
                      # This button is different for the RAT compared to
                      # normal invocation.
    #
    $r->content_type('text/html');
    $r->send_http_header;
    return OK if $r->header_only;
    ##
    ## Prevent caching of the search interface window.  Hopefully this means
    ## we will get the launch=1 passed in a little more.
    &Apache::loncommon::no_cache($r);
    ## 
    ## Pick up form fields passed in the links.
    ##
    &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},
             ['catalogmode','launch','acts','mode','form','element','pause',
              'phase','persistent_db_id','table','start','show',
              'cleargroupsort']);
    ##
    ## The following is a trick - we wait a few seconds if asked to so
    ##     the daemon running the search can get ahead of the daemon
    ##     printing the results.  We only need (theoretically) to do
    ##     this once, so the pause indicator is deleted
    ##
    if (exists($ENV{'form.pause'})) {
        sleep(1);
        delete($ENV{'form.pause'});
    }
    ##
    ## Initialize global variables
    ##
    my $domain  = $r->dir_config('lonDefDomain');
    $diropendb= "/home/httpd/perl/tmp/$ENV{'user.domain'}_$ENV{'user.name'}_searchcat.db";
    #
    # set the name of the persistent database
    #          $ENV{'form.persistent_db_id'} can only have digits in it.
    if (! exists($ENV{'form.persistent_db_id'}) ||
        ($ENV{'form.persistent_db_id'} =~ /\D/) ||
        ($ENV{'form.launch'} eq '1')) {
        $ENV{'form.persistent_db_id'} = time;
    }
    $bodytag=&Apache::loncommon::bodytag(undef,undef,undef,1);
    my $persistent_db_file = "/home/httpd/perl/tmp/".
        &Apache::lonnet::escape($domain).
            '_'.&Apache::lonnet::escape($ENV{'user.name'}).
                '_'.$ENV{'form.persistent_db_id'}.'_persistent_search.db';
    ##
    if (! &get_persistent_form_data($persistent_db_file)) {
        if ($ENV{'form.phase'} =~ /(run_search|results)/) {
            &Apache::lonnet::logthis("lonsearchcat:Unable to recover data ".
                                     "from $persistent_db_file");
            $r->print(<<END);
<html>
<head><title>LON-CAPA Search Error</title></head>
$bodytag
We were unable to retrieve data describing your search.  This is a serious
error and has been logged.  Please alert your LON-CAPA administrator.
</body>
</html>
END
            return OK;
        }
    }
    ##
    ## Clear out old values from groupsearch database
    ##
    untie %groupsearch_db if (tied(%groupsearch_db));
    if (($ENV{'form.cleargroupsort'} eq '1') || 
        (($ENV{'form.launch'} eq '1') && 
         ($ENV{'form.catalogmode'} eq 'groupsearch'))) {
	if (tie(%groupsearch_db,'GDBM_File',$diropendb,&GDBM_WRCREAT(),0640)) {
	    &start_fresh_session();
	    untie %groupsearch_db;
            delete($ENV{'form.cleargroupsort'});
	} else {
            # This is a stupid error to give to the user.  
            # It really tells them nothing.
	    $r->print('<html><head></head>'.$bodytag.
                      'Unable to tie hash to db file</body></html>');
	    return OK;
	}
    }
    ##
    ## Configure hidden fields
    ##
    $hidden_fields = '<input type="hidden" name="persistent_db_id" value="'.
        $ENV{'form.persistent_db_id'}.'" />'."\n";
    if (exists($ENV{'form.catalogmode'})) {
        $hidden_fields .= '<input type="hidden" name="catalogmode" value="'.
                $ENV{'form.catalogmode'}.'" />'."\n";
    }
    if (exists($ENV{'form.form'})) {
        $hidden_fields .= '<input type="hidden" name="form" value="'.
                $ENV{'form.form'}.'" />'."\n";
    }
    if (exists($ENV{'form.element'})) {
        $hidden_fields .= '<input type="hidden" name="element" value="'.
                $ENV{'form.element'}.'" />'."\n";
    }
    if (exists($ENV{'form.mode'})) {
        $hidden_fields .= '<input type="hidden" name="mode" value="'.
                $ENV{'form.mode'}.'" />'."\n";
    }
    ##
    ## Configure dynamic components of interface
    ##
    if ($ENV{'form.catalogmode'} eq 'interactive') {
        $closebutton="<input type='button' name='close' value='CLOSE' ";
        if ($ENV{'form.phase'} =~ /(results|run_search)/) {
	    $closebutton .="onClick='parent.close()'";
        } else {
            $closebutton .="onClick='self.close()'";
        }
        $closebutton .=">\n";
    } elsif ($ENV{'form.catalogmode'} eq 'groupsearch') {
        $closebutton="<input type='button' name='close' value='CLOSE' ";
        if ($ENV{'form.phase'} =~ /(results|run_search)/) {
	    $closebutton .="onClick='parent.close()'";
        } else {
            $closebutton .="onClick='self.close()'";
        }
        $closebutton .= ">";
        $importbutton=<<END;
<input type='button' name='import' value='IMPORT'
onClick='javascript:select_group()'>
END
    } else {
        $closebutton = '';
        $importbutton = '';
    }
    ##
    ## Sanity checks on form elements
    ##
    if (!defined($ENV{'form.viewselect'})) {
        if (($ENV{'form.catalogmode'} eq 'groupsearch') ||
            ($ENV{'form.catalogmode'} eq 'interactive')) {
            $ENV{'form.viewselect'} ="Compact View";
        } else {
            $ENV{'form.viewselect'} ="Detailed Citation View";
        }
    }
    $ENV{'form.phase'} = 'disp_basic' if (! exists($ENV{'form.phase'}));
    $ENV{'form.show'} = 20 if (! exists($ENV{'form.show'}));
    ##
    ## Switch on the phase
    ##
    if ($ENV{'form.phase'} eq 'disp_basic') {
        &print_basic_search_form($r,$closebutton);
    } elsif ($ENV{'form.phase'} eq 'disp_adv') {
        &print_advanced_search_form($r,$closebutton);
    } elsif ($ENV{'form.phase'} eq 'results') {
        &display_results($r,$importbutton,$closebutton);
    } elsif ($ENV{'form.phase'} =~ /^(sort|run_search)$/) {
        my ($query,$customquery,$customshow,$libraries,$pretty_string) =
            &get_persistent_data($persistent_db_file,
                 ['query','customquery','customshow',
                  'libraries','pretty_string']);
        if ($ENV{'form.phase'} eq 'sort') {
            &print_sort_form($r,$pretty_string);
        } elsif ($ENV{'form.phase'} eq 'run_search') {
            &run_search($r,$query,$customquery,$customshow,
                        $libraries,$pretty_string);
        }
    } elsif ($ENV{'form.phase'} eq 'course_search') {
        &course_search($r);
    } elsif(($ENV{'form.phase'} eq 'basic_search') ||
            ($ENV{'form.phase'} eq 'adv_search')) {
        $ENV{'form.searchmode'} = 'basic';
        if ($ENV{'form.phase'} eq 'adv_search') {
            $ENV{'form.searchmode'} = 'advanced';
        }
        # Set up table
        if (! defined(&create_results_table())) {
            $r->print(<<END);
<html><head><title>Search Error</title></head>
$bodytag
Unable to create table in which to store search results.  
The search has been aborted.
</body>
</html>
END
            return OK;
        }
        delete($ENV{'form.launch'});
        if (! &make_form_data_persistent($r,$persistent_db_file)) {
            $r->print(<<END);
<html><head><title>Search Error</title></head>
$bodytag
Unable to properly store search information.  The search has been aborted.
</body>
</html>
END
            return OK;
        }
        #
        # We are running a search
        my ($query,$customquery,$customshow,$libraries) = 
            (undef,undef,undef,undef);
        my $pretty_string;
        if ($ENV{'form.phase'} eq 'basic_search') {
            ($query,$pretty_string,$libraries) = 
                &parse_basic_search($r,$closebutton);
        } else {                      # Advanced search
            ($query,$customquery,$customshow,$libraries,$pretty_string) 
                = &parse_advanced_search($r,$closebutton);
            return OK if (! defined($query));
        }
        &make_persistent({ query => $query,
                           customquery => $customquery,
                           customshow => $customshow,
                           libraries => $libraries,
                           pretty_string => $pretty_string },
                         $persistent_db_file);
        ##
        ## Print out the frames interface
        ##
        &print_frames_interface($r);
    }
    return OK;
} 

######################################################################
######################################################################

sub course_search {
    my $r=shift;
    my $bodytag=&Apache::loncommon::bodytag('Course Search');
    my $pretty_search_string = '<b>'.$ENV{'form.courseexp'}.'</b>';
    my $search_string = $ENV{'form.courseexp'};
    my @New_Words;
    if ($ENV{'form.crsrelated'}) {
        ($search_string,@New_Words) = &related_version($ENV{'form.courseexp'});
        if (@New_Words) {
            $pretty_search_string .= " with related words: <b>@New_Words</b>.";
        } else {
            $pretty_search_string .= " with no related words.";
        }
    }
    my $fulltext=$ENV{'form.crsfulltext'};
    my @allwords=($search_string,@New_Words);
    $totalfound=0;
    $r->print('<html><head><title>LON-CAPA Course Search</title></head>'.
	      $bodytag.$pretty_search_string);
    $r->rflush();
# ======================================================= Go through the course
   $hashtied=0;
   undef %alreadyseen;
   %alreadyseen=();
    my $c=$r->connection;
   &tiehash();
   foreach (keys %hash) {
       if ($c->aborted()) { last; }
       if (($_=~/^src\_(.+)$/) && (!$alreadyseen{$hash{$_}})) {
           &checkonthis($r,$hash{$_},0,$hash{'title_'.$1},$fulltext,@allwords);
       }
   }
   &untiehash();
    unless ($totalfound) {
	$r->print('<p>No resources found.</p>');
    }
# =================================================== Done going through course
    $r->print('</body></html>');
}

# ---------------------------------------------------------------- tie the hash

sub tiehash {
    $hashtied=0;
    if ($ENV{'request.course.fn'}) {
        if (tie(%hash,'GDBM_File',$ENV{'request.course.fn'}.".db",
            &GDBM_READER(),0640)) {
                $hashtied=1;
        }
    }    
}

sub untiehash {
    if ($hashtied) { untie %hash; }
    $hashtied=0;
}

# =============================== This pulls up a resource and its dependencies

sub checkonthis {
    my ($r,$url,$level,$title,$fulltext,@allwords)=@_;
    $alreadyseen{$url}=1;
    $r->rflush();
    my $result=&Apache::lonnet::metadata($url,'title').' '.
               &Apache::lonnet::metadata($url,'subject').' '.
               &Apache::lonnet::metadata($url,'abstract').' '.
               &Apache::lonnet::metadata($url,'keywords');
    if (($url) && ($fulltext)) {
	$result.=&Apache::lonnet::ssi_body($url);
    }
    $result=~s/\s+/ /gs;
    my $applies=0;
    foreach (@allwords) {
        if ($_=~/\w/) {
	   if ($result=~/$_/si) {
	      $applies++;
           }
       }
    }
# Does this resource apply?
    if ($applies) {
       $r->print('<br />');
       for (my $i=0;$i<=$level*5;$i++) {
           $r->print('&nbsp;');
       }
       $r->print('<a href="'.$url.'" target="cat">'.
		 ($title?$title:$url).'</a><br />');
       $totalfound++;
    } elsif ($fulltext) {
       $r->print(' .');
    }
    $r->rflush();
# Check also the dependencies of this one
    my $dependencies=
                &Apache::lonnet::metadata($url,'dependencies');
    foreach (split(/\,/,$dependencies)) {
       if (($_=~/^\/res\//) && (!$alreadyseen{$_})) {
          &checkonthis($r,$_,$level+1,'',$fulltext,@allwords);
       }
    }
}

######################################################################
######################################################################

=pod 

=item &print_basic_search_form() 

Returns a scalar which holds html for the basic search form.

=cut

######################################################################
######################################################################

sub print_basic_search_form{
    my ($r,$closebutton) = @_;
    my $bodytag=&Apache::loncommon::bodytag('Search');
    my $scrout=<<"ENDDOCUMENT";
<html>
<head>
<title>The LearningOnline Network with CAPA</title>
<script type="text/javascript">
    function openhelp(val) {
	openhelpwin=open('/adm/help/searchcat.html','helpscreen',
	     'scrollbars=1,width=600,height=300');
	openhelpwin.focus();
    }
</script>
</head>
$bodytag
ENDDOCUMENT
if (&Apache::lonnet::allowed('bre',$ENV{'request.role.domain'})) {
    $scrout.=(<<ENDDOCUMENT);
<h1>Catalog Search</h1>
<form method="post" action="/adm/searchcat">
<input type="hidden" name="phase" value="basic_search" />
$hidden_fields
<p>
Enter terms or phrases separated by AND, OR, or NOT 
then press SEARCH below.
</p>
<p>
<table>
<tr><td>
ENDDOCUMENT
    $scrout.='&nbsp;'.&simpletextfield('basicexp',$ENV{'form.basicexp'},40).
        '&nbsp;';
    my $relatedcheckbox = &simplecheckbox('related',$ENV{'form.related'});
    my $domain = $r->dir_config('lonDefDomain');
    my $domaincheckbox = &simplecheckbox('domains',$domain);
    $scrout.=<<END;
</td><td><a
href="/adm/searchcat?phase=disp_adv&catalogmode=$ENV{'form.catalogmode'}&launch=$ENV{'form.launch'}"
>Advanced Search</a></td></tr>
<tr><td>$relatedcheckbox use related words</td>
    <td>$domaincheckbox only search domain <b>$domain</b></td></tr>
</table>
</p>
<p>
&nbsp;<input type="submit" name="basicsubmit" value='SEARCH' />&nbsp;
$closebutton
END
    $scrout.=&selectbox(undef,'viewselect',
			$ENV{'form.viewselect'},
			undef,undef,undef,
			sort(keys(%Views)));
    $scrout.=&selectbox(undef,'show',
			$ENV{'form.show'},
			undef,undef,undef,
			(10,20,50,100));
    $scrout.=<<ENDDOCUMENT;
per page.
</p>
</form>
ENDDOCUMENT
    }
    if ($ENV{'request.course.id'}) {
        $scrout.=(<<ENDCOURSESEARCH);
<hr />
<h1>Course Search</h1>    
<form method="post" action="/adm/searchcat">
<input type="hidden" name="phase" value="course_search" />
$hidden_fields
<p>
Enter terms or phrases, then press SEARCH below.
</p>
<p>
<table>
<tr><td>
ENDCOURSESEARCH
        $scrout.='&nbsp;'.
            &simpletextfield('courseexp',$ENV{'form.courseexp'},40);
        my $crscheckbox = 
            &simplecheckbox('crsfulltext',$ENV{'form.crsfulltext'});
        my $relcheckbox = 
            &simplecheckbox('crsrelated',$ENV{'form.crsrelated'});
        $scrout.=(<<ENDENDCOURSE);
</td></tr>
<tr><td>$relcheckbox use related words</td><td></td></tr>
<tr><td>$crscheckbox fulltext search (time consuming)</td><td></td></tr>
</table><p>
&nbsp;<input type="submit" name="coursesubmit" value='SEARCH' />
</p>
ENDENDCOURSE
    }
    $scrout.=(<<ENDDOCUMENT);
</body>
</html>
ENDDOCUMENT
    $r->print($scrout);
    return;
}
######################################################################
######################################################################

=pod 

=item &advanced_search_form() 

Returns a scalar which holds html for the advanced search form.

=cut

######################################################################
######################################################################

sub print_advanced_search_form{
    my ($r,$closebutton) = @_;
    my $advanced_buttons = <<"END";
<p>
<input type="submit" name="advancedsubmit" value='SEARCH' />
<input type="reset" name="reset" value='RESET' />
$closebutton
<input type="button" value="HELP" onClick="openhelp()" />
</p>
END
    if (!defined($ENV{'form.viewselect'})) {
        $ENV{'form.viewselect'} ="Detailed Citation View";
    }
    my $bodytag=&Apache::loncommon::bodytag('Advanced Catalog Search');
    my $scrout=<<"ENDHEADER";
<html>
<head>
<title>The LearningOnline Network with CAPA</title>
<script type="text/javascript">
    function openhelp(val) {
	openhelpwin=open('/adm/help/searchcat.html','helpscreen',
	     'scrollbars=1,width=600,height=300');
	openhelpwin.focus();
    }
</script>
</head>
$bodytag
Enter terms or phrases separated by search operators 
such as AND, OR, or NOT.<br />
<form method="post" action="/adm/searchcat">
$advanced_buttons
$hidden_fields
<input type="hidden" name="phase" value="adv_search" />
<table>
<tr><td><font color="#800000" face="helvetica"><b>VIEW:</b></font></td>
<td>
ENDHEADER
    $scrout.=&selectbox(undef,'viewselect',
			$ENV{'form.viewselect'},
			undef,undef,undef,
			sort(keys(%Views)));
    $scrout.='&nbsp;';
    $scrout.=&selectbox(undef,'show',
			$ENV{'form.show'},
			undef,undef,undef,
			(10,20,50,100));
    $scrout.='&nbsp;'.
        '<font color="#800000" face="helvetica">Per Page</font>';
    $scrout.="</td><td>Related<br />Words</td></tr>\n";
    $scrout.=&searchphrasefield_with_related('title',   'title'   ,
                                             $ENV{'form.title'});
    $scrout.=&searchphrasefield('author',  'author'  ,$ENV{'form.author'});
    $scrout.=&searchphrasefield_with_related('subject', 'subject' ,
                                             $ENV{'form.subject'});
    $scrout.=&searchphrasefield_with_related('keywords','keywords',
                                             $ENV{'form.keywords'});
    $scrout.=&searchphrasefield('URL',     'url'     ,$ENV{'form.url'});
    $scrout.=&searchphrasefield_with_related('notes',   'notes'   ,
                                             $ENV{'form.notes'});
    $scrout.=&searchphrasefield_with_related('abstract','abstract',
                                             $ENV{'form.abstract'});
    # Hack - an empty table row.
    $scrout.="<tr><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td></tr>\n";
    $scrout.=&searchphrasefield('file<br />extension','extension',
                        $ENV{'form.extension'});
    $scrout.="<tr><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td></tr>\n";
    $scrout.=&searchphrasefield('publisher<br />owner','owner',
				$ENV{'form.owner'});
    $scrout.="</table>\n";
    $ENV{'form.category'}='any' if (! defined($ENV{'form.category'}));
    $scrout.=&selectbox('File Category','category',
			$ENV{'form.category'},
			'any','Any category',
			undef,
			(&Apache::loncommon::filecategories()));
    $ENV{'form.language'}='any' unless length($ENV{'form.language'});
    #----------------------------------------------------------------
    # Allow restriction to multiple domains.
    #   I make the crazy assumption that there will never be a domain 'any'.
    #
    $ENV{'form.domains'} = 'any' if (! exists($ENV{'form.domains'}));
    my @allowed_domains = (ref($ENV{'form.domains'}) ? @{$ENV{'form.domains'}} 
                           :  ($ENV{'form.domains'}) );
    my %domain_hash = ();
    foreach (@allowed_domains) {
        $domain_hash{$_}++;
    }
    my @domains =&Apache::loncommon::get_domains();
    # adjust the size of the select box
    my $size = 4;
    $size = (scalar @domains < ($size - 1) ? scalar @domains + 1 : $size);
    $scrout.="\n".'<font color="#800000" face="helvetica"><b>'.
        'DOMAINS TO BE SEARCHED</b></font><br />'.
            '<select name="domains" size="'.$size.'" multiple>'."\n".
                '<option name="any" value="any" '.
                    ($domain_hash{'any'}? 'selected ' :'').
                        '>all domains</option>'."\n";
    foreach my $dom (sort @domains) {
        $scrout.="<option name=\"$dom\" ".
            ($domain_hash{$dom} ? 'selected ' :'').">$dom</option>\n";
    }
    $scrout.="</select>\n";
    #----------------------------------------------------------------
    $scrout.=&selectbox('Limit by language','language',
			$ENV{'form.language'},'any','Any Language',
			\&Apache::loncommon::languagedescription,
			(&Apache::loncommon::languageids),
			);
# ------------------------------------------------ Compute date selection boxes
    $scrout.=<<CREATIONDATESTART;
<p>
<font color="#800000" face="helvetica"><b>LIMIT BY CREATION DATE RANGE:</b>
</font>
<br />
between:
CREATIONDATESTART
    $scrout.=&dateboxes('creationdatestart',1,1,1976,
			$ENV{'form.creationdatestart_month'},
			$ENV{'form.creationdatestart_day'},
			$ENV{'form.creationdatestart_year'},
			);
    $scrout.="and:\n";
    $scrout.=&dateboxes('creationdateend',12,31,2051,
			$ENV{'form.creationdateend_month'},
			$ENV{'form.creationdateend_day'},
			$ENV{'form.creationdateend_year'},
			);
    $scrout.="</p>";
    $scrout.=<<LASTREVISIONDATESTART;
<p>
<font color="#800000" face="helvetica"><b>LIMIT BY LAST REVISION DATE RANGE:
</b></font>
<br />between:
LASTREVISIONDATESTART
    $scrout.=&dateboxes('lastrevisiondatestart',1,1,1976,
			$ENV{'form.lastrevisiondatestart_month'},
			$ENV{'form.lastrevisiondatestart_day'},
			$ENV{'form.lastrevisiondatestart_year'},
			);
    $scrout.=<<LASTREVISIONDATEEND;
and:
LASTREVISIONDATEEND
    $scrout.=&dateboxes('lastrevisiondateend',12,31,2051,
			$ENV{'form.lastrevisiondateend_month'},
			$ENV{'form.lastrevisiondateend_day'},
			$ENV{'form.lastrevisiondateend_year'},
			);
    $scrout.='</p>';
    $ENV{'form.copyright'}='any' unless length($ENV{'form.copyright'});
    $scrout.=&selectbox('Limit by copyright/distribution','copyright',
			 $ENV{'form.copyright'},
			 'any','Any copyright/distribution',
			 \&Apache::loncommon::copyrightdescription,
			 (&Apache::loncommon::copyrightids),
			 );
# ------------------------------------------- Compute customized metadata field
#    $scrout.=<<CUSTOMMETADATA;
#<p>
#<font color="#800000" face="helvetica"><b>LIMIT BY SPECIAL METADATA FIELDS:</b>
#</font>
#For resource-specific metadata, enter in an expression in the form of 
#<i>key</i>=<i>value</i> separated by operators such as AND, OR or NOT.<br />
#<b>Example:</b> grandmother=75 OR grandfather=85
#<br />
#CUSTOMMETADATA
#    $scrout.=&simpletextfield('custommetadata',$ENV{'form.custommetadata'});
#    $scrout.=<<CUSTOMSHOW;
#<p>
#<font color="#800000" face="helvetica"><b>SHOW SPECIAL METADATA FIELDS:</b>
#</font>
#Enter in a space-separated list of special metadata fields to show
#in a fielded listing for each record result.
#<br />
#CUSTOMSHOW
#    $scrout.=&simpletextfield('customshow',$ENV{'form.customshow'});
    $scrout.=<<ENDDOCUMENT;
$advanced_buttons
</form>
</body>
</html>
ENDDOCUMENT
    $r->print($scrout);
    return;
}

######################################################################
######################################################################

=pod 

=item &get_persistent_form_data

Inputs: filename of database

Outputs: returns undef on database errors.

This function is the reverse of &make_persistent() for form data.
Retrieve persistent data from %persistent_db.  Retrieved items will have their
values unescaped.  If a form value already exists in $ENV, it will not be
overwritten.  Form values that are array references may have values appended
to them.

=cut

######################################################################
######################################################################
sub get_persistent_form_data {
    my $filename = shift;
    return 0 if (! -e $filename);
    return undef if (! tie(%persistent_db,'GDBM_File',$filename,
                           &GDBM_READER(),0640));
    #
    # These make sure we do not get array references printed out as 'values'.
    my %arrays_allowed = ('form.domains'=>1);
    #
    # Loop through the keys, looking for 'form.'
    foreach my $name (keys(%persistent_db)) {
        next if ($name !~ /^form./);
        # Kludgification begins!
        if ($name eq 'form.domains' && 
            $ENV{'form.searchmode'} eq 'basic' &&
            $ENV{'form.phase'} ne 'disp_basic') {
            next;
        }
        # End kludge (hopefully)
        next if (exists($ENV{$name}));
        my @values = map { 
            &Apache::lonnet::unescape($_);
        } split(',',$persistent_db{$name});
        next if (@values <1);
        if ($arrays_allowed{$name}) {
            $ENV{$name} = [@values];
        } else {
            $ENV{$name} = $values[0] if ($values[0]);
        }
    }
    untie (%persistent_db);
    return 1;
}

######################################################################
######################################################################

=pod 

=item &get_persistent_data

Inputs: filename of database, ref to array of values to recover.

Outputs: array of values.  Returns undef on error.

This function is the reverse of &make_persistent();
Retrieve persistent data from %persistent_db.  Retrieved items will have their
values unescaped.  If the item contains commas (before unescaping), the
returned value will be an array pointer. 

=cut

######################################################################
######################################################################
sub get_persistent_data {
    my $filename = shift;
    my @Vars = @{shift()};
    my @Values;   # Return array
    return undef if (! -e $filename);
    return undef if (! tie(%persistent_db,'GDBM_File',$filename,
                           &GDBM_READER(),0640));
    foreach my $name (@Vars) {
        if (! exists($persistent_db{$name})) {
            push @Values, undef;
            next;
        }
        my @values = map { 
            &Apache::lonnet::unescape($_);
        } split(',',$persistent_db{$name});
        if (@values <= 1) {
            push @Values,$values[0];
        } else {
            push @Values,\@values;
        }
    }
    untie (%persistent_db);
    return @Values;
}

######################################################################
######################################################################

=pod 

=item &make_persistent() 

Inputs: Hash of values to save, filename of persistent database.

Store variables away to the %persistent_db.
Values will be escaped.  Values that are array pointers will have their
elements escaped and concatenated in a comma seperated string.  

=cut

######################################################################
######################################################################
sub make_persistent {
    my %save = %{shift()};
    my $filename = shift;
    return undef if (! tie(%persistent_db,'GDBM_File',
                           $filename,&GDBM_WRCREAT(),0640));
    foreach my $name (keys(%save)) {
        my @values = (ref($save{$name}) ? @{$save{$name}} : ($save{$name}));
        # We handle array references, but not recursively.
        my $store = join(',', map { &Apache::lonnet::escape($_); } @values );
        $persistent_db{$name} = $store;
    }
    untie(%persistent_db);
    return 1;
}

######################################################################
######################################################################

=pod 

=item &make_form_data_persistent() 

Inputs: filename of persistent database.

Store most form variables away to the %persistent_db.
Values will be escaped.  Values that are array pointers will have their
elements escaped and concatenated in a comma seperated string.  

=cut

######################################################################
######################################################################
sub make_form_data_persistent {
    my $r = shift;
    my $filename = shift;
    my %save;
    foreach (keys(%ENV)) {
        next if (!/^form/ || /submit/);
        $save{$_} = $ENV{$_};
    }
    return &make_persistent(\%save,$filename);
}

######################################################################
#                HTML form building functions                        #  
######################################################################

=pod 

=item HTML form building functions

=over 4

=cut

###############################################
###############################################

=pod

=item &simpletextfield() 

Inputs: $name,$value,$size

Returns a text input field with the given name, value, and size.  
If size is not specified, a value of 20 is used.

=cut

###############################################
###############################################

sub simpletextfield {
    my ($name,$value,$size)=@_;
    $size = 20 if (! defined($size));
    return '<input type="text" name="'.$name.
        '" size="'.$size.'" value="'.$value.'" />';
}

###############################################
###############################################

=pod

=item &simplecheckbox()

Inputs: $name,$value

Returns a simple check box with the given $name.
If $value eq 'on' the box is checked.

=cut

###############################################
###############################################

sub simplecheckbox{
    my ($name,$value,$checked)=@_;
    $checked="checked" if ($value eq 'on');
    return '<input type="checkbox" name="'.$name.'" value="'.$value.'" '.
        $checked.' />';
}

###############################################
###############################################

=pod

=item &fieldtitle()

Input: $title

Returns a scalar with html which will display $title as a search
field heading.

=cut

###############################################
###############################################

sub fieldtitle {
    my $title = uc(shift());
    return '<font color="#800000" face="helvetica"><b>'.$title.
        ':&nbsp;</b></font>';
}

###############################################
###############################################

=pod

=item &searchphrasefield()

Inputs: $title,$name,$value

Returns html for a title line and an input field for entering search terms.
The entry field (which is where the $name and $value are used) is a 50 column 
simpletextfield.  The html returned is for a row in a three column table.

=cut

###############################################
###############################################
    
sub searchphrasefield {
    my ($title,$name,$value)=@_;
    return '<tr><td>'.&fieldtitle($title).'</td><td>'.
        &simpletextfield($name,$value,50)."</td><td>&nbsp;</td></tr>\n";
}

###############################################
###############################################

=pod

=item &searchphrasefield_with_related()

Inputs: $title,$name,$value

Returns html for a title line and an input field for entering search terms
and a check box for 'related words'.  The entry field (which is where the 
$name and $value are used) is a 50 column simpletextfield.  The name of
the related words checkbox is "$name_related".

=cut

###############################################
###############################################
    
sub searchphrasefield_with_related {
    my ($title,$name,$value)=@_;
    return '<tr><td>'.&fieldtitle($title).'</td><td>'.
        &simpletextfield($name,$value,50).'</td><td align="center">&nbsp;'.
            &simplecheckbox($name.'_related',$ENV{'form.'.$name.'_related'}).
                "&nbsp;</td></tr>\n";
}

###############################################
###############################################

=pod

=item &dateboxes()

Returns html selection form elements for the specification of 
the day, month, and year.

=cut

###############################################
###############################################

sub dateboxes {
    my ($name,$defaultmonth,$defaultday,$defaultyear,
	$currentmonth,$currentday,$currentyear)=@_;
    ($defaultmonth,$defaultday,$defaultyear)=('','','');
    #
    # Day
    my $day=<<END;
<select name="${name}_day">
<option value='$defaultday'> </option>
END
    for (my $i = 1; $i<=31; $i++) {
	$day.="<option value=\"$i\">$i</option>\n";
    }
    $day.="</select>\n";
    $day=~s/(\"$currentday\")/$1 SELECTED/ if length($currentday);
    #
    # Month
    my $month=<<END;
<select name="${name}_month">
<option value='$defaultmonth'> </option>
END
    my $i = 1;
    foreach (qw/January February March April May June 
	     July August September October November December /){
	$month .="<option value=\"$i\">$_</option>\n";
	$i++;
    }
    $month.="</select>\n";
    $month=~s/(\"$currentmonth\")/$1 SELECTED/ if length($currentmonth);
    #
    # Year (obviously)
    my $year=<<END;
<select name="${name}_year">
<option value='$defaultyear'> </option>
END
    my $maxyear = 2051; 
    for (my $i = 1976; $i<=$maxyear; $i++) {
	$year.="<option value=\"$i\">$i</option>\n";
    }
    $year.="</select>\n";
    $year=~s/(\"$currentyear\")/$1 SELECTED/ if length($currentyear);
    return "$month$day$year";
}

###############################################
###############################################

=pod

=item &selectbox()

Returns a scalar containing an html <select> form.  

Inputs: 

=over 4

=item $title 

Printed above the select box, in uppercase.  If undefined, only a select
box will be returned, with no additional html.

=item $name 

The name element of the <select> tag.

=item $default 

The default value of the form.  Can be $anyvalue, or in @idlist.

=item $anyvalue 

The <option value="..."> used to indicate a default of 
none of the values.  Can be undef.

=item $anytag 

The text associate with $anyvalue above.

=item $functionref 

Each element in @idlist will be passed as a parameter 
to the function referenced here.  The return value of the function should
be a scalar description of the items.  If this value is undefined the 
description of each item in @idlist will be the item name.

=item @idlist 

The items to be selected from.  One of these or $anyvalue will be the 
value returned by the form element, $ENV{form.$name}.

=back

=cut

###############################################

sub selectbox {
    my ($title,$name,$default,$anyvalue,$anytag,$functionref,@idlist)=@_;
    if (! defined($functionref)) { $functionref = sub { $_[0]}; }
    my $selout='';
    if (defined($title)) {
        my $uctitle=uc($title);
        $selout="\n".'<p><font color="#800000" face="helvetica">'.
            '<b>'.$uctitle.': </b></font>';
    }
    $selout .= '<select name="'.$name.'">';
    unshift @idlist,$anyvalue if (defined($anyvalue));
    foreach (@idlist) {
        $selout.='<option value="'.$_.'"';
        if ($_ eq $default and !/^any$/) {
	    $selout.=' selected >'.&{$functionref}($_).'</option>';
	}
	elsif ($_ eq $default and /^$anyvalue$/) {
	    $selout.=' selected >'.$anytag.'</option>';
	}
        else {$selout.='>'.&{$functionref}($_).'</option>';}
    }
    return $selout.'</select>'.(defined($title)?'</p>':' ');
}

######################################################################
#                End of HTML form building functions                 #  
######################################################################

=pod

=back 

=cut


######################################################################
######################################################################

=pod 

=item &parse_advanced_search()

Parse advanced search form and return the following:

=over 4

=item $query Scalar containing an SQL query.

=item $customquery Scalar containing a custom query.

=item $customshow Scalar containing commands to show custom metadata.

=item $libraries_to_query Reference to array of domains to search.

=back

=cut

######################################################################
######################################################################
sub parse_advanced_search {
    my ($r,$closebutton)=@_;
    my $fillflag=0;
    my $pretty_search_string = "<br />\n";
    # Clean up fields for safety
    for my $field ('title','author','subject','keywords','url','version',
		   'creationdatestart_month','creationdatestart_day',
		   'creationdatestart_year','creationdateend_month',
		   'creationdateend_day','creationdateend_year',
		   'lastrevisiondatestart_month','lastrevisiondatestart_day',
		   'lastrevisiondatestart_year','lastrevisiondateend_month',
		   'lastrevisiondateend_day','lastrevisiondateend_year',
		   'notes','abstract','extension','language','owner',
		   'custommetadata','customshow','category') {
	$ENV{"form.$field"}=~s/[^\w\/\s\(\)\=\-\"\']//g;
    }
    foreach ('mode','form','element') {
	# is this required?  Hmmm.
	next unless (exists($ENV{"form.$_"}));
	$ENV{"form.$_"}=&Apache::lonnet::unescape($ENV{"form.$_"});
	$ENV{"form.$_"}=~s/[^\w\/\s\(\)\=\-\"\']//g;
    }
    # Preprocess the category form element.
    $ENV{'form.category'} = 'any' if (! defined($ENV{'form.category'}) ||
                                      ref($ENV{'form.category'}));
    #
    # Check to see if enough information was filled in
    for my $field ('title','author','subject','keywords','url','version',
		   'notes','abstract','category','extension','language',
                   'owner','custommetadata') {
	if (&filled($ENV{"form.$field"})) {
	    $fillflag++;
	}
    }
    unless ($fillflag) {
	&output_blank_field_error($r,$closebutton,'phase=disp_adv');
	return ;
    }
    # Turn the form input into a SQL-based query
    my $query='';
    my @queries;
    my $font = '<font color="#800000" face="helvetica">';
    # Evaluate logical expression AND/OR/NOT phrase fields.
    foreach my $field ('title','author','subject','notes','abstract','url',
		       'keywords','version','owner') {
	if ($ENV{'form.'.$field}) {
            my $searchphrase = $ENV{'form.'.$field};
            $pretty_search_string .= $font."$field</font> contains <b>".
                $searchphrase."</b>";
            if ($ENV{'form.'.$field.'_related'}) {
                my @New_Words;
                ($searchphrase,@New_Words) = &related_version($searchphrase);
                if (@New_Words) {
                    $pretty_search_string .= " with related words: ".
                        "<b>@New_Words</b>.";
                } else {
                    $pretty_search_string .= " with no related words.";
                }
            }
            $pretty_search_string .= "<br />\n";
	    push @queries,&build_SQL_query($field,$searchphrase);
        }
    }
    #
    # Make the 'mime' from 'form.category' and 'form.extension'
    #
    my $searchphrase;
    if (exists($ENV{'form.category'})    && 
        $ENV{'form.category'} !~ /^\s*$/ &&
        $ENV{'form.category'} ne 'any')     {
        my @extensions = &Apache::loncommon::filecategorytypes
                                                   ($ENV{'form.category'});
        if (scalar(@extensions) > 0) {
            $searchphrase = join(' OR ',@extensions);
        }
    }
    if (exists($ENV{'form.extension'}) && $ENV{'form.extension'} !~ /^\s*$/) {
        $searchphrase .= ' OR ' if (defined($searchphrase));
        my @extensions = split(/,/,$ENV{'form.extension'});
        $searchphrase .= join(' OR ',@extensions);
    }
    if (defined($searchphrase)) {
        push @queries,&build_SQL_query('mime',$searchphrase);
        $pretty_search_string .=$font.'mime</font> contains <b>'.
            $searchphrase.'</b><br />';
    }
    #####
    # Evaluate option lists
    if ($ENV{'form.language'} and $ENV{'form.language'} ne 'any') {
	push @queries,"(language like \"$ENV{'form.language'}\")";
        $pretty_search_string.=$font."language</font>= ".
            &Apache::loncommon::languagedescription($ENV{'form.language'}).
                "<br />\n";
    }
    if ($ENV{'form.copyright'} and $ENV{'form.copyright'} ne 'any') {
	push @queries,"(copyright like \"$ENV{'form.copyright'}\")";
        $pretty_search_string.=$font."copyright</font> = ".
            &Apache::loncommon::copyrightdescription($ENV{'form.copyright'}).
                "<br \>\n";
    }
    #
    # Evaluate date windows
    my $datequery=&build_date_queries(
			$ENV{'form.creationdatestart_month'},
			$ENV{'form.creationdatestart_day'},
			$ENV{'form.creationdatestart_year'},
			$ENV{'form.creationdateend_month'},
			$ENV{'form.creationdateend_day'},
			$ENV{'form.creationdateend_year'},
			$ENV{'form.lastrevisiondatestart_month'},
			$ENV{'form.lastrevisiondatestart_day'},
			$ENV{'form.lastrevisiondatestart_year'},
			$ENV{'form.lastrevisiondateend_month'},
			$ENV{'form.lastrevisiondateend_day'},
			$ENV{'form.lastrevisiondateend_year'},
			);
    # Test to see if date windows are legitimate
    if ($datequery=~/^Incorrect/) {
	&output_date_error($r,$datequery,$closebutton);
	return ;
    } elsif ($datequery) {
        # Here is where you would set up pretty_search_string to output
        # date query information.
	push @queries,$datequery;
    }
    # Process form information for custom metadata querying
    my $customquery=undef;
#    if ($ENV{'form.custommetadata'}) {
#        $pretty_search_string .=$font."Custom Metadata Search</font>: <b>".
#            $ENV{'form.custommetadata'}."</b><br />\n";
#	$customquery=&build_custommetadata_query('custommetadata',
#				      $ENV{'form.custommetadata'});
#    }
    my $customshow=undef;
#    if ($ENV{'form.customshow'}) {
#        $pretty_search_string .=$font."Custom Metadata Display</font>: <b>".
#            $ENV{'form.customshow'}."</b><br />\n";
#	$customshow=$ENV{'form.customshow'};
#	$customshow=~s/[^\w\s]//g;
#	my @fields=split(/\s+/,$customshow);
#	$customshow=join(" ",@fields);
#    }
    ## ---------------------------------------------------------------
    ## Deal with restrictions to given domains
    ## 
    my ($libraries_to_query,$pretty_domains_string) = 
        &parse_domain_restrictions();
    $pretty_search_string .= $pretty_domains_string."<br />\n";
    #
    if (@queries) {
	$query=join(" AND ",@queries);
	$query="select * from metadata where $query";
    } elsif ($customquery) {
        $query = '';
    }
    return ($query,$customquery,$customshow,$libraries_to_query,
            $pretty_search_string);
}

sub parse_domain_restrictions {
    my $libraries_to_query = undef;
    # $ENV{'form.domains'} can be either a scalar or an array reference.
    # We need an array.
    if (! exists($ENV{'form.domains'})) {
        return (undef,'');
    }
    my @allowed_domains;
    if (ref($ENV{'form.domains'})) {
        @allowed_domains =  @{$ENV{'form.domains'}};
    } else {
        @allowed_domains = ($ENV{'form.domains'});
    }
    my %domain_hash = ();
    my $pretty_domains_string;
    foreach (@allowed_domains) {
        $domain_hash{$_}++;
    }
    if ($domain_hash{'any'}) {
        $pretty_domains_string = "In all LON-CAPA domains.";
    } else {
        if (@allowed_domains > 1) {
            $pretty_domains_string = "In LON-CAPA domains:";
        } else {
            $pretty_domains_string = "In LON-CAPA domain ";
        }
        foreach (sort @allowed_domains) {
            $pretty_domains_string .= "<b>".$_."</b> ";
        }
        foreach (keys(%Apache::lonnet::libserv)) {
            if (exists($domain_hash{$Apache::lonnet::hostdom{$_}})) {
                push @$libraries_to_query,$_;
            }
        }
    }
    return ($libraries_to_query,$pretty_domains_string);
}

######################################################################
######################################################################

=pod 

=item &parse_basic_search() 

Parse the basic search form and return a scalar containing an sql query.

=cut

######################################################################
######################################################################
sub parse_basic_search {
    my ($r,$closebutton)=@_;
    # Clean up fields for safety
    for my $field ('basicexp') {
	$ENV{"form.$field"}=~s/[^\w\s\(\)\-]//g;
    }
    foreach ('mode','form','element') {
	# is this required?  Hmmm.
	next unless (exists($ENV{"form.$_"}));
	$ENV{"form.$_"}=&Apache::lonnet::unescape($ENV{"form.$_"});
	$ENV{"form.$_"}=~s/[^\w\/\s\(\)\=\-\"\']//g;
    }
    my ($libraries_to_query,$pretty_domains_string) = 
        &parse_domain_restrictions();
    # Check to see if enough is filled in
    unless (&filled($ENV{'form.basicexp'})) {
	&output_blank_field_error($r,$closebutton,'phase=disp_basic');
	return OK;
    }
    my $pretty_search_string = '<b>'.$ENV{'form.basicexp'}.'</b>';
    my $search_string = $ENV{'form.basicexp'};
    if ($ENV{'form.related'}) {
        my @New_Words;
        ($search_string,@New_Words) = &related_version($ENV{'form.basicexp'});
        if (@New_Words) {
            $pretty_search_string .= " with related words: <b>@New_Words</b>.";
        } else {
            $pretty_search_string .= " with no related words.";
        }
    }
    # Build SQL query string based on form page
    my $query='';
    my $concatarg=join(',',
		       ('title', 'author', 'subject', 'notes', 'abstract',
                        'keywords'));
    $concatarg='title' if $ENV{'form.titleonly'};
    $query=&build_SQL_query('concat_ws(" ",'.$concatarg.')',$search_string);
    if (defined($pretty_domains_string) && $pretty_domains_string ne '') {
        $pretty_search_string .= ' '.$pretty_domains_string;
    }
    $pretty_search_string .= "<br />\n";
    my $final_query = 'SELECT * FROM metadata WHERE '.$query;
#    &Apache::lonnet::logthis($final_query);
    return ($final_query,$pretty_search_string,
            $libraries_to_query);
}


######################################################################
######################################################################

=pod 

=item &related_version

Modifies an input string to include related words.  Words in the string
are replaced with parenthesized lists of 'OR'd words.  For example
"torque" is replaced with "(torque OR word1 OR word2 OR ...)".  

Note: Using this twice on a string is probably silly.

=cut

######################################################################
######################################################################
sub related_version {
    my $search_string = shift;
    my $result = $search_string;
    my %New_Words = ();
    while ($search_string =~ /(\w+)/cg) {
        my $word = $1;
        next if (lc($word) =~ /\b(or|and|not)\b/);
        my @Words = &Apache::loncommon::get_related_words($word);
        @Words = ($#Words>4? @Words[0..4] : @Words);
        foreach (@Words) { $New_Words{$_}++;}
        my $replacement = join " OR ", ($word,@Words);
        $result =~ s/(\b)$word(\b)/$1($replacement)$2/g;
    }
    return $result,sort(keys(%New_Words));
}

######################################################################
######################################################################

=pod 

=item &build_SQL_query() 

Builds a SQL query string from a logical expression with AND/OR keywords
using Text::Query and &recursive_SQL_query_builder()

=cut

######################################################################
######################################################################
sub build_SQL_query {
    my ($field_name,$logic_statement)=@_;
    my $q=new Text::Query('abc',
			  -parse => 'Text::Query::ParseAdvanced',
			  -build => 'Text::Query::Build');
    $q->prepare($logic_statement);
    my $matchexp=${$q}{'matchexp'}; chomp $matchexp;
    my $sql_query=&recursive_SQL_query_build($field_name,$matchexp);
    return $sql_query;
}

######################################################################
######################################################################

=pod 

=item &build_custommetadata_query() 

Constructs a custom metadata query using a rather heinous regular
expression.

=cut

######################################################################
######################################################################
sub build_custommetadata_query {
    my ($field_name,$logic_statement)=@_;
    my $q=new Text::Query('abc',
			  -parse => 'Text::Query::ParseAdvanced',
			  -build => 'Text::Query::BuildAdvancedString');
    $q->prepare($logic_statement);
    my $matchexp=${$q}{'-parse'}{'-build'}{'matchstring'};
    # quick fix to change literal into xml tag-matching
    # will eventually have to write a separate builder module
    # wordone=wordtwo becomes\<wordone\>[^\<] *wordtwo[^\<]*\<\/wordone\>
    $matchexp =~ s/(\w+)\\=([\w\\\+]+)?# wordone=wordtwo is changed to 
                 /\\<$1\\>?#           \<wordone\>
                   \[\^\\<\]?#        [^\<]         
                   \*$2\[\^\\<\]?#           *wordtwo[^\<]
                   \*\\<\\\/$1\\>?#                        *\<\/wordone\>
                   /g;
    return $matchexp;
}

######################################################################
######################################################################

=pod 

=item &recursive_SQL_query_build() 

Recursively constructs an SQL query.  Takes as input $dkey and $pattern.

=cut

######################################################################
######################################################################
sub recursive_SQL_query_build {
    my ($dkey,$pattern)=@_;
    my @matches=($pattern=~/(\[[^\]|\[]*\])/g);
    return $pattern unless @matches;
    foreach my $match (@matches) {
        $match=~/\[ (\w+)\s(.*) \]/;
        my ($key,$value)=($1,$2);
        my $replacement='';
        if ($key eq 'literal') {
            $replacement="($dkey LIKE \"\%$value\%\")";
        } elsif (lc($key) eq 'not') {
            $value=~s/LIKE/NOT LIKE/;
#          $replacement="($dkey not like $value)";
            $replacement="$value";
        } elsif ($key eq 'and') {
            $value=~/(.*[\"|\)]) ([|\(|\^].*)/;
            $replacement="($1 AND $2)";
	} elsif ($key eq 'or') {
            $value=~/(.*[\"|\)]) ([|\(|\^].*)/;
            $replacement="($1 OR $2)";
	}
	substr($pattern,
               index($pattern,$match),
               length($match),
               $replacement);
    }
    &recursive_SQL_query_build($dkey,$pattern);
}

######################################################################
######################################################################

=pod 

=item &build_date_queries() 

Builds a SQL logic query to check time/date entries.
Also reports errors (check for /^Incorrect/).

=cut

######################################################################
######################################################################
sub build_date_queries {
    my ($cmonth1,$cday1,$cyear1,$cmonth2,$cday2,$cyear2,
	$lmonth1,$lday1,$lyear1,$lmonth2,$lday2,$lyear2)=@_;
    my @queries;
    if ($cmonth1 or $cday1 or $cyear1 or $cmonth2 or $cday2 or $cyear2) {
	unless ($cmonth1 and $cday1 and $cyear1 and
		$cmonth2 and $cday2 and $cyear2) {
	    return "Incorrect entry for the creation date.  You must specify ".
		   "a starting month, day, and year and an ending month, ".
		   "day, and year.";
	}
	my $cnumeric1=sprintf("%d%2d%2d",$cyear1,$cmonth1,$cday1);
	$cnumeric1+=0;
	my $cnumeric2=sprintf("%d%2d%2d",$cyear2,$cmonth2,$cday2);
	$cnumeric2+=0;
	if ($cnumeric1>$cnumeric2) {
	    return "Incorrect entry for the creation date.  The starting ".
		   "date must occur before the ending date.";
	}
	my $cquery="(creationdate BETWEEN '$cyear1-$cmonth1-$cday1' AND '".
	           "$cyear2-$cmonth2-$cday2 23:59:59')";
	push @queries,$cquery;
    }
    if ($lmonth1 or $lday1 or $lyear1 or $lmonth2 or $lday2 or $lyear2) {
	unless ($lmonth1 and $lday1 and $lyear1 and
		$lmonth2 and $lday2 and $lyear2) {
	    return "Incorrect entry for the last revision date.  You must ".
		   "specify a starting month, day, and year and an ending ".
		   "month, day, and year.";
	}
	my $lnumeric1=sprintf("%d%2d%2d",$lyear1,$lmonth1,$lday1);
	$lnumeric1+=0;
	my $lnumeric2=sprintf("%d%2d%2d",$lyear2,$lmonth2,$lday2);
	$lnumeric2+=0;
	if ($lnumeric1>$lnumeric2) {
	    return "Incorrect entry for the last revision date.  The ".
		   "starting date must occur before the ending date.";
	}
	my $lquery="(lastrevisiondate BETWEEN '$lyear1-$lmonth1-$lday1' AND '".
	           "$lyear2-$lmonth2-$lday2 23:59:59')";
	push @queries,$lquery;
    }
    if (@queries) {
	return join(" AND ",@queries);
    }
    return '';
}

######################################################################
######################################################################

=pod

=item &copyright_check()

=cut

######################################################################
######################################################################
sub copyright_check {
    my $Metadata = shift;
    # Check copyright tags and skip results the user cannot use
    my (undef,undef,$resdom,$resname) = split('/',
                                              $Metadata->{'url'});
    # Check for priv
    if (($Metadata->{'copyright'} eq 'priv') && 
        (($ENV{'user.name'} ne $resname) &&
         ($ENV{'user.domain'} ne $resdom))) {
        return 0;
    }
    # Check for domain
    if (($Metadata->{'copyright'} eq 'domain') &&
        ($ENV{'user.domain'} ne $resdom)) {
        return 0;
    }
    return 1;
}


######################################################################
######################################################################

=pod

=item &ensure_db_and_table

Ensure we can get lonmysql to connect to the database and the table we
need exists.

Inputs: $r, table id

Returns: undef on error, 1 if the table exists.

=cut

######################################################################
######################################################################
sub ensure_db_and_table {
    my ($r,$table) = @_;
    ##
    ## Sanity check the table id.
    ##
    if (! defined($table) || $table eq '' || $table =~ /\D/ ) {
        $r->print("Unable to retrieve search results.  ".
                  "Unable to determine the table results were stored in.  ".
                  "</body></html>");
        return undef;
    }
    ##
    ## Make sure we can connect and the table exists.
    ##
    my $connection_result = &Apache::lonmysql::connect_to_db();
    if (!defined($connection_result)) {
        $r->print("Unable to connect to the MySQL database where your results".
                  " are stored. </body></html>");
        &Apache::lonnet::logthis("lonsearchcat: unable to get lonmysql to".
                                 " connect to database.");
        &Apache::lonnet::logthis(&Apache::lonmysql::get_error());
        return undef;
    }
    my $table_check = &Apache::lonmysql::check_table($table);
    if (! defined($table_check)) {
        $r->print("A MySQL error has occurred.</form></body></html>");
        &Apache::lonnet::logthis("lonmysql was unable to determine the status".
                                 " of table ".$table);
        return undef;
    } elsif (! $table_check) {
        $r->print("The table of results could not be found.");
        &Apache::lonnet::logthis("The user requested a table, ".$table.
                                 ", that could not be found.");
        return undef;
    }
    return 1;
}

######################################################################
######################################################################

=pod

=item &print_sort_form

=cut

######################################################################
######################################################################
sub print_sort_form {
    my ($r,$pretty_query_string) = @_;
    ##
    my %SortableFields = 
        (id        => 'Default',
         title     => 'Title',
         author    => 'Author',
         subject   => 'Subject',
         url       => 'URL',
         version   => 'Version Number',
         mime      => 'Mime type',
         lang      => 'Language',
         owner     => 'Owner/Publisher',
         copyright => 'Copyright',
         hostname  => 'Host',
         creationdate     => 'Creation Date',
         lastrevisiondate => 'Revision Date',
     );
    ##
    my $table = $ENV{'form.table'};
    return if (! &ensure_db_and_table($r,$table));
    ##
    ## Get the number of results 
    ##
    my $total_results = &Apache::lonmysql::number_of_rows($table);
    if (! defined($total_results)) {
        $r->print("A MySQL error has occurred.</form></body></html>");
        &Apache::lonnet::logthis("lonmysql was unable to determine the number".
                                 " of rows in table ".$table);
        &Apache::lonnet::logthis(&Apache::lonmysql::get_error());
        return;
    }
    my $result;
    $result.=<<END;
<html>
<head>
<script>
    function change_sort() {
        var newloc = "/adm/searchcat?phase=results";
        newloc += "&persistent_db_id=$ENV{'form.persistent_db_id'}";
        newloc += "&sortby=";
        newloc += document.forms.statusform.elements.sortby.value;
        parent.resultsframe.location= newloc;
    }
</script>
<title>Results</title>
</head>
$bodytag
<form name="statusform" action="" method="post">
<input type="hidden" name="Queue" value="" />
END

#<h2>Sort Results</h2>
#Sort by: <select size="1" name="sortby" onchange="javascript:change_sort();">
#    $ENV{'form.sortby'} = 'id' if (! defined($ENV{'form.sortby'}));
#    foreach (keys(%SortableFields)) {
#        $result.="<option name=\"$_\"";
#        if ($_ eq $ENV{'form.sortby'}) {
#            $result.=" selected ";
#        }
#        $result.=" >$SortableFields{$_}</option>\n";
#    }
#    $result.="</select>\n";
    my $revise = &revise_button();
    $result.=<<END;
<p>
There are $total_results matches to your query. $revise
</p><p>
Search:$pretty_query_string
</p>
</form>
</body>
</html>
END
    $r->print($result);
    return;
}

#####################################################################
#####################################################################

=pod

=item MySQL Table Description

MySQL table creation requires a precise description of the data to be
stored.  The use of the correct types to hold data is vital to efficient
storage and quick retrieval of records.  The columns must be described in
the following format:

=cut

#####################################################################
#####################################################################

my @Datatypes = 
    ( { name => 'id', 
        type => 'MEDIUMINT',
        restrictions => 'UNSIGNED NOT NULL',
        primary_key  => 'yes',
        auto_inc     => 'yes' },
      { name => 'title',     type=>'TEXT'},
      { name => 'author',    type=>'TEXT'},
      { name => 'subject',   type=>'TEXT'},
      { name => 'url',       type=>'TEXT', restrictions => 'NOT NULL' },
      { name => 'keywords',  type=>'TEXT'},
      { name => 'version',   type=>'TEXT'},
      { name => 'notes',     type=>'TEXT'},
      { name => 'abstract',  type=>'TEXT'},
      { name => 'mime',      type=>'TEXT'},
      { name => 'lang',      type=>'TEXT'},
      { name => 'owner',     type=>'TEXT'},
      { name => 'copyright', type=>'TEXT'},
      { name => 'hostname',  type=>'TEXT'},
      #--------------------------------------------------
      { name => 'creationdate',     type=>'DATETIME'},
      { name => 'lastrevisiondate', type=>'DATETIME'},
      #--------------------------------------------------
      );

my @Fullindicies = 
    qw/title/;
#    qw/title author subject abstract mime language owner copyright/;
    
######################################################################
######################################################################

=pod

=item &create_results_table()

Creates the table of search results by calling lonmysql.  Stores the
table id in $ENV{'form.table'}

Inputs: none.

Returns: the identifier of the table on success, undef on error.

=cut

######################################################################
######################################################################
sub create_results_table {
    my $table = &Apache::lonmysql::create_table
        ( { columns => \@Datatypes,
            FULLTEXT => [{'columns' => \@Fullindicies},],
        } );
    if (defined($table)) {
        $ENV{'form.table'} = $table;
        return $table;
    } 
    return undef; # Error...
}

######################################################################
######################################################################

=pod

=item Search Status update functions

Each of the following functions changes the values of one of the
input fields used to display the search status to the user.  The names
should be explanatory.

Inputs: Apache request handler ($r), text to display.

Returns: Nothing.

=over 4

=item &update_count_status()

=item &update_status()

=item &update_seconds()

=back

=cut

######################################################################
######################################################################
sub update_count_status {
    my ($r,$text) = @_;
    $text =~ s/\'/\\\'/g;
    $r->print
        ("<script>document.statusform.count.value = ' $text'</script>\n");
    $r->rflush();
}

sub update_status {
    my ($r,$text) = @_;
    $text =~ s/\'/\\\'/g;
    $r->print
        ("<script>document.statusform.status.value = ' $text'</script>\n");
    $r->rflush();
}

sub update_seconds {
    my ($r,$text) = @_;
    $text =~ s/\'/\\\'/g;
    $r->print
        ("<script>document.statusform.seconds.value = ' $text'</script>\n");
    $r->rflush();
}

######################################################################
######################################################################

=pod

=item &revise_button

Inputs: None

Returns: html string for a 'revise search' button.

=cut

######################################################################
######################################################################
sub revise_button {
    my $revise_phase = 'disp_basic';
    $revise_phase = 'disp_adv' if ($ENV{'form.searchmode'} eq 'advanced');
    my $newloc = '/adm/searchcat'.
        '?persistent_db_id='.$ENV{'form.persistent_db_id'}.
            '&cleargroupsort=1'.
            '&phase='.$revise_phase;
    my $result = qq{<input type="button" value="Revise search" name="revise"} .
        qq{ onClick="parent.location='$newloc';" /> };
    return $result;
}

######################################################################
######################################################################

=pod

=item &run_search 

=cut

######################################################################
######################################################################
sub run_search {
    my ($r,$query,$customquery,$customshow,$serverlist,$pretty_string) = @_;
    my $connection = $r->connection;
    #
    # Timing variables
    #
    my $starttime = time;
    my $max_time  = 30;  # seconds for the search to complete
    #
    # Print run_search header
    #
    $r->print(<<END);
<html>
<head><title>Search Status</title></head>
$bodytag
<form name="statusform" action="" method="post">
<input type="hidden" name="Queue" value="" />
END
    # Check to see if $pretty_string has more than one carriage return.
    # Assume \n s are following <br /> s and truncate the value.
    # (there is probably a better way)...
    my @Lines = split /<br \/>/,$pretty_string;
    if (@Lines > 2) {
        $pretty_string = join '<br \>',(@Lines[0..2],'....<br />');
    }
    $r->print("Search: ".$pretty_string);
    $r->rflush();
    #
    # Determine the servers we need to contact.
    #
    my @Servers_to_contact;
    if (defined($serverlist)) {
        if (ref($serverlist) eq 'ARRAY') {
            @Servers_to_contact = @$serverlist;
        } else {
            @Servers_to_contact = ($serverlist);
        }
    } else {
        @Servers_to_contact = sort(keys(%Apache::lonnet::libserv));
    }
    my %Server_status;
    my $table =$ENV{'form.table'};
    if (! defined($table) || $table eq '' || $table =~ /\D/ ) {
        $r->print("Unable to determine table id to store search results in.".
                  "The search has been aborted.</body></html>");
        return;
    }
    my $table_status = &Apache::lonmysql::check_table($table);
    if (! defined($table_status)) {
        $r->print("Unable to determine status of table.</body></html>");
        &Apache::lonnet::logthis("Bogus table id of $table for ".
                                 "$ENV{'user.name'} @ $ENV{'user.domain'}");
        &Apache::lonnet::logthis("lonmysql error = ".
                                 &Apache::lonmysql::get_error());
        return;
    }
    if (! $table_status) {
        $r->print("The table id,$table, we tried to use is invalid.".
                  "The search has been aborted.</body></html>");
        return;
    }
    ##
    ## Prepare for the big loop.
    ##
    my $hitcountsum;
    my $server; 
    my $status;
    my $revise = &revise_button();
    $r->print(<<END);
<table>
<tr><th>Status</th><th>Total Matches</th><th>Time Remaining</th><th></th></tr>
<tr>
<td><input type="text" name="status"  value="" size="30" /></td>
<td><input type="text" name="count"   value="" size="10" /></td>
<td><input type="text" name="seconds" value="" size="8" /></td>
<td>$revise</td>
</tr>
</table>
</form>
END
    $r->rflush();
    my $time_remaining = $max_time - (time - $starttime) ;
    my $last_time = $time_remaining;
    &update_seconds($r,$time_remaining);
    &update_status($r,'contacting '.$Servers_to_contact[0]);
    while (($time_remaining > 0) &&
           ((@Servers_to_contact) || keys(%Server_status))) {
        # Send out a search request if it needs to be done.
        if (@Servers_to_contact) {
            # Contact one server
            my $server = shift(@Servers_to_contact);
            &update_status($r,'contacting '.$server);
            my $reply=&Apache::lonnet::metadata_query($query,$customquery,
                                                      $customshow,[$server]);
            ($server) = keys(%$reply);
            $Server_status{$server} = $reply->{$server};
        } else {
            # wait a sec. to give time for files to be written
            # This sleep statement is here instead of outside the else 
            # block because we do not want to pause if we have servers
            # left to contact.  
            if (scalar (keys(%Server_status))) {
                &update_status($r,
                               'waiting on '.(join(' ',keys(%Server_status))));
            }
            sleep(1); 
        }
        #
        #
        # Loop through the servers we have contacted but do not
        # have results from yet, looking for results.
        while (my ($server,$status) = each(%Server_status)) {
            last if ($connection->aborted());
            if ($status eq 'con_lost') {
                delete ($Server_status{$server});
                next;
            }
            $status=~/^([\.\w]+)$/; 
       	    my $datafile=$r->dir_config('lonDaemons').'/tmp/'.$1;
            if (-e $datafile && ! -e "$datafile.end") {
                &update_status($r,'Receiving results from '.$server);
                next;
            }
            last if ($connection->aborted());
            if (-e "$datafile.end") {
                &update_status($r,'Reading results from '.$server);
                if (-z "$datafile") {
                    delete($Server_status{$server});
                    next;
                }
                my $fh;
                if (!($fh=Apache::File->new($datafile))) { 
                    $r->print("Unable to open search results file for ".
                                  "server $server.  Omitting from search");
                    delete($Server_status{$server}); 
                   next;
                }
                # Read in the whole file.
                while (my $result = <$fh>) {
                    last if ($connection->aborted());
                    # handle custom fields?  Someday we will!
                    chomp($result);
                    next unless $result;
                    # Parse the result.
                    my %Fields = &parse_raw_result($result,$server);
                    $Fields{'hostname'} = $server;
                    next if (! &copyright_check(\%Fields));
                    # Store the result in the mysql database
                    my $result = &Apache::lonmysql::store_row($table,\%Fields);
                    if (! defined($result)) {
                        $r->print(&Apache::lonmysql::get_error());
                    }
                    # $r->print(&Apache::lonmysql::get_debug());
                    $hitcountsum ++;
                    $time_remaining = $max_time - (time - $starttime) ;
                    if ($last_time - $time_remaining > 0) {
                        &update_seconds($r,$time_remaining);
                        $last_time = $time_remaining;
                    }
                    if ($hitcountsum % 50 == 0) {
                        &update_count_status($r,$hitcountsum);
                    }
                } # End of foreach (@results)
                $fh->close();
                # $server is only deleted if the results file has been 
                # found and (successfully) opened.  This may be a bad idea.
                delete($Server_status{$server});
            }
            last if ($connection->aborted());
            &update_count_status($r,$hitcountsum);
        }
        last if ($connection->aborted());
        # Finished looping through the servers
        $starttime = time if (@Servers_to_contact);
        $time_remaining = $max_time - (time - $starttime) ;
        if ($last_time - $time_remaining > 0) {
            $last_time = $time_remaining;
            &update_seconds($r,$time_remaining);
        }
    }
    &update_status($r,'Search Complete'.$server);
    &update_seconds($r,0);
    &Apache::lonmysql::disconnect_from_db();
    # We have run out of time or run out of servers to talk to and
    # results to get.  
    $r->print("</body></html>");
    if ($ENV{'form.catalogmode'} ne 'groupsearch') {
        $r->print("<script>".
                      "window.location='/adm/searchcat?".
                      "phase=sort&".
                      "persistent_db_id=$ENV{'form.persistent_db_id'}';".
                  "</script>");
    }
    return;
}

######################################################################
######################################################################
=pod

=item &prev_next_buttons

=cut

######################################################################
######################################################################
sub prev_next_buttons {
    my ($current_min,$show,$total,$parms) = @_;
    return '' if ($show eq 'all'); # No links if you get them all at once.
    my $links;
    ##
    ## Prev
    my $prev_min = $current_min - $show;
    $prev_min = 1 if $prev_min < 1;
    if ($prev_min < $current_min) {
        $links .= qq{
<a href="/adm/searchcat?$parms&start=$prev_min&show=$show">prev</a>
};    
    } else {
        $links .= 'prev';
    }
    ##
    ## Pages.... Someday.
    ##
    $links .= qq{ &nbsp;
<a href="/adm/searchcat?$parms&start=$current_min&$show=$show">reload</a>
};
    ##
    ## Next
    my $next_min = $current_min + $show;
    $next_min = $current_min if ($next_min > $total);
    if ($next_min != $current_min) {
        $links .= qq{ &nbsp;
<a href="/adm/searchcat?$parms&start=$next_min&show=$show">next</a>
};    
    } else {
        $links .= '&nbsp;next';
    }
    return $links;
}
######################################################################
######################################################################

=pod

=item &display_results

=cut

######################################################################
######################################################################
sub display_results {
    my ($r,$importbutton,$closebutton) = @_;
    my $connection = $r->connection;
    $r->print(&search_results_header($importbutton,$closebutton));
    ##
    ## Set viewing function
    ##
    my $viewfunction = $Views{$ENV{'form.viewselect'}};
    if (!defined($viewfunction)) {
        $r->print("Internal Error - Bad view selected.\n");
        $r->rflush();
        return;
    }
    ##
    ## $checkbox_num is a count of the number of checkboxes output on the 
    ## page this is used only during catalogmode=groupsearch.
    my $checkbox_num = 0;
    ##
    ## Get the catalog controls setup
    ##
    my $action = "/adm/searchcat?phase=results";
    ##
    ## Deal with groupsearch
    ##
    if ($ENV{'form.catalogmode'} eq 'groupsearch') {
        if (! tie(%groupsearch_db,'GDBM_File',$diropendb,
                  &GDBM_WRCREAT(),0640)) {
            $r->print('Unable to store import results.</form></body></html>');
            $r->rflush();
            return;
        } 
    }
    ##
    ## Prepare the table for querying
    ##
    my $table = $ENV{'form.table'};
    return if (! &ensure_db_and_table($r,$table));
    ##
    ## Get the number of results 
    ##
    my $total_results = &Apache::lonmysql::number_of_rows($table);
    if (! defined($total_results)) {
        $r->print("A MySQL error has occurred.</form></body></html>");
        &Apache::lonnet::logthis("lonmysql was unable to determine the number".
                                 " of rows in table ".$table);
        &Apache::lonnet::logthis(&Apache::lonmysql::get_error());
        return;
    }
    ##
    ## Determine how many results we need to get
    ##
    $ENV{'form.start'} = 1      if (! exists($ENV{'form.start'}));
    $ENV{'form.show'}  = 'all'  if (! exists($ENV{'form.show'}));
    my $min = $ENV{'form.start'};
    my $max;
    if ($ENV{'form.show'} eq 'all') {
        $max = $total_results ;
    } else {
        $max = $min + $ENV{'form.show'} - 1;
        $max = $total_results if ($max > $total_results);
    }
    ##
    ## Output links (if necessary) for 'prev' and 'next' pages.
    ##
    $r->print
        ('<center>'.
         &prev_next_buttons($min,$ENV{'form.show'},$total_results,
                            "table=".$ENV{'form.table'}.
                            "&phase=results".
                            "&persistent_db_id=".$ENV{'form.persistent_db_id'})
         ."</center>\n"
         );
    if ($total_results == 0) {
        $r->print('<meta HTTP-EQUIV="Refresh" CONTENT="1">'.
                  '<h3>There are currently no results.</h3>'.
                  "</form></body></html>");
        return;
    } else {
        $r->print
            ("<center>Results $min to $max out of $total_results</center>\n");
    }
    ##
    ## Get results from MySQL table
    ##
    my @Results = &Apache::lonmysql::get_rows($table,
                                              'id>='.$min.' AND id<='.$max);
    ##
    ## Loop through the results and output them.
    ##
    foreach my $row (@Results) {
        if ($connection->aborted()) {
            &cleanup();
            return;
        }
        my %Fields = %{&parse_row(@$row)};
        my $output="<p>\n";
        my $prefix=&catalogmode_output($Fields{'title'},$Fields{'url'},
                                       $Fields{'id'},$checkbox_num++);
        # Render the result into html
        $output.= &$viewfunction($prefix,%Fields);
        # Print them out as they come in.
        $r->print($output);
        $r->rflush();
    }
    if (@Results < 1) {
        $r->print("There were no results matching your query");
    } else {
        $r->print
            ('<center>'.
             &prev_next_buttons($min,$ENV{'form.show'},$total_results,
                                "table=".$ENV{'form.table'}.
                                "&phase=results".
                                "&persistent_db_id=".
                                $ENV{'form.persistent_db_id'})
             ."</center>\n"
             );
    }
    $r->print("</form></body></html>");
    $r->rflush();
    untie %groupsearch_db if (tied(%groupsearch_db));
    return;
}

######################################################################
######################################################################

=pod

=item &catalogmode_output($title,$url,$fnum,$checkbox_num)

Returns html needed for the various catalog modes.  Gets inputs from
$ENV{'form.catalogmode'}.  Stores data in %groupsearch_db.

=cut

######################################################################
######################################################################
sub catalogmode_output {
    my $output = '';
    my ($title,$url,$fnum,$checkbox_num) = @_;
    if ($ENV{'form.catalogmode'} eq 'interactive') {
        $title=~ s/\'/\\\'/g;
        if ($ENV{'form.catalogmode'} eq 'interactive') {
            $output.=<<END 
<font size='-1'><INPUT TYPE="button" NAME="returnvalues" VALUE="SELECT"
onClick="javascript:select_data('$title','$url')">
</font>
END
        }
    } elsif ($ENV{'form.catalogmode'} eq 'groupsearch') {
        $groupsearch_db{"pre_${fnum}_link"}=$url;
        $groupsearch_db{"pre_${fnum}_title"}=$title;
        $output.=<<END;
<font size='-1'>
<input type="checkbox" name="returnvalues" value="SELECT"
onClick="javascript:queue($checkbox_num,$fnum)" />
</font>
END
    }
    return $output;
}
######################################################################
######################################################################

=pod

=item &parse_row

Parse a row returned from the database.

=cut

######################################################################
######################################################################
sub parse_row {
    my @Row = @_;
    my %Fields;
    for (my $i=0;$i<=$#Row;$i++) {
        $Fields{$Datatypes[$i]->{'name'}}=&Apache::lonnet::unescape($Row[$i]);
    }
    $Fields{'language'} = 
        &Apache::loncommon::languagedescription($Fields{'lang'});
    $Fields{'copyrighttag'} =
        &Apache::loncommon::copyrightdescription($Fields{'copyright'});
    $Fields{'mimetag'} =
        &Apache::loncommon::filedescription($Fields{'mime'});
    return \%Fields;
}

###########################################################
###########################################################

=pod

=item &parse_raw_result()

Takes a line from the file of results and parse it.  Returns a hash 
with keys for the following fields:
'title', 'author', 'subject', 'url', 'keywords', 'version', 'notes', 
'abstract', 'mime', 'lang', 'owner', 'copyright', 'creationdate', 
'lastrevisiondate'.

In addition, the following tags are set by calling the appropriate 
lonnet function: 'language', 'cprtag', 'mimetag'.

The 'title' field is set to "Untitled" if the title field is blank.

'abstract' and 'keywords' are truncated to 200 characters.

=cut

###########################################################
###########################################################
sub parse_raw_result {
    my ($result,$hostname) = @_;
    # Check for a comma - if it is there then we do not need to unescape the
    # string.  There seems to be some kind of problem with some items in
    # the database - the entire string gets sent out unescaped...?
    unless ($result =~ /,/) {
        $result = &Apache::lonnet::unescape($result);
    }
    my @fields=map {
        &Apache::lonnet::unescape($_);
    } (split(/\,/,$result));
    my ($title,$author,$subject,$url,$keywords,$version,
        $notes,$abstract,$mime,$lang,
        $creationdate,$lastrevisiondate,$owner,$copyright)=@fields;
    my %Fields = 
        ( title     => &Apache::lonnet::unescape($title),
          author    => &Apache::lonnet::unescape($author),
          subject   => &Apache::lonnet::unescape($subject),
          url       => &Apache::lonnet::unescape($url),
          keywords  => &Apache::lonnet::unescape($keywords),
          version   => &Apache::lonnet::unescape($version),
          notes     => &Apache::lonnet::unescape($notes),
          abstract  => &Apache::lonnet::unescape($abstract),
          mime      => &Apache::lonnet::unescape($mime),
          lang      => &Apache::lonnet::unescape($lang),
          owner     => &Apache::lonnet::unescape($owner),
          copyright => &Apache::lonnet::unescape($copyright),
          creationdate     => &Apache::lonnet::unescape($creationdate),
          lastrevisiondate => &Apache::lonnet::unescape($lastrevisiondate)
        );
    $Fields{'language'} = 
        &Apache::loncommon::languagedescription($Fields{'lang'});
    $Fields{'copyrighttag'} =
        &Apache::loncommon::copyrightdescription($Fields{'copyright'});
    $Fields{'mimetag'} =
        &Apache::loncommon::filedescription($Fields{'mime'});
    if ($Fields{'author'}=~/^(\s*|error)$/) {
        $Fields{'author'}="Unknown Author";
    }
    # Put spaces in the keyword list, if needed.
    $Fields{'keywords'}=~ s/,([A-z])/, $1/g; 
    if ($Fields{'title'}=~ /^\s*$/ ) { 
        $Fields{'title'}='Untitled'; 
    }
    unless ($ENV{'user.adv'}) {
        # What is this anyway?
        $Fields{'keywords'} = '- not displayed -';
        $Fields{'notes'}    = '- not displayed -';
        $Fields{'abstract'} = '- not displayed -';
        $Fields{'subject'}  = '- not displayed -';
    }
    if (length($Fields{'abstract'})>200) {
        $Fields{'abstract'} = 
            substr($Fields{'abstract'},0,200).'...';
    }
    if (length($Fields{'keywords'})>200) {
        $Fields{'keywords'} =
            substr($Fields{'keywords'},0,200).'...';
    }
    return %Fields;
}

###########################################################
###########################################################

=pod

=item &handle_custom_fields()

=cut

###########################################################
###########################################################
sub handle_custom_fields {
    my @results = @{shift()};
    my $customshow='';
    my $extrashow='';
    my @customfields;
    if ($ENV{'form.customshow'}) {
        $customshow=$ENV{'form.customshow'};
        $customshow=~s/[^\w\s]//g;
        my @fields=map {
            "<font color=\"#008000\">$_:</font><!-- $_ -->";
        } split(/\s+/,$customshow);
        @customfields=split(/\s+/,$customshow);
        if ($customshow) {
            $extrashow="<ul><li>".join("</li><li>",@fields)."</li></ul>\n";
        }
    }
    my $customdata='';
    my %customhash;
    foreach my $result (@results) {
        if ($result=~/^(custom\=.*)$/) { # grab all custom metadata
            my $tmp=$result;
            $tmp=~s/^custom\=//;
            my ($k,$v)=map {&Apache::lonnet::unescape($_);
                        } split(/\,/,$tmp);
            $customhash{$k}=$v;
        }
    }
    return ($extrashow,\@customfields,\%customhash);
}

######################################################################
######################################################################

=pod

=item &search_results_header

Output the proper html headers and javascript code to deal with different 
calling modes.

Takes most inputs directly from %ENV, except $mode.  

=over 4

=item $mode is either (at this writing) 'Basic' or 'Advanced'

=back

The following environment variables are checked:

=over 4

=item 'form.catalogmode' 

Checked for 'interactive' and 'groupsearch'.

=item 'form.mode'

Checked for existance & 'edit' mode.

=item 'form.form'

=item 'form.element'

=back

=cut

######################################################################
######################################################################
sub search_results_header {
    my ($importbutton,$closebutton) = @_;
    my $result = '';
    # output beginning of search page
    # conditional output of script functions dependent on the mode in
    # which the search was invoked
    if ($ENV{'form.catalogmode'} eq 'interactive'){
	if (! exists($ENV{'form.mode'}) || $ENV{'form.mode'} ne 'edit') {
            $result.=<<SCRIPT;
<script type="text/javascript">
    function select_data(title,url) {
	changeTitle(title);
	changeURL(url);
	parent.close();
    }
    function changeTitle(val) {
	if (parent.opener.inf.document.forms.resinfo.elements.t) {
	    parent.opener.inf.document.forms.resinfo.elements.t.value=val;
	}
    }
    function changeURL(val) {
	if (parent.opener.inf.document.forms.resinfo.elements.u) {
	    parent.opener.inf.document.forms.resinfo.elements.u.value=val;
	}
    }
</script>
SCRIPT
        } elsif ($ENV{'form.mode'} eq 'edit') {
            my $form = $ENV{'form.form'};
            my $element = $ENV{'form.element'};
            $result.=<<SCRIPT;
<script type="text/javascript">
function select_data(title,url) {
    changeURL(url);
    parent.close();
}
function changeTitle(val) {
}
function changeURL(val) {
    if (parent.targetwin.document) {
        parent.targetwin.document.forms["$form"].elements["$element"].value=val;
    } else {
	var url = 'forms[\"$form\"].elements[\"$element\"].value';
        alert("Unable to transfer data to "+url);
    }
}
</script>
SCRIPT
        }
    }
    $result.=<<SCRIPT if $ENV{'form.catalogmode'} eq 'groupsearch';
<script type="text/javascript">
    function queue(checkbox_num,val) {
        if (document.forms.results.returnvalues.length != "undefined" &&
            typeof(document.forms.results.returnvalues.length) == "number") {
            if (document.forms.results.returnvalues[checkbox_num].checked) {
                parent.statusframe.document.forms.statusform.elements.Queue.value +='1a'+val+'b';
            } else {
                parent.statusframe.document.forms.statusform.elements.Queue.value +='0a'+val+'b';
            }
        } else {
            if (document.forms.results.returnvalues.checked) {
                parent.statusframe.document.forms.statusform.elements.Queue.value +='1a'+val+'b';
            } else {
                parent.statusframe.document.forms.statusform.elements.Queue.value +='0a'+val+'b';
            }
        }
    }
    function select_group() {
	parent.window.location=
    "/adm/groupsort?mode=$ENV{'form.mode'}&catalogmode=groupsearch&acts="+
	    parent.statusframe.document.forms.statusform.elements.Queue.value;
    }
</script>
SCRIPT
    $result.=<<END;
</head>
$bodytag
<form name="results" method="post" action="" >
<input type="hidden" name="Queue" value="" />
$importbutton
END
    return $result;
}

######################################################################
######################################################################
sub search_status_header {
    return <<ENDSTATUS;
<html><head><title>Search Status</title></head>
$bodytag
<h3>Search Status</h3>
Sending search request to LON-CAPA servers.<br />
ENDSTATUS
}

sub results_link {
    my $basic_link   = "/adm/searchcat?"."&table=".$ENV{'form.table'}.
        "&persistent_db_id=".$ENV{'form.persistent_db_id'};
    my $results_link = $basic_link."&phase=results".
        "&pause=1"."&start=1";
    return $results_link;
}

######################################################################
######################################################################
sub print_frames_interface {
    my $r = shift;
    my $basic_link = "/adm/searchcat?"."&table=".$ENV{'form.table'}.
        "&persistent_db_id=".$ENV{'form.persistent_db_id'};
    my $run_search_link = $basic_link."&phase=run_search";
    my $results_link = &results_link();
    my $result = <<"ENDFRAMES";
<html>
<head>
<script>
var targetwin = opener;
var queue = '';
</script>
<title>LON-CAPA Digital Library Search Results</title>
</head>
<frameset rows="150,*">
    <frame name="statusframe"  src="$run_search_link">
    <frame name="resultsframe" src="$results_link">
</frameset>
</html>
ENDFRAMES

    $r->print($result);
    return;
}

######################################################################
######################################################################

=pod 

=item Metadata Viewing Functions

Output is a HTML-ified string.
Input arguments are title, author, subject, url, keywords, version,
notes, short abstract, mime, language, creation date,
last revision date, owner, copyright, hostname, and
extra custom metadata to show.

=over 4

=item &detailed_citation_view() 

=cut

######################################################################
######################################################################
sub detailed_citation_view {
    my ($prefix,%values) = @_;
    my $result=<<END;
<b>$prefix<a href="http://$ENV{'HTTP_HOST'}$values{'url'}" 
    target='search_preview'>$values{'title'}</a></b>
<p>
<b>$values{'author'}</b>, <i>$values{'owner'}</i><br />

<b>Subject:       </b> $values{'subject'}<br />
<b>Keyword(s):    </b> $values{'keywords'}<br />
<b>Notes:         </b> $values{'notes'}<br />
<b>MIME Type:     </b> $values{'mimetag'}<br />
<b>Language:      </b> $values{'language'}<br />
<b>Copyright/Distribution:</b> $values{'cprtag'}<br />
</p>
$values{'extrashow'}
<p>
$values{'shortabstract'}
</p>
<hr align='left' width='200' noshade />
END
    return $result;
}

######################################################################
######################################################################

=pod 

=item &summary_view() 

=cut
######################################################################
######################################################################
sub summary_view {
    my ($prefix,%values) = @_;
    my $result=<<END;
$prefix<a href="http://$ENV{'HTTP_HOST'}$values{'url'}" 
   target='search_preview'>$values{'author'}</a><br />
$values{'title'}<br />
$values{'owner'} -- $values{'lastrevisiondate'}<br />
$values{'copyrighttag'}<br />
$values{'extrashow'}
</p>
<hr align='left' width='200' noshade />
END
    return $result;
}

######################################################################
######################################################################

=pod 

=item &compact_view() 

=cut

######################################################################
######################################################################
sub compact_view {
    my ($prefix,%values) = @_;
    my $result=<<END;
$prefix <a href="http://$ENV{'HTTP_HOST'}$values{'url'}"  target='search_preview'>
$values{'title'}</a>
<b>$values{'author'}</b><br />
END
    return $result;
}


######################################################################
######################################################################

=pod 

=item &fielded_format_view() 

=cut

######################################################################
######################################################################
sub fielded_format_view {
    my ($prefix,%values) = @_;
    my $result=<<END;
$prefix
<b>URL: </b> <a href="http://$ENV{'HTTP_HOST'}$values{'url'}" 
              target='search_preview'>$values{'url'}</a>
<br />
<b>Title:</b> $values{'title'}<br />
<b>Author(s):</b> $values{'author'}<br />
<b>Subject:</b> $values{'subject'}<br />
<b>Keyword(s):</b> $values{'keywords'}<br />
<b>Notes:</b> $values{'notes'}<br />
<b>MIME Type:</b> $values{'mimetag'}<br />
<b>Language:</b> $values{'language'}<br />
<b>Creation Date:</b> $values{'creationdate'}<br />
<b>Last Revision Date:</b> $values{'lastrevisiondate'}<br />
<b>Publisher/Owner:</b> $values{'owner'}<br />
<b>Copyright/Distribution:</b> $values{'copyrighttag'}<br />
<b>Repository Location:</b> $values{'hostname'}<br />
<b>Abstract:</b> $values{'shortabstract'}<br />
$values{'extrashow'}
</p>
<hr align='left' width='200' noshade />
END
    return $result;
}

######################################################################
######################################################################

=pod 

=item &xml_sgml_view() 

=back 

=cut

######################################################################
######################################################################
sub xml_sgml_view {
    my ($prefix,%values) = @_;
    my $result=<<END;
$prefix
<pre>
&lt;LonCapaResource&gt;
&lt;url&gt;$values{'url'}&lt;/url&gt;
&lt;title&gt;$values{'title'}&lt;/title&gt;
&lt;author&gt;$values{'author'}&lt;/author&gt;
&lt;subject&gt;$values{'subject'}&lt;/subject&gt;
&lt;keywords&gt;$values{'keywords'}&lt;/keywords&gt;
&lt;notes&gt;$values{'notes'}&lt;/notes&gt;
&lt;mimeInfo&gt;
&lt;mime&gt;$values{'mime'}&lt;/mime&gt;
&lt;mimetag&gt;$values{'mimetag'}&lt;/mimetag&gt;
&lt;/mimeInfo&gt;
&lt;languageInfo&gt;
&lt;language&gt;$values{'lang'}&lt;/language&gt;
&lt;languagetag&gt;$values{'language'}&lt;/languagetag&gt;
&lt;/languageInfo&gt;
&lt;creationdate&gt;$values{'creationdate'}&lt;/creationdate&gt;
&lt;lastrevisiondate&gt;$values{'lastrevisiondate'}&lt;/lastrevisiondate&gt;
&lt;owner&gt;$values{'owner'}&lt;/owner&gt;
&lt;copyrightInfo&gt;
&lt;copyright&gt;$values{'copyright'}&lt;/copyright&gt;
&lt;copyrighttag&gt;$values{'copyrighttag'}&lt;/copyrighttag&gt;
&lt;/copyrightInfo&gt;
&lt;repositoryLocation&gt;$values{'hostname'}&lt;/repositoryLocation&gt;
&lt;shortabstract&gt;$values{'shortabstract'}&lt;/shortabstract&gt;
&lt;/LonCapaResource&gt;
</pre>
$values{'extrashow'}
<hr align='left' width='200' noshade />
END
    return $result;
}

######################################################################
######################################################################

=pod 

=item &filled() see if field is filled.

=cut

######################################################################
######################################################################
sub filled {
    my ($field)=@_;
    if ($field=~/\S/ && $field ne 'any') {
	return 1;
    }
    else {
	return 0;
    }
}

######################################################################
######################################################################

=pod 

=item &output_blank_field_error()

Output a complete page that indicates the user has not filled in enough
information to do a search.

Inputs: $r (Apache request handle), $closebutton, $parms.

Returns: nothing

$parms is extra information to include in the 'Revise search request' link.

=cut

######################################################################
######################################################################
sub output_blank_field_error {
    my ($r,$closebutton,$parms)=@_;
    # make query information persistent to allow for subsequent revision
    $r->print(<<BEGINNING);
<html>
<head>
<title>The LearningOnline Network with CAPA</title>
BEGINNING
    $r->print(<<RESULTS);
</head>
$bodytag
<img align='right' src='/adm/lonIcons/lonlogos.gif' />
<h1>Search Catalog</h1>
<form method="post" action="/adm/searchcat">
$hidden_fields
<a href="/adm/searchcat?$parms&persistent_db_id=$ENV{'form.persistent_db_id'}"
>Revise search request</a>&nbsp;
$closebutton
<hr />
<h3>Unactionable search query.</h3>
<p>
You did not fill in enough information for the search to be started.
You need to fill in relevant fields on the search page in order 
for a query to be processed.
</p>
</body>
</html>
RESULTS
}

######################################################################
######################################################################

=pod 

=item &output_date_error()

Output a full html page with an error message.

Inputs: 

    $r, the request pointer.
    $message, the error message for the user.
    $closebutton, the specialized close button needed for groupsearch.

=cut

######################################################################
######################################################################
sub output_date_error {
    my ($r,$message,$closebutton)=@_;
    # make query information persistent to allow for subsequent revision
    $r->print(<<RESULTS);
<html>
<head>
<title>The LearningOnline Network with CAPA</title>
</head>
$bodytag
<img align='right' src='/adm/lonIcons/lonlogos.gif' />
<h1>Search Catalog</h1>
<form method="post" action="/adm/searchcat">
$hidden_fields
<input type='button' value='Revise search request'
onClick='this.form.submit();' />
$closebutton
<hr />
<h3>Error</h3>
<p>
$message
</p>
</body>
</html>
RESULTS
}

######################################################################
######################################################################

=pod 

=item &start_fresh_session()

Cleans the global %groupsearch_db by removing all fields which begin with
'pre_' or 'store'.

=cut

######################################################################
######################################################################
sub start_fresh_session {
    delete $groupsearch_db{'mode_catalog'};
    foreach (keys %groupsearch_db) {
        if ($_ =~ /^pre_/) {
            delete $groupsearch_db{$_};
        }
        if ($_ =~ /^store/) {
	    delete $groupsearch_db{$_};
	}
    }
}

1;

sub cleanup {
    if (tied(%groupsearch_db)) {
	&Apache::lonnet::logthis('Cleanup searchcat: groupsearch_db');
        unless (untie(%groupsearch_db)) {
	  &Apache::lonnet::logthis('Failed cleanup searchcat: groupsearch_db');
        }
    }
    &untiehash();
    &Apache::lonmysql::disconnect_from_db();
}

__END__

=pod

=back 

=cut
