# The LearningOnline Network with CAPA
#
# $Id: lonstatistics.pm,v 1.78 2003/06/20 18:50:17 matthew Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
# (Navigate problems for statistical reports
#
###

=pod

=head1 NAME

lonstatistics

=head1 SYNOPSIS

Main handler for statistics and chart.

=head1 PACKAGES USED

    use strict;
    use Apache::Constants qw(:common :http);
    use Apache::lonnet();
    use Apache::lonhomework;
    use Apache::loncommon;
    use Apache::loncoursedata;
    use Apache::lonhtmlcommon;
    use Apache::lonproblemanalysis;
    use Apache::lonproblemstatistics;
    use Apache::lonstudentassessment;
    use Apache::lonpercentage;
    use Apache::lonmysql;
=over 4

=cut

package Apache::lonstatistics;

use strict;
use Apache::Constants qw(:common :http);
use vars qw(
    @FullClasslist 
    @Students
    @Sections 
    @SelectedSections
    %StudentData
    @StudentDataOrder
    @SelectedStudentData
    $top_map 
    @Sequences 
    @SelectedMaps
    @Assessments);

use Apache::lonnet();
use Apache::lonhomework;
use Apache::loncommon;
use Apache::loncoursedata;
use Apache::lonhtmlcommon;
use Apache::lonproblemanalysis();
use Apache::lonproblemstatistics();
use Apache::lonstudentassessment();
use Apache::lonpercentage;
use Apache::lonmysql;
use Time::HiRes;

#######################################################
#######################################################

=pod

=item Package Variables

=item @FullClasslist The full classlist

=item @Students The students we are concerned with for this invocation

=item @Sections The sections available in this class

=item $curr_student The student currently being examined

=item $prev_student The student previous in the classlist

=item $next_student The student next in the classlist

=over

=cut 

#######################################################
#######################################################
#
# Classlist variables
#
my $curr_student;
my $prev_student;
my $next_student;

#######################################################
#######################################################

=pod

=item &clear_classlist_variables()

undef the following package variables:

=over

=item @FullClasslist

=item @Students

=item @Sections

=item @SelectedSections

=item %StudentData

=item @StudentDataOrder

=item @SelectedStudentData

=item $curr_student

=item $prev_student

=item $next_student

=back

=cut

#######################################################
#######################################################
sub clear_classlist_variables {
    undef(@FullClasslist);
    undef(@Students);
    undef(@Sections);
    undef(@SelectedSections);
    undef(%StudentData);
    undef(@SelectedStudentData);
    undef($curr_student);
    undef($prev_student);
    undef($next_student);
}

#######################################################
#######################################################

=pod

=item &PrepareClasslist()

Build up the classlist information.  The classlist information is kept in
the following package variables:

=over

=item @FullClasslist

=item @Students

=item @Sections

=item @SelectedSections

=item %StudentData

=item @SelectedStudentData

=item $curr_student

=item $prev_student

=item $next_student

=back

$curr_student, $prev_student, and $next_student may not be defined, depending
upon the calling context.

=cut

#######################################################
#######################################################
sub PrepareClasslist {
    my %Sections;
    &clear_classlist_variables();
    #
    # Retrieve the classlist
    my $cid  = $ENV{'request.course.id'};
    my $cdom = $ENV{'course.'.$cid.'.domain'};
    my $cnum = $ENV{'course.'.$cid.'.num'};
    my ($classlist,$field_names) = &Apache::loncoursedata::get_classlist($cid,
                                                                  $cdom,$cnum);
    if (exists($ENV{'form.Section'})) {
        if (ref($ENV{'form.Section'})) {
            @SelectedSections = @{$ENV{'form.Section'}};
        } elsif ($ENV{'form.Section'} !~ /^\s*$/) {
            @SelectedSections = ($ENV{'form.Section'});
        }
    }
    @SelectedSections = ('all') if (! @SelectedSections);
    foreach (@SelectedSections) {
        if ($_ eq 'all') {
            @SelectedSections = ('all');
        }
    }
    #
    # Deal with instructors with restricted section access
    if ($ENV{'request.course.sec'} !~ /^\s*$/) {
        @SelectedSections = ($ENV{'request.course.sec'});
    }
    #
    # Set up %StudentData
    @StudentDataOrder = qw/fullname username domain id section status/;
    foreach my $field (@StudentDataOrder) {
        $StudentData{$field}->{'title'} = $field;
        $StudentData{$field}->{'base_width'} = length($field);
        $StudentData{$field}->{'width'} = 
                               $StudentData{$field}->{'base_width'};
    }
    #
    # get the status requested
    my $requested_status = 'Active';
    $requested_status = $ENV{'form.Status'} if (exists($ENV{'form.Status'}));
    #
    # Process the classlist
    while (my ($student,$student_data) = each (%$classlist)) {
        my $studenthash = ();
        for (my $i=0; $i< scalar(@$field_names);$i++) {
            my $field = $field_names->[$i];
            # Store the data
            $studenthash->{$field}=$student_data->[$i];
            # Keep track of the width of the fields
            next if (! exists($StudentData{$field}));
            my $length = length($student_data->[$i]);
            if ($StudentData{$field}->{'width'} < $length) {
                $StudentData{$field}->{'width'} = $length; 
            }
        }
        push (@FullClasslist,$studenthash);
        #
        # Build up a list of sections
        my $section = $studenthash->{'section'};
        if (! defined($section) || $section =~/^\s*$/ || $section == -1) {
            $studenthash->{'section'} = 'none';
            $section = $studenthash->{'section'};
        }
        $Sections{$section}++;
        #
        # Only put in the list those students we are interested in
        foreach my $sect (@SelectedSections) {
            if ( (($sect eq 'all') || 
                  ($section eq $sect)) &&
                 (($studenthash->{'status'} eq $requested_status) || 
                  ($requested_status eq 'Any')) 
                 ){
                push (@Students,$studenthash);
                last;
            }
        }
    }
    #
    # Put the consolidated section data in the right place
    if ($ENV{'request.course.sec'} !~ /^\s*$/) {
        @Sections = ($ENV{'request.course.sec'});
    } else {
        @Sections = sort {$a cmp $b} keys(%Sections);
        unshift(@Sections,'all'); # Put 'all' at the front of the list
    }
    #
    # Sort the Students
    my $sortby = 'fullname';
    $sortby = $ENV{'form.sort'} if (exists($ENV{'form.sort'}));
    my @TmpStudents = sort { $a->{$sortby} cmp $b->{$sortby} ||
                             $a->{'fullname'} cmp $b->{'fullname'} } @Students;
    @Students = @TmpStudents;
    # 
    # Now deal with that current student thing....
    $curr_student = undef;
    if (exists($ENV{'form.SelectedStudent'})) {
        my ($current_uname,$current_dom) = 
            split(':',$ENV{'form.SelectedStudent'});
        my $i;
        for ($i = 0; $i<=$#Students; $i++) {
            next if (($Students[$i]->{'username'} ne $current_uname) || 
                     ($Students[$i]->{'domain'}   ne $current_dom));
            $curr_student = $Students[$i];
            last; # If we get here, we have our student.
        }
        if (defined($curr_student)) {
            if ($i == 0) {
                $prev_student = undef;
            } else {
                $prev_student = $Students[$i-1];
            }
            if ($i == $#Students) {
                $next_student = undef;
            } else {
                $next_student = $Students[$i+1];
            }
        }
    }
    #
    if (exists($ENV{'form.StudentData'})) {
        if (ref($ENV{'form.StudentData'}) eq 'ARRAY') {
            @SelectedStudentData = @{$ENV{'form.StudentData'}};
        } else {
            @SelectedStudentData = ($ENV{'form.StudentData'});
        }
    } else {
        @SelectedStudentData = ('username');
    }
    foreach (@SelectedStudentData) {
        if ($_ eq 'all') {
            @SelectedStudentData = ('all');
            last;
        }
    }
    #
    return;
}


#######################################################
#######################################################

=pod

=item get_students

Returns a list of the selected students

=cut

#######################################################
#######################################################
sub get_students {
    if (! @Students) {
        &PrepareClasslist()
    }
    return @Students;
}

#######################################################
#######################################################

=pod

=item &current_student()

Returns a pointer to a hash containing data about the currently
selected student.

=cut

#######################################################
#######################################################
sub current_student { 
    return $curr_student;
}

#######################################################
#######################################################

=pod

=item &previous_student()

Returns a pointer to a hash containing data about the student prior
in the list of students.  Or something.  

=cut

#######################################################
#######################################################
sub previous_student { 
    return $prev_student;
}

#######################################################
#######################################################

=pod

=item &next_student()

Returns a pointer to a hash containing data about the next student
to be viewed.

=cut

#######################################################
#######################################################
sub next_student { 
    return $next_student;
}

#######################################################
#######################################################

=pod

=item &clear_sequence_variables()

=cut

#######################################################
#######################################################
sub clear_sequence_variables {
    undef($top_map);
    undef(@Sequences);
    undef(@Assessments);
}

#######################################################
#######################################################

=pod

=item &SetSelectedMaps($elementname)

Sets the @SelectedMaps array from $ENV{'form.'.$elementname};

=cut

#######################################################
#######################################################
sub SetSelectedMaps {
    my $elementname = shift;
    if (exists($ENV{'form.'.$elementname})) {
        if (ref($ENV{'form.'.$elementname})) {
            @SelectedMaps = @{$ENV{'form.'.$elementname}};
        } else {
            @SelectedMaps = ($ENV{'form.'.$elementname});
        }
    } else {
        @SelectedMaps = ('all');
    }
}


#######################################################
#######################################################

=pod

=item &Sequences_with_Assess()

Returns an array containing the subset of @Sequences which contain
assessments.

=cut

#######################################################
#######################################################
sub Sequences_with_Assess {
    my @Sequences_to_Show;
    foreach my $map_symb (@SelectedMaps) {
        foreach my $sequence (@Sequences) {
            next if ($sequence->{'symb'} ne $map_symb && $map_symb ne 'all');
            next if ($sequence->{'num_assess'} < 1);
            push (@Sequences_to_Show,$sequence);
        }
    }
    return @Sequences_to_Show;
}

#######################################################
#######################################################

=pod

=item &PrepareCourseData($r)

=cut

#######################################################
#######################################################
sub PrepareCourseData {
    my ($r) = @_;
    &clear_sequence_variables();
    my ($top,$sequences,$assessments) = 
        &Apache::loncoursedata::get_sequence_assessment_data();
    if (! defined($top) || ! ref($top)) {
        # There has been an error, better report it
        &Apache::lonnet::logthis('top is undefined');
        return;
    }
    $top_map = $top if (ref($top));
    @Sequences = @{$sequences} if (ref($sequences) eq 'ARRAY');
    @Assessments = @{$assessments} if (ref($assessments) eq 'ARRAY');
    #
    # Compute column widths
    foreach my $seq (@Sequences) {
        my $name_length = length($seq->{'title'});
        my $num_parts = $seq->{'num_assess_parts'};
        #
        # Use 3 digits for each the sum and total, which means 7 total...
        my $num_col = $num_parts+7;
        if ($num_col < $name_length) {
            $num_col = $name_length;
        }
        $seq->{'base_width'} = $name_length;
        $seq->{'width'} = $num_col;
    }
    return;
}

#######################################################
#######################################################

=pod

=item &log_sequence($sequence,$recursive,$padding)

Write data about the sequence to a logfile.  If $recursive is not
undef the data is written recursively.  $padding is used for recursive
calls.

=cut

#######################################################
#######################################################
sub log_sequence {
    my ($seq,$recursive,$padding) = @_;
    $padding = '' if (! defined($padding));
    if (ref($seq) ne 'HASH') {
        &Apache::lonnet::logthis('log_sequence passed bad sequnce');
        return;
    }
    &Apache::lonnet::logthis($padding.'sequence '.$seq->{'title'});
    while (my($key,$value) = each(%$seq)) {
        next if ($key eq 'contents');
        if (ref($value) eq 'ARRAY') {
            for (my $i=0;$i< scalar(@$value);$i++) {
                &Apache::lonnet::logthis($padding.$key.'['.$i.']='.
                                         $value->[$i]);
            }
        } else {
            &Apache::lonnet::logthis($padding.$key.'='.$value);
        }
    }
    if (defined($recursive)) {
        &Apache::lonnet::logthis($padding.'-'x20);
        &Apache::lonnet::logthis($padding.'contains:');
        foreach my $item (@{$seq->{'contents'}}) {
            if ($item->{'type'} eq 'container') {
                &log_sequence($item,$recursive,$padding.'    ');
            } else {
                &Apache::lonnet::logthis($padding.'title = '.$item->{'title'});
                while (my($key,$value) = each(%$item)) {
                    next if ($key eq 'title');
                    if (ref($value) eq 'ARRAY') {
                        for (my $i=0;$i< scalar(@$value);$i++) {
                            &Apache::lonnet::logthis($padding.$key.'['.$i.']='.
                                                     $value->[$i]);
                        }
                    } else {
                        &Apache::lonnet::logthis($padding.$key.'='.$value);
                    }
                }
            }
        }
        &Apache::lonnet::logthis($padding.'end contents of '.$seq->{'title'});
        &Apache::lonnet::logthis($padding.'-'x20);
    }
    return;
}

##############################################
##############################################

=pod 

=item &StudentDataSelect($elementname,$status,$numvisible,$selected)

Returns html for a selection box allowing the user to choose one (or more) 
of the fields of student data available (fullname, username, id, section, etc)

=over 4

=item $elementname The name of the HTML form element

=item $status 'multiple' or 'single' selection box

=item $numvisible The number of options to be visible

=back

=cut

##############################################
##############################################
sub StudentDataSelect {
    my ($elementname,$status,$numvisible)=@_;
    if ($numvisible < 1) {
        return;
    }
    #
    # Build the form element
    my $Str = "\n";
    $Str .= '<select name="'.$elementname.'" ';
    if ($status ne 'single') {
        $Str .= 'multiple="true" ';
    }
    $Str .= 'size="'.$numvisible.'" >'."\n";
    #
    # Deal with 'all'
    $Str .= '    <option value="all" ';
    foreach (@SelectedStudentData) {
        if ($_ eq 'all') {
            $Str .= 'selected ';
            last;
        }
    }
    $Str .= ">all</option>\n";
    #
    # Loop through the student data fields
    foreach my $item (@StudentDataOrder) {
        $Str .= '    <option value="'.$item.'" ';
        foreach (@SelectedStudentData) {
            if ($item eq $_ ) {
                $Str .= 'selected ';
                last;
            }
        }
        $Str .= '>'.$item."</option>\n";
    }
    $Str .= "</select>\n";
    return $Str;
}

##############################################
##############################################

=pod 

=item &MapSelect($elementname,$status,$numvisible,$restriction) 

Returns html for a selection box allowing the user to choose one (or more) 
of the sequences in the course.  The values of the sequences are the symbs.
If the top sequence is selected, the value 'top' will result.

=over 4

=item $elementname The name of the HTML form element

=item $status 'multiple' or 'single' selection box

=item $numvisible The number of options to be visible

=item $restriction Code reference to subroutine which returns true or 
false.  The code must expect a reference to a sequence data structure.

=back

=cut

##############################################
##############################################
sub MapSelect {
    my ($elementname,$status,$numvisible,$restriction)=@_;
    if ($numvisible < 1) {
        return;
    }
    #
    # Set up array of selected items
    &SetSelectedMaps($elementname);
    #
    # Set up the restriction call
    if (! defined($restriction)) {
        $restriction = sub { 1; };
    }
    #
    # Build the form element
    my $Str = "\n";
    $Str .= '<select name="'.$elementname.'" ';
    if ($status ne 'single') {
        $Str .= 'multiple="true" ';
    }
    $Str .= 'size="'.$numvisible.'" >'."\n";
    #
    # Deal with 'all'
    foreach (@SelectedMaps) {
        if ($_ eq 'all') {
            @SelectedMaps = ('all');
            last;
        }
    }
    #
    # Put in option for 'all'
    $Str .= '    <option value="all" ';
    foreach (@SelectedMaps) {
        if ($_ eq 'all') {
            $Str .= 'selected ';
            last;
        }
    }
    $Str .= ">all</option>\n";
    #
    # Loop through the sequences
    foreach my $seq (@Sequences) {
        next if (! $restriction->($seq));
        $Str .= '    <option value="'.$seq->{'symb'}.'" ';
        foreach (@SelectedMaps) {
            if ($seq->{'symb'} eq $_) {
                $Str .= 'selected ';
                last;
            }
        }
        $Str .= '>'.$seq->{'title'}."</option>\n";
    }
    $Str .= "</select>\n";
    return $Str;
}

##############################################
##############################################

=pod 

=item &SectionSelect($elementname,$status,$numvisible) 

Returns html for a selection box allowing the user to choose one (or more) 
of the sections in the course.  

Uses the package variables @Sections and @SelectedSections
=over 4

=item $elementname The name of the HTML form element

=item $status 'multiple' or 'single' selection box

=item $numvisible The number of options to be visible

=back

=cut

##############################################
##############################################
sub SectionSelect {
    my ($elementname,$status,$numvisible)=@_;
    if ($numvisible < 1) {
        return;
    }
    #
    # Make sure we have the data we need to continue
    if (! @Sections) {
        &PrepareClasslist()
    }
    #
    # Build the form element
    my $Str = "\n";
    $Str .= '<select name="'.$elementname.'" ';
    if ($status ne 'single') {
        $Str .= 'multiple="true" ';
    }
    $Str .= 'size="'.$numvisible.'" >'."\n";
    #
    # Loop through the sequences
    foreach my $s (@Sections) {
        $Str .= '    <option value="'.$s.'" ';
        foreach (@SelectedSections) {
            if ($s eq $_) {
                $Str .= 'selected ';
                last;
            }
        }
        $Str .= '>'.$s."</option>\n";
    }
    $Str .= "</select>\n";
    return $Str;
}

##################################################
##################################################
sub DisplayClasslist {
    my ($r)=@_;
    #
    my @Fields = ('fullname','username','domain','id','section');
    #
    my $Str='';
    if (! @Students) {
        if ($SelectedSections[0] eq 'all') { 
            if (lc($ENV{'form.Status'}) eq 'any') {
                $Str .= '<h2>There are no students in the course.</h2>';
            } elsif (lc($ENV{'form.Status'}) eq 'active') {
                $Str .= '<h2>There are no currently enrolled students in '.
                    'the course.</h2>';
            } elsif (lc($ENV{'form.Status'}) eq 'expired') {
                $Str .= '<h2>There are no previously enrolled '.
                    'students in the course.</h2>';
            }
        } else { 
            my $sections;
            if (@SelectedSections == 1) {
                $sections = 'section '.$SelectedSections[0];
            } elsif (@SelectedSections > 2) {
                $sections = 'sections '.join(', ',@SelectedSections);
                $sections =~ s/, ([^,])*$/, and $1/;
            } else {
                $sections = 'sections '.join(' and ',@SelectedSections);
            }
            if (lc($ENV{'form.Status'}) eq 'any') {
                $Str .= '<h2>There are no students in '.$sections.'.</h2>';
            } elsif (lc($ENV{'form.Status'}) eq 'active') {
                $Str .= '<h2>There are no currently enrolled students '.
                    'in '.$sections.'.</h2>';
            } elsif (lc($ENV{'form.Status'}) eq 'expired') {
                $Str .= '<h2>There are no previously enrolled students '.
                    'in '.$sections.'.</h2>';
            }
        }
        $Str.= '<a href="/adm/statistics?reportSelected=student_assessment">'.
            'Return to the chart.</a>';
        $r->print($Str);
        $r->rflush();
        return;
    }

    # "Click" is asinine but it is probably not my place to change the world.
    $Str .= '<h2>Click on a students name or username to view their chart</h2>';
    $Str .= '<table border="0"><tr><td bgcolor="#777777">'."\n";
    $Str .= '<table border="0" cellpadding="3"><tr bgcolor="#e6ffff">'."\n";
    foreach my $field (@Fields) {
        $Str .= '<th><a href="/adm/statistics?reportSelected=classlist&sort='.$field.'">'.$field.
            '</a></th>';
    }
    $Str .= '</tr>'."\n";
    #
    my $alternate = 0;
    foreach my $student (@Students) { # @Students is a package variable
        my $sname = $student->{'username'}.':'.$student->{'domain'};
        if($alternate) {
            $Str .= '<tr bgcolor="#ffffe6">';
        } else {
            $Str .= '<tr bgcolor="#ffffc6">';
        }
        $alternate = ($alternate + 1) % 2;
        #
        foreach my $field (@Fields) {
            $Str .= '<td>';
            if ($field eq 'fullname' || $field eq 'username') {
                $Str .= '<a href="/adm/statistics?reportSelected=';
                $Str .= &Apache::lonnet::escape('student_assessment');
                $Str .= '&sort='.&Apache::lonnet::escape($ENV{'form.sort'});
                $Str .= '&SelectedStudent=';
                $Str .= &Apache::lonnet::escape($sname).'">';
                $Str .= $student->{$field}.'&nbsp';
                $Str .= '</a>';
            } else {
                $Str .= $student->{$field};
            }
            $Str .= '</td>';
        }
        $Str .= "</tr>\n";
    }
    $Str .= '</table></td></tr></table>'."\n";
    #
    $r->print($Str);
    $r->rflush();
    #
    return;
}

##############################################
##############################################
sub CreateMainMenu {
    my ($status,$reports,$current)=@_;
    #
    my $Str = '';
    #
    $Str  = '<input type="hidden" name="reportSelected" value="'.$current.'">';
#    $Str .= '<table border="0"><tbody><tr>'."\n";
#    $Str .= '<td align="center"><b>Report:</b></td>'."\n";
#    $Str .= '<td align="center">';
#    $Str .= '<select name="reportSelected" '.
#        'onchange="document.Statistics.submit()">'."\n";
#    foreach (sort(keys(%$reports))) {
#        $Str .= '<option value="'.$_.'"';
#        if($current eq $_) {
#            $Str .= ' selected';
#        }
#        $Str .= '>'.$reports->{$_}.'</option>'."\n";
#    }
#    $Str .= '</select></td>'."\n";
#    #
#    $Str .= '<td>'.('&nbsp;'x30).'</td>';
#    $Str .= '<td align="center">'.
#        '<input type="submit" name="ClearCache" value="Clear Caches" />'.
#            "</td>\n";
#    $Str .= '</tr></tbody></table>'."\n";
#    $Str .= '<hr>'."\n";
    #
    return $Str;
}

##############################################
##############################################
sub handler {
    my $r=shift;
    my $c = $r->connection();
    #
    # Check for overloading
    my $loaderror=&Apache::lonnet::overloaderror($r);
    if ($loaderror) { return $loaderror; }
    $loaderror=
       &Apache::lonnet::overloaderror($r,
         $ENV{'course.'.$ENV{'request.course.id'}.'.home'});
    if ($loaderror) { return $loaderror; }
    #
    # Check for access
    if (! &Apache::lonnet::allowed('vgr',$ENV{'request.course.id'})) {
        $ENV{'user.error.msg'}=
            $r->uri.":vgr:0:0:Cannot view grades for complete course";
        if (! &Apache::lonnet::allowed('vgr',
                      $ENV{'request.course.id'}.'/'.$ENV{'request.course.sec'})) {
            $ENV{'user.error.msg'}=
                $r->uri.":vgr:0:0:Cannot view grades with given role";
            return HTTP_NOT_ACCEPTABLE;
        }
    }
    #
    # Set document type for header only
    if($r->header_only) {
        if ($ENV{'browser.mathml'}) {
            $r->content_type('text/xml');
        } else {
            $r->content_type('text/html');
        }
        &Apache::loncommon::no_cache($r);
        $r->send_http_header;
        return OK;
    }
    #
    # Send the header
    $r->content_type('text/html');
    $r->send_http_header;
    #
    # Extract form elements from query string
    &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},
                                            ['sort','reportSelected',
                                             'SelectedStudent']);
    if (! exists($ENV{'form.reportSelected'})) {
        $ENV{'form.reportSelected'} = 'student_assessment';
    }
    #
    # Give the LON-CAPA page header
    $r->print(&Apache::lonhtmlcommon::Title('Course Statistics and Charts'));
    $r->rflush();
    #
    if (! &Apache::lonmysql::verify_sql_connection()) {
        my $serveradmin = $r->dir_config('lonAdmEMail');
        $r->print(<<END);
<h2><font color="Red">Unable to connect to database!</font></h2>
<p>
Please notify the server administrator <b>$serveradmin</b>.
</p><p>
Course Statistics and Charts cannot be retrieved until the database is
restarted.  Your data is intact but cannot be displayed at this time.
</p>
</body>
</html>
END
        return;
    }
    #
    # Clean out the caches
    if (exists($ENV{'form.ClearCache'})) {
        &Apache::loncoursedata::delete_caches($ENV{'requres.course.id'});
    }
    #
    # Set up the statistics and chart environment
    &PrepareClasslist();
    &PrepareCourseData($r);
    #
    # Begin form output
    $r->print('<form name="Statistics" ');
    $r->print('method="post" action="/adm/statistics">');
    #
    # Print main menu
    my %reports = ('classlist'          => 'Class list',
                   'problem_statistics' => 'Problem Statistics',
                   'student_assessment' => 'Problem Status Chart',
#                   'percentage'         => 'Correct-problems Plot',
#                   'option_response'    => 'Option Response Analysis',
#                   'activitylog'        => 'Activity Log',
                   );
    $r->print(&CreateMainMenu($ENV{'form.status'},
                              \%reports,$ENV{'form.reportSelected'}));
    $r->rflush();
    #
    my $GoToPage = $ENV{'form.reportSelected'};
    if($GoToPage eq 'activitylog') {
#        &Apache::lonproblemstatistics::Activity();
    } elsif($GoToPage eq 'problem_statistics') {
        &Apache::lonproblemstatistics::BuildProblemStatisticsPage($r,$c);
    } elsif($GoToPage eq 'option_response') {
#        &Apache::lonproblemanalysis::BuildProblemAnalysisPage($r,$c);
    } elsif($GoToPage eq 'student_assessment') {
        &Apache::lonstudentassessment::BuildStudentAssessmentPage($r,$c);
    } elsif($GoToPage eq 'DoDiffGraph' || $GoToPage eq 'PercentWrongGraph') {
#        &Apache::lonproblemstatistics::BuildGraphicChart($r,$c);
    } elsif($GoToPage eq 'Correct-problems Plot') {
#	&Apache::lonpercentage::BuildPercentageGraph($r,$c);
    }
    #
    $r->print("</form>\n");
    $r->print("</body>\n</html>\n");
    $r->rflush();
    #
    return OK;
}

1;

#######################################################
#######################################################

=pod

=back

=cut

#######################################################
#######################################################

__END__

