# The LearningOnline Network
# Syllabus
#
# $Id: lonsyllabus.pm,v 1.21 2003/07/10 10:53:18 www Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#

package Apache::lonsyllabus;

use strict;
use Apache::Constants qw(:common);
use Apache::loncommon;
use Apache::lonnet;
use Apache::lontexconvert;
use Apache::lonfeedback;
use Apache::lonannounce;

sub handler {
    my $r = shift;
    $r->content_type('text/html');
    $r->send_http_header;
    return OK if $r->header_only;

# ------------------------------------------------------------ Print the screen
    $r->print(<<ENDDOCUMENT);
<html>
<head>
<title>The LearningOnline Network with CAPA</title>
ENDDOCUMENT
    my (undef,undef,$cdom,$cnum)=split(/\//,$r->uri);
# Is this even a course?
    my $homeserver=&Apache::lonnet::homeserver($cnum,$cdom); 
    if ($homeserver eq 'no_host') {
	$r->print('</head><body>No syllabus available</body></html>');
        return OK;
    }
# --------------------------------------------------------- The syllabus fields
    my %syllabusfields=(
       'aaa_instructorinfo' => 'Instructor Information',
       'bbb_description'    => 'Course Description',
       'ccc_prereq'         => 'Prerequisites',
       'cdc_classhours'     => 'Class Hours',
       'ddd_officehours'    => 'Office Hours',
       'eee_helproom'       => 'Helproom Hours',
       'efe_projectinfo'    => 'Project Information',
       'fff_examinfo'       => 'Exam Information',
       'fgf_deadlines'      => 'Deadlines',
       'ggg_grading'        => 'Grading Information',
       'hhh_readings'       => 'Readings',
       'iii_coursepack'     => 'Coursepack',
       'jjj_weblinks'       => 'Web Links',
       'kkk_textbook'       => 'Textbook',
       'lll_includeurl'     => 'URLs To Include in Syllabus');

# ------------------------------------------------------------ Get query string
    &Apache::loncommon::get_unprocessed_cgi
                        ($ENV{'QUERY_STRING'},['forcestudent','register']);
# ----------------------------------------------------- Force menu registration
    my $addentries='';
    if ($ENV{'form.register'}) {
       $addentries=' onLoad="'.&Apache::lonmenu::loadevents().
	   '" onUnload="'.&Apache::lonmenu::unloadevents().'"';
       $r->print(&Apache::lonmenu::registerurl(1));
    }
# --------------------------------------------------------------- Force Student
    my $forcestudent='';
    if ($ENV{'form.forcestudent'}) { $forcestudent='student'; };
       
# ------------------------------------- There is such a course, get environment
    my %courseenv=&Apache::lonnet::dump('environment',$cdom,$cnum);
    $r->print('</head>'.&Apache::loncommon::bodytag
       ("Syllabus",$forcestudent,$addentries,'',$cdom,$ENV{'form.register'}));
    $r->print('<h1>'.$courseenv{'description'}.'</h1><h3>'.
              $Apache::lonnet::domaindescription{$cdom}.'</h3>');
# -------------------------------------------------------------- Announcements?
    $r->print(&Apache::lonannounce::showday(time,2,
                         &Apache::lonannounce::readcalendar($cdom.'_'.$cnum)));
# -------------------------------------------------------- Get course personnel
    my %coursepersonnel=&Apache::lonnet::get_course_adv_roles($cdom.'/'.$cnum);
    $r->print('<table border="2">');
    foreach (sort keys %coursepersonnel) {
	$r->print('<tr><td>'.$_.'</td><td>');
        foreach (split(/\,/,$coursepersonnel{$_})) {
	    my ($puname,$pudom)=split(/\:/,$_);
	    $r->print(' '.&Apache::loncommon::aboutmewrapper(
                          &Apache::loncommon::plainname($puname,
                          $pudom),$puname,$pudom));
	}
        $r->print('</td></tr>');
    }
    $r->print('</table>');
# ---------------------------------------------------------- Load syllabus info
    my %syllabus=&Apache::lonnet::dump('syllabus',$cdom,$cnum);
    my $allowed=0;

# This handler might be called anonymously ...
# ----------------------------------------------------- Only if not public call
    if ($ENV{'user.environment'}) {
# does this user have privileges to post, etc?
       if ($ENV{'request.course.id'}) {
          $allowed=&Apache::lonnet::allowed('srm',$ENV{'request.course.id'});
          if ($forcestudent) { $allowed=0; }
       }
       if ($allowed) {
          $r->print('<p>'.
&Apache::loncommon::help_open_topic('Uploaded_Templates_TextBoxes','Help with filling in text boxes').'</p><p>This syllabus can be publicly viewed at <tt>http://'.
		    $Apache::lonnet::hostname{$homeserver}.$r->uri.'</tt>'.
               &Apache::loncommon::help_open_topic('Syllabus_ExtLink').'</p>'.
	  '<p><a href="'.$r->uri.'?forcestudent=1"><font size="+1">Show Public View</font></a>'.
          &Apache::loncommon::help_open_topic('Uploaded_Templates_PublicView').
          '</p>');
      }
       if (($allowed) && ($ENV{'form.storesyl'})) {
	   foreach (keys %syllabusfields) {
               my $field=$ENV{'form.'.$_};
               $field=~s/\s+$//s;
               $field=&Apache::lonfeedback::clear_out_html($field,1);
	       $syllabus{$_}=$field;
               if ($_ eq 'lll_includeurl') { # clean up included URLs
                  my $field='';
	          foreach (split(/\n/,$syllabus{$_})) {
		      my $url=$_;
# get rid of leading and trailing spaces
                      $url=~s/^\s+//;
                      $url=~s/\s+$//;
                      if ($url=~/^http\:\/\/([^\/]+)\/(.+)$/) {
                          my $remainder=$2;
# remove the hostname from internal URLs
		          foreach (keys %Apache::lonnet::hostname) {
                              if ($1=~/$Apache::lonnet::hostname{$_}/i) {
			         $url=$remainder;
			      }
		          }
		      }
# norm internal URLs
                      unless ($url=~/^http\:/) {
		          $url=&Apache::lonnet::clutter($url);
                      }
# re-assemble field
                      if ($url) {
		          $field.=$url."\n";
                      }
		  }
                  $syllabus{$_}=$field;
	      }
           }
           $syllabus{'uploaded.domain'}=$ENV{'user.domain'};
           $syllabus{'uploaded.name'}=$ENV{'user.name'};
           $syllabus{'uploaded.lastmodified'}=time;
           &Apache::lonnet::put('syllabus',\%syllabus,$cdom,$cnum);
       }
    }
# ---------------------------------------------------------------- Get syllabus
    if (($syllabus{'uploaded.lastmodified'}) || ($allowed)) {
       my $lastmod=$syllabus{'uploaded.lastmodified'};
       $lastmod=($lastmod?localtime($lastmod):'never');

       $r->print('<table><tr><td>Uploaded:</td><td>'.
 &Apache::loncommon::aboutmewrapper(
 &Apache::loncommon::plainname($syllabus{'uploaded.name'},
                     $syllabus{'uploaded.domain'}),$syllabus{'uploaded.name'},
                     $syllabus{'uploaded.domain'}).
 '</td></tr><tr><td>Last updated:</td><td>'.
                     $lastmod.
                     '</td></tr></table><p>');
       if ($allowed) {
	   $r->print('<form method="post">');
       }
       foreach (sort keys %syllabusfields) {
          if (($syllabus{$_}) || ($allowed)) {
              my $message=$syllabus{$_};
              if ($_ eq 'lll_includeurl') { # this is the "included" field
		  my $urls=$message;
                  $message='';
                  foreach my $filelink (split(/\n/,$urls)) {
		      my $output='';
# embed style?
		      my ($curfext)=($filelink=~/\.([^\.]+)$/);
                      my $embstyle=&Apache::loncommon::fileembstyle($curfext);
	              if (($embstyle eq 'ssi') || ($curfext=~/\/$/)) {
# make ssi call and remove everything but the body contents
			  $output=&Apache::lonnet::ssi_body($filelink);
	              } elsif ($embstyle eq 'img') {
# embed as an image
                         $output='<img src="'.$filelink.'" />';
		      }
                      $message.='<p>'.$output.'</p>';
                  }
                  if ($allowed) {
                     $r->print('<h3>'.$syllabusfields{$_}.
          &Apache::loncommon::help_open_topic('Syllabus_URLs').'</h3>'.
	  '<p><a href="'.$r->uri.'?forcestudent=1"><font size="+1">Show Public View</font></a>'.
 &Apache::loncommon::help_open_topic('Uploaded_Templates_PublicView').'</p>');
		 } else {
                  $r->print($message);
                 } 
              } else {
                 $message=~s/\n/\<br \/\>/g;
                 $message
           =~s/(https*\:\/\/[^\s]+)/\<a href=\"$1\"\>\<tt\>$1\<\/tt\>\<\/a\>/g;
	         $message=&Apache::lontexconvert::msgtexconverted($message);
                 $r->print('<h3>'.$syllabusfields{$_}.'</h3><blockquote>'.
                        $message.'</blockquote>');
	     }
              if ($allowed) {
                 $r->print('<br /><textarea cols="80" rows="6" name="'.$_.'">'.
			   $syllabus{$_}.
           '</textarea><input type="submit" name="storesyl" value="Store" />');
	      }
	  }
       }
       if ($allowed) {
	   $r->print('</form>');
       }
       $r->print('</p>');
    } else {
       $r->print('<p>No syllabus information provided.</p>');
    }
    $r->print('</body></html>');
    return OK;
} 

1;
__END__
