#
# $Id: classcalc.pm,v 1.10 2003/06/23 19:58:18 matthew Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
# The LearningOnline Network with CAPA
# Spreadsheet/Grades Display Handler
#
# POD required stuff:

=head1 NAME

classcalc

=head1 SYNOPSIS

=head1 DESCRIPTION

=over 4

=cut

###################################################
###                 CourseSheet                 ###
###################################################
package Apache::classcalc;

use strict;
use Apache::Constants qw(:common :http);
use Apache::loncoursedata();
use Apache::lonhtmlcommon();
use Apache::Spreadsheet;
use Apache::studentcalc;
use Apache::lonstatistics();
use HTML::Entities();
use Spreadsheet::WriteExcel;
use Apache::lonnet;
use Time::HiRes;

@Apache::classcalc::ISA = ('Apache::Spreadsheet');

##
## Package variable
##

my @Students;

sub initialize {
    &Apache::lonstatistics::clear_classlist_variables();
    @Students = &Apache::lonstatistics::get_students();
    return;
}

sub html_header {
    my $self = shift;
    my ($toprow,$bottomrow);
    &Apache::lonstatistics::clear_classlist_variables();
    foreach (['Sections',&Apache::lonstatistics::SectionSelect('Section','multpile',3)],
             ['Enrollment Status',&Apache::lonhtmlcommon::StatusOptions(undef,undef,3)],
             ['Output Format',&Apache::Spreadsheet::output_selector()]) {
        my ($name,$selector) = @{$_};
        $toprow .= '<th align="center"><b>'.$name.'</b></th>';
        $bottomrow .= '<td>'.$selector.'</td>';
    }
    return "<p>\n<table>\n".
        "<tr>".$toprow."</tr>\n".
        "<tr>".$bottomrow."</tr>\n".
        "</table>\n</p>";
}

sub get_title {
    my $self = shift;
    # Section info should be included
    my @title = ($self->{'coursedesc'}, scalar(localtime(time)) );
    return @title;
}

sub get_html_title {
    my $self = shift;
    my ($classcalc_title,$time) = $self->get_title();
    my $title = '<h1>'.$classcalc_title."</h1>\n".'<h3>'.$time."</h3>\n";
    return $title;
}

sub parent_link {
    return '';
}

sub outsheet_html {
    my $self = shift;
    my ($r) = @_;
    ###################################
    # Determine table structure
    ###################################
    my $importcolor = '#88FF88';
    my $exportcolor = '#BBBBFF';
    my $num_uneditable = 26;
    my $num_left = 52-$num_uneditable;
    my $tableheader =<<"END";
<p>
<table border="2">
<tr>
  <th colspan="2" rowspan="2"><font size="+2">Course</font></th>
  <td bgcolor="$importcolor" colspan="$num_uneditable">
      <b><font size="+1">Import</font></b></td>
  <td colspan="$num_left">
      <b><font size="+1">Calculations</font></b></td>
</tr><tr>
END
    my $label_num = 0;
    foreach (split(//,'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz')){
        if ($label_num<$num_uneditable) { 
            $tableheader.='<th bgcolor="'.$importcolor.'">';
        } else {
            $tableheader.='<th>';
        }
        $tableheader.="<b><font size=+1>$_</font></b></th>";
        $label_num++;
    }
    $tableheader.="</tr>\n";
    #
    $r->print($tableheader);
    #
    # Print out template row
    $r->print('<tr><td>Template</td><td>&nbsp;</td>'.
	      $self->html_template_row($num_uneditable,$importcolor).
              "</tr>\n");
    #
    # Print out summary/export row
    $r->print('<tr><td>Summary</td><td>0</td>'.
	      $self->html_export_row($exportcolor)."</tr>\n");
    #
    # Prepare to output rows
    $tableheader =<<"END";
</p><p>
<table border="2">
<tr><th>Row</th>
  <th>student</th><th>username</th><th>domain</th>
  <th>section</th><th>status</th>
END
    foreach (split(//,'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz')){
	if ($label_num<$num_uneditable) { 
            $tableheader.='<th bgcolor="'.$importcolor.'">';
        } else {
            $tableheader.='<th>';
        }
        $tableheader.="<b><font size=+1>$_</font></b></th>";
    }
    #
    my $num_output = 0;
    foreach my $student (@Students) {
	if ($num_output++ % 50 == 0) {
	    $r->print("</table>\n".$tableheader);
	}
	my $rownum = $self->get_row_number_from_key
	    ($student->{'username'}.':'.$student->{'domain'});
        my $link = '<a href="/adm/studentcalc?sname='.$student->{'username'}.
            '&sdomain='.$student->{'domain'}.'">';
        $student->{'section'} = 'none' if ($student->{'section'} eq '-1');
	$r->print('<tr>'.'<td>'.$rownum.'</td>'.
		  '<td>'.$student->{'fullname'}.'</td>'.
		  '<td>'.$link.$student->{'username'}.'</a></td>'.
		  '<td>'.$student->{'domain'}  .'</td>'.
		  '<td>'.$student->{'section'} .'</td>'.
		  '<td>'.$student->{'status'}  .'</td>'.
		  $self->html_row($num_uneditable,$rownum,$exportcolor,
                                  $importcolor).
                  "</tr>\n");
    }
    $r->print("</table></p>\n");
    return;
}

sub excel_rows {
    # writes the meat of the spreadsheet to an excel worksheet.  Called
    # by Spreadsheet::outsheet_excel;
    my $self = shift;
    my ($worksheet,$cols_output,$rows_output) = @_;
    #
    # Write a header row
    $cols_output = 0;
    foreach my $value ('fullname','username','domain','section','status') {
        $worksheet->write($rows_output,$cols_output++,$value);
    }
    $rows_output++;    
    #
    # Write each students row
    foreach my $student (@Students) {
        $cols_output = 0;
	my $rownum = $self->get_row_number_from_key
	    ($student->{'username'}.':'.$student->{'domain'});
        $student->{'section'} = 'none' if ($student->{'section'} eq '-1');
        my @studentdata = ($student->{'fullname'},
                           $student->{'username'},
                           $student->{'domain'},
                           $student->{'section'},
                           $student->{'status'});
        $self->excel_output_row($worksheet,$rownum,$rows_output++,
                                @studentdata);
    }
    return;
}

sub csv_rows {
    # writes the meat of the spreadsheet to an excel worksheet.  Called
    # by Spreadsheet::outsheet_excel;
    my $self = shift;
    my ($filehandle) = @_;
    #
    # Write a header row

    $self->csv_output_row($filehandle,undef,
                          ('fullname','username','domain','section','status'));
    #
    # Write each students row
    foreach my $student (@Students) {
	my $rownum = $self->get_row_number_from_key
	    ($student->{'username'}.':'.$student->{'domain'});
        $student->{'section'} = 'none' if ($student->{'section'} eq '-1');
        my @studentdata = ($student->{'fullname'},
                           $student->{'username'},
                           $student->{'domain'},
                           $student->{'section'},
                           $student->{'status'});
        $self->csv_output_row($filehandle,$rownum,@studentdata);
    }
    return;
}

sub outsheet_recursive_excel {
    my $self = shift;
    my ($r) = @_;
} 

sub compute {
    my $self = shift;
    my ($r) = @_;
    $self->initialize_safe_space();
    my %c = $self->constants();
    my %f = $self->formulas();
    my %prog_state=&Apache::lonhtmlcommon::Create_PrgWin
        ($r,'Spreadsheet Computation Status',
         'Spreadsheet Computation', scalar(@Students));
    &Apache::lonhtmlcommon::Update_PrgWin($r,\%prog_state,
                                          'Processing course structure');
    &Apache::studentcalc::initialize_package();
    &Apache::lonhtmlcommon::Update_PrgWin($r,\%prog_state,
                                          'Processing first student');
    foreach my $student (@Students) {
        my $sname = $student->{'username'}.':'.$student->{'domain'};
	my $studentsheet = Apache::studentcalc->new
	    ($student->{'username'},$student->{'domain'},undef);
	my @exportdata = $studentsheet->export_data();
	my $rownum = $self->get_row_number_from_key($sname);
        $f{'A'.$rownum} = $sname;
        $self->{'row_source'}->{$rownum} = $sname;
        $c{'A'.$rownum} = shift(@exportdata);
	foreach (split(//,'BCDEFGHIJKLMNOPQRSTUVWXYZ')) {
            my $cell = $_.$rownum;
            my $data = shift(@exportdata);
            if (defined($data)) {
                $f{$cell} = 'import';
                $c{$cell} = $data;
            }
	}
        &Apache::lonhtmlcommon::Increment_PrgWin($r,\%prog_state,
                                                 'last student');
    }
    &Apache::lonhtmlcommon::Close_PrgWin($r,\%prog_state);
    $r->rflush();
    $self->constants(\%c);
    $self->formulas(\%f);
    $self->calcsheet();
    $self->save() if ($self->need_to_save());
}

1;

__END__
