#
# $Id: lonspreadsheet.pm,v 1.20 2003/08/17 18:50:12 matthew Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
# The LearningOnline Network with CAPA
# Spreadsheet/Grades Display Handler
#
# POD required stuff:

=head1 NAME

lonspreadsheet

=head1 SYNOPSIS

Spreadsheet interface to internal LON-CAPA data

=head1 DESCRIPTION

Lonspreadsheet provides course coordinators the ability to manage their
students grades online.  The students are able to view their own grades, but
not the grades of their peers.  The spreadsheet is highly customizable,
offering the ability to use Perl code to manipulate data, as well as many
built-in functions.

=head2 Functions available to user of lonspreadsheet

=over 4

=cut


package Apache::lonspreadsheet;
            
use strict;
use Apache::classcalc();
use Apache::studentcalc();
use Apache::assesscalc();
use Apache::Constants qw(:common :http);
use Apache::lonnet;
use Apache::lonhtmlcommon;
use HTML::Entities();

##
## HTML utility subroutines really should go in lonhtmlcommon
##

sub textfield {
    my ($title,$name,$value)=@_;
    return "\n<p><b>$title:</b><br>".
        '<input type=text name="'.$name.'" size=80 value="'.$value.'">';
}

sub hiddenfield {
    my ($name,$value)=@_;
    return '<input type=hidden name="'.$name.'" value="'.$value.'" />'."\n";
}

sub selectbox {
    my ($title,$name,$value,%options)=@_;
    my $selout="\n<p><b>$title:</b><br>".'<select name="'.$name.'">';
    foreach (sort keys(%options)) {
        $selout.='<option value="'.$_.'"';
        if ($_ eq $value) { $selout.=' selected'; }
        $selout.='>'.$options{$_}.'</option>';
    }
    return $selout.'</select>';
}

sub file_dialogs {
    my ($spreadsheet) = @_;
    my $bgcolor = "#FFFFFF";
    my $sheettype = $spreadsheet->{'type'};
    my $result = '';
    my $message = '';
    ##
    ## Deal with saving the spreadsheet
    if ((exists($ENV{'form.save'}) || exists($ENV{'form.makedefault'})) && 
        exists($ENV{'form.savefilename'})) {
        $spreadsheet->filename($ENV{'form.savefilename'});
        my $save_status = $spreadsheet->save();
        if ($save_status ne 'ok') {
            $message .= "An error occurred while saving the spreadsheet".
                "There error is:".$save_status;
            return $result;
        } else {
            $message .= "Spreadsheet saved as ".$ENV{'form.savefilename'};
        }
    } elsif (exists($ENV{'form.newformula'}) && 
             exists($ENV{'form.cell'})       && 
             $ENV{'form.cell'} ne '' ) {
        ##
        ## Make any requested modifications to the spreadsheet
        $spreadsheet->modify_cell($ENV{'form.cell'},
                                  $ENV{'form.newformula'});
        $spreadsheet->save_tmp();
        # output that we are dealing with a temporary file
        $result .=&hiddenfield('workcopy',$sheettype);
        if ($ENV{'form.newformula'} !~ /^\s*$/) {
            $message .='<table><tr>'.
              '<td valign="top"><pre>Cell '.$ENV{'form.cell'}.' = </pre></td>'.
              '<td><pre>'.$ENV{'form.newformula'}."</pre></td></tr></table>\n";
        } else {
            $message .= 'Deleted contents of cell '.$ENV{'form.cell'}.'.';
        }
    }
    ##
    ## Editing code
    $result .=&hiddenfield('cell','').
              &hiddenfield('newformula','');
    ##
    ## Create the save and load dialogs
    my $filename = $spreadsheet->filename();
    my $truefilename = $filename;
    if ($spreadsheet->is_default()) {
        $filename = 'Default';
    }
    my $save_dialog = '<nobr>'.
        '<input type="submit" name="save" value="Save as" /> '.
        '<input type="text" name="savefilename" size="30" value="'.
        $truefilename.'" />'.
        '</nobr>';
    my $makedefault_dialog = '<input type="submit" name="makedefault" '.
        'value="Save as & Make This Sheet the Default"/>';
    #
    my $link = '<a href="javascript:openbrowser'.
        "('sheet','loadfilename','spreadsheet')\">Select Spreadsheet File</a>";
    my $load_dialog = <<END;
<table bgcolor="$bgcolor">
<tr><td><input type="submit" name="load" value="Load" /></td>
    <td><nobr>
        <input type="text" name="loadfilename" size="20" value="$filename" />
        $link</nobr>
    </td></tr>
<tr><td>&nbsp;</td><td>
    <select name="fileselect" onchange="document.sheet.loadfilename.value=document.sheet.fileselect.value" >
END
    my $default_filename_set = 0;
    foreach my $sheetfilename ($spreadsheet->othersheets()) {
        $load_dialog .= '    <option name="'.$sheetfilename.'"';
        if ($filename eq $sheetfilename) {
            $load_dialog .= ' selected';
            $default_filename_set = 1;
        }
        $load_dialog .= '>'.$sheetfilename."</option>\n";
    }
    $load_dialog .= "</td><td>&nbsp;</td></tr>\n</table>\n";
        #
    $result .=<<END;
<!-- load / save dialogs -->
<table cellspacing="2">
<tr>
    <td>$load_dialog</td>
    <td>
        <table bgcolor="$bgcolor">
        <tr><td>$save_dialog</td></tr>
        <tr><td align="center">$makedefault_dialog</td></tr>
        </table>
    </td>
</tr>
</table>
END
    return ($result,$message);
}

sub handler {
    my $r=shift;
    #
    # Overload checking
    #
    # Check this server
    my $loaderror=&Apache::lonnet::overloaderror($r);
    if ($loaderror) { return $loaderror; }
    # Check the course homeserver
    $loaderror= &Apache::lonnet::overloaderror($r,
                      $ENV{'course.'.$ENV{'request.course.id'}.'.home'});
#    if ($loaderror) { return $loaderror; } 
    #
    # HTML Header
    #
    if ($r->header_only) {
        $r->content_type('text/html');
        $r->send_http_header;
        return OK;
    }
    #
    # Roles Checking
    #
    # Needs to be in a course
    if (! $ENV{'request.course.fn'}) { 
        # Not in a course, or not allowed to modify parms
        $ENV{'user.error.msg'}=
            $r->uri.":opa:0:0:Cannot modify spreadsheet";
        return HTTP_NOT_ACCEPTABLE; 
    }
    my $courseid = $ENV{'request.course.id'};
    #
    # Do not allow students to continue if standard grading is in effect.
    if ($ENV{'request.role'} =~ /^st\./) {
        if ($ENV{'course.'.$courseid.'.grading'} eq 'standard') {
            return HTTP_NOT_ACCEPTABLE;
        }
    }
    #
    # Get query string for limited number of parameters
    #
    &Apache::loncommon::get_unprocessed_cgi
        ($ENV{'QUERY_STRING'},['sname','sdomain','usymb','filename','recalc']);
    #
    # Deal with restricted student permissions 
    #
    if ($ENV{'request.role'} =~ /^st\./) {
        delete $ENV{'form.cell'}       if (exists($ENV{'form.cell'}));
        delete $ENV{'form.newformula'} if (exists($ENV{'form.newformula'}));
    }
    #
    # Determine basic information about the spreadsheet
    my ($sheettype) = ($r->uri=~/\/(\w+)$/);
    #
    my $symb   = undef;
    $symb = $ENV{'form.usymb'} if (exists($ENV{'form.usymb'}));
    my $name   = $ENV{'user.name'};
    my $domain = $ENV{'user.domain'};
    if (exists($ENV{'form.sname'})) {
        $name   = $ENV{'form.sname'};
        $domain = $ENV{'form.sdomain'};
    }
    ##
    ## Check permissions
    my $allowed_to_edit = &Apache::lonnet::allowed('mgr',
                                                $ENV{'request.course.id'});
    # Only those instructors/tas/whatevers with complete access
    # (not section restricted) are able to modify spreadsheets.
    my $allowed_to_view =  &Apache::lonnet::allowed('vgr',
                                                $ENV{'request.course.id'});
    if (! $allowed_to_view) {
        $allowed_to_view = &Apache::lonnet::allowed('vgr',
                    $ENV{'request.course.id'}.'/'.$ENV{'request.course.sec'});
        # Those who are restricted by section are allowed to view.
        # The routines in lonstatistics which decide which students' 
        # will be shown take care of the restriction by section.
    }
    #
    # Only those able to view others grades will be allowed to continue 
    # if they are not requesting their own.
    if ($sheettype eq 'classcalc') {
        if (! $allowed_to_view) {
	    $ENV{'user.error.msg'}=
		$r->uri.":vgr:0:0:Access Permission Denied";
	    return HTTP_NOT_ACCEPTABLE; 
	}
    }
    if ((($name   ne $ENV{'user.name'} ) ||
         ($domain ne $ENV{'user.domain'})) && $sheettype ne 'classcalc') {
        # Check that the student is in their section?
        if (exists($ENV{'request.course.sec'}) && 
            $ENV{'request.course.sec'} ne '' ) {
            my $stu_sec = &Apache::lonnet::usection($domain,$name,
                                                    $ENV{'request.course.id'});
            if ($stu_sec ne $ENV{'request.course.sec'}) {
		$ENV{'user.error.msg'}=
		    $r->uri.":vgr:0:0:Requested student not in your section.";
		return HTTP_NOT_ACCEPTABLE; 
            }
        }
    }

    #
    # Open page, try to prevent browser cache.
    #
    $r->content_type('text/html');
    &Apache::loncommon::no_cache($r);
    $r->send_http_header;

    #
    # Header....
    #
    $r->print('<html><head><title>LON-CAPA Spreadsheet</title>');
    my $nothing = &Apache::lonhtmlcommon::javascript_nothing();
    ##
    ## Spit out the javascript required for editing
    ##
    if ($allowed_to_edit) {
        my $extra_javascript = 
            &Apache::loncommon::browser_and_searcher_javascript();
        $r->print(<<ENDSCRIPT);
<script language="JavaScript">

    $extra_javascript

    var editwin;

    function celledit(cellname,cellformula) {
        var edit_text = '';
        // cellformula may contain less-than and greater-than symbols, so
        // we need to escape them?  
        edit_text +='<html><head><title>Cell Edit Window</title></head><body>';
        edit_text += '<form name="editwinform">';
        edit_text += '<center><h3>Cell '+cellname+'</h3>';
        edit_text += '<textarea name="newformula" cols="60" rows="12"';
        edit_text += ' wrap="off" >'+cellformula+'</textarea>';
        edit_text += '</br>';
        edit_text += '<input type="button" name="accept" value="Accept"';
        edit_text += ' onClick=\\\'javascript:';
        edit_text += 'opener.document.sheet.cell.value=';
        edit_text +=     '"'+cellname+'";';
        edit_text += 'opener.document.sheet.newformula.value=';
        edit_text +=     'document.editwinform.newformula.value;';
        edit_text += 'opener.document.sheet.submit();';
        edit_text += 'self.close()\\\' />';
        edit_text += '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';
        edit_text += '<input type="button" name="abort" ';
        edit_text +=     'value="Discard Changes"';
        edit_text += ' onClick="javascript:self.close()" />';
        edit_text += '</center></body></html>';

        if (editwin != null && !(editwin.closed) ) {
            editwin.close();
        }

        editwin = window.open($nothing,'CellEditWin','height=280,width=480,scrollbars=no,resizeable=yes,alwaysRaised=yes,dependent=yes',true);
        editwin.document.write(edit_text);
    }
</script>
ENDSCRIPT
    }
    $r->print('</head>'.&Apache::loncommon::bodytag('Grades Spreadsheet').
              '<form action="'.$r->uri.'" name="sheet" method="post">');
    $r->print(&hiddenfield('sname'  ,$ENV{'form.sname'}).
              &hiddenfield('sdomain',$ENV{'form.sdomain'}).
              &hiddenfield('usymb'  ,$ENV{'form.usymb'}));
    $r->rflush();
    ##
    ## Determine the filename to use
    my $filename = undef;
    if ($allowed_to_edit) {
        $filename = $ENV{'form.filename'} if (exists($ENV{'form.filename'}));
        #
        if (exists($ENV{'form.load'}) && exists($ENV{'form.loadfilename'})) {
            $filename = $ENV{'form.loadfilename'};
            $ENV{'form.workcopy'} = 'no';
        }
    }
    ##
    ## Take care of "backdoor" spreadsheet expiration / recalc stuff
    if ($allowed_to_edit && exists($ENV{'form.recalc'})) {
        if ($ENV{'form.recalc'} eq 'ilovewastingtime') {
            &Apache::lonnet::logthis('ilovewastingtime');
            # expire ALL spreadsheets
            &Apache::lonnet::expirespread('','','studentcalc');
            &Apache::lonnet::expirespread('','','assesscalc');
        } elsif ($ENV{'form.recalc'} =~ /^symb:/) {
            # expire for all students on this symb
            my ($symb) = ($ENV{'form.recalc'} =~ /^symb:(.*)$/);
            &Apache::lonnet::logthis('symb = '.$symb);
            &Apache::lonnet::expirespread('','','assesscalc',$symb);
            &Apache::lonnet::expirespread('','','studentcalc');
        } elsif ($ENV{'form.recalc'} =~ /^student:/) {
            # expire all assessment spreadsheets for this user
            my ($sname,$sdom) = ($ENV{'form.recalc'}=~/^student:(.*):(.*)$/);
            &Apache::lonnet::logthis('student = '.$sname.':'.$sdom);
            if (defined($sname) && defined($sdom)) {
                &Apache::lonnet::expirespread($sname,$sdom,'assesscalc');
                &Apache::lonnet::expirespread($sname,$sdom,'studentcalc');
            }
        }
    }
    ##
    ## Make the spreadsheet
    &Apache::Spreadsheet::initialize_spreadsheet_package();
    my $spreadsheet = undef;
    if ($sheettype eq 'classcalc') {
        $spreadsheet = Apache::classcalc->new($name,$domain,$filename,undef);
    } elsif ($sheettype eq 'studentcalc') {
        $spreadsheet = Apache::studentcalc->new($name,$domain,$filename,undef);
    } elsif ($sheettype eq 'assesscalc' && 
             defined($symb) && 
             $allowed_to_edit) {
        $spreadsheet = Apache::assesscalc->new($name,$domain,$filename,$symb);
    } else {
        return HTTP_NOT_ACCEPTABLE;
    }
    if (! defined($spreadsheet)) {
        # error error - run in circles, scream and shout
        return;
    }
    $spreadsheet->initialize();
    #
    # Output selector
    ##
    ## Editing/loading/saving
    if ($allowed_to_edit) {
        my ($html,$action_message) = &file_dialogs($spreadsheet);
        if ($ENV{'form.makedefault'}) {
            $spreadsheet->make_default();
            if ($action_message) {
                $action_message .= '<br />';
            }
            $action_message .= 'Made this spreadsheet the default';
            if ($sheettype eq 'classcalc') {
                $action_message .= ' for the course';
            } elsif ($sheettype eq 'studentcalc') {
                $action_message .= ' for all students';
            } elsif ($sheettype eq 'assesscalc') {
                $action_message .= ' for all assessments';
            }
            $action_message .= '.';
        }
        $r->print('<table><tr><td>'.$spreadsheet->html_header().'</td>'.
                  '<td valign="bottom">'.$html."</td></tr></table>\n");
        if ($action_message ne '') {
            $r->print(<<END);
<table>
<tr><td valign="top"><b>Last Action:</b></td>
    <td>&nbsp;</td>
    <td>$action_message</td>
</tr>
</table>
END
        }
        $r->rflush();
    } else {
        $r->print('<table><tr><td>'.$spreadsheet->html_header().
                  "</td></tr></table>\n");
    }
    $r->rflush();
    #
    $r->print("<table><tr>");
    if ($sheettype eq 'classcalc') {
        $r->print('<td><input type="submit" value="Generate Spreadsheet" />'.
                  '</td>');
    }
#    if ($allowed_to_view) {
#        $r->print('<td>'.
#                  &Apache::loncommon::help_open_topic("Spreadsheet_About",
#                                                      'Spreadsheet Help').
#                  '</td>');
#    }
#    if ($allowed_to_edit) {
#        $r->print('<td>'.
#                  &Apache::loncommon::help_open_topic("Spreadsheet_Editing",
#                                                      'Editing Help').
#                  '</td>');
#    }
    $r->print('</tr></table>');
    #
    # Keep track of the filename
    $r->print(&hiddenfield('filename',$filename));
    #
    # Keep track of the number of times we have been called, sort of.
    $r->print(&hiddenfield('not_first_run','whatever'));
    #
    if (exists($ENV{'form.not_first_run'}) || $sheettype ne 'classcalc') {
        $r->print($spreadsheet->get_html_title());
        if ($allowed_to_view || $allowed_to_edit) {
            $r->print($spreadsheet->parent_link());
        }
        $r->rflush();
        $spreadsheet->display($r);
    }
    $r->print('</form></body></html>');
    return OK;
}

1;

__END__

