# The LearningOnline Network with CAPA
#
# $Id: lonstudentassessment.pm,v 1.64 2003/06/24 21:44:09 albertel Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
# (Navigate problems for statistical reports
#
#######################################################
#######################################################

=pod

=head1 NAME

lonstudentassessment

=head1 SYNOPSIS

Presents assessment data about a student or a group of students.

=head1 Subroutines

=over 4 

=cut

#######################################################
#######################################################

package Apache::lonstudentassessment;

use strict;
use Apache::lonstatistics;
use Apache::lonhtmlcommon;
use Apache::loncoursedata;
use Apache::lonnet; # for logging porpoises
use Spreadsheet::WriteExcel;

#######################################################
#######################################################
=pod

=item Package Variables

=over 4

=item $Statistics Hash ref to store student data.  Indexed by symb,
      contains hashes with keys 'score' and 'max'.

=cut

#######################################################
#######################################################

my $Statistics;

#######################################################
#######################################################

=pod

=item $show_links 'yes' or 'no' for linking to student performance data

=item $output_mode 'html', 'excel', or 'csv' for output mode

=item $show 'all', 'totals', or 'scores' determines how much data is output

=item $data  determines what performance data is shown

=item $datadescription A short description of the output data selected.

=item $base 'tries' or 'scores' determines the base of the performance shown

=item $single_student_mode evaluates to true if we are showing only one
student.

=cut

#######################################################
#######################################################
my $show_links;
my $output_mode;
my $data;
my $base;
my $datadescription;
my $single_student_mode;

#######################################################
#######################################################
# End of package variable declarations

=pod

=back

=cut

#######################################################
#######################################################

=pod

=item &BuildStudentAssessmentPage()

Inputs: 

=over 4

=item $r Apache Request

=item $c Apache Connection 

=back

=cut

#######################################################
#######################################################
sub BuildStudentAssessmentPage {
    my ($r,$c)=@_;
    undef($Statistics);
    $single_student_mode = 1 if ($ENV{'form.SelectedStudent'});
    if ($ENV{'form.selectstudent'}) {
        &Apache::lonstatistics::DisplayClasslist($r);
        return;
    }
    #
    # Print out the HTML headers for the interface
    #    This also parses the output mode selector
    #    This step must *always* be done.
    $r->print(&CreateInterface());
    $r->print('<input type="hidden" name="notfirstrun" value="true" />');
    $r->print('<input type="hidden" name="sort" value="'.
              $ENV{'form.sort'}.'" />');
    $r->rflush();
    #
    if (! exists($ENV{'form.notfirstrun'}) && ! $single_student_mode) {
        return;
    }
    #
    my $initialize     = \&html_initialize;
    my $output_student = \&html_outputstudent;
    my $finish         = \&html_finish;
    #
    if ($output_mode eq 'excel') {
        $initialize     = \&excel_initialize;
        $output_student = \&excel_outputstudent;
        $finish         = \&excel_finish;
    } elsif ($output_mode eq 'csv') {
        $initialize     = \&csv_initialize;
        $output_student = \&csv_outputstudent;
        $finish         = \&csv_finish;
    }
    #
    if($c->aborted()) {  return ; }
    #
    # Determine which students we want to look at
    my @Students;
    if ($single_student_mode) {
        @Students = (&Apache::lonstatistics::current_student());
        $r->print(&next_and_previous_buttons());
        $r->rflush();
    } else {
        @Students = @Apache::lonstatistics::Students;
    }
    #
    # Perform generic initialization tasks
    #       Since we use lonnet::EXT to retrieve problem weights,
    #       to ensure current data we must clear the caches out.
    #       This makes sure that parameter changes at the student level
    #       are immediately reflected in the chart.
    &Apache::lonnet::clear_EXT_cache_status();
    #
    # Call the initialize routine selected above
    $initialize->($r);
    foreach my $student (@Students) {
        if($c->aborted()) { 
            $finish->($r);
            return ; 
        }
        # Call the output_student routine selected above
        $output_student->($r,$student);
    }
    # Call the "finish" routine selected above
    $finish->($r);
    #
    return;
}

#######################################################
#######################################################
sub next_and_previous_buttons {
    my $Str = '';
    $Str .= '<input type="hidden" name="SelectedStudent" value="'.
        $ENV{'form.SelectedStudent'}.'" />';
    #
    # Build the previous student link
    my $previous = &Apache::lonstatistics::previous_student();
    my $previousbutton = '';
    if (defined($previous)) {
        my $sname = $previous->{'username'}.':'.$previous->{'domain'};
        $previousbutton .= '<input type="button" value="'.
            'Previous Student ('.
            $previous->{'username'}.'@'.$previous->{'domain'}.')'.
            '" onclick="document.Statistics.SelectedStudent.value='.
            "'".$sname."'".';'.
            'document.Statistics.submit();" />';
    } else {
        $previousbutton .= '<input type="button" value="'.
            'Previous student (none)'.'" />';
    }
    #
    # Build the next student link
    my $next = &Apache::lonstatistics::next_student();
    my $nextbutton = '';
    if (defined($next)) {
        my $sname = $next->{'username'}.':'.$next->{'domain'};
        $nextbutton .= '<input type="button" value="'.
            'Next Student ('.
            $next->{'username'}.'@'.$next->{'domain'}.')'.
            '" onclick="document.Statistics.SelectedStudent.value='.
            "'".$sname."'".';'.
            'document.Statistics.submit();" />';
    } else {
        $nextbutton .= '<input type="button" value="'.
            'Next student (none)'.'" />';
    }
    #
    # Build the 'all students' button
    my $all = '';
    $all .= '<input type="button" value="All Students" '.
            '" onclick="document.Statistics.SelectedStudent.value='.
            "''".';'.'document.Statistics.submit();" />';
    $Str .= $previousbutton.('&nbsp;'x5).$all.('&nbsp;'x5).$nextbutton;
    return $Str;
}

#######################################################
#######################################################

sub get_student_fields_to_show {
    my @to_show = @Apache::lonstatistics::SelectedStudentData;
    foreach (@to_show) {
        if ($_ eq 'all') {
            @to_show = @Apache::lonstatistics::StudentDataOrder;
            last;
        }
    }
    return @to_show;
}

#######################################################
#######################################################

=pod

=item &CreateInterface()

Called by &BuildStudentAssessmentPage to create the top part of the
page which displays the chart.

Inputs: None

Returns:  A string containing the HTML for the headers and top table for 
the chart page.

=cut

#######################################################
#######################################################
sub CreateInterface {
    my $Str = '';
#    $Str .= &CreateLegend();
    $Str .= '<table cellspacing="5">'."\n";
    $Str .= '<tr>';
    $Str .= '<td align="center"><b>Sections</b></td>';
    $Str .= '<td align="center"><b>Student Data</b></td>';
    $Str .= '<td align="center"><b>Enrollment Status</b></td>';
    $Str .= '<td align="center"><b>Sequences and Folders</b></td>';
    $Str .= '<td align="center"><b>Output Format</b>'.
        &Apache::loncommon::help_open_topic("Chart_Output_Formats").
        '</td>';
    $Str .= '<td align="center"><b>Output Data</b>'.
        &Apache::loncommon::help_open_topic("Chart_Output_Data").
        '</td>';
    $Str .= '</tr>'."\n";
    #
    $Str .= '<tr><td align="center">'."\n";
    $Str .= &Apache::lonstatistics::SectionSelect('Section','multiple',5);
    $Str .= '</td><td align="center">';
    my $only_seq_with_assessments = sub { 
        my $s=shift;
        if ($s->{'num_assess'} < 1) { 
            return 0;
        } else { 
            return 1;
        }
    };
    $Str .= &Apache::lonstatistics::StudentDataSelect('StudentData','multiple',
                                                      5,undef);
    $Str .= '</td><td>'."\n";
    $Str .= &Apache::lonhtmlcommon::StatusOptions(undef,undef,5);
    $Str .= '</td><td>'."\n";
    $Str .= &Apache::lonstatistics::MapSelect('Maps','multiple,all',5,
                                              $only_seq_with_assessments);
    $Str .= '</td><td>'."\n";
    $Str .= &CreateAndParseOutputSelector();
    $Str .= '</td><td>'."\n";
    $Str .= &CreateAndParseOutputDataSelector();
    $Str .= '</td></tr>'."\n";
    $Str .= '</table>'."\n";
    $Str .= '<input type="submit" value="Generate Chart" />';
    $Str .= '&nbsp;'x5;
    $Str .= '<input type="submit" name="selectstudent" '.
                                  'value="Select One Student" />';
    $Str .= '&nbsp;'x5;
    $Str .= '<input type="submit" name="ClearCache" value="Clear Caches" />';
    $Str .= '&nbsp;'x5;
    $Str .= '<br />';
    return $Str;
}

#######################################################
#######################################################

=pod

=item &CreateAndParseOutputSelector()

=cut

#######################################################
#######################################################
my @OutputOptions = 
    ({ name  => 'HTML, with links',
       value => 'html, with links',
       description => 'Output HTML with each symbol linked to the problem '.
	   'which generated it.',
       mode => 'html',
       show_links => 'yes',
       },
     { name  => 'HTML, with all links',
       value => 'html, with all links',
       description => 'Output HTML with each symbol linked to the problem '.
	   'which generated it.  '.
           'This includes links for unattempted problems.',
       mode => 'html',
       show_links => 'all',
       },
     { name  => 'HTML, without links',
       value => 'html, without links',
       description => 'Output HTML.  By not including links, the size of the'.
	   ' web page is greatly reduced.  If your browser crashes on the '.
	   'full display, try this.',
       mode => 'html',
       show_links => 'no',
           },
     { name  => 'Excel',
       value => 'excel',
       description => 'Output an Excel file (compatable with Excel 95).',
       mode => 'excel',
       show_links => 'no',
   },
     { name  => 'CSV',
       value => 'csv',
       description => 'Output a comma seperated values file suitable for '.
           'import into a spreadsheet program.  Using this method as opposed '.
           'to Excel output allows you to organize your data before importing'.
           ' it into a spreadsheet program.',
       mode => 'csv',
       show_links => 'no',
           },
     );

sub OutputDescriptions {
    my $Str = '';
    $Str .= "<h2>Output Formats</h2>\n";
    $Str .= "<dl>\n";
    foreach my $outputmode (@OutputOptions) {
	$Str .="    <dt>".$outputmode->{'name'}."</dt>\n";
	$Str .="        <dd>".$outputmode->{'description'}."</dd>\n";
    }
    $Str .= "</dl>\n";
    return $Str;
}

sub CreateAndParseOutputSelector {
    my $Str = '';
    my $elementname = 'chartoutputmode';
    #
    # Format for output options is 'mode, restrictions';
    my $selected = 'html, without links';
    if (exists($ENV{'form.'.$elementname})) {
        if (ref($ENV{'form.'.$elementname} eq 'ARRAY')) {
            $selected = $ENV{'form.'.$elementname}->[0];
        } else {
            $selected = $ENV{'form.'.$elementname};
        }
    }
    #
    # Set package variables describing output mode
    $show_links  = 'no';
    $output_mode = 'html';
    foreach my $option (@OutputOptions) {
        next if ($option->{'value'} ne $selected);
        $output_mode = $option->{'mode'};
        $show_links  = $option->{'show_links'};
    }

    #
    # Build the form element
    $Str = qq/<select size="5" name="$elementname">/;
    foreach my $option (@OutputOptions) {
        $Str .= "\n".'    <option value="'.$option->{'value'}.'"';
        $Str .= " selected " if ($option->{'value'} eq $selected);
        $Str .= ">".$option->{'name'}."<\/option>";
    }
    $Str .= "\n</select>";
    return $Str;
}

##
## Data selector stuff
##
my @OutputDataOptions =
    (
     { name  => 'Scores',
       base  => 'scores',
       value => 'scores',
       shortdesc => 'Score on each Problem Part',
       longdesc =>'The students score on each problem part, computed as'.
           'the part weight * part awarded',
       },
     { name  => 'Scores Sum',
       base  => 'scores',
       value => 'sum only',
       shortdesc => 'Sum of Scores on each Problem Part',
       longdesc =>'The total of the scores of the student on each problem'.
           ' part in the sequences or folders selected.',
       },
     { name  => 'Scores Sum & Maximums',
       base  => 'scores',
       value => 'sum and total',
       shortdesc => 'Total Score and Maximum Possible for each '.
           'Sequence or Folder',
       longdesc => 'The score of each student as well as the '.
           ' maximum possible on each Sequence or Folder.',
       },
     { name  => 'Scores Summary Table Only',
       base  => 'scores',
       value => 'final table scores',
       shortdesc => 'Summary of Scores',
       longdesc  => 'The average score on each sequence or folder for the '.
           'selected students.',
       },
     { name  =>'Tries',
       base  =>'tries',
       value => 'tries',
       shortdesc => 'Number of Tries before success on each Problem Part',
       longdesc =>'The number of tries before success on each problem part.',
       },
     { name  =>'Parts Correct',
       base  =>'tries',
       value => 'parts correct',
       shortdesc => 'Number of Problem Parts completed successfully.',
       longdesc => 'The Number of Problem Parts completed successfully'.
           ' on each sequence or folder.',
       },
     { name  =>'Parts Correct & Maximums',
       base  =>'tries',
       value => 'parts correct total',
       shortdesc => 'Number of Problem Parts completed successfully.',
       longdesc => 'The Number of Problem Parts completed successfully and '.
           'the maximum possible for each student',
       },
     { name  => 'Parts Summary Table Only',
       base  => 'tries',
       value => 'final table parts',
       shortdesc => 'Summary of Parts Correct',
       longdesc  => 'A summary table of the average number of problem parts '.
           'students were able to get correct on each sequence.',
       },
     );

sub HTMLifyOutputDataDescriptions {
    my $Str = '';
    $Str .= "<h2>Output Data</h2>\n";
    $Str .= "<dl>\n";
    foreach my $option (@OutputDataOptions) {
        $Str .= '    <dt>'.$option->{'name'}.'</dt>';
        $Str .= '<dd>'.$option->{'longdesc'}.'</dd>'."\n";
    }
    $Str .= "</dl>\n";
    return $Str;
}

sub CreateAndParseOutputDataSelector {
    my $Str = '';
    my $elementname = 'chartoutputdata';
    #
    my $selected = 'scores';
    if (exists($ENV{'form.'.$elementname})) {
        if (ref($ENV{'form.'.$elementname} eq 'ARRAY')) {
            $selected = $ENV{'form.'.$elementname}->[0];
        } else {
            $selected = $ENV{'form.'.$elementname};
        }
    }
    #
    $data = 'scores';
    foreach my $option (@OutputDataOptions) {
        if ($option->{'value'} eq $selected) {
            $data = $option->{'value'};
            $base = $option->{'base'};
            $datadescription = $option->{'shortdesc'};
        }
    }
    #
    # Build the form element
    $Str = qq/<select size="5" name="$elementname">/;
    foreach my $option (@OutputDataOptions) {
        $Str .= "\n".'    <option value="'.$option->{'value'}.'"';
        $Str .= " selected " if ($option->{'value'} eq $data);
        $Str .= ">".$option->{'name'}."<\/option>";
    }
    $Str .= "\n</select>";
    return $Str;

}

#######################################################
#######################################################

=pod

=head2 HTML output routines

=item &html_initialize($r)

Create labels for the columns of student data to show.

=item &html_outputstudent($r,$student)

Return a line of the chart for a student.

=item &html_finish($r)

=cut

#######################################################
#######################################################
{
    my $padding;
    my $count;

    my $nodata_count; # The number of students for which there is no data
    my %prog_state;   # progress state used by loncommon PrgWin routines

sub html_initialize {
    my ($r) = @_;
    #
    $padding = ' 'x3;
    $count = 0;
    $nodata_count = 0;
    #
    $r->print("<h3>".$ENV{'course.'.$ENV{'request.course.id'}.'.description'}.
              "&nbsp;&nbsp;".localtime(time)."</h3>");

    if ($data !~ /^final table/) {
        $r->print("<h3>".$datadescription."</h3>");        
    }
    #
    # Set up progress window for 'final table' display only
    if ($data =~ /^final table/) {
        my $studentcount = scalar(@Apache::lonstatistics::Students); 
        %prog_state=&Apache::lonhtmlcommon::Create_PrgWin
            ($r,'Summary Table Status',
             'Summary Table Compilation Progress', $studentcount);
    }
    my $Str = "<pre>\n";
    # First, the @StudentData fields need to be listed
    my @to_show = &get_student_fields_to_show();
    foreach my $field (@to_show) {
        my $title=$Apache::lonstatistics::StudentData{$field}->{'title'};
        my $base =$Apache::lonstatistics::StudentData{$field}->{'base_width'};
        my $width=$Apache::lonstatistics::StudentData{$field}->{'width'};
        $Str .= $title.' 'x($width-$base).$padding;
    }
    # Now the selected sequences need to be listed
    foreach my $sequence (&Apache::lonstatistics::Sequences_with_Assess()){
        my $title = $sequence->{'title'};
        my $base  = $sequence->{'base_width'};
        my $width = $sequence->{'width'};
        $Str .= $title.' 'x($width-$base).$padding;
    }
    $Str .= "total</pre>\n";
    $Str .= "<pre>";
    #
    # Check for suppression of output
    if ($data =~ /^final table/) {
        $Str = '';
    }
    $r->print($Str);
    $r->rflush();
    return;
}

sub html_outputstudent {
    my ($r,$student) = @_;
    my $Str = '';
    #
    if($count++ % 5 == 0 && $count > 0 && $data !~ /^final table/) {
        $r->print("</pre><pre>");
    }
    # First, the @StudentData fields need to be listed
    my @to_show = &get_student_fields_to_show();
    foreach my $field (@to_show) {
        my $title=$student->{$field};
        my $base = length($title);
        my $width=$Apache::lonstatistics::StudentData{$field}->{'width'};
        $Str .= $title.' 'x($width-$base).$padding;
    }
    # Get ALL the students data
    my %StudentsData;
    my @tmp = &Apache::loncoursedata::get_current_state
        ($student->{'username'},$student->{'domain'},undef,
         $ENV{'request.course.id'});
    if ((scalar @tmp > 0) && ($tmp[0] !~ /^error:/)) {
        %StudentsData = @tmp;
    }
    if (scalar(@tmp) < 1) {
        $nodata_count++;
        return if ($data =~ /^final table/);
        $Str .= '<font color="blue">No Course Data</font>'."\n";
        $r->print($Str);
        $r->rflush();
        return;
    }
    #
    # By sequence build up the data
    my $studentstats;
    my $PerformanceStr = '';
    foreach my $seq (&Apache::lonstatistics::Sequences_with_Assess()) {
        my ($performance,$performance_length,$score,$seq_max,$rawdata);
        if ($base eq 'tries') {
            ($performance,$performance_length,$score,$seq_max,$rawdata) =
                &StudentTriesOnSequence($student,\%StudentsData,
                                        $seq,$show_links);
        } else {
            ($performance,$performance_length,$score,$seq_max,$rawdata) =
                &StudentPerformanceOnSequence($student,\%StudentsData,
                                              $seq,$show_links);
        }
        my $ratio = sprintf("%3d",$score).'/'.sprintf("%3d",$seq_max);
        #
        if ($data eq 'sum and total' || $data eq 'parts correct total') {
            $performance  = $ratio;
            $performance .= ' 'x($seq->{'width'}-length($ratio));
        } elsif ($data eq 'sum only' || $data eq 'parts correct') {
            $performance  = $score;
            $performance .= ' 'x($seq->{'width'}-length($score));
        } else {
            # Pad with extra spaces
            $performance .= ' 'x($seq->{'width'}-$performance_length-
                                 length($ratio)
                                 ).$ratio;
        }
        #
        $Str .= $performance.$padding;
        #
        $studentstats->{$seq->{'symb'}}->{'score'}= $score;
        $studentstats->{$seq->{'symb'}}->{'max'}  = $seq_max;
    }
    #
    # Total it up and store the statistics info.
    my ($score,$max) = (0,0);
    while (my ($symb,$seq_stats) = each (%{$studentstats})) {
        $Statistics->{$symb}->{'score'} += $seq_stats->{'score'};
        if ($Statistics->{$symb}->{'max'} < $seq_stats->{'max'}) {
            $Statistics->{$symb}->{'max'} = $seq_stats->{'max'};
        }
        $score += $seq_stats->{'score'};
        $max   += $seq_stats->{'max'};
    }
    $Str .= ' '.' 'x(length($max)-length($score)).$score.'/'.$max;
    $Str .= " \n";
    #
    # Check for suppressed output and update the progress window if so...
    if ($data =~ /^final table/) {
        $Str = '';
        &Apache::lonhtmlcommon::Increment_PrgWin($r,\%prog_state,
                                                 'last student');
    }
    #
    $r->print($Str);
    #
    $r->rflush();
    return;
}    

sub html_finish {
    my ($r) = @_;
    #
    # Check for suppressed output and close the progress window if so
    if ($data =~ /^final table/) {
        &Apache::lonhtmlcommon::Close_PrgWin($r,\%prog_state);
    } else {
        $r->print("</pre>\n"); 
    }
    if ($single_student_mode) {
        $r->print(&SingleStudentTotal());
    } else {
        $r->print(&StudentAverageTotal());
    }
    $r->rflush();
    return;
}

sub StudentAverageTotal {
    my $Str = "<h3>Summary Tables</h3>\n";
    my $num_students = scalar(@Apache::lonstatistics::Students);
    my $total_ave = 0;
    my $total_max = 0;
    $Str .= '<table border=2 cellspacing="1">'."\n";
    $Str .= "<tr><th>Title</th><th>Average</th><th>Maximum</th></tr>\n";
    foreach my $seq (&Apache::lonstatistics::Sequences_with_Assess()) {
        my $ave;
        if ($num_students > $nodata_count) {
            $ave = int(100*($Statistics->{$seq->{'symb'}}->{'score'}/
                            ($num_students-$nodata_count)))/100;
        } else {
            $ave = 0;
        }
        $total_ave += $ave;
        my $max = $Statistics->{$seq->{'symb'}}->{'max'};
        $total_max += $max;
        if ($ave == 0) {
            $ave = "0.00";
        }
        $ave .= '&nbsp;';
        $max .= '&nbsp;&nbsp;&nbsp;';
        $Str .= '<tr><td>'.$seq->{'title'}.'</td>'.
            '<td align="right">'.$ave.'</td>'.
                '<td align="right">'.$max.'</td></tr>'."\n";
    }
    $total_ave = int(100*$total_ave)/100; # only two digit
    $Str .= "</table>\n";
    $Str .= '<table border=2 cellspacing="1">'."\n";
    $Str .= '<tr><th>Number of Students</th><th>Average</th>'.
        "<th>Maximum</th></tr>\n";
    $Str .= '<tr><td>'.($num_students-$nodata_count).'</td>'.
        '<td>'.$total_ave.'</td><td>'.$total_max.'</td>';
    $Str .= "</table>\n";
    return $Str;
}

sub SingleStudentTotal {
    my $student = &Apache::lonstatistics::current_student();
    my $Str = "<h3>Summary table for ".$student->{'fullname'}." ".
        $student->{'username'}.'@'.$student->{'domain'}."</h3>\n";
    $Str .= '<table border=2 cellspacing="1">'."\n";
    $Str .= 
        "<tr><th>Sequence or Folder</th><th>Score</th><th>Maximum</th></tr>\n";
    my $total = 0;
    my $total_max = 0;
    foreach my $seq (&Apache::lonstatistics::Sequences_with_Assess()) {
        my $value = $Statistics->{$seq->{'symb'}}->{'score'};
        my $max = $Statistics->{$seq->{'symb'}}->{'max'};
        $Str .= '<tr><td>'.$seq->{'title'}.'</td>'.
            '<td align="right">'.$value.'</td>'.
                '<td align="right">'.$max.'</td></tr>'."\n";
        $total += $value;
        $total_max +=$max;
    }
    $Str .= '<tr><td><b>Total</b></td>'.
        '<td align="right">'.$total.'</td>'.
        '<td align="right">'.$total_max."</td></tr>\n";
    $Str .= "</table>\n";
    return $Str;
}

}

#######################################################
#######################################################

=pod

=head2 EXCEL subroutines

=item &excel_initialize($r)

=item &excel_outputstudent($r,$student)

=item &excel_finish($r)

=cut

#######################################################
#######################################################
{

my $excel_sheet;
my $excel_workbook;

my $filename;
my $rows_output;
my $cols_output;

my %prog_state; # progress window state
my $request_aborted;

sub excel_initialize {
    my ($r) = @_;
    #
    $request_aborted = undef;
    my $total_columns = scalar(&get_student_fields_to_show());
    foreach my $seq (&Apache::lonstatistics::Sequences_with_Assess()) {
        # Add 2 because we need a 'sum' and 'total' column for each
        $total_columns += $seq->{'num_assess_parts'}+2;
    }
    if ($data eq 'tries' && $total_columns > 255) {
        $r->print(<<END);
<h2>Unable to Complete Request</h2>
<p>
LON-CAPA is unable to produce your Excel spreadsheet because your selections
will result in more than 255 columns.  Excel allows only 255 columns in a
spreadsheet.
</p><p>
You may consider reducing the number of <b>Sequences or Folders</b> you
have selected.  
</p><p>
LON-CAPA can produce <b>CSV</b> files of this data or Excel files of the
summary data (<b>Parts Correct</b> or <b>Parts Correct & Totals</b>).
</p>
END
       $request_aborted = 1;
    }
    if ($data eq 'scores' && $total_columns > 255) {
        $r->print(<<END);
<h2>Unable to Complete Request</h2>
<p>
LON-CAPA is unable to produce your Excel spreadsheet because your selections
will result in more than 255 columns.  Excel allows only 255 columns in a
spreadsheet.
</p><p>
You may consider reducing the number of <b>Sequences or Folders</b> you
have selected.  
</p><p>
LON-CAPA can produce <b>CSV</b> files of this data or Excel files of the
summary data (<b>Scores Sum</b> or <b>Scores Sum & Totals</b>).
</p>
END
       $request_aborted = 1;
    }
    if ($data =~ /^final table/) {
        $r->print(<<END);
<h2>Unable to Complete Request</h2>
<p>
The <b>Summary Table (Scores)</b> option is not available for non-HTML output.
</p>
END
       $request_aborted = 1;
    }
    return if ($request_aborted);
    #
    $filename = '/prtspool/'.
        $ENV{'user.name'}.'_'.$ENV{'user.domain'}.'_'.
            time.'_'.rand(1000000000).'.xls';
    #
    $excel_workbook = undef;
    $excel_sheet = undef;
    #
    $rows_output = 0;
    $cols_output = 0;
    #
    # Create sheet
    $excel_workbook = Spreadsheet::WriteExcel->new('/home/httpd'.$filename);
    #
    # Check for errors
    if (! defined($excel_workbook)) {
        $r->log_error("Error creating excel spreadsheet $filename: $!");
        $r->print("Problems creating new Excel file.  ".
                  "This error has been logged.  ".
                  "Please alert your LON-CAPA administrator");
        return ;
    }
    #
    # The excel spreadsheet stores temporary data in files, then put them
    # together.  If needed we should be able to disable this (memory only).
    # The temporary directory must be specified before calling 'addworksheet'.
    # File::Temp is used to determine the temporary directory.
    $excel_workbook->set_tempdir($Apache::lonnet::tmpdir);
    #
    # Add a worksheet
    my $sheetname = $ENV{'course.'.$ENV{'request.course.id'}.'.description'};
    if (length($sheetname) > 31) {
        $sheetname = substr($sheetname,0,31);
    }
    $excel_sheet = $excel_workbook->addworksheet($sheetname);
    #
    # Put the course description in the header
    $excel_sheet->write($rows_output,$cols_output++,
                   $ENV{'course.'.$ENV{'request.course.id'}.'.description'});
    $cols_output += 3;
    #
    # Put a description of the sections listed
    my $sectionstring = '';
    my @Sections = @Apache::lonstatistics::SelectedSections;
    if (scalar(@Sections) > 1) {
        if (scalar(@Sections) > 2) {
            my $last = pop(@Sections);
            $sectionstring = "Sections ".join(', ',@Sections).', and '.$last;
        } else {
            $sectionstring = "Sections ".join(' and ',@Sections);
        }
    } else {
        if ($Sections[0] eq 'all') {
            $sectionstring = "All sections";
        } else {
            $sectionstring = "Section ".$Sections[0];
        }
    }
    $excel_sheet->write($rows_output,$cols_output++,$sectionstring);
    $cols_output += scalar(@Sections);
    #
    # Put the date in there too
    $excel_sheet->write($rows_output++,$cols_output++,
                        'Compiled on '.localtime(time));
    #
    $cols_output = 0;
    $excel_sheet->write($rows_output++,$cols_output++,$datadescription);
    #
    if ($data eq 'tries' || $data eq 'scores') {
        $rows_output++;
    }
    #
    # Add the student headers
    $cols_output = 0;
    foreach my $field (&get_student_fields_to_show()) {
        $excel_sheet->write($rows_output,$cols_output++,$field);
    }
    my $row_offset = 0;
    if ($data eq 'tries' || $data eq 'scores') {
        $row_offset = -1;
    }
    #
    # Add the remaining column headers
    foreach my $seq (&Apache::lonstatistics::Sequences_with_Assess()) {
        $excel_sheet->write($rows_output+$row_offset,
                            $cols_output,$seq->{'title'});
        if ($data eq 'tries' || $data eq 'scores') {
            foreach my $res (@{$seq->{'contents'}}) {
                next if ($res->{'type'} ne 'assessment');
                if (scalar(@{$res->{'parts'}}) > 1) {
                    foreach my $part (@{$res->{'parts'}}) {
                        $excel_sheet->write($rows_output,
                                            $cols_output++,
                                            $res->{'title'}.' part '.$part);
                    }
                } else {
                    $excel_sheet->write($rows_output,
                                        $cols_output++,
                                        $res->{'title'});
                }
            }
            $excel_sheet->write($rows_output,$cols_output++,'score');
            $excel_sheet->write($rows_output,$cols_output++,'maximum');
        } elsif ($data eq 'sum and total' || $data eq 'parts correct total') {
            $excel_sheet->write($rows_output+1,$cols_output,'score');
            $excel_sheet->write($rows_output+1,$cols_output+1,'maximum');
            $cols_output += 2;
        } else {
            $cols_output++;
        }
    }
    #
    # Bookkeeping
    if ($data eq 'sum and total' || $data eq 'parts correct total') {
        $rows_output += 2;
    } else {
        $rows_output += 1;
    }
    #
    # Output a row for MAX
    $cols_output = 0;
    foreach my $field (&get_student_fields_to_show()) {
        if ($field eq 'username' || $field eq 'fullname' || 
            $field eq 'id') {
            $excel_sheet->write($rows_output,$cols_output++,'Maximum');
        } else {
            $excel_sheet->write($rows_output,$cols_output++,'');
        }
    }
    #
    # Add the maximums for each sequence or assessment
    foreach my $seq (&Apache::lonstatistics::Sequences_with_Assess()) {
        my $weight;
        my $max = 0;
        foreach my $resource (@{$seq->{'contents'}}) {
            next if ($resource->{'type'} ne 'assessment');
            foreach my $part (@{$resource->{'parts'}}) {
                $weight = 1;
                if ($base eq 'scores') {
                    $weight = &Apache::lonnet::EXT
                        ('resource.'.$part.'.weight',$resource->{'symb'},
                         undef,undef,undef);
                    if (!defined($weight) || ($weight eq '')) { 
                        $weight=1;
                    }
                }
                if ($data eq 'scores') {
                    $excel_sheet->write($rows_output,$cols_output++,$weight);
                } elsif ($data eq 'tries') {
                    $excel_sheet->write($rows_output,$cols_output++,'');
                }
                $max += $weight;
            }
        } 
        if (! ($data eq 'sum only' || $data eq 'parts correct')) {
            $excel_sheet->write($rows_output,$cols_output++,'');
        }
        $excel_sheet->write($rows_output,$cols_output++,$max);
    }
    $rows_output++;
    #
    # Let the user know what we are doing
    my $studentcount = scalar(@Apache::lonstatistics::Students); 
    $r->print("<h1>Compiling Excel spreadsheet for ".
              $studentcount.' student');
    $r->print('s') if ($studentcount > 1);
    $r->print("</h1>\n");
    $r->rflush();
    #
    # Initialize progress window
    %prog_state=&Apache::lonhtmlcommon::Create_PrgWin
        ($r,'Excel File Compilation Status',
         'Excel File Compilation Progress', $studentcount);
    #
    &Apache::lonhtmlcommon::Update_PrgWin($r,\%prog_state,
                                          'Processing first student');
    return;
}

sub excel_outputstudent {
    my ($r,$student) = @_;
    return if ($request_aborted);
    return if (! defined($excel_sheet));
    $cols_output=0;
    #
    # Write out student data
    my @to_show = &get_student_fields_to_show();
    foreach my $field (@to_show) {
        $excel_sheet->write($rows_output,$cols_output++,$student->{$field});
    }
    #
    # Get student assessment data
    my %StudentsData;
    my @tmp = &Apache::loncoursedata::get_current_state($student->{'username'},
                                                        $student->{'domain'},
                                                        undef,
                                                   $ENV{'request.course.id'});
    if ((scalar @tmp > 0) && ($tmp[0] !~ /^error:/)) {
        %StudentsData = @tmp;
    }
    #
    # Write out sequence scores and totals data
    foreach my $seq (&Apache::lonstatistics::Sequences_with_Assess()) {
        my ($performance,$performance_length,$score,$seq_max,$rawdata);
        if ($base eq 'tries') {
            ($performance,$performance_length,$score,$seq_max,$rawdata) =
                &StudentTriesOnSequence($student,\%StudentsData,
                                        $seq,'no');
        } else {
            ($performance,$performance_length,$score,$seq_max,$rawdata) =
                &StudentPerformanceOnSequence($student,\%StudentsData,
                                              $seq,'no');
        }
        if ($data eq 'tries' || $data eq 'scores') {
            foreach my $value (@$rawdata) {
                $excel_sheet->write($rows_output,$cols_output++,$value);
            }
            $excel_sheet->write($rows_output,$cols_output++,$score);
            $excel_sheet->write($rows_output,$cols_output++,$seq_max);
        } elsif ($data eq 'sum and total' || $data eq 'sum only' || 
            $data eq 'parts correct' || $data eq 'parts correct total') {
            $excel_sheet->write($rows_output,$cols_output++,$score);
        }
        if ($data eq 'sum and total' || $data eq 'parts correct total') {
            $excel_sheet->write($rows_output,$cols_output++,$seq_max);
        }
    }
    #
    # Bookkeeping
    $rows_output++; 
    $cols_output=0;
    #
    # Update the progress window
    &Apache::lonhtmlcommon::Increment_PrgWin($r,\%prog_state,'last student');
    return;
}

sub excel_finish {
    my ($r) = @_;
    return if ($request_aborted);
    return if (! defined($excel_sheet));
    #
    # Write the excel file
    $excel_workbook->close();
    my $c = $r->connection();
    #
    return if($c->aborted());
    #
    # Close the progress window
    &Apache::lonhtmlcommon::Close_PrgWin($r,\%prog_state);
    #
    # Tell the user where to get their excel file
    $r->print('<br />'.
              '<a href="'.$filename.'">Your Excel spreadsheet.</a>'."\n");
    $r->rflush();
    return;
}

}
#######################################################
#######################################################

=pod

=head2 CSV output routines

=item &csv_initialize($r)

=item &csv_outputstudent($r,$student)

=item &csv_finish($r)

=cut

#######################################################
#######################################################
{

my $outputfile;
my $filename;
my $request_aborted;
my %prog_state; # progress window state

sub csv_initialize{
    my ($r) = @_;
    # 
    # Clean up
    $filename = undef;
    $outputfile = undef;
    undef(%prog_state);
    #
    # Deal with unimplemented requests
    $request_aborted = undef;
    if ($data =~ /final table/) {
        $r->print(<<END);
<h2>Unable to Complete Request</h2>
<p>
The <b>Summary Table (Scores)</b> option is not available for non-HTML output.
</p>
END
       $request_aborted = 1;
    }
    return if ($request_aborted);

    #
    # Open a file
    $filename = '/prtspool/'.
        $ENV{'user.name'}.'_'.$ENV{'user.domain'}.'_'.
            time.'_'.rand(1000000000).'.csv';
    unless ($outputfile = Apache::File->new('>/home/httpd'.$filename)) {
        $r->log_error("Couldn't open $filename for output $!");
        $r->print("Problems occured in writing the csv file.  ".
                  "This error has been logged.  ".
                  "Please alert your LON-CAPA administrator.");
        $outputfile = undef;
    }
    #
    # Datestamp
    my $description = $ENV{'course.'.$ENV{'request.course.id'}.'.description'};
    print $outputfile '"'.&Apache::loncommon::csv_translate($description).'",'.
        '"'.&Apache::loncommon::csv_translate(scalar(localtime(time))).'"'.
            "\n";
    #
    # Print out the headings
    my $Str = '';
    my $Str2 = undef;
    foreach my $field (&get_student_fields_to_show()) {
        if ($data eq 'sum only') {
            $Str .= '"'.&Apache::loncommon::csv_translate($field).'",';
        } elsif ($data eq 'sum and total' || $data eq 'parts correct total') {
            $Str .= '"",'; # first row empty on the student fields
            $Str2 .= '"'.&Apache::loncommon::csv_translate($field).'",';
        } elsif ($data eq 'scores' || $data eq 'tries' || 
                 $data eq 'parts correct') {
            $Str  .= '"",';
            $Str2 .= '"'.&Apache::loncommon::csv_translate($field).'",';
        }
    }
    foreach my $seq (&Apache::lonstatistics::Sequences_with_Assess()) {
        if ($data eq 'sum only' || $data eq 'parts correct') {
            $Str .= '"'.&Apache::loncommon::csv_translate($seq->{'title'}).
                '",';
        } elsif ($data eq 'sum and total' || $data eq 'parts correct total') {
            $Str  .= '"'.&Apache::loncommon::csv_translate($seq->{'title'}).
                '","",';
            $Str2 .= '"score","total possible",';
        } elsif ($data eq 'scores' || $data eq 'tries') {
            $Str  .= '"'.&Apache::loncommon::csv_translate($seq->{'title'}).
                '",';
            $Str .= '"",'x($seq->{'num_assess_parts'}-1+2);
            foreach my $res (@{$seq->{'contents'}}) {
                next if ($res->{'type'} ne 'assessment');
                foreach my $part (@{$res->{'parts'}}) {
                    $Str2 .= '"'.&Apache::loncommon::csv_translate($res->{'title'}.', Part '.$part).'",';
                }
            }
            $Str2 .= '"score","total possible",';
        }
    }
    chop($Str);
    $Str .= "\n";
    print $outputfile $Str;
    if (defined($Str2)) {
        chop($Str2);
        $Str2 .= "\n";
        print $outputfile $Str2;
    }
    #
    # Initialize progress window
    my $studentcount = scalar(@Apache::lonstatistics::Students);
    %prog_state=&Apache::lonhtmlcommon::Create_PrgWin
        ($r,'CSV File Compilation Status',
         'CSV File Compilation Progress', $studentcount);
    return;
}

sub csv_outputstudent {
    my ($r,$student) = @_;
    return if ($request_aborted);
    return if (! defined($outputfile));
    my $Str = '';
    #
    # Output student fields
    my @to_show = &get_student_fields_to_show();
    foreach my $field (@to_show) {
        $Str .= '"'.&Apache::loncommon::csv_translate($student->{$field}).'",';
    }
    #
    # Get student assessment data
    my %StudentsData;
    my @tmp = &Apache::loncoursedata::get_current_state($student->{'username'},
                                                        $student->{'domain'},
                                                        undef,
                                                   $ENV{'request.course.id'});
    if ((scalar @tmp > 0) && ($tmp[0] !~ /^error:/)) {
        %StudentsData = @tmp;
    }
    #
    # Output performance data
    foreach my $seq (&Apache::lonstatistics::Sequences_with_Assess()) {
        my ($performance,$performance_length,$score,$seq_max,$rawdata);
        if ($base eq 'tries') {
            ($performance,$performance_length,$score,$seq_max,$rawdata) =
                &StudentTriesOnSequence($student,\%StudentsData,
                                        $seq,'no');
        } else {
            ($performance,$performance_length,$score,$seq_max,$rawdata) =
                &StudentPerformanceOnSequence($student,\%StudentsData,
                                              $seq,'no');
        }
        if ($data eq 'sum only' || $data eq 'parts correct') {
            $Str .= '"'.$score.'",';
        } elsif ($data eq 'sum and total' || $data eq 'parts correct total') {
            $Str  .= '"'.$score.'","'.$seq_max.'",';
        } elsif ($data eq 'scores' || $data eq 'tries') {
            $Str .= '"'.join('","',(@$rawdata,$score,$seq_max)).'",';
        }
    }
    chop($Str);
    $Str .= "\n";
    print $outputfile $Str;
    #
    # Update the progress window
    &Apache::lonhtmlcommon::Increment_PrgWin($r,\%prog_state,'last student');
    return;
}

sub csv_finish {
    my ($r) = @_;
    return if ($request_aborted);
    return if (! defined($outputfile));
    close($outputfile);
    #
    my $c = $r->connection();
    return if ($c->aborted());
    #
    # Close the progress window
    &Apache::lonhtmlcommon::Close_PrgWin($r,\%prog_state);
    #
    # Tell the user where to get their csv file
    $r->print('<br />'.
              '<a href="'.$filename.'">Your csv file.</a>'."\n");
    $r->rflush();
    return;
    
}

}

#######################################################
#######################################################

=pod

=item &StudentTriesOnSequence()

Inputs:

=over 4

=item $student

=item $studentdata Hash ref to all student data

=item $seq Hash ref, the sequence we are working on

=item $links if defined we will output links to each resource.

=back

=cut

#######################################################
#######################################################
sub StudentTriesOnSequence {
    my ($student,$studentdata,$seq,$links) = @_;
    $links = 'no' if (! defined($links));
    my $Str = '';
    my ($sum,$max) = (0,0);
    my $performance_length = 0;
    my @TriesData = ();
    my $tries;
    foreach my $resource (@{$seq->{'contents'}}) {
        next if ($resource->{'type'} ne 'assessment');
        my $resource_data = $studentdata->{$resource->{'symb'}};
        my $value = '';
        foreach my $partnum (@{$resource->{'parts'}}) {
            $tries = undef;
            $max++;
            $performance_length++;
            my $symbol = ' '; # default to space
            #
            if (exists($resource_data->{'resource.'.$partnum.'.solved'})) {
                my $status = $resource_data->{'resource.'.$partnum.'.solved'};
                if ($status eq 'correct_by_override') {
                    $symbol = '+';
                    $sum++;
                } elsif ($status eq 'incorrect_by_override') {
                    $symbol = '-';
                } elsif ($status eq 'ungraded_attempted') {
                    $symbol = '#';
                } elsif ($status eq 'incorrect_attempted')  {
                    $symbol = '.';
                } elsif ($status eq 'excused') {
                    $symbol = 'x';
                    $max--;
                } elsif ($status eq 'correct_by_student' &&
                    exists($resource_data->{'resource.'.$partnum.'.tries'})){
                    $tries = $resource_data->{'resource.'.$partnum.'.tries'};
                    if ($tries > 9) {
                        $symbol = '*';
                    } elsif ($tries > 0) {
                        $symbol = $tries;
                    } else {
                        $symbol = ' ';
                    }
                    $sum++;
                } elsif (exists($resource_data->{'resource.'.
                                                     $partnum.'.tries'})){
                    $symbol = '.';
                } else {
                    $symbol = ' ';
                }
            } else {
                # Unsolved.  Did they try?
                if (exists($resource_data->{'resource.'.$partnum.'.tries'})){
                    $symbol = '.';
                } else {
                    $symbol = ' ';
                }
            }
            #
            if (! defined($tries)) {
                $tries = $symbol;
            }
            push (@TriesData,$tries);
            #
            if ( ($links eq 'yes' && $symbol ne ' ') ||
                 ($links eq 'all')) {
                if (length($symbol) > 1) {
                    &Apache::lonnet::logthis('length of symbol "'.$symbol.'" > 1');
                }
                $symbol = '<a href="/adm/grades'.
                    '?symb='.&Apache::lonnet::escape($resource->{'symb'}).
                        '&student='.$student->{'username'}.
                            '&userdom='.$student->{'domain'}.
                                '&command=submission">'.$symbol.'</a>';
            }
            $value .= $symbol;
        }
        $Str .= $value;
    }
    if ($seq->{'randompick'}) {
        $max = $seq->{'randompick'};
    }
    return ($Str,$performance_length,$sum,$max,\@TriesData);
}

#######################################################
#######################################################

=pod

=item &StudentPerformanceOnSequence()

Inputs:

=over 4

=item $student

=item $studentdata Hash ref to all student data

=item $seq Hash ref, the sequence we are working on

=item $links if defined we will output links to each resource.

=back

=cut

#######################################################
#######################################################
sub StudentPerformanceOnSequence {
    my ($student,$studentdata,$seq,$links) = @_;
    $links = 'no' if (! defined($links));
    my $Str = ''; # final result string
    my ($score,$max) = (0,0);
    my $performance_length = 0;
    my $symbol;
    my @ScoreData = ();
    my $partscore;
    foreach my $resource (@{$seq->{'contents'}}) {
        next if ($resource->{'type'} ne 'assessment');
        my $resource_data = $studentdata->{$resource->{'symb'}};
        foreach my $part (@{$resource->{'parts'}}) {
            $partscore = undef;
            my $weight = &Apache::lonnet::EXT('resource.'.$part.'.weight',
                                              $resource->{'symb'},
                                              $student->{'domain'},
                                              $student->{'username'},
                                              $student->{'section'});
            if (!defined($weight) || ($weight eq '')) { 
                $weight=1;
            }
            #
            $max += $weight; # see the 'excused' branch below...
            $performance_length++; # one character per part
            $symbol = ' '; # default to space
            #
            my $awarded = 0;
            if (exists($resource_data->{'resource.'.$part.'.awarded'})) {
                $awarded = $resource_data->{'resource.'.$part.'.awarded'};
            }
            #
            $partscore = $weight*$awarded;
            $score += $partscore;
            $symbol = $partscore; 
            if (length($symbol) > 1) {
                $symbol = '*';
            }
            if (exists($resource_data->{'resource.'.$part.'.solved'})) {
                my $status = $resource_data->{'resource.'.$part.'.solved'};
                if ($status eq 'excused') {
                    $symbol = 'x';
                    $max -= $weight; # Do not count 'excused' problems.
                }
            } else {
                # Unsolved.  Did they try?
                if (exists($resource_data->{'resource.'.$part.'.tries'})){
                    $symbol = '.';
                } else {
                    $symbol = ' ';
                }
            }
            #
            if (! defined($partscore)) {
                $partscore = $symbol;
            }
            push (@ScoreData,$partscore);
            #
            if ( ($links eq 'yes' && $symbol ne ' ') || ($links eq 'all')) {
                $symbol = '<a href="/adm/grades'.
                    '?symb='.&Apache::lonnet::escape($resource->{'symb'}).
                    '&student='.$student->{'username'}.
                    '&userdom='.$student->{'domain'}.
                    '&command=submission">'.$symbol.'</a>';
            }
            $Str .= $symbol;
        }
    }
    return ($Str,$performance_length,$score,$max,\@ScoreData);
}

#######################################################
#######################################################

=pod

=item &CreateLegend()

This function returns a formatted string containing the legend for the
chart.  The legend describes the symbols used to represent grades for
problems.

=cut

#######################################################
#######################################################
sub CreateLegend {
    my $Str = "<p><pre>".
              "   1  correct by student in 1 try\n".
              "   7  correct by student in 7 tries\n".
              "   *  correct by student in more than 9 tries\n".
	      "   +  correct by hand grading or override\n".
              "   -  incorrect by override\n".
	      "   .  incorrect attempted\n".
	      "   #  ungraded attempted\n".
              "      not attempted (blank field)\n".
	      "   x  excused".
              "</pre><p>";
    return $Str;
}

#######################################################
#######################################################

=pod 

=back

=cut

#######################################################
#######################################################

1;

__END__
