# The LearningOnline Network with CAPA
# Handler to show differences between file versions
#
# $Id: londiff.pm,v 1.11 2003/02/03 18:03:53 harris41 Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
#
# (Handler to retrieve an old version of a file
#
# (Publication Handler
# 
# (TeX Content Handler
#
# 05/29/00,05/30,10/11 Gerd Kortemeyer)
#
# 11/28,11/29,11/30,12/01,12/02,12/04,12/23 Gerd Kortemeyer
# 03/23 Guy Albertelli
# 03/24,03/29 Gerd Kortemeyer)
#
# 03/31,04/03 Gerd Kortemeyer)
#
# 05/02/01,05/09 Gerd Kortemeyer
#
###

package Apache::londiff;

use strict;
use Apache::File;
use File::Copy;
use Algorithm::Diff qw(diff);
use Apache::Constants qw(:common :http :methods);
use Apache::loncacc;
use Apache::lonnet();
use Apache::loncommon();

sub handler {

  my $r=shift;

# Get query string for limited number of parameters

   &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},
                                       ['filename','versiontwo','versionone']);
# Get the files

  my $cuname=$ENV{'user.name'};
  my $cudom=$ENV{'user.domain'};

  if ($ENV{'form.filename'}=~/^\/res\//) {
      ($cudom,$cuname,$ENV{'form.filename'})=
       ($ENV{'form.filename'}=~/^\/res\/(\w+)\/(\w+)\/(.*)$/);
  } else {
   unless (($cuname,$cudom)=
    &Apache::loncacc::constructaccess($ENV{'form.filename'},
                                      $r->dir_config('lonDefDomain'))) {
     $r->log_reason($cuname.' at '.$cudom.
         ' trying to get diffs file '.$ENV{'form.filename'}.
         '  - not authorized', 
         $r->filename); 
     return HTTP_NOT_ACCEPTABLE;
   }
  }

  my $efn=$ENV{'form.filename'};

  $efn=~s/\/\~(\w+)//g;

  my @f1=();
  my @f2=();

  $r->content_type('text/html');
  $r->send_http_header;

  $r->print('<html><head><title>LON-CAPA Construction Diffs</title></head>');

  $r->print(&Apache::loncommon::bodytag('Resource Differences'));

  
  $r->print('<h1>Compare versions of <tt>'.$efn.'</tt></h1>');
   
       if (($cuname ne $ENV{'user.name'}) || ($cudom ne $ENV{'user.domain'})) {
          $r->print('<h3><font color=red>Co-Author: '.$cuname.' at '.$cudom.
               '</font></h3>');
      }


 if (&Apache::loncommon::fileembstyle(($efn=~/\.(\w+)$/)) eq
      'ssi') {
  $r->print('<p><font color="red">');
  if ($ENV{'form.versionone'} eq 'priv') {
      my $fn='/home/'.$cuname.'/public_html/'.$efn;
      if (-e $fn) {
	  my $fh=Apache::File->new($fn);
          my $line;
          while($line=<$fh>) {
             chomp($line);
             $f1[$#f1+1]=$line;
	 }
      }
      $r->print('<b>Construction Space Version</b>');
  } else {
      my $fn=
       '/home/httpd/html//res/'.$cudom.'/'.$cuname.'/';
      if ($ENV{'form.versionone'}) {
         my ($main,$suffix)=($efn=~/^(.+)\.(\w+)$/);
         $fn.=$main.'.'.$ENV{'form.versionone'}.'.'.$suffix;
	 $r->print('<b>Version '.$ENV{'form.versionone'}.'</b>');
      } else {
         $fn.=$efn;
	 $r->print('<b>Current Version</b>');
      }
      @f1=split(/\n/,&Apache::lonnet::getfile($fn));      
  }

  $r->print('</font><br />versus<br /><font color="green">');

  if ($ENV{'form.versiontwo'} eq 'priv') {
      my $fn='/home/'.$cuname.'/public_html/'.$efn;
      if (-e $fn) {
	  my $fh=Apache::File->new($fn);
          my $line;
          while($line=<$fh>) {
             chomp($line);
             $f2[$#f2+1]=$line;
	 }
      }
      $r->print('<b>Construction Space Version</b>');
  } else {
      my $fn=
       '/home/httpd/html/res/'.$cudom.'/'.$cuname.'/';
      if ($ENV{'form.versiontwo'}) {
         my ($main,$suffix)=($efn=~/^(.+)\.(\w+)$/);
         $fn.=$main.'.'.$ENV{'form.versiontwo'}.'.'.$suffix;
	 $r->print('<b>Version '.$ENV{'form.versiontwo'}.'</b>');
      } else {
         $fn.=$efn;
	 $r->print('<b>Current Version</b>');
      }
      @f2=split(/\n/,&Apache::lonnet::getfile($fn));      
  }
  $r->print('</font></p>');
# Run diff

  my $diffs = diff(\@f1, \@f2);

# Start page output

  my $chunk;
  my $line;

  $r->print('<pre>');

  foreach $chunk (@$diffs) {

    foreach $line (@$chunk) {
      my ($sign, $lineno, $text) = @$line;
      $text=~s/\</\&lt\;/g;
      $text=~s/\>/\&gt\;/g;
      $lineno=substr($lineno.'        ',0,7);
      $r->print('<font color='.(($sign eq '+')?'green':'red').'>'.
                $sign.' '.$lineno.' '.$text."</font>\n");
    }
    $r->print("<hr>\n");
  }
  $r->print('</pre>');

} else {
    $r->print('<h1><font color=red>Binary File</font></h1>');
}
  $r->print('</body></html>');
  return OK;  
}


1;
__END__




