# The LearningOnline Network with CAPA
# Construction Space Directory Lister
#
# $Id: lonpubdir.pm,v 1.37 2003/08/03 01:20:02 www Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
# 
# (TeX Content Handler
#
# YEAR=2000
# 05/29/00,05/30,10/11 Gerd Kortemeyer)
#
# 11/28,11/29,11/30,12/01,12/02,12/04,12/23 Gerd Kortemeyer
# YEAR=2001
# 03/23 Guy Albertelli
# 03/24,03/29 Gerd Kortemeyer)
# 03/31,04/03,05/09,06/23,08/18,08/20 Gerd Kortemeyer
# 12/28 Gerd Kortemeyer
#
###

package Apache::lonpubdir;

use strict;
use Apache::File;
use File::Copy;
use Apache::Constants qw(:common :http :methods);
use Apache::loncacc;
use Apache::loncommon();

sub handler {

  my $r=shift;

  my $fn;



  $fn = getEffectiveUrl($r);

  # Validate access to the construction space and get username@domain.

  my $uname;
  my $udom;

  ($uname,$udom)=
    &Apache::loncacc::constructaccess(
             $fn,$r->dir_config('lonDefDomain')); 
  unless (($uname) && ($udom)) {
     $r->log_reason($uname.' at '.$udom.
         ' trying to list directory '.$ENV{'form.filename'}.
         ' ('.$fn.') - not authorized', 
         $r->filename); 
     return HTTP_NOT_ACCEPTABLE;
  }

  # Remove trailing / from directory name.

  $fn=~s/\/$//;

  unless ($fn) { 
     $r->log_reason($ENV{'user.name'}.' at '.$ENV{'user.domain'}.
         ' trying to list empty directory', $r->filename); 
     return HTTP_NOT_FOUND;
  } 

# ----------------------------------------------------------- Start page output

  my $thisdisfn=$fn;
  $thisdisfn=~s/^\/home\/$uname\/public_html//;	# subdirectory part of
                                                # construction space. 
  my $docroot=$r->dir_config('lonDocRoot');     # Apache  londocument root.

  my $resdir=$docroot.'/res/'.$udom.'/'.$uname.$thisdisfn; # Resource directory
  my $targetdir=$udom.'/'.$uname.$thisdisfn; # Publiction target directory.
  my $linkdir='/priv/'.$uname.$thisdisfn;      # Full URL name of constr space.



  &startpage($r, $uname, $udom, $thisdisfn);   # Put out the start of page.
  
  # Start off the diretory table.

  $r->print('<table border=2>'.
	    '<tr><th>Actions</th><th>Name</th><th>Title</th>'.
	    '<th>Status</th><th>Last Modified</th></tr>');

  my $filename;
  my $dirptr=16384;		# Mask indicating a directory in stat.cmode.

  opendir(DIR,$fn);
  my @files=sort(readdir(DIR));
  foreach my $filename (@files) {
     my ($cdev,$cino,$cmode,$cnlink,
         $cuid,$cgid,$crdev,$csize,
         $catime,$cmtime,$cctime,
         $cblksize,$cblocks)=stat($fn.'/'.$filename);

     my $extension='';
     if ($filename=~/\.(\w+)$/) { $extension=$1; }
     if ($cmode&$dirptr) {
	 putdirectory($r, $thisdisfn, $linkdir, $filename, $cmtime);
     } elsif (&Apache::loncommon::fileembstyle($extension) ne 'hdn') {
	 putresource($r, $uname, $filename, $thisdisfn, $resdir, 
		     $targetdir, $linkdir, $cmtime);
     } else {
	# "hidden" extension and not a directory, so hide it away.
     }
  }
  closedir(DIR);

  $r->print('</table></body></html>');
  return OK;  
}
#
#  Gets the effective URL of the request and returns it:
#    $effn = getEffectiveUrl($r);
#       $r  - The Apache Request object.
sub getEffectiveUrl {
    my $r = shift;
    my $fn;
    
    if ($ENV{'form.filename'}) {	# If a form filename is defined.
	$fn=$ENV{'form.filename'};
	#
	#   Replace the ~username of the URL with /home/username/public_html
	#   so that we don't have to worry about ~ expansion internally.
	#
	$fn=~s/^http\:\/\/[^\/]+\///;
        $fn=~s/^\///;
        $fn=~s/\~(\w+)/\/home\/$1\/public_html/;
	
	#  Remove trailing / strings (?) 
	
	$fn=~s/\/[^\/]+$//;
    } else {
	#   If no form is defined, use request filename.
	$fn = $r->filename();
	my $lonDocRoot=$r->dir_config('lonDocRoot');
	if ( $fn =~ /$lonDocRoot/ ) {
	    #internal authentication, needs fixup.
	    $fn = $r->uri(); # non users do not get the full path request
                             # through SCRIPT_FILENAME
	    $fn=~s|^/~(\w+)|/home/$1/public_html|;
	}
    }
    $fn=~s/\/+/\//g;
    return $fn;
}
#
#   Output the header of the page.  This includes:
#   - The HTML header 
#   - The H1/H3  stuff which includes the directory.
#
#     startpage($r, $uame, $udom, $thisdisfn);
#      $r     - The apache request object.
#      $uname - User name.
#      $udom  - Domain name the user is logged in under.
#      $thisdisfn - Displayable version of the filename.

sub startpage {
    my ($r, $uname, $udom, $thisdisfn) = @_;
    
    $r->content_type('text/html');
    $r->send_http_header;
    
    $r->print('<html><head><title>LON-CAPA Construction Space</title></head>');
    
    $r->print(&Apache::loncommon::bodytag(undef,undef,undef,1));
    my $pubdirscript=(<<ENDPUBDIRSCRIPT);
<script>
// Store directory location for menu bar to find

parent.lastknownpriv='/~$uname/$thisdisfn/';

// Confirmation dialogues

    function pubdir(theform) {
	if (confirm('Publish complete directory?')) {
	    theform.submit();
        }
    }
   function pubrecdir(theform) {
	if (confirm('Publish directory and all subdirectories?')) {
            theform.pubrec.value='1';
	    theform.submit();
        }
    }
</script>
ENDPUBDIRSCRIPT

    $r->print('<h1>Construction Space Directory <tt>'.
	      $thisdisfn.'/</tt></h1>'.
	      '<script type="text/javascript">top.document.title = \''.
	      $thisdisfn.'/ - LON-CAPA Construction Space\';</script>'.
	      $pubdirscript.
              '<form method="post" action="/adm/publish" target="_parent">'.
              '<table><tr><td><input type="hidden" name="filename" value="/~'.
               $uname.$thisdisfn.'/" />'.
              '<input type="button" onClick="pubdir(this.form);" value="Publish Directory" />'.
              '<input type="hidden" name="pubrec" value="" />'.
              '<input type="button" onClick="pubrecdir(this.form);" value="Publish Directory and Sub Directories" /></td><td>'.
'<input type="button" onClick="window.location='."'/~".
               $uname.$thisdisfn."/default.meta'".'" value="Edit Directory Catalog Information" /></td></tr><tr><td><input type="checkbox" name="forcerepub" /> Force publication of unmodified files.</td><td>&nbsp;</td></tr></table></form>');
    
    if (($uname ne $ENV{'user.name'}) || ($udom ne $ENV{'user.domain'})) {
	$r->print('<h3>Co-Author: '.$uname.' at '.$udom.
		  '</h3>');
    }
}

#
#   Get the title string or "[untitled]" if the file has no title metadata:
#   Without the latter substitution, it's impossible to examine metadata for
#   untitled resources.  Resources may be legitimately untitled, to prevent
#   searches from locating them.
#
#   $str = getTitleString($fullname);
#       $fullname - Fully qualified filename to check.
#
sub getTitleString {
    my $fullname = shift;
    my $title    = &Apache::lonnet::metadata($fullname, 'title');

    unless ($title) {
	$title = "[untitled]";
    }
    return $title;
}


#
#  Put out a directory table row:
#    putdirectory(r, base, here, dirname, modtime)
#      r       - Apache request object.
#      reqfile - File in request.
#      here    - Where we are in directory tree.
#      dirname - Name of directory special file.
#      modtime - Encoded modification time.
# 
sub putdirectory {
    my ($r, $reqfile, $here, $dirname, $modtime) = @_;
  
    # construct the display filename: the directory name unless ..:
    
    my $disfilename = $dirname;
    if ($dirname eq '..') {
	$disfilename = '<i>Parent Directory</i>';
    }
    unless (( ($dirname eq '..') && ($reqfile eq '')) ||
	    ($dirname eq '.')) {
	$r->print('<tr bgcolor="#CCCCFF">'.
		  '<td>Go to ...</td>'.
		  '<td><a href="'.$here.'/'.$dirname.'/" target="_top">'.
		  $disfilename.'</a></td>'.
		        '<td>&nbsp;</td>'.
		  '<td>&nbsp;</td>'.
		  '<td>'.localtime($modtime).'</td>'.
		  "</tr>\n");
    }	
    return OK;
}
#
#   Put a table row for a file resource.
#
sub putresource {
    my ($r, $uname, $filename, $thisdisfn, 
	$resdir, $targetdir, $linkdir,
	$cmtime) = @_;

    my $status='Unpublished';
    my $bgcolor='#FFCCCC';
    my $title='&nbsp;';
    if (-e $resdir.'/'.$filename) {
	my ($rdev,$rino,$rmode,$rnlink,
	    $ruid,$rgid,$rrdev,$rsize,
	    $ratime,$rmtime,$rctime,
	    $rblksize,$rblocks)=stat($resdir.'/'.$filename);
	if ($rmtime>=$cmtime) {
	    $status='Published';
            $bgcolor='#CCFFCC';
	    $title='<a href="/res/'.$targetdir.'/'.$filename.
		'.meta" target=cat>'.
		getTitleString($targetdir.'/'.$filename, 'title').'</a>';
	} else {
	    $status='Modified';
            $bgcolor='#FFFFCC';
	    $title='<a href="/res/'.$targetdir.'/'.$filename.'.meta" target=cat>'.
		getTitleString($targetdir.'/'.$filename,'title').'</a>';
	    if (&Apache::loncommon::fileembstyle(($filename=~/\.(\w+)$/)) eq 'ssi') {
		$status.='<br><a href="/adm/diff?filename=/~'.$uname.
		    $thisdisfn.'/'.$filename.
		    '&versiontwo=priv" target=cat>Diffs</a>';
	    }
	}   
	$status.='<br><a href="/adm/retrieve?filename=/~'.$uname.
	    $thisdisfn.'/'.$filename.'" target=cat>Retrieve</a>';
    }
    my $editlink='';
    if ($filename=~/\.(xml|html|htm|xhtml|xhtm|sty)$/) {
	$editlink=' (<a href="'.$linkdir.'/'.$filename.'?forceedit=1" target="_top">Edit</a>)';
    }
    if ($filename=~/\.(problem|exam|quiz|assess|survey|form|library)$/) {
	$editlink=' (<a href="'.$linkdir.'/'.$filename.'?forceedit=1" target="_top">EditXML</a>)';
    }
    $r->print('<tr bgcolor="'.$bgcolor.'">'.
	      '<td><a target="_parent" href="/adm/publish?filename=/~'.
	      $uname.$thisdisfn.'/'.$filename.'">'.'Publish</a>'.
	      '</td>'.
	      '<td>'.
	      '<a href="'.$linkdir.'/'.$filename.'" target="_top">'.
               $filename.'</a>'.$editlink.
	      '</td>'.
	      '<td>'.$title.'</td>'.
	      '<td>'.$status.'</td>'.
	      '<td>'.localtime($cmtime).'</td>'.
	      "</tr>\n");
    return OK;
}
#
#   Categorize files in the directory.
#   For each file in a list of files in a file directory, 
#   the  file categorized as one of:
#    - directory  
#    - sequence
#    - problem 
#    - Other resource.
#
#   For each file the modification date is determined as well.
#   Returned is a list of sublists:
#    (directories, sequences, problems, other)
#   each of the sublists contains entries of the following form (sorted by
#   filename):
#     (filename, typecode, lastmodtime)
#
#   $list = CategorizeFiles($location, $files)
#       $location   - Directory in which the files live (relative to our
#                     execution.
#       $files      - list of files.
#
sub CategorizeFiles {
    my $location = shift;
    my $files    = shift;
}

1;
__END__

=head1 NAME

Apache::lonpubdir - Construction space directory lister

=head1 SYNOPSIS

Invoked (for various locations) by /etc/httpd/conf/srm.conf:

 <LocationMatch "^/\~.*/$">
 PerlAccessHandler       Apache::loncacc
 SetHandler perl-script
 PerlHandler Apache::lonpubdir
 ErrorDocument     403 /adm/login
 ErrorDocument     404 /adm/notfound.html
 ErrorDocument     406 /adm/unauthorized.html
 ErrorDocument	  500 /adm/errorhandler
 </LocationMatch>

 <Location /adm/pubdir>
 PerlAccessHandler       Apache::lonacc
 SetHandler perl-script
 PerlHandler Apache::lonpubdir
 ErrorDocument     403 /adm/login
 ErrorDocument     404 /adm/notfound.html
 ErrorDocument     406 /adm/unauthorized.html
 ErrorDocument	  500 /adm/errorhandler
 </Location>

=head1 INTRODUCTION

This module publishes a directory of files.

This is part of the LearningOnline Network with CAPA project
described at http://www.lon-capa.org.

=head1 HANDLER SUBROUTINE

This routine is called by Apache and mod_perl.

=over 4

=item *

read in information

=item *

start page output

=item *

run through list of files and attempt to publish unhidden files

=back

=cut
