# The LearningOnline Network with CAPA
# Edit Handler for RAT Maps
#
# $Id: lonratedt.pm,v 1.45 2003/06/16 13:51:46 www Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
# (TeX Content Handler
#
# 05/29/00,05/30 Gerd Kortemeyer)
# 7/1,6/30 Gerd Kortemeyer

package Apache::lonratedt;

use strict;
use Apache::Constants qw(:common);
use Apache::lonnet;
use Apache::lonratsrv;
use Apache::lonsequence;
use Apache::loncommon;
use File::Copy;

use vars qw(@order @resources);


# Mapread read maps into global arrays @links and @resources, determines status
# sets @order - pointer to resources in right order
# sets @resources - array with the resources with correct idx
#
sub mapread {
    my $fn=shift;

    my @links;
    undef @links;
    undef @resources;
    undef @order;
    @resources=('');
    @order=();

    my ($outtext,$errtext)=&Apache::lonratsrv::loadmap($fn,'');
    if ($errtext) { return ($errtext,2); }

# -------------------------------------------------------------------- Read map
    foreach (split(/\<\&\>/,$outtext)) {
	my ($command,$number,$content)=split(/\<\:\>/,$_);
        if ($command eq 'objcont') {
	    $resources[$number]=$content;
        }
        if ($command eq 'objlinks') {
            $links[$number]=$content;
        }
        if ($command eq 'objparms') {
	    return('Map has resource parameters. Use advanced editor.',1);
        }
    }
# ------------------------------------------------------- Is this a linear map?
    my @starters=();
    my @endings=();
    undef @starters;
    undef @endings;

    foreach (@links) {
        if (defined($_)) {
	    my ($start,$end,$cond)=split(/\:/,$_);
            if ((defined($starters[$start])) || (defined($endings[$end]))) { 
		return
                 ('Map has branchings. Use advanced editor.',1);
            }
	    $starters[$start]=1;
	    $endings[$end]=1;
	    if ($cond) {
		return
                 ('Map has conditions. Use advanced editor.',1);
            }
	}

    }
    for (my $i=1; $i<=$#resources; $i++) {
        if (defined($resources[$i])) {
	    unless (($starters[$i]) || ($endings[$i])) {
                return
		 ('Map has unconnected resources. Use advanced editor.',1);
            }
        }
    }
# ---------------------------------------------- Did we just read an empty map?
    if ($#resources<1) {
        undef $resources[0];
	$resources[1]=':::start';
        $resources[2]=':::finish';
    }
# -------------------------------------------------- This is a linear map, sort

    my $startidx=0;
    my $endidx=0;
    for (my $i=0; $i<=$#resources; $i++) {
        if (defined($resources[$i])) {
            my ($title,$url,$ext,$type)=split(/\:/,$resources[$i]);
	    if ($type eq 'start') { $startidx=$i; }
            if ($type eq 'finish') { $endidx=$i; }
        }
    }
    my $k=0;
    my $currentidx=$startidx;
    $order[$k]=$currentidx;
    for (my $i=0; $i<=$#resources; $i++) {
        foreach (@links) {
	    my ($start,$end)=split(/\:/,$_);
            if ($start==$currentidx) {
		$currentidx=$end;
                $k++;
                $order[$k]=$currentidx;
                last;
            }
        }
        if ($currentidx==$endidx) { last; }
    }
    return $errtext;
}

# ---------------------------------------------- Read a map as well as possible
# Also used by the sequence handler
# Call lonsequence::attemptread to read from resource space
#
sub attemptread {
    my $fn=shift;

    my @links;
    undef @links;
    my @theseres;
    undef @theseres;

    my ($outtext,$errtext)=&Apache::lonratsrv::loadmap($fn,'');
    if ($errtext) { return @theseres }

# -------------------------------------------------------------------- Read map
    foreach (split(/\<\&\>/,$outtext)) {
	my ($command,$number,$content)=split(/\<\:\>/,$_);
        if ($command eq 'objcont') {
	    $theseres[$number]=$content;
        }
        if ($command eq 'objlinks') {
            $links[$number]=$content;
        }
    }

# --------------------------------------------------------------- Sort, sort of

    my @objsort=();
    undef @objsort;

    my @data1=();
    my @data2=();
    undef @data1;
    undef @data2;

    my $k;
    my $kj;
    my $j;
    my $ij;

   for ($k=1;$k<=$#theseres;$k++) {
      if (defined($theseres[$k])) {
         $objsort[$#objsort+1]=$k;
      }
   }

   for ($k=1;$k<=$#links;$k++) {
     if (defined($links[$k])) {
      @data1=split(/\:/,$links[$k]);
      $kj=-1;
      for (my $j=0;$j<=$#objsort;$j++) {
         if ((split(/\:/,$objsort[$j]))[0]==$data1[0]) {
            $kj=$j;
         }
      }
      if ($kj!=-1) { $objsort[$kj].=':'.$data1[1]; }
     }
   }
    for ($k=0;$k<=$#objsort;$k++) {
      for ($j=0;$j<=$#objsort;$j++) {
        if ($k!=$j) {
          @data1=split(/\:/,$objsort[$k]);
          @data2=split(/\:/,$objsort[$j]);
          my $dol=$#data1+1;
          my $dtl=$#data2+1;
          if ($dol+$dtl<1000) {
           for ($kj=1;$kj<$dol;$kj++) {
             if ($data1[$kj]==$data2[0]) {
                for ($ij=1;$ij<$dtl;$ij++) {
                   $data1[$#data1+1]=$data2[$ij];
                }
             }
           }
           for ($kj=1;$kj<$dtl;$kj++) {
             if ($data2[$kj]==$data1[0]) {
                 for ($ij=1;$ij<$dol;$ij++) {
                    $data2[$#data2+1]=$data1[$ij];
                 }
             }
           }
           $objsort[$k]=join(':',@data1);
           $objsort[$j]=join(':',@data2);
          }
         }
      } 
  }
# ---------------------------------------------------------------- Now sort out

    @objsort=sort {
      my @data1=split(/\:/,$a);
      my @data2=split(/\:/,$b);
      my $rvalue=0;
      my $k;
      for ($k=1;$k<=$#data1;$k++) {
         if ($data1[$k]==$data2[0]) { $rvalue--; }
      }
      for ($k=1;$k<=$#data2;$k++) {
         if ($data2[$k]==$data1[0]) { $rvalue++; }
      }
      if ($rvalue==0) { $rvalue=$#data2-$#data1; }
      $rvalue;
    } @objsort;

    my @outres=();
    undef @outres;

    for ($k=0;$k<=$#objsort;$k++) {
	$outres[$k]=$theseres[(split(/\:/,$objsort[$k]))[0]];
    }
    return @outres;
}

# --------------------------------------------------------- Build up RAT screen
sub ratedt {
  my ($r,$url)=@_;
  $r->print(<<ENDDOCUMENT);

<html>
<head>
<script language="JavaScript">
    var flag=0;
</script>
</head>
<frameset rows="1,50,*" border=0>
<frame name=server src="$url/loadonly/ratserver" noresize noscroll>
<frame name=code src="/adm/rat/code.html">
<frame name=mapout src="/adm/rat/map.html">
</frameset>
</html>

ENDDOCUMENT
}

# ---------------------------------------------------------------- Make buttons

sub buttons {
    my $adv=shift;
    my $output='<form method=post>';     
    if ($adv==1) {
	$output.='<input type=submit name=forceadv value="Edit">';
    } else {
        unless ($adv==2) {
           $output.='<input type=submit name=forcesmp value="Simple Edit">';
        }
	$output.='<input type=submit name=forceadv value="Advanced Edit">';
    }
    return $output.'</form><hr>';
}

# ----------------------------------------------------------- Paste into target
# modifies @order, @resources

sub pastetarget {
    my ($after,@which)=@_;
    my @insertorder=();
    foreach (@which) {
        if (defined($_)) {
	    my ($name,$url)=split(/\=/,$_);
            $name=&Apache::lonnet::unescape($name);
            $url=&Apache::lonnet::unescape($url);
            if ($url) {
	       my $idx=$#resources+1;
               $insertorder[$#insertorder+1]=$idx;
               my $ext='false';
               if ($url=~/^http\:\/\//) { $ext='true'; }
               $url=~s/\:/\&colon;/g;
               $name=~s/\:/\&colon;/g;
               $resources[$idx]=$name.':'.$url.':'.$ext.':normal:res';
	   }
        }
    }
    my @oldorder=splice(@order,$after);
    @order=(@order,@insertorder,@oldorder);
}

# ------------------------------------------------ Get start and finish correct
# modifies @resources

sub startfinish {
# Remove all start and finish
    foreach (@order) {
	my ($name,$url,$ext)=split(/\:/,$resources[$_]);
        if ($url=~/http\&colon\:\/\//) { $ext='true'; }
        $resources[$_]=$name.':'.$url.':'.$ext.':normal:res';
    }
# Garbage collection
    my $stillchange=1;
     while (($#order>1) && ($stillchange)) {
       $stillchange=0;
       for (my $i=0;$i<=$#order;$i++) {
 	  my ($name,$url,$ext)=split(/\:/,$resources[$order[$i]]);
          unless ($url) {
# Take out empty resource
              for (my $j=$i+1;$j<=$#order;$j++) {
                  $order[$j-1]=$order[$j];
	      }
              $#order--;
	      $stillchange=1;
              last;
          }
       }
    }
# Put in a start resource
   my ($name,$url,$ext)=split(/\:/,$resources[$order[0]]);
   $resources[$order[0]]=$name.':'.$url.':'.$ext.':start:res';
# Make sure this has at least start and finish
   if ($#order==0) {
       $resources[$#resources+1]='::false';
       $order[1]=$#resources;
   }
# Make the last one a finish resource
   ($name,$url,$ext)=split(/\:/,$resources[$order[$#order]]);
   $resources[$order[$#order]]=$name.':'.$url.':'.$ext.':finish:res';
}

# ------------------------------------------------------------------- Store map

sub storemap {
    my $realfn=shift;
    my $fn=$realfn;
# unless this is forced to work from the original file, use a temporary file
# instead
    unless (shift) {
       $fn=$realfn.'.tmp';
       unless (-e $fn) {
	  copy($realfn,$fn);
       }
    }
# store data either into tmp or real file
    &startfinish();
    my $output='graphdef<:>no';
    my $k=1;
    for (my $i=0; $i<=$#order; $i++) {
        if (defined($resources[$order[$i]])) {
	    $output.='<&>objcont<:>'.$order[$i].'<:>'.$resources[$order[$i]];
        }
        if (defined($order[$i+1])) {
	    if (defined($resources[$order[$i+1]])) {
               $output.='<&>objlinks<:>'.$k.'<:>'.
		   $order[$i].':'.$order[$i+1].':0';
	       $k++;
            }
        }
    }
    $output=~s/http\&colon\;\/\///g;
    $ENV{'form.output'}=$output;
    return 
     &Apache::lonratsrv::loadmap($fn,&Apache::lonratsrv::savemap($fn,''));
}

sub editscript {
    my $mode=shift;
    return(<<ENDSCRIPT);
var srch;
var srchflag=-1; // 1 means currently open
                 // 0 means closed (but has been open)
                 // -1 means never yet opened/defined
var srchmode='';

var idx;
var idxflag=-1; // 1 means currently open
                 // 0 means closed (but has been open)
                 // -1 means never yet opened/defined
var idxmode='';

// ------------------------------------------------------ Clears indexer window
function idxclear() {
  idx.document.clear();
}

// ------------------------------------------------------- Clears search window
function srchclear() {
  srch.document.clear();
}

// ------------------------------------------------------ Closes indexer window
function idxclose() {
  if (idx && !idx.closed) {
    idxflag=0;
    idx.close();
  }
}

// ------------------------------------------------------- Closes search window
function srchclose() {
  if (srch && !srch.closed) {
    srchflag=0;
    srch.close();
  }
}

// -------------------------------------------------------- Open indexer window
function idxopen(mode) {
   var options="scrollbars=1,resizable=1,menubar=0";
   idxmode=mode;
   idxflag=1;
   idx=open("/res/?launch=1&mode=$mode&catalogmode="+mode,"idxout",options);
   idx.focus();
}

// --------------------------------------------------------- Open search window
function srchopen(mode) {
   var options="scrollbars=1,resizable=1,menubar=0";
   srchmode=mode;
   srchflag=1;
   srch=open("/adm/searchcat?launch=1&mode=$mode&catalogmode="+mode,"srchout",options);
   srch.focus();
}
// ----------------------------------------------------- launch indexer browser
function groupsearch() {
   srchcheck('groupsearch');
}

function groupimport() {
   idxcheck('groupimport');
}
// ------------------------------------------------------- Do srch status check
function srchcheck(mode) {
   if (!srch || srch.closed || srchmode!=mode) {
      srchopen(mode);
   }
   srch.focus();
}

// -------------------------------------------------------- Do idx status check
function idxcheck(mode) {
   if (!idx || idx.closed || idxmode!=mode) {
      idxopen(mode);
   }
   idx.focus();
}


    var editbrowser;
    function openbrowser(formname,elementname,only,omit) {
        var url = '/res/?';
        if (editbrowser == null) {
            url += 'launch=1&';
        }
        url += 'catalogmode=interactive&';
        url += 'mode=edit&';
        url += 'form=' + formname + '&';
        if (only != null) {
            url += 'only=' + only + '&';
        } 
        if (omit != null) {
            url += 'omit=' + omit + '&';
        }
        url += 'element=' + elementname + '';
        var title = 'Browser';
        var options = 'scrollbars=1,resizable=1,menubar=0';
        options += ',width=700,height=600';
        editbrowser = open(url,title,options,'1');
        editbrowser.focus();
    }
ENDSCRIPT
}
# ------------------------------------------------------- Simple edit processor

sub smpedt {
   my ($r,$url,$errtext)=@_;
   my $buttons=&buttons(2);
   my $tmpfn=&Apache::lonnet::filelocation('',$url).'.tmp';
   my $targetmsg='';
   if ($ENV{'form.save'}) {
       $targetmsg='<b>Saving ...</b><br>';
       copy($tmpfn,&Apache::lonnet::filelocation('',$url));
       unlink($tmpfn);
       my ($errtext,$fatal)=
                           &mapread(&Apache::lonnet::filelocation('',$url),'');
   }
   if ($ENV{'form.revert'}) {
       $targetmsg='<b>Reverting ...</b><br>';
       unlink($tmpfn);
       my ($errtext,$fatal)=
                           &mapread(&Apache::lonnet::filelocation('',$url),'');
   }
   if (-e $tmpfn) {
      $targetmsg=
        '<b><font color="red">You are working with an unsaved version of your map.</font></b><br>';
      my ($errtext,$fatal)=&mapread($tmpfn,'');
   }
# ---------------------------------------------------------- Process form input

   my @importselect=();
   my @targetselect=();
   undef @importselect;
   undef @targetselect;
   if (defined($ENV{'form.importsel'})) {
       if (ref($ENV{'form.importsel'})) {
	   @importselect=sort(@{$ENV{'form.importsel'}});
       } else {
           @importselect=($ENV{'form.importsel'});
       }
   }
   if (defined($ENV{'form.target'})) {
       if (ref($ENV{'form.target'})) {
	   @targetselect=sort(@{$ENV{'form.target'}});
       } else {
           @targetselect=($ENV{'form.target'});
       }
   }
# ============================================================ Process commands

   my $targetdetail=$ENV{'form.targetdetail'};
   my $importdetail=$ENV{'form.curimpdetail'};

# ---------------------------------------------------- Importing from groupsort
   if (($ENV{'form.importdetail'}) && (!$ENV{'form.impfortarget'})) {

       $importdetail='';
       my @curimport=split(/\&/,$ENV{'form.curimpdetail'});

       my $lastsel;

       if (defined($importselect[-1])) {
	   $lastsel=$importselect[-1];
       } else {
           $lastsel=$#curimport;
       }

       for (my $i=0;$i<=$lastsel;$i++) {
           my ($name,$url)=split(/\=/,$curimport[$i]);
           if ($url) {
              $importdetail.='&'.$name.'='.$url;
	   }
       }

      $importdetail.='&'.$ENV{'form.importdetail'};

       for (my $i=$lastsel+1;$i<=$#curimport;$i++) {
           my ($name,$url)=split(/\=/,$curimport[$i]);
           if ($url) {
              $importdetail.='&'.$name.'='.$url;
	  }
       }
       $importdetail=~s/\&+/\&/g;
       $importdetail=~s/^\&//;

# ------------------------------------------------------------------- Clear all
   } elsif ($ENV{'form.clear'}) {
       $importdetail='';
# ------------------------------------------------------------ Discard selected
   } elsif ($ENV{'form.discard'}) {
       $importdetail='';
       my @curimport=split(/\&/,$ENV{'form.curimpdetail'});
       foreach (@importselect) {
	   $curimport[$_]='';
       }
       for (my $i=0;$i<=$#curimport;$i++) {
           my ($name,$url)=split(/\=/,$curimport[$i]);
           if ($url) {
              $importdetail.='&'.$name.'='.$url;
	   }
       }
# --------------------------------------------------------- Loading another map
   } elsif ($ENV{'form.loadmap'}) {
       $importdetail='';
       my @curimport=split(/\&/,$ENV{'form.curimpdetail'});

       my $lastsel;

       if (defined($importselect[-1])) {
	   $lastsel=$importselect[-1];
       } else {
           $lastsel=$#curimport;
       }

       for (my $i=0;$i<=$lastsel;$i++) {
           my ($name,$url)=split(/\=/,$curimport[$i]);
           if ($url) {
              $importdetail.='&'.$name.'='.$url;
	   }
       }

       foreach (
    &Apache::lonsequence::attemptread(&Apache::lonnet::filelocation('',$ENV{'form.importmap'}))) {
	   my ($name,$url)=split(/\:/,$_);
           if ($url) {
              $importdetail.='&'.&Apache::lonnet::escape($name).'='.
		 	         &Apache::lonnet::escape($url);
	  }
       }

       for (my $i=$lastsel+1;$i<=$#curimport;$i++) {
           my ($name,$url)=split(/\=/,$curimport[$i]);
           if ($url) {
              $importdetail.='&'.$name.'='.$url;
	  }
       }
       $importdetail=~s/\&+/\&/g;
       $importdetail=~s/^\&//;

# ------------------------------------------------ Groupimport/search to target
   } elsif ($ENV{'form.importdetail'}) {
       my $lastsel;
       if (defined($targetselect[-1])) {
	   $lastsel=$targetselect[-1];
       } else {
           $lastsel=$#order+1;
       }
       &pastetarget($lastsel,split(/\&/,$ENV{'form.importdetail'}));
       &storemap(&Apache::lonnet::filelocation('',$url));
# ------------------------------------------------------------------------- Cut
   } elsif (($ENV{'form.cut'}) || ($ENV{'form.copy'})) {
       $importdetail='';
       my @curimport=split(/\&/,$ENV{'form.curimpdetail'});

       my $lastsel;

       if (defined($importselect[-1])) {
	   $lastsel=$importselect[-1];
       } else {
           $lastsel=$#curimport;
       }

       for (my $i=0;$i<=$lastsel;$i++) {
           my ($name,$url)=split(/\=/,$curimport[$i]);
           if ($url) {
              $importdetail.='&'.$name.'='.$url;
	   }
       }

       foreach (@targetselect) {
	   my ($name,$url)=split(/\:/,$resources[$order[$_-1]]);
           if ($url) {
              $importdetail.='&'.&Apache::lonnet::escape($name).'='.
		 	         &Apache::lonnet::escape($url);
	  }
       }

       for (my $i=$lastsel+1;$i<=$#curimport;$i++) {
           my ($name,$url)=split(/\=/,$curimport[$i]);
           if ($url) {
              $importdetail.='&'.$name.'='.$url;
	  }
       }
       $importdetail=~s/\&+/\&/g;
       $importdetail=~s/^\&//;

       if ($ENV{'form.cut'}) {
           my @neworder=();
           for (my $i=0;$i<=$#order;$i++) {
               my $include=1;
               foreach (@targetselect) {
		   if ($_-1==$i) { $include=0; }
               }
               if ($include) { $neworder[$#neworder+1]=$order[$i]; }
           }
           @order=@neworder;
           &storemap(&Apache::lonnet::filelocation('',$url));      
       }

# ----------------------------------------------------------------------- Paste
   } elsif ($ENV{'form.paste'}) {
       my $lastsel;
       if (defined($targetselect[-1])) {
	   $lastsel=$targetselect[-1];
       } else {
           $lastsel=$#order+1;
       }
       my @newsequence;
       my @curimport=split(/\&/,$ENV{'form.curimpdetail'});
       foreach (@importselect) {
          $newsequence[$#newsequence+1]=$curimport[$_];
       }
       &pastetarget($lastsel,@newsequence);
       &storemap(&Apache::lonnet::filelocation('',$url));
# -------------------------------------------------------------------- Move up 
   } elsif ($ENV{'form.moveup'}) {
       foreach (sort @targetselect) {
           if ($_-1>0) {
              my $movethis=$order[$_-1];
              $order[$_-1]=$order[$_-2];
              $order[$_-2]=$movethis;
	  }
       }
       &storemap(&Apache::lonnet::filelocation('',$url));
# ------------------------------------------------------------------ Move down
   } elsif ($ENV{'form.movedown'}) {
       foreach (reverse sort @targetselect) {
           if ($_-1<$#order) {
              my $movethis=$order[$_-1];
              $order[$_-1]=$order[$_];
              $order[$_]=$movethis;
	   }
       }
       &storemap(&Apache::lonnet::filelocation('',$url));
# --------------------------------------------------------------------- Rename
   } elsif ($ENV{'form.renameres'}) {
       my $residx=$Apache::lonratedt::order[$ENV{'form.renameidx'}-1];
       my ($name,@resrest)=split(/\:/,$Apache::lonratedt::resources[$residx]);
       $name=$ENV{'form.renametitle'};
       $name=~s/\:/\&colon\;/g;
       $Apache::lonratedt::resources[$residx]=$name.':'.join(':',@resrest);
       &storemap(&Apache::lonnet::filelocation('',$url));
   }
# ------------------------------------------------------------ Assemble windows
   
   my $idx=-1;
   $importdetail='&'.$importdetail;
   $importdetail=~s/^\&+/\&/;
   my $importwindow=
       '<option value="-1"> ---- Import and Paste Area ---- </option>'.
     join("\n",map {
       $idx++;
       if ($_) { 
          my ($name,$url)=split(/\=/,$_);
          unless ($name) { $name=(split(/\//,$url))[-1]; }
          unless ($name) { $name='EMPTY'; }
          '<option value="'.$idx.'">'.&Apache::lonnet::unescape($name).
                                    '</option>';
      }
   } split(/\&/,$importdetail));

   $idx=0;
   $targetdetail='';
   my $targetwindow=       
       '<option value="0"> ------- Target Edit Map ------- </option>'.
     join("\n",map { 
       my ($name,$url)=split(/\:/,$resources[$_]);
       unless ($name) {  $name=(split(/\//,$url))[-1]; }
       unless ($name) { $name='EMPTY'; }
       $targetdetail.='&'.&Apache::lonnet::escape($name).'='.
	                  &Apache::lonnet::escape($url);
       $idx++;
       $name=~s/\&colon;/\:/g;
       '<option value="'.$idx.'">'.$name.'</option>';
   } @order);

# ----------------------------------------------------- Start simple RAT screen
   my $editscript=&editscript('simple');
   my $bodytag=&Apache::loncommon::bodytag(undef,undef,undef,1);
   $r->print(<<ENDSMPHEAD);
<html>
<head>
<script>

   $editscript

   function openview(entry) {
       var url=unescape((entry.split('='))[1]);
       var parts=new Array;
       parts=url.split('&colon;');
       url=parts.join(':');
       if (url) { open(url,'cat'); }
   }

   function viewtarget() {
       openview((document.forms.simpleedit.targetdetail.value.split('&'))
                [document.forms.simpleedit.target.selectedIndex]);
   }

   function viewimport() {
       openview((document.forms.simpleedit.curimpdetail.value.split('&'))
                [document.forms.simpleedit.importsel.selectedIndex]);
   }

   function renametarget() {
       var selidx=document.forms.simpleedit.target.selectedIndex;
       var entry=(document.forms.simpleedit.targetdetail.value.split('&'))
                [selidx];
       var oldname=unescape((entry.split('='))[0]);
       var nameparts=oldname.split('&colon;');
       oldname=unescape(nameparts.join(':'));
       nameparts=oldname.split('&#34;');
       oldname=unescape(nameparts.join('"'));
       nameparts=oldname.split('&#39;');
       oldname=unescape(nameparts.join("'"));
       newtitle=prompt('New Title',oldname);
       if (newtitle) {
           document.forms.simpleedit.renameres.value=1;
           document.forms.simpleedit.renameidx.value=selidx;
           document.forms.simpleedit.renametitle.value=newtitle;
	   document.forms.simpleedit.submit();
       }
   }

</script>
</head>                 
$bodytag
$buttons
<font color=red>$errtext</font>
<form name=simpleedit method=post>
<input type=hidden name=forcesmp value=1>
<input type=hidden name=renameres value=0>
<input type=hidden name=renametitle value=''>
<input type=hidden name=renameidx value=0>
<table>
    <tr><th width="40%">Temporary Assembly Workspace</th>
<th>&nbsp;</th>
<th width="40%">File: $url</th></tr>
<tr><td bgcolor="#FFFFCC">
<input type=button onClick="javascript:groupsearch()" value="Search">
<input type=button onClick="javascript:groupimport();" value="Import">
after selected
<hr>
<input type=text size=20 name=importmap>
<input type=button 
onClick="javascript:openbrowser('simpleedit','importmap','sequence,page','')"
value="Select Map"><input type=submit name=loadmap value="Load Map"><hr>
<input type=submit name="discard" value="Discard Selected">
<input type=submit name="clear" value="Clear All">
<input type=button onClick="javascript:viewimport()" value="View">

    </td><td>&nbsp;</td><td bgcolor="#FFFFCC">

<input type=button onClick=
"javascript:impfortarget.value=1;groupsearch()" value="Search">
<input type=button onClick=
"javascript:impfortarget.value=1;groupimport();" value="Import">
after selected
<hr>
<input type=submit name="moveup" value="Move Up">
<input type=submit name="movedown" value="Move Down">
<input type=button onClick="javascript:renametarget()" value="Rename">
<hr>$targetmsg
<input type=submit name="revert" value="Revert to Last Saved">
<input type=submit name="save" value="Save">
<input type=button onClick="javascript:viewtarget()" value="View">
</td></tr>

<tr><td bgcolor="#FFFFCC"><select name="importsel" size=10 multiple>
$importwindow
</select>
</td>
<td bgcolor="#FFFFAA" align="center">
Cut selected<br>
<input type=submit name=cut value='<<<'><p>
<hr>
Copy selected<br>
<input type=submit name=copy value='<--'><p>
<hr>
Paste after selected<br>
<input type=submit name=paste value='-->'>
</td>
<td bgcolor="#FFFFCC"><select name="target" size=10 multiple>
$targetwindow
</select>
</table>
<input type=hidden name=importdetail value="">
<input type=hidden name=curimpdetail value="$importdetail">
<input type=hidden name=targetdetail value="$targetdetail">
<input type=hidden name=impfortarget value="0">
</form>
</body></html>
ENDSMPHEAD
}

# ----------------------------------------------------------------- No such dir
sub nodir {
   my ($r,$dir)=@_;
   $dir=~s/^\/home\/\w+\/public\_html//;
   $r->print(<<ENDNODIR);
<html>
<body bgcolor='#FFFFFF'>
<h1>No such directory: $dir</h1>
</body>
</html>
ENDNODIR
}

# ---------------------------------------------------------------- View Handler

sub viewmap {
    my ($r,$url,$adv,$errtext)=@_;
    $r->print('<html>'.
     &Apache::loncommon::bodytag('Edit Content of a Map').
     &buttons($adv));
    if ($errtext) {
	$r->print($errtext.'<hr>');
    }
    my $idx=0;
    $r->print('<h1>'.$url.'</h1>');
    if ($adv) {
	$r->print('<p><b><font color="red">Map contents are not shown in order.</font></b></p><br />');
    }
    foreach (&attemptread(&Apache::lonnet::filelocation('',$url))) {
	if (defined($_)) {
            $idx++;
	    my ($title,$url)=split(/\:/,$_);
            $title=~s/\&colon\;/\:/g;
            $url=~s/\&colon\;/\:/g;
            unless ($title) { $title=(split(/\//,$url))[-1] };
            unless ($title) { $title='<i>Empty</i>'; }
            if ($url) {
		$r->print('<a href="'.&Apache::lonratsrv::qtescape($url).'">');
            }
            $r->print(&Apache::lonratsrv::qtescape($title));
            if ($url) { $r->print('</a>'); }
	    $r->print('<br>');
        }
    }
    $r->print('</body></html>');
}

# ================================================================ Main Handler

sub handler {
  my $r=shift;
  $r->content_type('text/html');
  $r->send_http_header;

  return OK if $r->header_only;

  my $url=$r->uri;
  my $fn=&Apache::lonnet::filelocation('',$url);

  my ($dir)=($fn=~/^(.+)\/[^\/]+$/);
  unless (-e $dir) {
      &nodir($r,$dir);
      return OK;
  }

# ------------------------------------------- Determine which tools can be used
  my $adv=0;

  unless ($ENV{'form.forcesmp'}) {
     if ($ENV{'form.forceadv'}) {
        $adv=1;
     } elsif (my $fh=Apache::File->new($fn)) {
	 my $allmap=join('',<$fh>);
         $adv=($allmap=~/\<map[^\>]+mode\s*\=\s*(\'|\")rat/is);
     }
  }

  my $errtext='';
  my $fatal=0;

# -------------------------------------------------------------------- Load map
  ($errtext,$fatal)=&mapread($fn,$errtext);

  if ($fatal==1) { $adv=1; }

# ----------------------------------- adv==1 now means "graphical MUST be used"

  if ($ENV{'form.forceadv'}) {
      &ratedt($r,$url);
  } elsif ($ENV{'form.forcesmp'}) {
      &smpedt($r,$url,$errtext);
  } else {
      &viewmap($r,$url,$adv,$errtext);
  }
  return OK;
}

1;
__END__







